/*
 * @(#)LibraryInfo.java  1.0.0 / 2008-05-01
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka;

import java.io.File;
import java.util.Locale;

import net.wrka.util.CommonConstants;
import net.wrka.util.GPTools;

/**
 * Information about the library and environment.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class LibraryInfo {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * True when the library is initialized.
   */
  public static final boolean INITIALIZED;

  /**
   * Library version.
   */
  public static final String LIBRARY_VERSION;

  /**
   * Library date.
   */
  public static final String LIBRARY_DATE;

  /**
   * Supported Java platform.
   */
  public static final String LIBRARY_PLATFORM;

  /**
   * Default locale.
   */
  public static final Locale LOCALE_DEFAULT;

  /**
   * Java home directory.
   */
  public static final String JAVA_HOME;

  /**
   * Java version.
   */
  public static final String JAVA_VERSION;

  /**
   * <code>True</code> if Java version is 1.4.
   */
  public static final boolean IS_JAVA_14;

  /**
   * <code>True</code> if Java version is 1.4.
   */
  public static final boolean IS_JAVA_14_OR_HIGHER;

  /**
   * <code>True</code> if Java version is 1.4.1.
   */
  public static final boolean IS_JAVA_141;

  /**
   * <code>True</code> if Java version is 1.4.1.
   */
  public static final boolean IS_JAVA_141_OR_HIGHER;

  /**
   * <code>True</code> if Java version is 1.4.2.
   */
  public static final boolean IS_JAVA_142;

  /**
   * <code>True</code> if Java version is 1.4.
   */
  public static final boolean IS_JAVA_142_OR_HIGHER;

  /**
   * <code>True</code> if Java version is 1.5.
   */
  public static final boolean IS_JAVA_15;

  /**
   * <code>True</code> if Java version is 1.5.
   */
  public static final boolean IS_JAVA_15_OR_HIGHER;

  /**
   * <code>True</code> if Java version is 1.6.
   */
  public static final boolean IS_JAVA_16;

  /**
   * <code>True</code> if Java version is 1.6.
   */
  public static final boolean IS_JAVA_16_OR_HIGHER;

  /**
   * The name of the operating system.
   */
  public static final String OS_NAME;

  /**
   * The version of the operating system.
   */
  public static final String OS_VERSION;

  /**
   * <code>True</code> if OS is Linux.
   */
  public static final boolean OS_LINUX;

  /**
   * <code>True</code> if OS is MSWindows.
   */
  public static final boolean OS_WINDOWS;

  /**
   * <code>True</code> if OS is Mac.
   */
  public static final boolean OS_MAC;

  /**
   * <code>True</code> if OS is OS2.
   */
  public static final boolean OS_OS2;

  /**
   * <code>True</code> if OS is BEOS.
   */
  public static final boolean OS_BEOS;

  /**
   * <code>True</code> if OS is Solaris.
   */
  public static final boolean OS_SOLARIS;

  /**
   * Line separator.
   */
  public static final String LINE_SEPARATOR;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static boolean discoverLibraryFiles;
  private static File libraryLocation;
  private static File libraryJarFile;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    LIBRARY_VERSION = "1.0.0"; //$NON-NLS-1$
    LIBRARY_DATE = "May 20, 2008"; //$NON-NLS-1$
    LIBRARY_PLATFORM = "1.4"; //$NON-NLS-1$

    LOCALE_DEFAULT = Locale.ENGLISH;
    String ls = System.getProperty("line.separator"); //$NON-NLS-1$
    if (ls == null) {
      ls = CommonConstants.STRING_DEFAULT_LINE_SEPARATOR;
    }
    LINE_SEPARATOR = ls;
    discoverLibraryFiles = true;

    JAVA_HOME = GPTools.getSystemProperty("java.home"); //$NON-NLS-1$
    JAVA_VERSION = GPTools.getSystemProperty("java.version"); //$NON-NLS-1$
    boolean javaFound = JAVA_VERSION.startsWith("1.6"); //$NON-NLS-1$
    if (javaFound) {
      IS_JAVA_16 = true;
      javaFound = true;
    } else {
      IS_JAVA_16 = false;
    }
    if (!javaFound && JAVA_VERSION.startsWith("1.5")) { //$NON-NLS-1$
      IS_JAVA_15 = true;
      javaFound = true;
    } else {
      IS_JAVA_15 = false;
    }
    if (!javaFound && JAVA_VERSION.startsWith("1.4.2")) { //$NON-NLS-1$
      IS_JAVA_142 = true;
      javaFound = true;
    } else {
      IS_JAVA_142 = false;
    }
    if (!javaFound && JAVA_VERSION.startsWith("1.4.1")) { //$NON-NLS-1$
      IS_JAVA_141 = true;
      javaFound = true;
    } else {
      IS_JAVA_141 = false;
    }
    if (!javaFound && JAVA_VERSION.startsWith("1.4")) { //$NON-NLS-1$
      IS_JAVA_14 = true;
    } else {
      IS_JAVA_14 = false;
    }
    IS_JAVA_16_OR_HIGHER = IS_JAVA_16 ||
         GPTools.isValidVersion(JAVA_VERSION, "1.6"); //$NON-NLS-1$
    IS_JAVA_15_OR_HIGHER = IS_JAVA_16_OR_HIGHER || IS_JAVA_15 ||
         GPTools.isValidVersion(JAVA_VERSION, "1.5"); //$NON-NLS-1$
    IS_JAVA_142_OR_HIGHER = IS_JAVA_15_OR_HIGHER || IS_JAVA_142 ||
         GPTools.isValidVersion(JAVA_VERSION, "1.4.2"); //$NON-NLS-1$
    IS_JAVA_141_OR_HIGHER = IS_JAVA_142_OR_HIGHER || IS_JAVA_141 ||
         GPTools.isValidVersion(JAVA_VERSION, "1.4.1"); //$NON-NLS-1$
    IS_JAVA_14_OR_HIGHER = IS_JAVA_141_OR_HIGHER || IS_JAVA_14 ||
         GPTools.isValidVersion(JAVA_VERSION, "1.4"); //$NON-NLS-1$
    OS_VERSION = GPTools.getSystemProperty("os.version"); //$NON-NLS-1$
    OS_NAME = GPTools.getSystemProperty("os.name"); //$NON-NLS-1$
    boolean systemFound = GPTools.startsWithIgnoreCase(OS_NAME,
                                                       "Linux"); //$NON-NLS-1$
    if (systemFound) {
      OS_LINUX = true;
      systemFound = true;
    } else {
      OS_LINUX = false;
    }
    if (!systemFound &&
        GPTools.startsWithIgnoreCase(OS_NAME, "Windows")) { //$NON-NLS-1$
      OS_WINDOWS = true;
      systemFound = true;
    } else {
      OS_WINDOWS = false;
    }
    if (!systemFound &&
        GPTools.startsWithIgnoreCase(OS_NAME, "Mac")) { //$NON-NLS-1$
      OS_MAC = true;
      systemFound = true;
    } else {
      OS_MAC = false;
    }
    if (!systemFound &&
        GPTools.startsWithIgnoreCase(OS_NAME, "OS2")) { //$NON-NLS-1$
      OS_OS2 = true;
      systemFound = true;
    } else {
      OS_OS2 = false;
    }
    if (!systemFound &&
        GPTools.startsWithIgnoreCase(OS_NAME, "Beos")) { //$NON-NLS-1$
      OS_BEOS = true;
      systemFound = true;
    } else {
      OS_BEOS = false;
    }
    if (!systemFound &&
        GPTools.startsWithIgnoreCase(OS_NAME, "Solaris")) { //$NON-NLS-1$
      OS_SOLARIS = true;
    } else {
      OS_SOLARIS = false;
    }

    INITIALIZED = true;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private LibraryInfo() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Returns library location.
   *
   * @return library location
   */
  public static File getLibraryLocation() {
    if (discoverLibraryFiles) {
      getLibraryFiles();
    }
    return libraryLocation;
  }

  /**
   * Returns library jar file.
   *
   * @return library jar file
   */
  public static File getLibraryJarFile() {
    if (discoverLibraryFiles) {
      getLibraryFiles();
    }
    return libraryJarFile;
  }

/*^**************************************************************************^*/
/*- Private Static Method.                                                   -*/
/*^**************************************************************************^*/

  private static void getLibraryFiles() {
    String libPaht = GPTools.getCodeSource(LibraryInfo.class);
    if (libPaht != null) {
      File f = new File(libPaht);
      if (f.exists()) {
        if (f.isFile()) {
          libraryJarFile = f;
          libraryLocation = f.getParentFile();
        } else {
          libraryJarFile = null;
          libraryLocation = f;
        }
      }
    }
    discoverLibraryFiles = false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-01 20:29:10 Slobodan
   Initial version

*****************************************************************************^*/
