/*
 * @(#)Clipboard.java  1.0.0 / 2008-05-05
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.awt;

import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;

import net.wrka.util.CommonConstants;

/**
 * Clipboard.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class Clipboard {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final java.awt.datatransfer.Clipboard SYSTEM_CLIPBOARD;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SYSTEM_CLIPBOARD = Toolkit.getDefaultToolkit().getSystemClipboard();
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private Clipboard() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Copies the specified string to the system SYSTEM_CLIPBOARD.
   *
   * @param string the string to be copied
   */
  public static void copyString(String string) {
    try {
      StringSelection stringSelection = new StringSelection(string);
      SYSTEM_CLIPBOARD.setContents(stringSelection, null);
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
  }

  /**
   * Pastes string from the system SYSTEM_CLIPBOARD.
   *
   * @return string from the system SYSTEM_CLIPBOARD
   */
  public static String pasteString() {
    String result = CommonConstants.STRING_EMPTY;
    try {
      Transferable contents = SYSTEM_CLIPBOARD.getContents(null);
      if ((contents != null) &&
          contents.isDataFlavorSupported(DataFlavor.stringFlavor)) {
        result = (String)contents.getTransferData(DataFlavor.stringFlavor);
      }
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
    return result;
  }

  /**
   * Returns <tt>true</tt> if SYSTEM_CLIPBOARD contains string.
   *
   * @return <tt>true</tt> if SYSTEM_CLIPBOARD contains string
   */
  public static boolean containsString() {
    try {
      Transferable contents = SYSTEM_CLIPBOARD.getContents(null);
      return ((contents != null) &&
              contents.isDataFlavorSupported(DataFlavor.stringFlavor));
    } catch (Throwable t) { // NOPMD
      return false;
    }
  }

  /**
   * Returns <tt>true</tt> if SYSTEM_CLIPBOARD contains image.
   *
   * @return <tt>true</tt> if SYSTEM_CLIPBOARD contains image
   */
  public static boolean containsImage() {
    try {
      Transferable contents = SYSTEM_CLIPBOARD.getContents(null);
      return ((contents != null) &&
              contents.isDataFlavorSupported(DataFlavor.imageFlavor));
    } catch (Throwable t) { // NOPMD
      return false;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-05 22:08:13 Slobodan
   Initial version

*****************************************************************************^*/
