/*
 * @(#)FileUtils.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.io;

import java.io.File;
import java.util.Locale;

/**
 * File Utilities.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class FileUtils {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private FileUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns file extension.
   *
   * @param file the file
   *
   * @return file extension
   */
  public static String getFileExtension(File file) {
    if (file != null) {
      String filename = file.getName();
      int i = filename.lastIndexOf('.');
      if ((i > 0) && (i < filename.length() - 1)) {
        return filename.substring(i + 1).toLowerCase(Locale.getDefault());
      }
    }
    return null;
  }

  /**
   * Changes file extension of the specified file.
   *
   * @param fileName the file name
   * @param fileExt the new file extension
   *
   * @return file name with changed extension
   */
  public static String changeFileExt(String fileName, String fileExt) {
    if ((fileName == null) || (fileName.length() == 0)) {
      throw new IllegalArgumentException("Invalid file name"); //$NON-NLS-1$
    }
    if ((fileExt == null) || (fileExt.length() == 0)) {
      throw new IllegalArgumentException(
                                       "Invalid file extension"); //$NON-NLS-1$
    }
    int ldi = fileName.lastIndexOf('.');
    if (ldi > 0) {
      StringBuffer sb = new StringBuffer(fileName.substring(0, ldi + 1));
      sb.append(fileExt);
      return sb.toString();
    }
    return fileName;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 14:06:12 Slobodan
   Initial version

*****************************************************************************^*/
