/*
 * @(#)ExternalLauncher.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.natives;

import java.net.URL;

import net.wrka.LibraryInfo;

/**
 * Cross platform external program launcher.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ExternalLauncher {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * <tt>True</tt> if the launcher is available.
   */
  public static final boolean AVAILABLE;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                -*/
/*^**************************************************************************^*/

  private static final String WIN_BROWSER;
  private static final String NETSCAPE;
  private static final String NETSCAPE_RMT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    WIN_BROWSER =  "rundll32 url.dll,FileProtocolHandler "; //$NON-NLS-1$
    NETSCAPE = "netscape"; //$NON-NLS-1$
    NETSCAPE_RMT = "netscape -remote openURL("; //$NON-NLS-1$
    boolean available = NativeManager.hasNativeSupport();
    if (!available) {
      if (LibraryInfo.OS_WINDOWS) {
        available = true;
      } else {
        try {
          Runtime.getRuntime().exec("netscape --version"); //$NON-NLS-1$
          available = true;
        } catch (Throwable t) { // NOPMD
          // empty catch block
        }
      }
    }
    AVAILABLE = available;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private ExternalLauncher() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Platform specific opening of the specified url.
   *
   * @param url the url
   *
   * @return <code>false</code> if operation failed
   */
  public static boolean openURL(String url) {
    try {
      return openURL(new URL(url));
    } catch (Throwable t) { // NOPMD
      return false;
    }
  }

  /**
   * Platform specific opening of the specified URL.
   *
   * @param url the URL
   *
   * @return <code>false</code> if operation failed
   */
  public static boolean openURL(URL url) {
    if ((url != null) && AVAILABLE) {
      if (NativeManager.hasNativeSupport()) {
        return NativeManager.getNativeSupport().openURL(url);
      }
      try {
        if (LibraryInfo.OS_WINDOWS) {
          String cmd = WIN_BROWSER.concat(url.toExternalForm());
          Runtime.getRuntime().exec(cmd);
        } else if (LibraryInfo.OS_MAC) {
          String[] commandLine = {
            NETSCAPE, url.toExternalForm()
          };
          Runtime.getRuntime().exec(commandLine);
        } else {
          StringBuffer sb = new StringBuffer(NETSCAPE_RMT);
          sb.append(url.toExternalForm());
          sb.append(')');
          Process process = Runtime.getRuntime().exec(sb.toString());
          if (process.waitFor() != 0) {
            sb.setLength(0);
            sb.append(NETSCAPE);
            sb.append(' ');
            sb.append(url.toExternalForm());
            Runtime.getRuntime().exec(sb.toString());
          }
        }
        return true;
      } catch (Throwable t) { // NOPMD
        // empty catch block
      }
    }
    return false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 13:48:14 Slobodan
   Initial version

*****************************************************************************^*/
