/*
 * @(#)NativeSupport.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.natives;

import java.io.File;
import java.net.URL;

/**
 * Native platform support.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface NativeSupport {

  /**
   * Returns <code>true</code> if support is successfully loaded.
   *
   * @return <code>true</code> if support is successfully loaded
   */
  public boolean isLoaded();

  /**
   * Opens the specified file with platform default program.
   *
   * @param fileName the file name
   *
   * @return <code>false</code> if operation failed
   */
  public boolean openFile(String fileName);

  /**
   * Opens the specified file with platform default program.
   *
   * @param file the file
   *
   * @return <code>false</code> if operation failed
   */
  public boolean openFile(File file);

  /**
   * Opens the specified url with platform default program.
   *
   * @param url the url
   *
   * @return <code>false</code> if operation failed
   */
  public boolean openURL(String url);

  /**
   * Opens the specified url with platform default program.
   *
   * @param url the url
   *
   * @return <code>false</code> if operation failed
   */
  public boolean openURL(URL url);

  /**
   * Executes the specified command.
   *
   * @param command the command
   *
   * @return <code>false</code> if operation failed
   */
  public boolean exec(String command);

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 13:39:36 Slobodan
   Initial version

*****************************************************************************^*/
