/*
 * @(#)AboutAction.java  1.0.0 / 2008-05-14
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.util.ResourceBundle;

import javax.swing.Icon;

import net.wrka.swing.resources.BuiltInResources;

/**
 * Action About.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class AboutAction extends CommonAction {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Name resource key (the value is {@value}).
   */
  public static final String RK_NAME_KEY = "AboutAction.name"; //$NON-NLS-1$

  /**
   * Description resource key (the value is {@value}).
   */
  public static final String RK_SHORT_DESCRIPTION =
                                  "AboutAction.shortDescription"; //$NON-NLS-1$

  /**
   * Small icon resource key (the value is {@value}).
   */
  public static final String RK_SMALL_ICON =
                                         "AboutAction.smallIcon"; //$NON-NLS-1$

  /**
   * Action command resource key (the value is {@value}).
   */
  public static final String RK_ACTION_COMMAND_KEY =
                                  "AboutAction.actionCommandKey"; //$NON-NLS-1$

  /**
   * Mnemonic resource key (the value is {@value}).
   */
  public static final String RK_MNEMONIC_KEY =
                                        "AboutAction.memonicKey"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new AboutAction object.
   */
  public AboutAction() {
    super();
  }

  /**
   * Constructs new AboutAction object.
   *
   * @param name the action name
   */
  public AboutAction(String name) {
    super(name);
  }

  /**
   * Constructs new AboutAction object.
   *
   * @param name the action name
   * @param description the short description
   */
  public AboutAction(String name, String description) {
    super(name, description);
  }

  /**
   * Constructs new AboutAction object.
   *
   * @param name the action name
   * @param icon the icon icon
   */
  public AboutAction(String name, Icon icon) {
    super(name, icon);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setDefaults() {
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                              "ActionResources"); //$NON-NLS-1$
    setName(rb, RK_NAME_KEY);
    setShortDescription(rb, RK_SHORT_DESCRIPTION);
    setSmallIcon(rb, RK_SMALL_ICON);
    setActionCommandKey(rb, RK_ACTION_COMMAND_KEY);
    setMnemonicKey(rb, RK_MNEMONIC_KEY);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-03-07 15:10:02 Slobodan
   Initial version

*****************************************************************************^*/
