/*
 * @(#)ActionUtils.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.KeyStroke;

/**
 * Action Utilities.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ActionUtils {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private ActionUtils() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Setups the specified action.
   *
   * @param action the Action
   * @param name the action name
   * @param mnemonic the action mnemonic
   * @param accelerator the action accelerator
   * @param icon the action icon
   * @param tooltip the action tooltip
   */
  public static void setupAction(Action action, String name, int mnemonic,
                                 KeyStroke accelerator, Icon icon,
                                 String tooltip) {
    action.putValue(Action.NAME, name);
    action.putValue(Action.MNEMONIC_KEY, new Integer(mnemonic));
    if (accelerator != null) {
      action.putValue(Action.ACCELERATOR_KEY, accelerator);
    }
    if (icon != null) {
      action.putValue(Action.SMALL_ICON, icon);
    }
    if (tooltip != null) {
      action.putValue(Action.SHORT_DESCRIPTION, tooltip);
    }
  }

  /**
   * Setups the specified action.
   *
   * @param action the Action
   * @param name the action name
   * @param mnemonic the action mnemonic
   */
  public static void setupAction(Action action, String name, int mnemonic) {
    setupAction(action, name, mnemonic, null, null, null);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-14 20:41:36 Slobodan
   Initial version

*****************************************************************************^*/
