/*
 * @(#)CloseAction.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.util.ResourceBundle;

import javax.swing.Icon;

import net.wrka.swing.resources.BuiltInResources;

/**
 * Close Action.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class CloseAction extends CommonAction {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Name resource key (the value is {@value}).
   */
  public static final String RK_NAME = "CloseAction.name"; //$NON-NLS-1$

  /**
   * Short description resource key (the value is {@value}).
   */
  public static final String RK_SHORT_DESCRIPTION =
                                  "CloseAction.shortDescription"; //$NON-NLS-1$
  /**
   * Long description resource key (the value is {@value}).
   */
  public static final String RK_LONG_DESCRIPTION =
                                   "CloseAction.longDescription"; //$NON-NLS-1$

  /**
   * Small icon resource key (the value is {@value}).
   */
  public static final String RK_SMALL_ICON =
                                         "CloseAction.smallIcon"; //$NON-NLS-1$

  /**
   * Action command resource key (the value is {@value}).
   */
  public static final String RK_ACTION_COMMAND_KEY =
                                  "CloseAction.actionCommandKey"; //$NON-NLS-1$

  /**
   * Accelerator resource key (the value is {@value}).
   */
  public static final String RK_ACCELERATOR_KEY =
                                    "CloseAction.acceleratorKey"; //$NON-NLS-1$

  /**
   * Mnemonic resource key (the value is {@value}).
   */
  public static final String RK_MNEMONIC_KEY =
                                        "CloseAction.memonicKey"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new CloseAction object.
   */
  public CloseAction() {
    super();
  }

  /**
   * Constructs new CloseAction object.
   *
   * @param name the action name
   */
  public CloseAction(String name) {
    super(name);
  }

  /**
   * Constructs new CloseAction object.
   *
   * @param name the action name
   * @param description the action description
   */
  public CloseAction(String name, String description) {
    super(name, description);
  }

  /**
   * Constructs new CloseAction object.
   *
   * @param name the action name
   * @param icon the action icon
   */
  public CloseAction(String name, Icon icon) {
    super(name, icon);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setDefaults() {
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                              "ActionResources"); //$NON-NLS-1$
    setName(rb, RK_NAME);
    setShortDescription(rb, RK_SHORT_DESCRIPTION);
    setLongDescription(rb, RK_LONG_DESCRIPTION);
    setSmallIcon(rb, RK_SMALL_ICON);
    setActionCommandKey(rb, RK_ACTION_COMMAND_KEY);
    setAcceleratorKey(rb, RK_ACCELERATOR_KEY);
    setMnemonicKey(rb, RK_MNEMONIC_KEY);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 15:14:07 Slobodan
   Initial version

*****************************************************************************^*/
