/*
 * @(#)LaunchAction.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Icon;
import javax.swing.SwingUtilities;

import net.wrka.natives.ExternalLauncher;

/**
 * Launch Action.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class LaunchAction extends AbstractAction implements Runnable {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 5430366222144074008L;

/*^**************************************************************************^*/
/*- Protected Field.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Link string.
   */
  protected String link;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new LaunchAction object.
   *
   * @param name the action name
   * @param shortDescription the short description
   * @param smallIcon the small icon
   * @param link the link
   *
   * @throws IllegalArgumentException if link is <tt>null</tt>
   */
  public LaunchAction(String name, String shortDescription, Icon smallIcon,
                      String link) {
    super();
    if (link == null) {
      throw new IllegalArgumentException("Null link"); //$NON-NLS-1$
    }
    this.link = link;
    if (name != null) {
      putValue(NAME, name);
    }
    if (shortDescription != null) {
      putValue(SHORT_DESCRIPTION, shortDescription);
    }
    if (smallIcon != null) {
      putValue(SMALL_ICON, smallIcon);
    }
  }

/*^**************************************************************************^*/
/*- ActionListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    SwingUtilities.invokeLater(this);
  }

/*^**************************************************************************^*/
/*- Runnable Implementation.                                                 -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void run() {
    ExternalLauncher.openURL(link);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 17:48:32 Slobodan
   Initial version

*****************************************************************************^*/
