/*
 * @(#)ErrorDialog.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.ResourceBundle;

import javax.swing.FocusManager;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.UIManager;

import net.wrka.awt.AwtTools;
import net.wrka.swing.components.JWxButton;
import net.wrka.swing.resources.BuiltInResources;

/**
 * Critical Error Dialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class ErrorDialog extends JDialog implements WindowListener {

/*^**************************************************************************^*/
/*- Package-Private Static Method.                                           -*/
/*^**************************************************************************^*/

  static void display() {
    Window activeWindow = FocusManager.getCurrentManager().getActiveWindow();
    if (activeWindow instanceof Dialog) {
      new ErrorDialog((Dialog)activeWindow);
    } else if (activeWindow instanceof Frame) {
      new ErrorDialog((Frame)activeWindow);
    } else {
      new ErrorDialog((Frame)null);
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Constructors.                                            -*/
/*^**************************************************************************^*/

  ErrorDialog(Dialog parent) {
    super(parent, true);
  }

  ErrorDialog(Frame parent) {
    super(parent, true);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void dialogInit() {
    super.dialogInit();
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    Container cp = getContentPane();
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                                     AppContainer.RESOURCES);
    setTitle(AppContainer.getApplicationName().concat(
                             rb.getString(AppContainer.RK_ERROR_DIALOG_TITLE)));
    setResizable(false);
    cp.setLayout(new GridBagLayout());
    GridBagConstraints gbc = new GridBagConstraints();
    gbc.insets = new Insets(4, 8, 0, 22);
    JLabel jle = new JLabel(rb.getString(
                            AppContainer.RK_ERROR_DIALOG_MSG_EXCEPTION));
    gbc.anchor = GridBagConstraints.SOUTH;
    gbc.gridwidth = 2;
    gbc.gridy = 0;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.gridx = 1;
    cp.add(jle, gbc);
    jle = new JLabel(rb.getString(
                               AppContainer.RK_ERROR_DIALOG_MSG_TERMINATE));
    gbc.anchor = GridBagConstraints.NORTH;
    gbc.gridwidth = 2;
    gbc.insets.top = 2;
    gbc.gridx = 1;
    gbc.gridy = 1;
    cp.add(jle, gbc);
    TerminateButton jbTerminate = new TerminateButton(
                          rb.getString(AppContainer.RK_ERROR_DIALOG_TERMINATE));
    IgnoreButton jbIgnore = new IgnoreButton(
                            rb.getString(AppContainer.RK_ERROR_DIALOG_IGNORE));
    DetailsButton jbStackTrace = new DetailsButton(
                            rb.getString(AppContainer.RK_ERROR_DIALOG_DETAILS));
    JPanel tmp = new JPanel(new GridLayout(1, 3, 4, 2));
    tmp.add(jbTerminate);
    tmp.add(jbIgnore);
    tmp.add(jbStackTrace);
    gbc.gridx = 2;
    gbc.gridy = 2;
    gbc.anchor = GridBagConstraints.EAST;
    gbc.insets.bottom = 8;
    gbc.insets.top = 16;
    gbc.insets.right = 8;
    cp.add(tmp, gbc);
    try {
      Icon errIcon = (Icon)UIManager.getDefaults().
                                     get("OptionPane.errorIcon"); //$NON-NLS-1$
      jle = new JLabel(errIcon);
      gbc.insets.top = 10;
      gbc.insets.left = 10;
      gbc.insets.right = 8;
      gbc.insets.bottom = 8;
      gbc.gridx = 0;
      gbc.gridy = 0;
      gbc.gridwidth = 1;
      gbc.gridheight = 2;
      cp.add(jle, gbc);
    } catch (Throwable t) { // NOPMD
      // empty catch body
    }
    addWindowListener(this);
    FocusManager.getCurrentManager().focusNextComponent(this);
    getRootPane().setDefaultButton(jbTerminate);
    pack();
    AwtTools.screenCenterWindow(this);
    setVisible(true);
  }

/*^**************************************************************************^*/
/*- WindowListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void windowClosing(WindowEvent e) {
    AppContainer.shutDownApplicaiton(-1);
  }

  /**
   * {@inheritDoc}
   */
  public void windowClosed(WindowEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void windowOpened(WindowEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void windowIconified(WindowEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void windowDeiconified(WindowEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void windowActivated(WindowEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void windowDeactivated(WindowEvent e) {
    // empty method body
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Terminate Button.
   */
  static final class TerminateButton extends JWxButton {
    /**
     * Constructs new TerminateButton object.
     *
     * @param label the button label
     */
    public TerminateButton(String label) {
      super(label);
      setDefaultCapable(true);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      AppContainer.shutDownApplicaiton(-1);
    }
  }

  /**
   * Ignore Button.
   */
  final class IgnoreButton extends JWxButton {
    /**
     * Constructs new IgnoreButton object.
     *
     * @param label the button label
     */
    public IgnoreButton(String label) {
      super(label);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      AppContainer.getErrorOutputStream().reset();
      dispose();
    }
  }

  /**
   * Details Button.
   */
  final class DetailsButton extends JWxButton {
    /**
     * Constructs new DetailsButton object.
     *
     * @param label the button label
     */
    public DetailsButton(String label) {
      super(label);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      new StackTrace(ErrorDialog.this);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 09:45:13 Slobodan
   Initial version

*****************************************************************************^*/
