/*
 * @(#)ErrorOutputStream.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

import java.io.ByteArrayOutputStream;

/**
 * Error Output Stream.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class ErrorOutputStream extends ByteArrayOutputStream {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int MAX_SIZE = 1024;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean errorOccurred;
  private String toStr;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ErrorOutputStream.
   */
  public ErrorOutputStream() {
    super(MAX_SIZE);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void write(int b) {
    synchronized (this) {
      if (count < MAX_SIZE) {
        if (toStr != null) {
          toStr = null;
        }
        buf[count++] = (byte)b;
        if (!errorOccurred) {
          errorOccurred = true;
          fireError();
        }
      } //else IGNORE
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(byte[] b, int off, int len) {
    synchronized (this) {
      if ((count) < MAX_SIZE) {
        if (toStr != null) {
          toStr = null;
        }
        super.write(b, off, len);
        if (!errorOccurred) {
          errorOccurred = true;
          fireError();
        }
      } //else IGNORE
    }
  }

  /**
   * {@inheritDoc}
   */
  public void reset() {
    synchronized (this) {
      count = 0;
      errorOccurred = false;
    }
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    synchronized (this) {
      if (toStr == null) {
        toStr = new String(buf, 0, count);
      }
      return toStr;
    }
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void fireError() {
    AppContainer.fireError();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 09:22:16 Slobodan
   Initial version

*****************************************************************************^*/
