/*
 * @(#)ExceptionWatchThreadGroup.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.appcontainer;

/**
 * Print Exception Thread Group.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ExceptionWatchThreadGroup extends ThreadGroup {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ExceptionWatchThreadGroup object.
   *
   * @param name the thread group name
   */
  public ExceptionWatchThreadGroup(String name) {
    super(name);
  }

  /**
   * Constructs new ExceptionWatchThreadGroup object.
   *
   * @param name the thread group name
   * @param deamon <tt>true</tt> if the thread group is deamon
   */
  public ExceptionWatchThreadGroup(String name, boolean deamon) {
    super(name);
    setDaemon(deamon);
  }

  /**
   * Constructs new ExceptionWatchThreadGroup object.
   *
   * @param parent the group parent
   * @param name the thread group name
   */
  public ExceptionWatchThreadGroup(ThreadGroup parent, String name) {
    super(parent, name);
  }

  /**
   * Constructs new ExceptionWatchThreadGroup object.
   *
   * @param parent the group parent
   * @param name the thread group name
   * @param deamon <tt>true</tt> if the thread group is deamon
   */
  public ExceptionWatchThreadGroup(ThreadGroup parent, String name,
                                   boolean deamon) {
    super(parent, name);
    setDaemon(deamon);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void uncaughtException(Thread t, Throwable e) {
    if (!(e instanceof ThreadDeath)) {
      e.printStackTrace(System.err);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 09:46:23 Slobodan
   Initial version

*****************************************************************************^*/
