/*
 * @(#)DarkShadowLineBorder.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.borders;

import java.awt.Color;
import java.awt.Component;
import java.awt.Insets;

import javax.swing.LookAndFeel;
import javax.swing.border.LineBorder;

import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Dark shadow line border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class DarkShadowLineBorder extends LineBorder
                                  implements LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Public Static Final Field.                                               -*/
/*^**************************************************************************^*/

  /**
   * Singleton instance.
   */
  public static final DarkShadowLineBorder SHARED_INSTANCE;

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 7389691207677511251L;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SHARED_INSTANCE = new DarkShadowLineBorder();
  }

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Border insets.
   */
  protected final Insets insets;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new DarkShadowLineBorder object.
   */
  public DarkShadowLineBorder() {
    super(null);
    initColors();
    insets = new Insets(thickness, thickness, thickness, thickness);
    LookAndFeelManager.addLnFListener(this); // NOTE: this is weak listener list
  }

  /**
   * Constructs new DarkShadowLineBorder object.
   *
   * @param thickness the line thickness
   */
  public DarkShadowLineBorder(int thickness)  {
    this();
    this.thickness = thickness;
  }

  /**
   * Constructs new DarkShadowLineBorder object.
   *
   * @param thickness line thickness
   * @param roundedCorners whether or not border corners should be round
   */
  public DarkShadowLineBorder(int thickness, boolean roundedCorners)  {
    this();
    this.thickness = thickness;
    this.roundedCorners = roundedCorners;
  }

  /**
   * Constructs new DarkShadowLineBorder object.
   *
   * @param color line color
   */
  public DarkShadowLineBorder(Color color) {
    this(color, 1, false);
  }

  /**
   * Constructs new DarkShadowLineBorder object.
   *
   * @param color line color
   * @param thickness line thickness
   */
  public DarkShadowLineBorder(Color color, int thickness)  {
    this(color, thickness, false);
  }

  /**
   * Constructs new DarkShadowLineBorder object.
   *
   * @param color line color
   * @param thickness line thickness
   * @param roundedCorners whether or not border corners should be round
   */
  public DarkShadowLineBorder(Color color, int thickness,
                              boolean roundedCorners)  {
    super(color, thickness, roundedCorners);
    insets = new Insets(thickness, thickness, thickness, thickness);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)insets.clone();
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    initColors();
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Initializes colors.
   */
  protected void initColors() {
    lineColor = LookAndFeelManager.getControlDarkShadow();
    if (lineColor == null) {
      lineColor = Color.BLACK;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 21:51:59 Slobodan
   Initial version

*****************************************************************************^*/
