/*
 * @(#)GlassPaneDialog.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.CompoundBorder;

import net.wrka.swing.borders.GlassPaneDialogBorder;
import net.wrka.swing.borders.ThinBevelBorder;

/**
 * Glass Pane Dialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class GlassPaneDialog extends JPanel {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  private final JWxGlassPane glassPane;
  private final String cancelMessage;
  private final JLabel iconLabel;
  private final JLabel blockMessageLabel;
  private final JButton buttonCancel;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GlassPaneDialog object.
   *
   * @param glassPane the glass pane
   * @param blockMessage the block message
   * @param cancelMessage the cancel message
   * @param cancelButtonText the cancel button text
   */
  public GlassPaneDialog(JWxGlassPane glassPane, String blockMessage,
                                   String cancelMessage,
                                   String cancelButtonText) {
    super(new BorderLayout());
    this.glassPane = glassPane;
    this.cancelMessage = cancelMessage;
    JPanel panel = new JPanel(new GridBagLayout());
    iconLabel = new JLabel();
    blockMessageLabel = new JLabel(blockMessage);
    buttonCancel = new JButton(cancelButtonText);
    setOpaque(false);
    setMinimumSize(new Dimension(200, 80));
    setPreferredSize(new Dimension(210, 90));
    setMaximumSize(new Dimension(Short.MAX_VALUE, Short.MAX_VALUE));
    setBorder(new CompoundBorder(new GlassPaneDialogBorder(),
                                 new ThinBevelBorder(ThinBevelBorder.
                                                                TYPE_RAISED)));
    panel.setOpaque(true);
    GridBagConstraints gbc = new GridBagConstraints(
        0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
        GridBagConstraints.NONE, new Insets(14, 14, 8, 8), 0, 0);
    panel.add(iconLabel, gbc);
    gbc.gridx = 1;
    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.anchor = GridBagConstraints.CENTER;
    panel.add(blockMessageLabel, gbc);
    gbc.gridx = 0;
    gbc.gridy = 1;
    gbc.gridwidth = 2;
    gbc.insets.top = 12;
    gbc.anchor = GridBagConstraints.CENTER;
    panel.add(buttonCancel, gbc);
    add(panel);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns glass pane.
   *
   * @return glass pane
   */
  public JWxGlassPane getGlassPane() {
    return glassPane;
  }

  /**
   * Returns icon.
   *
   * @return Icon
   */
  public Icon getIcon() {
    return iconLabel.getIcon();
  }

  /**
   * Returns blocking message.
   *
   * @return blocking message
   */
  public String getBlockMessage() {
    return blockMessageLabel.getText();
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    iconLabel.setVisible(visible);
    buttonCancel.setVisible(visible);
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Cancels dialog.
   */
  protected void doCancel() {
    blockMessageLabel.setText(cancelMessage);
    buttonCancel.setVisible(false);
    iconLabel.setVisible(false);
    if (glassPane != null) {
      glassPane.cancel();
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Cancel Button.
   */
  protected class CancelButton extends JWxButton {
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      doCancel();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 22:07:57 Slobodan
   Initial version

*****************************************************************************^*/
