/*
 * @(#)JWxButton.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;

import net.wrka.util.CommonConstants;

/**
 * Extended JButton.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxButton extends JButton {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private boolean mouseIn;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxButton object.
   */
  public JWxButton() {
    super();
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param action the action
   */
  public JWxButton(Action action) {
    super(action);
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param icon the icon
   */
  public JWxButton(Icon icon) {
    super(icon);
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param text the text
   */
  public JWxButton(String text) {
    super(text);
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param text the text
   * @param icon the icon
   */
  public JWxButton(String text, Icon icon) {
    super(text, icon);
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param icon the icon
   * @param tooltip the tooltip
   */
  public JWxButton(Icon icon, String tooltip) {
    super(CommonConstants.STRING_EMPTY, icon);
    if ((tooltip != null) && (tooltip.length() > 0)) {
      setToolTipText(tooltip);
    }
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param text the text
   * @param tooltip the tooltip
   */
  public JWxButton(String text, String tooltip) {
    super(text);
    if ((tooltip != null) && (tooltip.length() > 0)) {
      setToolTipText(tooltip);
    }
  }

  /**
   * Constructs new JWxButton object.
   *
   * @param title the title
   * @param toolTip the tooltip
   * @param icon the icon
   * @param rolloverIcon the rollover icon
   */
  public JWxButton(String title, String toolTip, Icon icon, Icon rolloverIcon) {
    super(title, icon);
    if ((toolTip != null) && (toolTip.length() > 0)) {
      setToolTipText(toolTip);
    }
    if (rolloverIcon != null) {
      setRolloverIcon(rolloverIcon);
    }
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if mouse is inside button area.
   *
   * @return <tt>true</tt> if mouse is inside button area
   */
  public boolean isMouseIn() {
    return mouseIn;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setEnabled(boolean enabled) {
    super.setEnabled(enabled);
    onEnabled(enabled);
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    switch (e.getID()) { // NOPMD
      case MouseEvent.MOUSE_ENTERED:
        onMouseEntered(e);
        break;
      case MouseEvent.MOUSE_EXITED:
        onMouseExited(e);
        break;
      case MouseEvent.MOUSE_CLICKED:
        onMouseClicked(e);
        break;
      case MouseEvent.MOUSE_PRESSED:
        onMousePressed(e);
        break;
      case MouseEvent.MOUSE_RELEASED:
        onMouseReleased(e);
        break;
    }
    super.processMouseEvent(e);
  }

  /**
   * {@inheritDoc}
   */
  protected void fireActionPerformed(ActionEvent e) {
    onAction(e);
    super.fireActionPerformed(e);
  }

  /**
   * {@inheritDoc}
   */
  protected void init(String text, Icon icon) {
    super.init(text, icon);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Called on action.
   *
   * @param e the action event
   */
  protected void onAction(ActionEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on enabled.
   *
   * @param enabled <tt>true</tt> if enabled
   */
  protected void onEnabled(boolean enabled) {
    if (enabled && isMouseIn()) {
      getModel().setRollover(true);
    }
  }

  /**
   * Called on mouse entered.
   *
   * @param e the mouse event
   */
  protected void onMouseEntered(MouseEvent e) {
    mouseIn = true;
  }

  /**
   * Called on mouse exited.
   *
   * @param e the mouse event
   */
  protected void onMouseExited(MouseEvent e) {
    mouseIn = false;
  }

  /**
   * Called on mouse clicked.
   *
   * @param e the mouse event
   */
  protected void onMouseClicked(MouseEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on mouse pressed.
   *
   * @param e the mouse event
   */
  protected void onMousePressed(MouseEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on mouse released.
   *
   * @param e the mouse event
   */
  protected void onMouseReleased(MouseEvent e) { // NOPMD
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 19:47:25 Slobodan
   Initial version

*****************************************************************************^*/
