/*
 * @(#)JWxCheckBoxMenuItem.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JCheckBoxMenuItem;

/**
 * Check Box Menu Item.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxCheckBoxMenuItem extends JCheckBoxMenuItem {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   */
  public JWxCheckBoxMenuItem() {
    super();
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param icon the icon
   */
  public JWxCheckBoxMenuItem(Icon icon) {
    super(icon);
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param text the text
   */
  public JWxCheckBoxMenuItem(String text) {
    super(text);
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param action the action
   */
  public JWxCheckBoxMenuItem(Action action) {
    super(action);
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param text the text
   * @param icon the icon
   */
  public JWxCheckBoxMenuItem(String text, Icon icon) {
    super(text, icon);
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param text the text
   * @param selected <tt>true</tt> if selected
   */
  public JWxCheckBoxMenuItem(String text, boolean selected) {
    super(text, selected);
  }

  /**
   * Constructs new JWxCheckBoxMenuItem object.
   *
   * @param text the text
   * @param icon the icon
   * @param selected <tt>true</tt> if selected
   */
  public JWxCheckBoxMenuItem(String text, Icon icon, boolean selected) {
    super(text, icon, selected);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void configurePropertiesFromAction(Action a) {
    super.configurePropertiesFromAction(a);
    Object selected = a.getValue("selected"); //$NON-NLS-1$
    if (selected instanceof Boolean) {
      setSelected(((Boolean)selected).booleanValue());
    }
  }

  /**
   * {@inheritDoc}
   */
  protected PropertyChangeListener createActionPropertyChangeListener(
                                                                    Action a) {
    return new WrappedPropertyChangeListener(
                                  super.createActionPropertyChangeListener(a));
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Wrapped Property Change Listener.
   */
  final class WrappedPropertyChangeListener implements PropertyChangeListener {
    private final PropertyChangeListener pcl;
    /**
     * Constructs new WrappedPropertyChangeListener object.
     *
     * @param pcl the property change listener
     */
    public WrappedPropertyChangeListener(PropertyChangeListener pcl) {
      this.pcl = pcl;
    }
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent evt) {
      pcl.propertyChange(evt);
      if ("selected".equals(evt.getPropertyName())) { //$NON-NLS-1$
        Object selected = evt.getNewValue();
        if (selected instanceof Boolean) {
          setSelected(((Boolean)selected).booleanValue());
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 23:12:41 Slobodan
   Initial version

*****************************************************************************^*/
