/*
 * @(#)JWxDialog.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.event.WindowListener;
import java.awt.event.WindowStateListener;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.MenuSelectionManager;

import net.wrka.awt.AwtTools;
import net.wrka.swing.patch.PatchManager;

/**
 * Extended JDialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxDialog extends JDialog {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Default position.
   */
  public static final int POSITION_DEFAULT = 0;

  /**
   * Center position.
   */
  public static final int POSITION_SCREEN_CENTER = 1;

  /**
   * Parent centered position.
   */
  public static final int POSITION_PARENT_CENTER = 2;

  /**
   * Custom position.
   */
  public static final int POSITION_CUSTOM = 3;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int position;
  private DialogAdapter dialogAdapter;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxDialog object.
   */
  public JWxDialog() {
    super((Frame)null, false);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the owner frame
   */
  public JWxDialog(Frame owner) {
    super(owner, false);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the owner frame
   * @param modal <tt>true</tt> if model
   */
  public JWxDialog(Frame owner, boolean modal) {
    super(owner, null, modal);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the owner frame
   * @param title the title
   */
  public JWxDialog(Frame owner, String title) {
    super(owner, title, false);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the owner frame
   * @param title the title
   * @param modal <tt>true</tt> if model
   */
  public JWxDialog(Frame owner, String title, boolean modal) {
    super(owner, title, modal);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the dialog owner
   */
  public JWxDialog(Dialog owner) {
    super(owner, false);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the dialog owner
   * @param modal <tt>true</tt> if modal
   */
  public JWxDialog(Dialog owner, boolean modal) {
    super(owner, null, modal);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the dialog owner
   * @param title the title
   */
  public JWxDialog(Dialog owner, String title) {
    super(owner, title, false);
  }

  /**
   * Constructs new JWxDialog object.
   *
   * @param owner the dialog owner
   * @param title the title
   * @param modal <tt>true</tt> if modal
   */
  public JWxDialog(Dialog owner, String title, boolean modal) {
    super(owner, title, modal);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the <code>JWRootPane</code> object for this dialog.
   *
   * @return the <code>JWRootPane</code> object for this dialog
   */
  public JWxRootPane getJWxRootPane() {
    if (rootPane instanceof JWxRootPane) {
      return (JWxRootPane)rootPane; // FB: OK
    }
    return null;
  }

  /**
   * Returns the status bar set on this dialog.
   *
   * @return the status bar for this dialog
   */
  public JWxStatusBar getJWxStatusBar() {
    JWxRootPane jwxrp = getJWxRootPane();
    if (jwxrp != null) {
      return jwxrp.getJWxStatusBar();
    }
    return null;
  }

  /**
   * Sets the status bar for this dialog.
   *
   * @param statusBar the status bar being placed in the dialog
   */
  public void setJWxStatusBar(JWxStatusBar statusBar) {
    JWxRootPane jwxrp = getJWxRootPane();
    if (jwxrp != null) {
      jwxrp.setJWxStatusBar(statusBar);
    }
  }

  /**
   * Returns position of this dialog.
   *
   * @return dialog position
   */
  public int getDialogPositon() {
    return position;
  }

  /**
   * Sets position of this dialog.
   *
   * @param position dialog position
   */
  public void setDialogPosition(int position) {
    this.position = position;
  }

  /**
   * Disables user inputs.
   */
  public void block() {
    Component gp = getGlassPane();
    if (gp instanceof JWxGlassPane) {
      ((JWxGlassPane)gp).block();
    }
  }

  /**
   * Enables user inputs.
   */
  public void release() {
    Component gp = getGlassPane();
    if (gp instanceof JWxGlassPane) {
      ((JWxGlassPane)gp).release();
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void show() { // @deprecated as of java 1.1 but OK here.
    switch (position) { // NOPMD
      case POSITION_SCREEN_CENTER:
        AwtTools.screenCenterWindow(this);
        position = POSITION_CUSTOM;
        break;
      case POSITION_PARENT_CENTER:
        AwtTools.parentCenterWindow(this);
        position = POSITION_CUSTOM;
        break;
    }
    super.show(); // @deprecated as of java 1.1 but OK here.
  }

  /**
   * Sets location of this frame.
   *
   * @param x x - coordinate
   * @param y y - coordinate
   */
  public void setLocation(int x, int y) {
    super.setLocation(x, y);
    position = POSITION_CUSTOM;
  }

  /**
   * Shows error.
   *
   * @param title error dialog title
   * @param message error message
   */
  public final void showError(String title, Object message) {
    JOptionPane.showMessageDialog(this, message, title,
                                  JOptionPane.ERROR_MESSAGE);
  }

  /**
   * Shows warning.
   *
   * @param title error dialog title
   * @param message warning message
   */
  public final void showWarning(String title, Object message) {
    JOptionPane.showMessageDialog(this, message, title,
                                  JOptionPane.WARNING_MESSAGE);
  }

  /**
   * Shows information.
   *
   * @param title error dialog title
   * @param message information message
   */
  public final void showInformation(String title, Object message) {
    JOptionPane.showMessageDialog(this, message, title,
                                  JOptionPane.INFORMATION_MESSAGE);
  }

  /**
   * {@inheritDoc}
   */
  protected JRootPane createRootPane() {
    return new JWxRootPane(true);
  }

  /**
   * {@inheritDoc}
   */
  protected void dialogInit() {
    super.dialogInit();
    if (getOwner() == null) {
      position = POSITION_SCREEN_CENTER;
    } else {
      position = POSITION_PARENT_CENTER;
    }
    dialogAdapter = new DialogAdapter();
    addDialogListeners();
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Adds frame listeners.
   */
  protected void addDialogListeners() {
    addComponentListener(dialogAdapter);
    addWindowListener(dialogAdapter);
  }

  /**
   * Invoked when the component's size changes.
   *
   * @param ce ComponentEvent
   */
  protected void onResize(ComponentEvent ce) {
    Dimension mindim = getMinimumSize();
    if (mindim != null) {
      Dimension newdim = getSize();
      if (newdim.width < mindim.width) {
        newdim.width = mindim.width;
      }
      if (newdim.height < mindim.height) {
        newdim.height = mindim.height;
      }
      setSize(newdim);
    }
  }

  /**
   * Invoked when the component's position changes.
   *
   * @param ce ComponentEvent
   */
  protected void onMoved(ComponentEvent ce) {
    // empty method body
  }

  /**
   * Invoked when the component has been made visible.
   *
   * @param ce ComponentEvent
   */
  protected void onShown(ComponentEvent ce) {
    // empty method body
  }

  /**
   * Invoked when the component has been made invisible.
   *
   * @param ce ComponentEvent
   */
  protected void onHidden(ComponentEvent ce) {
    // empty method body
  }

  /**
   * Invoked when a window has been opened.
   *
   * @param we WindowEvent
   */
  protected void onOpened(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window is in the process of being closed.
   *
   * @param we WindowEvent
   */
  protected void onClosing(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window has been closed.
   *
   * @param we WindowEvent
   */
  protected void onClosed(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window is iconified.
   *
   * @param we WindowEvent
   */
  protected void onIconified(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window is de-iconified.
   *
   * @param we WindowEvent
   */
  protected void onDeiconified(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window is activated.
   *
   * @param we WindowEvent
   */
  protected void onActivated(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when a window is de-activated.
   *
   * @param we WindowEvent
   */
  protected void onDeactivated(WindowEvent we) {
    if (PatchManager.APPLAY_PATCH_14) {
      MenuSelectionManager.defaultManager().clearSelectedPath();
    }
  }

  /**
   * Invoked when a window state is changed.
   *
   * @param we WindowEvent
   */
  protected void onStateChanged(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when the Window is set to be the focused Window.
   *
   * @param we WindowEvent
   */
  protected void onGainedFocus(WindowEvent we) {
    // empty method body
  }

  /**
   * Invoked when the Window is no longer the focused Window.
   *
   * @param we WindowEvent
   */
  protected void onLostFocus(WindowEvent we) {
    // empty method body
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Dialog Adapter.
   */
  final class DialogAdapter implements ComponentListener, WindowListener,
                                       WindowStateListener,
                                       WindowFocusListener {
    /**
     * {@inheritDoc}
     */
    public void componentResized(ComponentEvent ce) {
      onResize(ce);
    }
    /**
     * {@inheritDoc}
     */
    public void componentMoved(ComponentEvent ce) {
      onMoved(ce);
    }
    /**
     * {@inheritDoc}
     */
    public void componentShown(ComponentEvent ce) {
      onShown(ce);
    }
    /**
     * {@inheritDoc}
     */
    public void componentHidden(ComponentEvent ce) {
      onHidden(ce);
    }
    /**
     * {@inheritDoc}
     */
    public void windowOpened(WindowEvent we) {
      onOpened(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowClosing(WindowEvent we) {
      onClosing(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowClosed(WindowEvent we) {
      onClosed(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowIconified(WindowEvent we) {
      onIconified(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowDeiconified(WindowEvent we) {
      onDeiconified(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowActivated(WindowEvent we) {
      onActivated(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowDeactivated(WindowEvent we) {
      onDeactivated(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowStateChanged(WindowEvent we) {
      onStateChanged(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowGainedFocus(WindowEvent we) {
      onGainedFocus(we);
    }
    /**
     * {@inheritDoc}
     */
    public void windowLostFocus(WindowEvent we) {
      onLostFocus(we);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 20:31:32 Slobodan
   Initial version

*****************************************************************************^*/
