/*
 * @(#)JWxLogoPanel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Paint;

import javax.swing.JPanel;

import net.wrka.swing.graphics.FastGradientPaint;

/**
 * Logo Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxLogoPanel extends JPanel {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Secondary background property.
   */
  public static final String SECONDARY_BACKGROUND =
                                           "secondaryBackground"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Color secondaryBackground;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxLogoPanel object.
   */
  public JWxLogoPanel() {
    this(Color.WHITE);
  }

  /**
   * Constructs new JWxLogoPanel object.
   *
   * @param secondaryBackground the secondary background
   */
  public JWxLogoPanel(Color secondaryBackground) {
    super(new BorderLayout());
    this.secondaryBackground = secondaryBackground;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns secondary background color.
   *
   * @return secondary background color
   */
  public Color getSecondaryBackground() {
    if (secondaryBackground == null) {
      return getBackground();
    }
    return secondaryBackground;
  }

  /**
   * Sets secondary background color.
   *
   * @param secondaryBackground the color
   */
  public void setSecondaryBackground(Color secondaryBackground) {
    Color oldSB = secondaryBackground;
    this.secondaryBackground = secondaryBackground;
    repaint();
    firePropertyChange(SECONDARY_BACKGROUND, oldSB, secondaryBackground);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void paintComponent(Graphics g) {
    if (isOpaque()) {
      Graphics2D g2D = (Graphics2D)g;
      Paint gradient = new FastGradientPaint(getSecondaryBackground(),
                                             getBackground(), false);
      g2D.setPaint(gradient);
      g2D.fillRect(getX(), getY(), getWidth(), getHeight());
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 08:40:49 Slobodan
   Initial version

*****************************************************************************^*/
