/*
 * @(#)JWxPlafComboBox.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;

import javax.swing.JComboBox;
import javax.swing.SwingUtilities;

import net.wrka.swing.components.models.PlafComboBoxModel;
import net.wrka.swing.plaf.UIUpdater;

/**
 * Look and Feel ComboBox.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class JWxPlafComboBox extends JComboBox {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Object selectedObject;
  private UIUpdater uiUpdater;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxPlafComboBox object.
   */
  public JWxPlafComboBox() {
    this(null);
  }

  /**
   * Constructs new JWxPlafComboBox object.
   *
   * @param c target component
   */
  public JWxPlafComboBox(Component c) {
    super(PlafComboBoxModel.getInstance());
    setComponent(c);
  }

/*^**************************************************************************^*/
/*- Public methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets target component.
   *
   * @param c target component
   */
  public void setComponent(Component c) {
    if (c == null) {
      uiUpdater = null;
    } else {
      uiUpdater = new UIUpdater(c);
    }
  }

  /**
   * Returns target component.
   *
   * @return target component
   */
  public Component getComponent() {
    if (uiUpdater == null) {
      return null;
    }
    return uiUpdater.getComponent();
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setSelectedItem(Object anObject) {
    if (((selectedObject != null) && (!selectedObject.equals(anObject))) ||
        (selectedObject == null) && (anObject != null)) {
      super.setSelectedItem(anObject);
      selectedObject = anObject;
      if (uiUpdater != null) {
        SwingUtilities.invokeLater(uiUpdater);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:23:40 Slobodan
   Initial version

*****************************************************************************^*/
