/*
 * @(#)JWxPlafThemeComboBox.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.Component;

import javax.swing.JComboBox;
import javax.swing.SwingUtilities;
import javax.swing.event.ListDataEvent;

import net.wrka.swing.components.models.PlafThemeComboBoxModel;
import net.wrka.swing.plaf.UIUpdater;

/**
 *  Look and Feel Chooser Theme ComboBox.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class JWxPlafThemeComboBox extends JComboBox {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Object selectedObject;
  private UIUpdater uiUpdater;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxPlafThemeComboBox object.
   */
  public JWxPlafThemeComboBox() {
    this(null);
  }

  /**
   * Constructs new JWxPlafThemeComboBox object.
   *
   * @param c the target component
   */
  public JWxPlafThemeComboBox(Component c) {
    super(PlafThemeComboBoxModel.getInstance());
    setComponent(c);
    Object selectedItem = getSelectedItem();
    setEnabled((selectedItem != null) &&
               (!selectedItem.equals(PlafThemeComboBoxModel.NO_THEME)));
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets target component.
   *
   * @param c target component
   */
  public void setComponent(Component c) {
    if (c == null) {
      uiUpdater = null;
    } else {
      uiUpdater = new UIUpdater(c);
    }
  }

  /**
   * Returns target component.
   *
   * @return target component
   */
  public Component getComponent() {
    if (uiUpdater == null) {
      return null;
    }
    return uiUpdater.getComponent();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void contentsChanged(ListDataEvent e) {
    super.contentsChanged(e);
    setEnabled(!getSelectedItem().equals(PlafThemeComboBoxModel.NO_THEME));
  }

  /**
   * {@inheritDoc}
   */
  public void setSelectedItem(Object anObject) {
    if (((selectedObject != null) && (!selectedObject.equals(anObject))) ||
        (selectedObject == null) && (anObject != null)) {
      super.setSelectedItem(anObject);
      selectedObject = anObject;
      if (uiUpdater != null) {
        SwingUtilities.invokeLater(uiUpdater);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 14:00:56 Slobodan
   Initial version

*****************************************************************************^*/
