/*
 * @(#)JWxToggleButton.java  1.0.0 / 2008-05-21
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components;

import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JToggleButton;

import net.wrka.util.CommonConstants;

/**
 * Extended JToggleButton.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxToggleButton extends JToggleButton {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private boolean mouseIn;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxToggleButton object.
   */
  public JWxToggleButton() {
    super(CommonConstants.STRING_EMPTY, null, false);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param action the action
   */
  public JWxToggleButton(Action action) {
    super(action);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param icon the icon
   */
  public JWxToggleButton(Icon icon) {
    super(CommonConstants.STRING_EMPTY, icon);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param text the text
   */
  public JWxToggleButton(String text) {
    super(text, null);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param text the text
   * @param icon the icon
   */
  public JWxToggleButton(String text, Icon icon) {
    super(text, icon, false);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param text the text
   * @param icon the icon
   * @param selected <tt>true</tt> if selected
   */
  public JWxToggleButton(String text, Icon icon, boolean selected) {
    super(text, icon, selected);
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param text the text
   * @param tooltip the tooltip
   * @param icon the icon
   * @param selected <tt>true</tt> if selected
   */
  public JWxToggleButton(String text, String tooltip, Icon icon,
                         boolean selected) {
    super(text, icon, selected);
    if ((tooltip != null) && (tooltip.length() != 0)) {
      setToolTipText(tooltip);
    }
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param title the title
   * @param icon the icon
   * @param rolloverIcon the rollover icon
   */
  public JWxToggleButton(String title, Icon icon, Icon rolloverIcon) {
    super(title, icon);
    if (rolloverIcon != null) {
      setRolloverIcon(rolloverIcon);
    }
  }

  /**
   * Constructs new JWxToggleButton object.
   *
   * @param title the title
   * @param tooltip the tooltip
   * @param icon the icon
   * @param rolloverIcon the rollover icon
   */
  public JWxToggleButton(String title, String tooltip, Icon icon,
                         Icon rolloverIcon) {
    super(title, icon);
    if ((tooltip != null) && (tooltip.length() != 0)) {
      setToolTipText(tooltip);
    }
    if (rolloverIcon != null) {
      setRolloverIcon(rolloverIcon);
    }
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if mouse is inside button area.
   *
   * @return <tt>true</tt> if mouse is inside button area
   */
  public boolean isMouseIn() {
    return mouseIn;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setEnabled(boolean enabled) {
    super.setEnabled(enabled);
    onEnabled(enabled);
  }

  /**
   * {@inheritDoc}
   */
  protected void processMouseEvent(MouseEvent e) {
    switch (e.getID()) { // NOPMD
      case MouseEvent.MOUSE_ENTERED:
        onMouseEntered(e);
        break;
      case MouseEvent.MOUSE_EXITED:
        onMouseExited(e);
        break;
      case MouseEvent.MOUSE_CLICKED:
        onMouseClicked(e);
        break;
      case MouseEvent.MOUSE_PRESSED:
        onMousePressed(e);
        break;
      case MouseEvent.MOUSE_RELEASED:
        onMouseReleased(e);
        break;
    }
    super.processMouseEvent(e);
  }

  /**
   * {@inheritDoc}
   */
  protected void fireActionPerformed(ActionEvent e) {
    onAction(e);
    super.fireActionPerformed(e);
  }

  /**
   * {@inheritDoc}
   */
  protected void configurePropertiesFromAction(Action a) {
    super.configurePropertiesFromAction(a);
    Object selected = a.getValue("selected"); //$NON-NLS-1$
    if (selected instanceof Boolean) {
      setSelected(((Boolean)selected).booleanValue());
    }
  }

  /**
   * {@inheritDoc}
   */
  protected PropertyChangeListener createActionPropertyChangeListener(
                                                                    Action a) {
    PropertyChangeListener pcl = super.createActionPropertyChangeListener(a);
    return new WrappedPropertyChangeListener(pcl);
  }

  /**
   * {@inheritDoc}
   */
  protected void init(String text, Icon icon) {
    super.init(text, icon);
    enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Called on action.
   *
   * @param e the action event
   */
  protected void onAction(ActionEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on enabled.
   *
   * @param enabled <tt>true</tt> if enabled
   */
  protected void onEnabled(boolean enabled) {
    if (enabled && isMouseIn()) {
      getModel().setRollover(true);
    }
  }

  /**
   * Called on mouse entered.
   *
   * @param e the mouse event
   */
  protected void onMouseEntered(MouseEvent e) {
    mouseIn = true;
  }

  /**
   * Called on mouse exited.
   *
   * @param e the mouse event
   */
  protected void onMouseExited(MouseEvent e) {
    mouseIn = false;
  }

  /**
   * Called on mouse clicked.
   *
   * @param e the mouse event
   */
  protected void onMouseClicked(MouseEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on mouse pressed.
   *
   * @param e the mouse event
   */
  protected void onMousePressed(MouseEvent e) { // NOPMD
    // empty method body
  }

  /**
   * Called on mouse released.
   *
   * @param e the mouse event
   */
  protected void onMouseReleased(MouseEvent e) { // NOPMD
    // empty method body
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Wrapped Property Change Listener.
   */
  final class WrappedPropertyChangeListener implements PropertyChangeListener {
    private final PropertyChangeListener pcl;
    /**
     * Constructs new WrappedPropertyChangeListener object.
     *
     * @param pcl the property change listener
     */
    public WrappedPropertyChangeListener(PropertyChangeListener pcl) {
      this.pcl = pcl;
    }
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent evt) {
      pcl.propertyChange(evt);
      if ("selected".equals(evt.getPropertyName())) { //$NON-NLS-1$
        Object selected = evt.getNewValue();
        if (selected instanceof Boolean) {
          setSelected(((Boolean)selected).booleanValue());
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-21 20:06:20 Slobodan
   Initial version

*****************************************************************************^*/
