/*
 * @(#)AboutDialog.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.about;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

import javax.swing.JTabbedPane;

import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxDialog;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.components.JWxImageIconComponent;
import net.wrka.swing.components.JWxLogoPanel;
import net.wrka.swing.components.JWxSeparator;
import net.wrka.swing.resources.BuiltInResources;

/**
 * About Box Dialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class AboutDialog extends JWxDialog {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final Insets NULL_INSETS;
  private static final Insets TABP_INSETS;
  private static final Insets CLBT_INSETS;
  private static final Insets SEP_INSETS;
  private static final Dimension SEP_DIM;
  private static final Dimension LOGO_DIM;
  private static final int ABOUT_WIDTH;
  private static final int ABOUT_HEIGHT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    NULL_INSETS = new Insets(0, 0, 0, 0);
    TABP_INSETS = new Insets(13, 6, 6, 6);
    CLBT_INSETS = new Insets(0, 4, 6, 8);
    SEP_INSETS = new Insets(0, 1, 1, 2);
    SEP_DIM = new Dimension(1, 3);
    LOGO_DIM = new Dimension(1, 70);
    ABOUT_WIDTH = 426;
    ABOUT_HEIGHT = 427;
  }

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private AboutBox aboutBox;
  private JTabbedPane tabPane;

/*^**************************************************************************^*/
/*- Public constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new AboutDialog object.
   *
   * @param owner the dialog owner
   * @param aboutBox the about box
   */
  public AboutDialog(Frame owner, AboutBox aboutBox) {
    super(owner, true);
    initDialog(aboutBox);
  }

  /**
   * Constructs new AboutDialog object.
   *
   * @param owner the dialog owner
   * @param aboutBox the about box
   */
  public AboutDialog(Dialog owner, AboutBox aboutBox) {
    super(owner, true);
    initDialog(aboutBox);
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates all about panels.
   */
  public void updatePanels() {
    JTabbedPane tabPane = this.tabPane;
    int size = tabPane.getTabCount();
    for (int i = 0; i < size; i++) {
      AboutPanel aboutPanel = (AboutPanel)tabPane.getComponentAt(i);
      aboutPanel.update();
    }
    if (size > 0) {
      tabPane.setSelectedIndex(0);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void onClosed(WindowEvent we) {
    aboutBox.dialog = null;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void initDialog(AboutBox aboutBox) {
    this.aboutBox = aboutBox;
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    setSize(ABOUT_WIDTH, ABOUT_HEIGHT);
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                                     AboutBox.RESOURCES);
    setTitle(rb.getString(AboutBox.RK_DIALOG_TITLE));
    setResizable(false);
    JWxGriddedPanel cp = new JWxGriddedPanel(NULL_INSETS);
    JWxLogoPanel lp = new JWxLogoPanel();
    lp.add(new JWxImageIconComponent(aboutBox.logoIcon), BorderLayout.CENTER);
    lp.setPreferredSize(LOGO_DIM);
    lp.setMinimumSize(LOGO_DIM);
    cp.addComponent(lp, 0, 0, 1, 1, JWxGriddedPanel.GBC_NORTH,
                    JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    cp.addComponent(new JWxSeparator(SEP_DIM, SEP_INSETS, JWxSeparator.TOP),
                    0, 1, 1, 1, JWxGriddedPanel.GBC_NORTH,
                    JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    tabPane = new JTabbedPane();
    int size = aboutBox.aboutGroups.size();
    for (int i = 0; i < size; i++) {
      AboutGroup ag = (AboutGroup)aboutBox.aboutGroups.get(i);
      tabPane.addTab(ag.getGroupName(), ag.getGroupPanel());
    }
    cp.addComponent(tabPane, 0, 2, 1, 1, JWxGriddedPanel.GBC_CENTER,
                    JWxGriddedPanel.GBC_BOTH, TABP_INSETS, 1.0, 1.0);
    cp.addComponent(new CloseButton(rb.getString(
                                              AboutBox.RK_DIALOG_CLOSE_BUTTON)),
                    0, 3, 1, 1, JWxGriddedPanel.GBC_EAST,
                    JWxGriddedPanel.GBC_NONE, CLBT_INSETS, 1.0, 0.0, 3, 0);
    setContentPane(cp);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Close Button.
   */
  final class CloseButton extends JWxButton {
    /**
     * Constructs new CloseButton object.
     *
     * @param label the label
     */
    public CloseButton(String label) {
      super(label);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      dispose();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 21:44:48 Slobodan
   Initial version

*****************************************************************************^*/
