/*
 * @(#)SimpleAboutBox.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.about;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Insets;
import java.awt.event.ActionEvent;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;

import net.wrka.awt.AwtTools;
import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxSeparator;
import net.wrka.swing.resources.BuiltInResources;

/**
 * Simple About Box.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SimpleAboutBox extends JDialog {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int ABOUT_WIDTH = 330;
  private static final int ABOUT_HEIGHT = 380;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleAboutBox object.
   *
   * @param owner the owner
   * @param title the title
   * @param aboutlogo logo picture
   * @param projecttitle project title picture
   * @param closeLabel close label
   */
  public SimpleAboutBox(Frame owner, String title, ImageIcon aboutlogo,
                        ImageIcon projecttitle, String closeLabel) {
    super(owner, title, true);
    initDialog(aboutlogo, projecttitle, closeLabel);
  }

  /**
   * Constructs new SimpleAboutBox object.
   *
   * @param owner the owner
   * @param title the title
   * @param aboutlogo logo picture
   * @param projecttitle project title picture
   * @param closeLabel close label
   */
  public SimpleAboutBox(Dialog owner, String title, ImageIcon aboutlogo,
                        ImageIcon projecttitle, String closeLabel) {
    super(owner, title, true);
    initDialog(aboutlogo, projecttitle, closeLabel);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Displays new about box.
   *
   * @param owner about box owner
   * @param title about box title
   * @param aboutlogo about box logo image
   * @param projecttitle about box title image
   * @param closeLabel close label
   */
  public static void show(JFrame owner, String title, ImageIcon aboutlogo,
                          ImageIcon projecttitle, String closeLabel) {
    SimpleAboutBox aboutBox = new SimpleAboutBox(owner, title, aboutlogo,
                                                       projecttitle,
                                                       closeLabel);
    aboutBox.setVisible(true);
  }

  /**
   * Displays new about box.
   *
   * @param owner about box owner
   * @param title about box title
   * @param aboutlogo about box logo image
   * @param projecttitle about box title image
   * @param closeLabel close label
   */
  public static void show(JDialog owner, String title, ImageIcon aboutlogo,
                          ImageIcon projecttitle, String closeLabel) {
    SimpleAboutBox aboutBox = new SimpleAboutBox(owner, title, aboutlogo,
                                                       projecttitle,
                                                       closeLabel);
    aboutBox.setVisible(true);
  }

/*^**************************************************************************^*/
/*- Private Helper Method.                                                   -*/
/*^**************************************************************************^*/

  private void initDialog(ImageIcon aboutlogo, ImageIcon projecttitle,
                          String closeLabel) {
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    setSize(ABOUT_WIDTH, ABOUT_HEIGHT);
    AwtTools.parentCenterWindow(this);
    setResizable(false);
    Container cp = getContentPane();
    CloseButton closeButton = new CloseButton(closeLabel);
    cp.add(new JLabel(aboutlogo), BorderLayout.NORTH);
    cp.add(new JLabel(projecttitle), BorderLayout.CENTER);
    JPanel jpSouth = new JPanel(null);
    Dimension jpSouthDim = new Dimension(ABOUT_WIDTH, 69);
    jpSouth.setSize(jpSouthDim);
    jpSouth.setPreferredSize(jpSouthDim);
    jpSouth.setMinimumSize(jpSouthDim);
    JWxSeparator jpBottomSep = new JWxSeparator(new Dimension(220, 20),
                                                new Insets(3, 3, 3, 3),
                                                JWxSeparator.VERTICAL_CENTER,
                                                false);
    addCompAtPos(jpSouth,
                 new JLabel(BuiltInResources.INSTANCE.
                             getImageIcon("Powered.png")), 16, 0); //$NON-NLS-1$
    addCompAtPos(jpSouth, jpBottomSep,
                 getWidth() - jpBottomSep.getPreferredSize().width - 18, 0);
    addCompAtPos(jpSouth, closeButton,
                 getWidth() - closeButton.getPreferredSize().width - 20,
                 jpSouth.getPreferredSize().height -
                 closeButton.getPreferredSize().height - 15);
    getContentPane().add(jpSouth, BorderLayout.SOUTH);
    pack();
  }

  private void addCompAtPos(JComponent parent, JComponent c, int x, int y) {
    c.setBounds(x, y, c.getPreferredSize().width, c.getPreferredSize().height);
    parent.add(c);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Close Button.
   */
  final class CloseButton extends JWxButton {
    /**
     * Constructs new CloseButton object.
     *
     * @param label the button label
     */
    public CloseButton(String label) {
      super(label);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      dispose();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 21:38:24 Slobodan
   Initial version

*****************************************************************************^*/
