/*
 * @(#)SystemAboutPanel.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.about;

import java.awt.Dimension;
import java.awt.Insets;
import java.text.DecimalFormat;
import java.util.ResourceBundle;

import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.components.JWxTable;
import net.wrka.swing.components.models.SystemPreferencesTableModel;
import net.wrka.swing.resources.BuiltInResources;

/**
 * System Information Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class SystemAboutPanel extends AboutPanel {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final Insets NULL_INSETS;
  private static final Insets PROP_INSETS;
  private static final Insets HEAP_INSETS;
  private static final Insets ALOC_INSETS;
  private static final Insets USED_INSETS;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    NULL_INSETS = new Insets(0, 0, 0, 0);
    PROP_INSETS = new Insets(6, 8, 4, 8);
    HEAP_INSETS = new Insets(10, 8, 2, 8);
    ALOC_INSETS = new Insets(2, 8, 4, 8);
    USED_INSETS = ALOC_INSETS;
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final DecimalFormat decfu;
  private final DecimalFormat decfa;
  private final JLabel jlUsed;
  private final JLabel jlAllocated;
  private final Runtime runtime;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SystemAboutPanel object.
   */
  public SystemAboutPanel() {
    super(NULL_INSETS);
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                                            AboutBox.RESOURCES);
    decfu = new DecimalFormat(rb.getString(AboutBox.RK_SYSTEM_JVM_USED));
    decfa = new DecimalFormat(rb.getString(AboutBox.RK_SYSTEM_JVM_ALLOCATED));
    jlUsed = new JLabel();
    jlAllocated = new JLabel();
    SystemPreferencesTableModel tableModel =
        new SystemPreferencesTableModel(
                               rb.getString(AboutBox.RK_SYSTEM_PROPERTY_NAME),
                               rb.getString(AboutBox.RK_SYSTEM_PROPERTY_VALUE));
    JWxTable jtSysProps = new JWxTable(tableModel);
    jtSysProps.setIntercellSpacing(new Dimension(0, 1));
    tableModel.update();
    jtSysProps.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    addComponent(new JLabel(rb.getString(AboutBox.RK_SYSTEM_PROPERTIES)), 0, 0,
                 2, 1, JWxGriddedPanel.GBC_EAST, JWxGriddedPanel.GBC_HORIZONTAL,
                 PROP_INSETS, 1.0, 0.0);
    addComponent(new JScrollPane(jtSysProps), 0, 1, 2, 1,
                 JWxGriddedPanel.GBC_EAST, JWxGriddedPanel.GBC_BOTH, null,
                 1.0, 1.0);
    addComponent(new JLabel(rb.getString(AboutBox.RK_SYSTEM_JVM_HEAP)), 0, 2,
                 2, 1, JWxGriddedPanel.GBC_EAST, JWxGriddedPanel.GBC_HORIZONTAL,
                 HEAP_INSETS, 1.0, 0.0);
    addComponent(jlAllocated, 0, 3, 1, 1, JWxGriddedPanel.GBC_EAST,
                 JWxGriddedPanel.GBC_HORIZONTAL, ALOC_INSETS, 0.0, 0.0);
    addComponent(jlUsed, 1, 3, 1, 1, JWxGriddedPanel.GBC_EAST,
                 JWxGriddedPanel.GBC_HORIZONTAL, USED_INSETS, 0.0, 0.0);
    runtime = Runtime.getRuntime();
    updateHeap();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Updates heap indicators.
   */
  public void updateHeap() {
    jlAllocated.setText(decfa.format(runtime.totalMemory() / 1048576));
    jlUsed.setText(decfu.format((runtime.totalMemory() - runtime.freeMemory()) /
                                1048576));
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void update() {
    updateHeap();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 21:40:41 Slobodan
   Initial version

*****************************************************************************^*/
