/*
 * @(#)SystemPreferencesTableModel.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.models;

import java.util.Arrays;
import java.util.Enumeration;
import java.util.Properties;

import javax.swing.table.AbstractTableModel;

import net.wrka.util.CommonConstants;

/**
 * System Preferences Table Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SystemPreferencesTableModel extends AbstractTableModel {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final Properties properties;
  private final String nameColTitle;
  private final String valueColTitle;
  private Object[] sortedProperties;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SystemPreferencesTableModel object.
   *
   * @param nameColTitle the "name" column title
   * @param valueColTitle the "value" column title
   */
  public SystemPreferencesTableModel(String nameColTitle,
                                     String valueColTitle) {
    super();
    if (nameColTitle == null) {
      this.nameColTitle = CommonConstants.STRING_EMPTY;
    } else {
      this.nameColTitle = nameColTitle;
    }
    if (valueColTitle == null) {
      this.valueColTitle = CommonConstants.STRING_EMPTY;
    } else {
      this.valueColTitle = valueColTitle;
    }
    properties = System.getProperties();
    sortedProperties = new Object[0];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Updates this model.
   */
  public void update() {
    sortedProperties = new Object[properties.size()];
    Enumeration keys = properties.keys();
    for (int i = 0; i < sortedProperties.length; i++) {
      sortedProperties[i] = keys.nextElement();
    }
    Arrays.sort(sortedProperties);
    fireTableDataChanged();
  }

  /**
   * Clears this model.
   */
  public void clear() {
    sortedProperties = new Object[0];
    fireTableDataChanged();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns the number of the columns.
   *
   * @return the number of the columns
   */
  public int getColumnCount() {
    return 2;
  }

  /**
   * Returns the name of the column.
   *
   * @param column the column number
   *
   * @return the name of the column
   */
  public String getColumnName(int column) {
    switch (column) {
      case 0:
        return nameColTitle;
      case 1:
        return valueColTitle;
      default:
        return CommonConstants.STRING_EMPTY;
    }
  }

  /**
   * Returns the number of the rows.
   *
   * @return the number of the rows
   */
  public int getRowCount() {
    return sortedProperties.length;
  }

  /**
   * Returns value at specified position.
   *
   * @param row row position
   * @param column column position
   *
   * @return value at specified position
   */
  public Object getValueAt(int row, int column) {
    switch (column) {
      case 0:
        return sortedProperties[row];
      case 1:
        return properties.get(sortedProperties[row]);
      default:
        return CommonConstants.STRING_EMPTY;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 13:52:18 Slobodan
   Initial version

*****************************************************************************^*/
