/*
 * @(#)WxTitlePaneUI.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.plaf;

import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;

import net.wrka.swing.components.JWxTitlePane;

/**
 * TitlePaneUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class WxTitlePaneUI extends ComponentUI {

/*^**************************************************************************^*/
/*- Protected Static Field.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Property prefix.
   */
  protected static final String PROPERTY_PREFIX =
                                                "WxTitlePaneUI."; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void installUI(JComponent c) {
    if (c instanceof JWxTitlePane) {
      JWxTitlePane tp = (JWxTitlePane)c; // FB: OK
      installDefaults(tp);
      installListeners(tp);
      installKeyboardActions(tp);
    } else {
      throw new IllegalArgumentException("Invalid component"); //$NON-NLS-1$
    }
  }

  /**
   * {@inheritDoc}
   */
  public void uninstallUI(JComponent c) {
    if (c instanceof JWxTitlePane) {
      JWxTitlePane tp = (JWxTitlePane)c; // FB: OK
      uninstallKeyboardActions(tp);
      uninstallListeners(tp);
      uninstallDefaults(tp);
    } else {
      throw new IllegalArgumentException("Invalid component"); //$NON-NLS-1$
    }
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize(JComponent c) {
    JWxTitlePane tp = (JWxTitlePane)c;  // FB: OK
    Dimension cd = new Dimension(tp.getContentPanel().getPreferredSize());
    Dimension td = tp.getTitlePanel().getPreferredSize();
    cd.height = cd.height + td.height;
    if (cd.width < td.width) {
      cd.width = td.width;
    }
    Border b = tp.getBorder();
    if (b != null) {
      Insets ins = b.getBorderInsets(tp);
      if (ins != null) {
        cd.width = cd.width + ins.left + ins.right;
        cd.height = cd.height + ins.top + ins.bottom;
      }
    }
    return cd;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize(JComponent c) {
    JWxTitlePane tp = (JWxTitlePane)c;  // FB: OK
    Dimension cd = new Dimension(tp.getContentPanel().getMinimumSize());
    Dimension td = tp.getTitlePanel().getMinimumSize();
    cd.height = cd.height + td.height;
    if (cd.width < td.width) {
      cd.width = td.width;
    }
    Border b = tp.getBorder();
    if (b != null) {
      Insets ins = b.getBorderInsets(tp);
      if (ins != null) {
        cd.width = cd.width + ins.left + ins.right;
        cd.height = cd.height + ins.top + ins.bottom;
      }
    }
    return cd;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize(JComponent c) {
    JWxTitlePane tp = (JWxTitlePane)c;  // FB: OK
    Dimension cd = new Dimension(tp.getContentPanel().getMaximumSize());
    Dimension td = tp.getTitlePanel().getMaximumSize();
    cd.height = cd.height + td.height;
    if (cd.width < td.width) {
      cd.width = td.width;
    }
    Border b = tp.getBorder();
    if (b != null) {
      Insets ins = b.getBorderInsets(tp);
      if (ins != null) {
        cd.width = cd.width + ins.left + ins.right;
        cd.height = cd.height + ins.top + ins.bottom;
      }
    }
    return cd;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns property prefix.
   *
   * @return property prefix
   */
  protected String getPropertyPrefix() {
    return PROPERTY_PREFIX;
  }

  /**
   * Installs defaults.
   *
   * @param tp the title pane
   */
  protected void installDefaults(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

  /**
   * Installs listeners.
   *
   * @param tp the title pane
   */
  protected void installListeners(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

  /**
   * Installs keyboard actions.
   *
   * @param tp the title pane
   */
  protected void installKeyboardActions(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

  /**
   * Uninstalls keyboard actions.
   *
   * @param tp the title pane
   */
  protected void uninstallKeyboardActions(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

  /**
   * Uninstalls listeners.
   *
   * @param tp the title pane
   */
  protected void uninstallListeners(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

  /**
   * Uninstalls defaults.
   *
   * @param tp the title pane
   */
  protected void uninstallDefaults(JWxTitlePane tp) { // NOPMD
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 17:48:19 Slobodan
   Initial version

*****************************************************************************^*/
