/*
 * @(#)PropertyStatusBarPanel.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.statusbar;

import java.awt.Dimension;
import java.awt.LayoutManager;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;

import net.wrka.util.CommonConstants;

/**
 * Name-Value Pair Status Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class PropertyStatusBarPanel extends JComponent {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Property name label.
   */
  protected JLabel nameLabel;

  /**
   * Property value label.
   */
  protected JLabel valueLabel;

  /**
   * Show tool tip flag.
   */
  protected boolean showToolTip;

  /**
   * Preferred size set flag.
   */
  protected boolean preferredSizeSet;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PropertyStatusBarPanel object.
   *
   * @param name the property name
   * @param showToolTip show tool tips
   */
  public PropertyStatusBarPanel(String name, boolean showToolTip) {
    super();
    super.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
    nameLabel = new JLabel();
    valueLabel = new JLabel();
    setPropertyName(name);
    add(nameLabel);
    add(Box.createHorizontalStrut(3));
    add(valueLabel);
    this.showToolTip = showToolTip;
  }

  /**
   * Constructs new PropertyStatusBarPanel object.
   *
   * @param name the property name
   * @param value the property value
   * @param showToolTip show tool tips
   */
  public PropertyStatusBarPanel(String name, String value,
                                boolean showToolTip) {
    this(name, showToolTip);
    setPropertyValue(value);
  }

  /**
   * Constructs new PropertyStatusBarPanel object.
   *
   * @param name the property name
   */
  public PropertyStatusBarPanel(String name) {
    this(name, false);
  }

  /**
   * Constructs new PropertyStatusBarPanel object.
   *
   * @param name the property name
   * @param value the property value
   */
  public PropertyStatusBarPanel(String name, String value) {
    this(name, value, false);
  }

  /**
   * Constructs new PropertyStatusBarPanel object.
   *
   * @param showToolTip show tool tips
   */
  public PropertyStatusBarPanel(boolean showToolTip) {
    this(CommonConstants.STRING_EMPTY, showToolTip);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns property name.
   *
   * @return property name
   */
  public String getPropertyName() {
    return nameLabel.getText();
  }

  /**
   * Sets property name.
   *
   * @param name property name
   */
  public void setPropertyName(String name) {
    nameLabel.setText(name);
  }

  /**
   * Clears property name.
   */
  public void clearPropertyName() {
    nameLabel.setText(CommonConstants.STRING_EMPTY);
  }

  /**
   * Returns property value.
   *
   * @return property value
   */
  public String getPropertyValue() {
    return valueLabel.getText();
  }

  /**
   * Sets property value.
   *
   * @param value property value
   */
  public void setPropertyValue(String value) {
    valueLabel.setText(value);
    if (showToolTip) {
      valueLabel.setToolTipText(value);
    }
  }

  /**
   * Clears property value.
   */
  public void clearProperValue() {
    valueLabel.setText(CommonConstants.STRING_EMPTY);
    if (showToolTip) {
      valueLabel.setToolTipText(CommonConstants.STRING_EMPTY);
    }
  }

  /**
   * Returns property name label.
   *
   * @return property name label
   */
  public JLabel getPropertyNameLabel() {
    return nameLabel;
  }

  /**
   * Returns property value label.
   *
   * @return property value label
   */
  public JLabel getPropertyValueLabel() {
    return valueLabel;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setPreferredSize(Dimension preferredSize) {
    super.setPreferredSize(preferredSize);
    preferredSizeSet = true;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize() {
    if (preferredSizeSet) {
      return super.getPreferredSize();
    }
    return new Dimension(0, 0);
  }

  /**
   * {@inheritDoc}
   */
  public void setLayout(LayoutManager mgr) {
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 21:28:31 Slobodan
   Initial version

*****************************************************************************^*/

