/*
 * @(#)StatusBarPanel.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.statusbar;

import java.awt.BorderLayout;

import javax.swing.JComponent;

import net.wrka.swing.components.plaf.StatusBarPanelUI;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.basic.BasicStatusBarPanelUI;

/**
 * Status Panel Container.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StatusBarPanel extends JComponent {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final String UI_CLASS_ID = "StatusBarPanelUI"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StatusBarPanel object.
   */
  public StatusBarPanel() {
    super();
    setLayout(new BorderLayout());
    updateUI();
  }

  /**
   * Constructs new StatusBarPanel object.
   *
   * @param c the status component
   */
  public StatusBarPanel(JComponent c) {
    this();
    add(c);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns look and feel delegate.
   *
   * @return look and feel delegate
   */
  public StatusBarPanelUI getUI() {
    return (StatusBarPanelUI)ui;
  }

  /**
   * Sets the look and feel delegate for this component.
   *
   * @param ui the new UI delegate
   */
  public void setUI(StatusBarPanelUI ui) {
    if (this.ui != null) {
      this.ui.uninstallUI(this);
    }
    StatusBarPanelUI oldUI = (StatusBarPanelUI)this.ui;
    if (ui == null) {
      if (this.ui == null) {
        this.ui = BasicStatusBarPanelUI.createUI(this);
      }
      ((StatusBarPanelUI)this.ui).installUI(this);
    } else {
      this.ui = ui;
      ui.installUI(this);
      firePropertyChange("UI", oldUI, this.ui); //$NON-NLS-1$
    }
    revalidate();
    repaint();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((StatusBarPanelUI)LookAndFeelManager.getComponentUI(this));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 21:14:33 Slobodan
   Initial version

*****************************************************************************^*/
