/*
 * @(#)StatusBarSeparator.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.statusbar;

import java.awt.Dimension;

import javax.swing.JComponent;

import net.wrka.swing.components.plaf.StatusBarSeparatorUI;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.basic.BasicStatusSeparatorUI;

/**
 * Status bar separator.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StatusBarSeparator extends JComponent {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final String UI_CLASS_ID =
                                          "StatusBarSeparatorUI"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StatusBarSeparator object.
   */
  public StatusBarSeparator() {
    this(2, 18);
  }

  /**
   * Constructs new StatusBarSeparator object.
   *
   * @param width the separator width
   * @param height the separator height
   */
  public StatusBarSeparator(int width, int height) {
    super();
    Dimension pd = new Dimension(width, height);
    setPreferredSize(pd);
    setMinimumSize(pd);
    setMaximumSize(pd);
    updateUI();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns look and feel delegate.
   *
   * @return look and feel delegate
   */
  public StatusBarSeparatorUI getUI() {
    return (StatusBarSeparatorUI)ui;
  }

  /**
   * Sets the look and feel delegate for this component.
   *
   * @param ui the new UI delegate
   */
  public void setUI(StatusBarSeparatorUI ui) {
    if (this.ui != null) {
      this.ui.uninstallUI(this);
    }
    StatusBarSeparatorUI oldUI = (StatusBarSeparatorUI)this.ui;
    if (ui == null) {
      if (this.ui == null) {
        this.ui = BasicStatusSeparatorUI.createUI(this);
      }
      ((StatusBarSeparatorUI)this.ui).installUI(this);
    } else {
      this.ui = ui;
      ui.installUI(this);
      firePropertyChange("UI", oldUI, this.ui); //$NON-NLS-1$
    }
    revalidate();
    repaint();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getUIClassID() {
    return UI_CLASS_ID;
  }

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    setUI((StatusBarSeparatorUI)LookAndFeelManager.getComponentUI(this));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 20:33:24 Slobodan
   Initial version

*****************************************************************************^*/
