/*
 * @(#)LineCounterPanel.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JPanel;
import javax.swing.JViewport;
import javax.swing.Scrollable;
import javax.swing.text.JTextComponent;

/**
 * Line Counter Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class LineCounterPanel extends JPanel implements Scrollable {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final JTextComponent jtc;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new WxLineCounterPanel object.
   *
   * @param jtc the text component
   * @param startLine the start line
   */
  public LineCounterPanel(JTextComponent jtc, int startLine) {
    super(new BorderLayout());
    if (jtc == null) {
      throw new IllegalArgumentException("Null Text Component"); //$NON-NLS-1$
    }
    this.jtc = jtc;
    add(jtc, BorderLayout.CENTER);
    add(new LineEnumerator(jtc, startLine), BorderLayout.WEST);
  }

  /**
   * Constructs new MJLineCounterPanel object.
   *
   * @param jtc the text component
   */
  public LineCounterPanel(JTextComponent jtc) {
    this(jtc, 0);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean getScrollableTracksViewportHeight() { // NOPMD
    Container parent = getParent();
    if (parent instanceof JViewport) {
      return (((JViewport)parent).getHeight() > getPreferredSize().height);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public boolean getScrollableTracksViewportWidth() { // NOPMD
    Container parent = getParent();
    if (parent instanceof JViewport) {
      return (((JViewport)parent).getWidth() > getPreferredSize().width);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredScrollableViewportSize() {
    return getPreferredSize();
  }

  /**
   * {@inheritDoc}
   */
  public int getScrollableBlockIncrement(Rectangle visibleRect, int orientation,
                                         int direction) {
    return jtc.getScrollableBlockIncrement(visibleRect, orientation, direction);
  }

  /**
   * {@inheritDoc}
   */
  public int getScrollableUnitIncrement(Rectangle visibleRect, int orientation,
                                        int direction) {
    return jtc.getScrollableUnitIncrement(visibleRect, orientation, direction);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 08:57:16 Slobodan
   Initial version

*****************************************************************************^*/
