/*
 * @(#)TextSearchDialog.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ResourceBundle;

import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Caret;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxDialog;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.util.SimpleArrayList;
import net.wrka.util.ref.WeakArrayList;

/**
 * Text Search Dialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class TextSearchDialog extends JPanel {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Property text component changed.
   */
  public static final String TEXT_COMPONENT_CHANGED;

  /**
   * Resources.
   */
  public static final String RESOURCES;

  /**
   * Resource key <i>dialog title</i>.
   */
  public static final String RK_DIALOG_TITLE;

  /**
   * Resource key <i>text field title</i>.
   */
  public static final String RK_TEXTFIELD_TITLE;

  /**
   * Resource key <i>replace tab title</i>.
   */
  public static final String RK_TAB_REPLACE_TITLE;

  /**
   * Resource key <i>close button text</i>.
   */
  public static final String RK_BUTTON_CLOSE_TEXT;

  /**
   * Resource key <i>find what label text</i>.
   */
  public static final String RK_LABEL_FINDWHAT_TEXT;

  /**
   * Resource key <i>find button text</i>.
   */
  public static final String RK_BUTTON_FIND_TEXT;

  /**
   * Resource key <i>replace label text</i>.
   */
  public static final String RK_LABEL_REPLACE_TEXT;

  /**
   * Resource key <i>replace button text</i>.
   */
  public static final String RK_BUTTON_REPLACE_TEXT;

  /**
   * Resource key <i>replace all button text</i>.
   */
  public static final String RK_BUTTON_REPLACEALL_TEXT;

  /**
   * Resource key <i>options title</i>.
   */
  public static final String RK_OPTIONS_TITLE;

  /**
   * Resource key <i>options words only</i>.
   */
  public static final String RK_OPTIONS_WORDS_ONLY;

  /**
   * Resource key <i>options match case</i>.
   */
  public static final String RK_OPTIONS_MATCH_CASE;

  /**
   * Resource key <i>options search up</i>.
   */
  public static final String RK_OPTIONS_SERACH_UP;

  /**
   * Resource key <i>options search down</i>.
   */
  public static final String RK_OPTIONS_SERACH_DOWN;

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final SimpleArrayList FIND_HISTORY;
  private static final HistoryComboBoxModel FIND_HISTORY_COMBOBOX_MODEL;
  private static final SimpleArrayList REPLACE_HISTORY;
  private static final HistoryComboBoxModel REPLACE_HISTORY_COMBOBOX_MODEL;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    RESOURCES = "TextSearchDialogResources"; //$NON-NLS-1$
    RK_DIALOG_TITLE = "TextSearchDialog.dialog.title"; //$NON-NLS-1$
    RK_TEXTFIELD_TITLE = "TextSearchDialog.tabFind.title"; //$NON-NLS-1$
    RK_TAB_REPLACE_TITLE = "TextSearchDialog.tabReplace.title"; //$NON-NLS-1$
    RK_BUTTON_CLOSE_TEXT = "TextSearchDialog.buttonClose.text"; //$NON-NLS-1$
    RK_LABEL_FINDWHAT_TEXT =
                           "TextSearchDialog.labelFindWhat.text"; //$NON-NLS-1$
    RK_BUTTON_FIND_TEXT = "TextSearchDialog.buttonFindNext.text"; //$NON-NLS-1$
    RK_LABEL_REPLACE_TEXT = "TextSearchDialog.labelReplace.text"; //$NON-NLS-1$
    RK_BUTTON_REPLACE_TEXT =
                           "TextSearchDialog.buttonReplace.text"; //$NON-NLS-1$
    RK_BUTTON_REPLACEALL_TEXT =
                        "TextSearchDialog.buttonReplaceAll.text"; //$NON-NLS-1$
    RK_OPTIONS_TITLE = "TextSearchDialog.options.title"; //$NON-NLS-1$
    RK_OPTIONS_WORDS_ONLY = "TextSearchDialog.options.wordsOnly"; //$NON-NLS-1$
    RK_OPTIONS_MATCH_CASE = "TextSearchDialog.options.matchCase"; //$NON-NLS-1$
    RK_OPTIONS_SERACH_UP = "TextSearchDialog.options.searchUp"; //$NON-NLS-1$
    RK_OPTIONS_SERACH_DOWN =
                           "TextSearchDialog.options.searchDown"; //$NON-NLS-1$
    TEXT_COMPONENT_CHANGED = "TextComponentChanged"; //$NON-NLS-1$
    FIND_HISTORY = new SimpleArrayList();
    FIND_HISTORY_COMBOBOX_MODEL = new HistoryComboBoxModel(FIND_HISTORY);
    REPLACE_HISTORY = new SimpleArrayList();
    REPLACE_HISTORY_COMBOBOX_MODEL = new HistoryComboBoxModel(REPLACE_HISTORY);
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  JTextComponent jtc;
  JWxDialog dialog;
  final ResourceBundle rb;
  final JTabbedPane tabPane;
  final JComboBox jcbFind;
  final JComboBox jcbReplaceFind;
  final JComboBox jcbReplace;
  final OptionsPanel findOptionsPanel;
  final OptionsPanel replaceOptionsPanel;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TextSearchDialog object.
   *
   * @param jtc JTextComponent
   *
   * @throws NullPointerException if jtc is <tt>null</tt>
   */
  public TextSearchDialog(JTextComponent jtc) {
    super(new BorderLayout());
    if (jtc == null) {
      throw new IllegalArgumentException("Null text component"); //$NON-NLS-1$
    }
    this.jtc = jtc;
    rb = BuiltInResources.INSTANCE.getResourceBundle(RESOURCES);
    setBorder(new EmptyBorder(2, 3, 3, 3));
    jcbFind = new JComboBox(FIND_HISTORY_COMBOBOX_MODEL);
    jcbFind.setEditable(true);
    jcbReplaceFind = new JComboBox(FIND_HISTORY_COMBOBOX_MODEL);
    jcbReplaceFind.setEditable(true);
    jcbReplace = new JComboBox(REPLACE_HISTORY_COMBOBOX_MODEL);
    jcbReplace.setEditable(true);
    //jcbReplaceFind.setNextFocusableComponent(jcbReplace);
    findOptionsPanel = createFindOptionsPanel();
    replaceOptionsPanel = createReplaceOptionsPanel();
    tabPane = createTabPane();
    addSearchTabs();
    add(tabPane);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns text component.
   *
   * @return text component
   */
  public JTextComponent getTextComponent() {
    return jtc;
  }

  /**
   * Sets new text component.
   *
   * @param jtc the text component
   */
  public void setTextComponent(JTextComponent jtc) {
    if (jtc == null) {
      throw new IllegalArgumentException("Null text component"); //$NON-NLS-1$
    }
    if (this.jtc != jtc) { // NOPMD
      JTextComponent oldjtc = this.jtc;
      this.jtc = jtc;
      firePropertyChange(TEXT_COMPONENT_CHANGED, oldjtc, jtc);
    }
  }

  /**
   * Sets text to find.
   *
   * @param text the text
   */
  public void setFindText(String text) {
    if (text != null) {
      jcbFind.setSelectedItem(text);
    }
  }

  /**
   * Sets text to find.
   *
   * @param text the text
   */
  public void setReplaceText(String text) {
    if (text != null) {
      jcbReplaceFind.setSelectedItem(text);
    }
  }

  /**
   * Shows text find dialog.
   */
  public void find() {
    find(null);
  }

  /**
   * Shows text find dialog.
   *
   * @param text text to search for
   */
  public void find(String text) {
    if (dialog == null) {
      dialog = createDialog();
    }
    setFindText(text);
    tabPane.setSelectedIndex(0);
    dialog.setVisible(true);
    jcbFind.requestFocus();
  }

  /**
   * Find next occurrence of the previously specified text.
   */
  public void findNext() {
    find((String)jcbFind.getSelectedItem(), findOptionsPanel.searchUp(),
         !findOptionsPanel.matchCase(), findOptionsPanel.wordsOnly());
    jtc.requestFocus();
  }

  /**
   * Shows text replace dialog.
   */
  public void replace() {
    replace(null);
  }

  /**
   * Shows text replace dialog.
   *
   * @param text text to be replaced
   */
  public void replace(String text) {
    if (dialog == null) {
      dialog = createDialog();
    }
    setReplaceText(text);
    tabPane.setSelectedIndex(1);
    dialog.setVisible(true);
    jcbReplaceFind.requestFocus();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    if (dialog != null) {
      dialog.pack();
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates tab pane.
   *
   * @return tab pane
   */
  protected JTabbedPane createTabPane() {
    return new JTabbedPane();
  }

  /**
   * Adds search tabs.
   */
  protected void addSearchTabs() {
    tabPane.addTab(rb.getString(RK_TEXTFIELD_TITLE), createFindPanel());
    tabPane.addTab(rb.getString(RK_TAB_REPLACE_TITLE), createReplacePanel());
  }

  /**
   * Creates find panel.
   *
   * @return find panel
   */
  protected JPanel createFindPanel() {
    JWxGriddedPanel jwxgpFindPanel = new JWxGriddedPanel();
    jwxgpFindPanel.setBorder(new EmptyBorder(8, 8, 6, 8));
    jwxgpFindPanel.addComponent(
                   new JLabel(rb.getString(RK_LABEL_FINDWHAT_TEXT)), 0, 0, 1, 1,
                   JWxGriddedPanel.GBC_WEST, JWxGriddedPanel.GBC_BOTH);
    jwxgpFindPanel.addComponent(jcbFind, 1, 0, 1, 1, JWxGriddedPanel.GBC_WEST,
                                JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    jwxgpFindPanel.addComponent(new FindButton(), 2, 0, 1, 1,
                                JWxGriddedPanel.GBC_EAST,
                                JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpFindPanel.addComponent(new CloseButton(), 2, 1, 1, 1,
                                JWxGriddedPanel.GBC_EAST,
                                JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpFindPanel.addComponent(findOptionsPanel, 0, 2, 2, 1,
                                JWxGriddedPanel.GBC_WEST,
                                JWxGriddedPanel.GBC_BOTH);
    return jwxgpFindPanel;
  }

  /**
   * Creates replace panel.
   *
   * @return replace panel
   */
  protected JPanel createReplacePanel() {
    JWxGriddedPanel jwxgpReplacePanel = new JWxGriddedPanel();
    jwxgpReplacePanel.setBorder(new EmptyBorder(8, 8, 6, 4));
    jwxgpReplacePanel.addComponent(
        new JLabel(rb.getString(RK_LABEL_FINDWHAT_TEXT)), 0, 0, 1, 1,
        JWxGriddedPanel.GBC_WEST, JWxGriddedPanel.GBC_NONE);
    jwxgpReplacePanel.addComponent(
                                new JLabel(rb.getString(RK_LABEL_REPLACE_TEXT)),
                                0, 1, 1, 1);
    jwxgpReplacePanel.addComponent(jcbReplaceFind, 1, 0, 1, 1,
                                   JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0,
                                   0.0);
    jwxgpReplacePanel.addComponent(jcbReplace, 1, 1, 1, 1,
                                   JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpReplacePanel.addComponent(new ReplaceButton(), 2, 0, 1, 1,
                                   JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpReplacePanel.addComponent(new ReplaceAllButton(), 2, 1, 1, 1,
                                   JWxGriddedPanel.GBC_CENTER,
                                   JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpReplacePanel.addComponent(new CloseButton(), 2, 2, 1, 1,
                                   JWxGriddedPanel.GBC_NORTH,
                                   JWxGriddedPanel.GBC_HORIZONTAL);
    jwxgpReplacePanel.addComponent(replaceOptionsPanel, 0, 2, 2, 1,
                                   JWxGriddedPanel.GBC_WEST,
                                   JWxGriddedPanel.GBC_BOTH);
    return jwxgpReplacePanel;
  }

  /**
   * Creates find options panel.
   *
   * @return find options panel
   */
  protected OptionsPanel createFindOptionsPanel() {
    return new OptionsPanel();
  }

  /**
   * Creates replace options panel.
   *
   * @return replace options panel
   */
  protected OptionsPanel createReplaceOptionsPanel() {
    return new OptionsPanel();
  }

  /**
   * Creates dialog.
   *
   * @return dialog
   */
  protected JWxDialog createDialog() {
    Component p = null;
    for (p = jtc; p != null; p = p.getParent()) {
      if (p instanceof Window) {
        break;
      }
    }
    JWxDialog jwxd = null;
    if (p instanceof Dialog) {
      jwxd = new JWxDialog((Dialog)p, rb.getString(RK_DIALOG_TITLE), false);
    } else if (p instanceof Frame) {
      jwxd = new JWxDialog((Frame)p, rb.getString(RK_DIALOG_TITLE), false);
    } else {
      throw new IllegalArgumentException(
                                    "Null dialog parent window"); //$NON-NLS-1$
    }
    Container cp = jwxd.getContentPane();
    jwxd.setResizable(false);
    cp.add(this);
    jwxd.pack();
    return jwxd;
  }

  /**
   * Saves find history.
   */
  protected void saveToFindHistory() {
    FIND_HISTORY_COMBOBOX_MODEL.rememberCurrentItem();
  }

  /**
   * Saves replace history.
   */
  protected void saveToReplaceHistory() {
    FIND_HISTORY_COMBOBOX_MODEL.rememberCurrentItem();
    REPLACE_HISTORY_COMBOBOX_MODEL.rememberCurrentItem();
  }

  /**
   * Finds the specified text.
   *
   * @param text the text
   * @param up the direction
   * @param ignoreCase the ignore case flag
   * @param whole the whole word flag
   *
   * @return <tt>true</tt> if the text is found
   */
  protected boolean find(String text, boolean up, boolean ignoreCase,
                         boolean whole) {
    if (text != null) {
      jtc.requestFocus();
      saveToFindHistory();
      SearchSegment sseg = new SearchSegment(text, up, ignoreCase);
      Document doc = jtc.getDocument();
      int dlen = doc.getLength();
      if (dlen == 0) {
        return false;
      }
      int cpos;
      Caret caret = jtc.getCaret();
      int mark = caret.getMark();
      int dot = caret.getDot();
      if (up) {
        cpos = Math.min(mark, dot);
      } else {
        cpos = Math.max(mark, dot);
      }
      if (cpos < 0) {
        cpos = 0;
      }
      try {
        if (up) {
          doc.getText(0, cpos, sseg);
        } else {
          doc.getText(cpos, dlen - 1 - cpos, sseg);
        }
      } catch (BadLocationException bae) {
        return false;
      }
      int si;
      if (whole) {
        si = sseg.findWhole();
      } else {
        si = sseg.find();
      }
      if (si >= 0) {
        if (up) {
          caret.setDot(si + text.length());
          caret.moveDot(si);
        } else {
          caret.setDot(si);
          caret.moveDot(si + text.length());
        }
        return true;
      }
    }
    return false;
  }

  /**
   * Performs replace.
   */
  protected void doReplace() {
    jtc.requestFocus();
    String stext = (String)jcbReplaceFind.getSelectedItem();
    String rtext = (String)jcbReplace.getSelectedItem();
    if ((stext != null) && (rtext != null)) {
      saveToReplaceHistory();
      boolean up = replaceOptionsPanel.searchUp();
      if (find(stext, up, !replaceOptionsPanel.matchCase(),
               replaceOptionsPanel.wordsOnly())) {
        Caret caret = jtc.getCaret();
        if (up) {
          int ncp = jtc.getCaretPosition();
          jtc.replaceSelection(rtext);
          caret.setDot(ncp + rtext.length());
          caret.moveDot(ncp);
        } else {
          int mark = caret.getMark();
          jtc.replaceSelection(rtext);
          caret.setDot(mark);
          caret.moveDot(mark + rtext.length());
        }
      }
    }
  }

  /**
   * Performs replace all operation.
   */
  protected void replaceAll() {
    jtc.requestFocus();
    String stext = (String)jcbReplaceFind.getSelectedItem();
    String rtext = (String)jcbReplace.getSelectedItem();
    if ((stext != null) && (rtext != null)) {
      saveToReplaceHistory();
      int cp = jtc.getCaretPosition();
      jtc.setCaretPosition(jtc.getDocument().getLength());
      while (find((String)jcbReplaceFind.getSelectedItem(),
                  true, !replaceOptionsPanel.matchCase(),
                  replaceOptionsPanel.wordsOnly())) {
        jtc.replaceSelection(rtext);
      }
      jtc.setCaretPosition(cp);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected boolean processKeyBinding(KeyStroke ks, KeyEvent e, int condition,
                                      boolean pressed) {
    int keycode = e.getKeyCode();
    if ((dialog != null) && (keycode == KeyEvent.VK_ESCAPE)) {
      dialog.dispose();
      return true;
    }
    if (keycode == KeyEvent.VK_ENTER) {
      if (tabPane.getSelectedIndex() == 0) {
        find((String)jcbFind.getSelectedItem(), findOptionsPanel.searchUp(),
             !findOptionsPanel.matchCase(), findOptionsPanel.wordsOnly());
      } else if (tabPane.getSelectedIndex() == 1) {
        doReplace();
      }
      jtc.requestFocus();
    }
    return super.processKeyBinding(ks, e, condition, pressed);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Close Button.
   */
  protected class CloseButton extends JWxButton implements Runnable {
    /**
     * Constructs new CloseButton object.
     */
    public CloseButton() {
      super();
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                 getResourceBundle(RESOURCES);
      setText(rb.getString(RK_BUTTON_CLOSE_TEXT));
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      dialog.dispose();
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      SwingUtilities.invokeLater(this);
    }
  }

  /**
   * Find Button.
   */
  protected class FindButton extends JWxButton implements Runnable {
    /**
     * Constructs new FindButton object.
     */
    public FindButton() {
      super();
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                  getResourceBundle(RESOURCES);
      setText(rb.getString(RK_BUTTON_FIND_TEXT));
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      dialog.dispose();
      find((String)jcbFind.getSelectedItem(), findOptionsPanel.searchUp(),
           !findOptionsPanel.matchCase(), findOptionsPanel.wordsOnly());
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      SwingUtilities.invokeLater(this);
    }
  }

  /**
   * Replace Button.
   */
  protected class ReplaceButton extends JWxButton implements Runnable {
    /**
     * Constructs new ReplaceButton object.
     */
    public ReplaceButton() {
      super();
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                  getResourceBundle(RESOURCES);
      setText(rb.getString(RK_BUTTON_REPLACE_TEXT));
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      doReplace();
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      SwingUtilities.invokeLater(this);
    }
  }

  /**
   * Replace All Button.
   */
  protected class ReplaceAllButton extends JWxButton implements Runnable {
    /**
     * Constructs new ReplaceAllButton object.
     */
    public ReplaceAllButton() {
      super();
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                  getResourceBundle(RESOURCES);
      setText(rb.getString(RK_BUTTON_REPLACEALL_TEXT));
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      replaceAll();
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      SwingUtilities.invokeLater(this);
    }
  }

  /**
   * Options Panel.
   */
  protected static class OptionsPanel extends JWxGriddedPanel {

    // Private Fields. ---------------------------------------------------------

    private final JCheckBox jcbWordsOnly;
    private final JCheckBox jcbMatchCase;
    private final JRadioButton jrbSearchUp;
    private final JRadioButton jrbSearchDown;

    // Public Constructor. -----------------------------------------------------

    /**
     * Constructs new OptionsPanel object.
     */
    public OptionsPanel() {
      super();
      ResourceBundle rb = BuiltInResources.INSTANCE.
                                                  getResourceBundle(RESOURCES);
      setBorder(new TitledBorder(rb.getString(RK_OPTIONS_TITLE)));
      jcbWordsOnly = new JCheckBox(rb.getString(RK_OPTIONS_WORDS_ONLY));
      jcbMatchCase = new JCheckBox(rb.getString(RK_OPTIONS_MATCH_CASE), true);
      jrbSearchUp = new JRadioButton(rb.getString(RK_OPTIONS_SERACH_UP));
      jrbSearchDown = new JRadioButton(rb.getString(RK_OPTIONS_SERACH_DOWN),
                                       true);
      ButtonGroup bg = new ButtonGroup();
      bg.add(jrbSearchUp);
      bg.add(jrbSearchDown);
      addComponent(jcbWordsOnly, 0, 0);
      addComponent(jcbMatchCase, 0, 1);
      addComponent(jrbSearchUp, 1, 0);
      addComponent(jrbSearchDown, 1, 1, 0, 0, JWxGriddedPanel.GBC_WEST,
                   JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    }

    // Public Methods. ---------------------------------------------------------

    /**
     * Returns <tt>true</tt> if <i>words only</i> is checked.
     *
     * @return <tt>true</tt> if <i>words only</i> is checked
     */
    public boolean wordsOnly() {
      return jcbWordsOnly.isSelected();
    }

    /**
     * Returns <tt>true</tt> if <i>match case</i> is checked.
     *
     * @return <tt>true</tt> if <i>match case</i> is checked
     */
    public boolean matchCase() {
      return jcbMatchCase.isSelected();
    }

    /**
     * Returns <tt>true</tt> if <i>search up</i> is checked.
     *
     * @return <tt>true</tt> if <i>search up</i> is checked
     */
    public boolean searchUp() {
      return jrbSearchUp.isSelected();
    }

  }

  /**
   * History ComboBox Model.
   */
  protected static class HistoryComboBoxModel implements ComboBoxModel {

    // Private Fields. ---------------------------------------------------------

    private final WeakArrayList listeners;
    private Object selectedObject;
    private final SimpleArrayList history;

    // Public Constructor. -----------------------------------------------------

    /**
     * Constructs new HistoryComboBoxModel object.
     *
     * @param history the history list
     */
    public HistoryComboBoxModel(SimpleArrayList history) {
      listeners = new WeakArrayList(7);
      this.history = history;
    }

    // Public Methods. ---------------------------------------------------------

    /**
     * Remembers current item.
     */
    public void rememberCurrentItem() {
      if (selectedObject != null) {
        int hs = history.size();
        if (hs == 0) {
          history.add(selectedObject);
          fireContentsChanged();
        } else {
          int ind = history.indexOf(selectedObject);
          if (ind < 0) {
            history.insert(0, selectedObject);
            fireContentsChanged();
          } else if (ind > 0) {
            history.remove(ind);
            history.insert(0, selectedObject);
            fireContentsChanged();
          }
        }
      }
    }

    // Overridden Methods. -----------------------------------------------------

    /**
     * {@inheritDoc}
     */
    public void setSelectedItem(Object anItem) {
      synchronized (listeners) {
        selectedObject = anItem;
        fireContentsChanged();
      }
    }

    /**
     * {@inheritDoc}
     */
    public Object getSelectedItem() {
      synchronized (listeners) {
        return selectedObject;
      }
    }

    /**
     * {@inheritDoc}
     */
    public int getSize() {
      synchronized (listeners) {
        return history.size();
      }
    }

    /**
     * {@inheritDoc}
     */
    public Object getElementAt(int index) {
      synchronized (listeners) {
        return history.get(index);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void addListDataListener(ListDataListener l) {
      synchronized (listeners) {
        listeners.add(l);
      }
    }

    /**
     * {@inheritDoc}
     */
    public void removeListDataListener(ListDataListener l) {
      synchronized (listeners) {
        listeners.remove(l);
      }
    }

    // Private Method. ---------------------------------------------------------

    private void fireContentsChanged() {
      ListDataEvent e = new ListDataEvent(this, ListDataEvent.CONTENTS_CHANGED,
                                          -1, -1);
      boolean hasnull = false;
      final WeakArrayList ls = listeners;
      for (int i = ls.size() - 1; i >= 0; i--) {
        ListDataListener ldl = (ListDataListener)ls.get(i);
        if (ldl == null) {
          hasnull = true;
        } else {
          ldl.contentsChanged(e);
        }
      }
      if (hasnull) {
        ls.expungeStaleReferences();
      }
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 21:24:37 Slobodan
   Initial version

*****************************************************************************^*/
