/*
 * @(#)AbstractCodeEditorKit.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.io.IOException;
import java.io.Reader;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.StyledEditorKit;

/**
 * Code EditorKit.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class AbstractCodeEditorKit extends StyledEditorKit {

/*^**************************************************************************^*/
/*- Protected Field.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Syntax highlight flag.
   */
  protected boolean syntaxHighlight;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new WxCodeEditorKit object.
   */
  public AbstractCodeEditorKit() { // NOPMD
    syntaxHighlight = true;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Enables/disables syntax highlighting.
   *
   * @param enable if <tt>true</tt> enable
   */
  public void setSyntaxHighlight(boolean enable) {
    syntaxHighlight = enable;
  }

  /**
   * Returns <tt>true</tt> if highlighting is enabled.
   *
   * @return <tt>true</tt> if highlighting is enabled
   */
  public boolean getSyntaxHighlight() { // NOPMD
    return syntaxHighlight;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Inserts content from the given stream.
   *
   * @param in  the stream to read from
   * @param doc the destination for the insertion.
   * @param pos the location in the document to place the content >= 0.
   *
   * @exception IOException on any I/O error
   * @exception BadLocationException if pos represents an invalid
   *            location within the document.
   */
  public void read(Reader in, Document doc,
                   int pos) throws IOException, BadLocationException {
    if (doc instanceof AbstractCodeDocument) {
      AbstractCodeDocument acd = (AbstractCodeDocument)doc;
      boolean eh = acd.getSyntaxHighlight();
      acd.setSyntaxHighlight(false);
      super.read(in, doc, pos);
      acd.setSyntaxHighlight(eh);
      if (eh) {
        acd.highlight();
      }
    } else {
      super.read(in, doc, pos);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:55:11 Slobodan
   Initial version

*****************************************************************************^*/
