/*
 * @(#)CodeEditorManager.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import net.wrka.swing.components.text.code.kits.JavaCodeEditorKit;
import net.wrka.swing.components.text.code.kits.XMLCodeEditorKit;

/**
 * Code Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class CodeEditorManager {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private CodeEditorManager() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns JavaCodeEditor.
   *
   * @return JavaCodeEditor
   */
  public static JWxCodeEditor getJavaCodeEditor() {
    return new JWxCodeEditor(new JavaCodeEditorKit());
  }

  /**
   * Returns JWxCodeViewer.
   *
   * @return JWxCodeViewer
   */
  public static JWxCodeViewer getJavaCodeViewer() {
    return new JWxCodeViewer(new JavaCodeEditorKit());
  }

  /**
   * Returns JWxCodeViewer.
   *
   * @param copyText copy action name
   * @param copyTextMnem copy action mnemonic
   * @param selectAllText select all action name
   * @param selectAlltMnem select all action mnemonic
   *
   * @return JWxCodeViewer
   */
  public static JWxCodeViewer getJavaCodeViewer(String copyText,
                                                int copyTextMnem,
                                                String selectAllText,
                                                int selectAlltMnem) {
    return new JWxCodeViewer(new JavaCodeEditorKit(), copyText, copyTextMnem,
                             selectAllText, selectAlltMnem);
  }

  /**
   * Returns XMLCodeEditor.
   *
   * @return XMLCodeEditor
   */
  public static JWxCodeEditor getXMLCodeEditor() {
    return new JWxCodeEditor(new XMLCodeEditorKit());
  }

  /**
   * Returns XMLCodeViewer.
   *
   * @return XMLCodeViewer
   */
  public static JWxCodeViewer getXMLCodeViewer() {
    return new JWxCodeViewer(new XMLCodeEditorKit());
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:54:38 Slobodan
   Initial version

*****************************************************************************^*/
