/*
 * @(#)JWxCodeEditor.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.awt.event.KeyEvent;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.StyledEditorKit;

import net.wrka.swing.components.text.JWxEditorPane;

/**
 * Code Editor.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxCodeEditor extends JWxEditorPane {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean sytnaxHighlight;
  private String tab;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxCodeEditor object.
   *
   * @param kit the WxCodeEditorKit
   */
  public JWxCodeEditor(StyledEditorKit kit) {
    super();
    setEditorKit(kit);
  }

  /**
   * Constructs new JWxCodeEditor object.
   *
   * @param text the text
   * @param kit the WxCodeEditorKit
   */
  public JWxCodeEditor(String text, StyledEditorKit kit) {
    super();
    setEditorKit(kit);
    setText(text);
    undoManager.discardAllEdits();
    updateEditActions();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns syntax highlighting.
   *
   * @return syntax highlighting
   */
  public boolean getSyntaxHighlight() { // NOPMD
    return sytnaxHighlight;
  }

  /**
   * Sets syntax highlighting.
   *
   * @param b if <tt>true</tt> syntax will be highlighted
   */
  public void setSyntaxHighlight(boolean b) {
    sytnaxHighlight = b;
    Document doc = getDocument();
    if (doc instanceof AbstractCodeDocument) {
      ((AbstractCodeDocument)doc).setSyntaxHighlight(sytnaxHighlight);
    }
    EditorKit kit = getEditorKit();
    if (kit instanceof AbstractCodeEditorKit) {
      ((AbstractCodeEditorKit)kit).setSyntaxHighlight(b);
    }
  }

  /**
   * Returns insert tab size.
   *
   * @return insert tab size
   */
  public int getInsertTabSize() {
    if (tab == null) {
      return 0;
    }
    return tab.length();
  }

  /**
   * Sets insert tab size.
   *
   * @param tabSize tab size
   */
  public void setInsertTabSize(int tabSize) {
    char[] instTabSeq = new char[tabSize];
    for (int i = tabSize - 1; i >= 0; i--) {
      instTabSeq[i] = ' ';
    }
    tab = new String(instTabSeq);
  }

/*^**************************************************************************^*/
/*- Overridden methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void undo() {
    super.undo();
    Document doc = getDocument();
    if (doc instanceof AbstractCodeDocument) {
      ((AbstractCodeDocument)doc).highlight();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void redo() {
    super.redo();
    Document doc = getDocument();
    if (doc instanceof AbstractCodeDocument) {
      ((AbstractCodeDocument)doc).highlight();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void processKeyEvent(java.awt.event.KeyEvent ke) {
    if ((ke.getKeyCode() != KeyEvent.VK_TAB) || (tab == null)) {
      super.processKeyEvent(ke);
    } else {
      try {
        if (ke.getID() == KeyEvent.KEY_PRESSED) {
          getDocument().insertString(getCaret().getDot(), tab, null);
        }
      } catch (BadLocationException e) { // NOPMD
        // empty catch block
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 22:11:59 Slobodan
   Initial version

*****************************************************************************^*/
