/*
 * @(#)JWxCodeViewer.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.io.IOException;
import java.io.InputStream;

import javax.swing.text.EditorKit;

import net.wrka.swing.actions.ActionUtils;
import net.wrka.swing.components.text.JWxEditorPane;

/**
 * Code Viewer.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class JWxCodeViewer extends JWxEditorPane {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new JWxCodeViewer object.
   *
   * @param kit the StyledEditorKit
   */
  public JWxCodeViewer(RegExCodeEditorKit kit) {
    this(kit, null, 0, null, 0);
  }

  /**
   * Constructs new JWxCodeViewer object.
   *
   * @param kit the StyledEditorKit
   * @param copyText copy action name
   * @param copyTextMnem copy action mnemonic
   * @param selectAllText select all action name
   * @param selectAllMnem select all action mnemonic
   */
  public JWxCodeViewer(RegExCodeEditorKit kit, String copyText,
                       int copyTextMnem, String selectAllText,
                       int selectAllMnem) {
    super();
    super.setEditable(false);
    if (copyText != null) {
      ActionUtils.setupAction(actionCopy, copyText, copyTextMnem);
    }
    if (selectAllText != null) {
      ActionUtils.setupAction(actionSelectAll, selectAllText, selectAllMnem);
    }
    super.setEditorKit(kit);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Loads codes from the specified input stream.
   *
   * @param is the InputStream
   *
   * @throws IOException if failed
   */
  public void load(InputStream is) throws IOException {
    super.read(is, null);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setEditable(boolean b) {
    // empty catch block
  }

  /**
   * {@inheritDoc}
   */
  protected void createUndoListner() {
    // empty catch block
  }

  /**
   * {@inheritDoc}
   */
  public void setEditorKit(EditorKit kit) {
    // empty catch block
  }

  /**
   * {@inheritDoc}
   */
  protected void createEditActions() {
    actionCopy = new ActionCopy();
    actionSelectAll = new ActionSelectAll();
  }

  /**
   * {@inheritDoc}
   */
  public void updateEditActions() {
    boolean enabled = isEnabled();
    boolean hasSelection = hasSelection();
    boolean cancopy = enabled && hasSelection;
    boolean canselectall = enabled && canSelectAll();
    if (actionCopy.isEnabled() != cancopy) {
      actionCopy.setEnabled(cancopy);
    }
    if (actionSelectAll.isEnabled() != canselectall) {
      actionSelectAll.setEnabled(canselectall);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void updateCaretActionStates(boolean hasSelection) {
    boolean enabled = isEnabled();
    actionCopy.setEnabled(enabled && hasSelection);
    actionSelectAll.setEnabled(enabled && canSelectAll());
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 22:15:32 Slobodan
   Initial version

*****************************************************************************^*/
