/*
 * @(#)RegExCodeDocument.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

/**
 * RE Code Document.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class RegExCodeDocument extends AbstractCodeDocument {

/*^**************************************************************************^*/
/*- Protected Field.                                                         -*/
/*^**************************************************************************^*/

  /**
   * The document type.
   */
  protected RegExTypes types;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new EditorDocument object.
   *
   * @param types the RegExTypes
   */
  public RegExCodeDocument(RegExTypes types) {
    this(true, types, "DialogInput", 12); //$NON-NLS-1$
  }

  /**
   * Constructs new EditorDocument object.
   *
   * @param sytnaxHighlight syntax highlight
   * @param types the RegExTypes
   */
  public RegExCodeDocument(boolean sytnaxHighlight, RegExTypes types) {
    this(sytnaxHighlight, types, "DialogInput", 12); //$NON-NLS-1$
  }

  /**
   * Constructs new EditorDocument object.
   *
   * @param sytnaxHighlight syntax highlight
   * @param types the RegExTypes
   * @param fontFamily font size
   * @param fontSize font family
   */
  public RegExCodeDocument(boolean sytnaxHighlight, RegExTypes types,
                        String fontFamily, int fontSize) {
    super(sytnaxHighlight, fontFamily, fontSize);
    this.types = types;
    types.setStyles(this);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void highlightAll() {
    String text;
    try {
      text = getText(0, getLength());
    } catch (Throwable t) { // NOPMD
      return;
    }
    highlighting = true;
    setCharacterAttributes(0, getLength(), getStyle("default"), //$NON-NLS-1$
                           true);
    RegExTokenizer.Token token;
    RegExTokenizer tokenizer = new RegExTokenizer(types, text);
    int typeCount = types.getTypeCount();
    while ((token = tokenizer.nextToken()) != null) {
      int pos = token.pos;
      String type = token.type;
      String word = token.text;
      int len = word.length();
      for (int i = 0; i < typeCount; i++) {
        String name = types.getTypeName(i);
        if (type.equals(name)) {
          if (types.getTypeForeground(i) == null) {
            String style = types.getStyleName(word);
            if (style != null) {
              setCharacterAttributes(pos, len, getStyle(style), false);
            }
          } else {
            setCharacterAttributes(pos, len, getStyle(name), false);
          }
        }
      }
    }
    highlighting = false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:48:24 Slobodan
   Initial version

*****************************************************************************^*/
