/*
 * @(#)RegExTokenizer.java  1.0.0 / 2008-05-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code;

import java.util.regex.Matcher;

/**
 * RegExTypes.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class RegExTokenizer {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final RegExTypes types;
  private final Matcher matcher;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new RegExTypes object.
   *
   * @param types the RegExTypes
   * @param text the text
   */
  public RegExTokenizer(RegExTypes types, CharSequence text) {
    this.types = types;
    matcher = types.getMatcher(text);
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns next token.
   *
   * @return next token
   */
  public Token nextToken() {
    if (matcher.find()) {
      return getToken(matcher.start());
    }
    return null;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private Token getToken(int pos) {
    int count = types.getTypeCount();
    RegExTypes tp = types;
    Matcher m = matcher;
    for (int i = 1; i <= count; i++) {
      String token = m.group(i);
      if (token != null) {
        return new Token(token, tp.getTypeName(i - 1), pos);
      }
    }
    return null;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Token class.
   */
  static final class Token {

    // Package-Private Fields. -------------------------------------------------

    String text;
    String type;
    int pos;

    // Public Constructors. ----------------------------------------------------

    /**
     * Constructs new Token object.
     *
     * @param text the text
     * @param type the type
     * @param pos the position
     */
    public Token(String text, String type, int pos) {
      this.text = text;
      this.type = type;
      this.pos = pos;
    }

    /**
     * {@inheritDoc}
     */
    public String toString() {
      StringBuffer sb = new StringBuffer(type);
      sb.append('(');
      sb.append(text);
      sb.append(", "); //$NON-NLS-1$
      sb.append(pos);
      sb.append(')');
      return sb.toString();
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-23 23:43:59 Slobodan
   Initial version

*****************************************************************************^*/
