/*
 * @(#)RegExJavaTypes.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code.kits;

import java.awt.Color;
import java.io.IOException;

import net.wrka.swing.components.text.code.KeywordManager;
import net.wrka.swing.components.text.code.RegExTypes;
import net.wrka.swing.components.text.code.resources.CodeEditorResourceManager;

/**
 * RegExJavaTypes.
 * (Based on article
 * http://www.ftponline.com/javapro/2003_07/magazine/columns/visualcomponents/
 * Created by Claude Duguay)
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class RegExJavaTypes extends RegExTypes {

/*^**************************************************************************^*/
/*- Package-Private Static Final Field.                                      -*/
/*^**************************************************************************^*/

  static final RegExJavaTypes INSTANCE;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final Color COMMENT_COLOR;
  private static final Color TEXT_COLOR;
  private static KeywordManager keywordManager;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    COMMENT_COLOR = new Color(0, 128, 0);
    TEXT_COLOR = new Color(0, 0, 213);
    INSTANCE = new RegExJavaTypes();
    keywordManager = new KeywordManager(2);
    try {
      keywordManager.addKeywordList(
          CodeEditorResourceManager.INSTANCE.
              getKeywordStream("java/reserved.properties")); //$NON-NLS-1$
      keywordManager.addKeywordList(
          CodeEditorResourceManager.INSTANCE.
              getKeywordStream("java/types.properties")); //$NON-NLS-1$
    } catch (IOException ioe) {
      keywordManager.clear();
      keywordManager = null;
    }
  }

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new RegExJavaTypes object.
   */
  public RegExJavaTypes() {
    super();
    addTokenType("linecomment", "//[^\n]*", //$NON-NLS-1$ //$NON-NLS-2$
                 COMMENT_COLOR);
    addTokenType("starcomment",  //$NON-NLS-1$
                 "/\\*[^*]*\\*+(?:[^/*][^*]*\\*+)*/", //$NON-NLS-1$
                 COMMENT_COLOR);
    addTokenType("text", //$NON-NLS-1$
                 "\"(?:\\\\.|[^\"\\\\])*\"", TEXT_COLOR); //$NON-NLS-1$
    addTokenType("char", //$NON-NLS-1$
                 "'(?:\\\\[^']+|[^'])'", TEXT_COLOR); //$NON-NLS-1$
    addTokenType("atom", "[a-zA-Z]\\w*", null); //$NON-NLS-1$  //$NON-NLS-2$
    addTokenType("number", //$NON-NLS-1$
                 "[0-9]+(\\.[0-9]+)?", TEXT_COLOR); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected KeywordManager getKeywordManager() {
    return keywordManager;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 08:46:30 Slobodan
   Initial version

*****************************************************************************^*/
