/*
 * @(#)CodeEditorResourceManager.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.components.text.code.resources;

import java.io.InputStream;

import net.wrka.swing.resources.ResourceManager;

/**
 * Code Resource Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class CodeEditorResourceManager extends ResourceManager {

/*^**************************************************************************^*/
/*- Public Static Field.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Singleton instance.
   */
  public static final CodeEditorResourceManager INSTANCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    INSTANCE = new CodeEditorResourceManager(null, "images"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final String KEYWORDS;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private CodeEditorResourceManager(String repos, String images) {
    super(repos, images);
    KEYWORDS = homeFolder.concat("keywords"); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Creates keyword stream form resource repository.
   *
   * @param fileName resource file name
   *
   * @return keyword stream
   */
  public InputStream getKeywordStream(String fileName) {
    if (fileName == null) {
      throw new IllegalArgumentException("Null fileName"); //$NON-NLS-1$
    }
    try {
      return classLoader.getResourceAsStream(concatResource(KEYWORDS,
                                                            fileName));
    } catch (Throwable t) { // NOPMD
      logError("Resource Manager cannot keywords", fileName, //$NON-NLS-1$
               KEYWORDS);
    }
    return null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 08:38:58 Slobodan
   Initial version

*****************************************************************************^*/
