/*
 * @(#)FastGradientPaint.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 *
 * The original code is developed by
 * Sebastian Ferreyra (sebastianf@citycolor.net).
 * It is published under the terms of the GNU Lesser General Public License.
 */
package net.wrka.swing.graphics;

import java.awt.Color;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;

/**
 * Fast Gradient Paint.
 *
 * @author Sebastian Ferreyra (sebastianf@citycolor.net)
 * @version original
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class FastGradientPaint implements Paint {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final int startColor;
  private final int endColor;
  private final boolean isVertical;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new FastGradientPaint object.
   *
   * @param sc the start color
   * @param ec the end color
   * @param isVertical if <tt>true</tt> gradient is vertical
   */
  public FastGradientPaint(Color sc, Color ec, boolean isVertical) {
    startColor = sc.getRGB();
    endColor = ec.getRGB();
    this.isVertical = isVertical;
  }

/*^**************************************************************************^*/
/*- Paint Interface Implementation.                                          -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public PaintContext createContext(ColorModel cm, Rectangle r,
                                    Rectangle2D r2d, AffineTransform xform,
                                    RenderingHints hints) {
    return new FastGradientPaintContext(cm, r, startColor, endColor,
                                        isVertical);
  }

  /**
   * {@inheritDoc}
   */
  public int getTransparency() {
    if ((((startColor & endColor) >> 24) & 0xFF) == 0xFF) {
      return OPAQUE;
    }
    return TRANSLUCENT;
  }

}

/*^*****************************************************************************
  File History:

 - Initial version Sebastian Ferreyra

 - 2008-05-17 22:51:28 Slobodan
   Adapted version

*****************************************************************************^*/
