/*
 * @(#)HelpHandler.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.help;

import javax.swing.tree.DefaultMutableTreeNode;

import net.wrka.util.stack.ObjectStack;
import net.wrka.xml.simple.XMLAttributes;
import net.wrka.xml.simple.sax.DefaultDocumentHandler;
import net.wrka.xml.simple.sax.ParsingException;

/**
 * SimpleHelp SAX handler.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class HelpHandler extends DefaultDocumentHandler {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String TOC = "toc"; //$NON-NLS-1$
  private static final String TOCITEM = "tocitem"; //$NON-NLS-1$
  private static final String TOPIC = "topic"; //$NON-NLS-1$
  private static final String NAME = "name"; //$NON-NLS-1$
  private static final String TARGET = "target"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final ObjectStack stack;
  private DefaultMutableTreeNode parent;
  private String key;
  private String value;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HelpHandler object.
   *
   * @param root the root node
   */
  public HelpHandler(DefaultMutableTreeNode root) {
    super();
    parent = root;
    stack = new ObjectStack();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void startElement(String tag,
                           XMLAttributes attrs) throws ParsingException {
    if (TOC.equals(tag)) {
      if ((attrs != null) && attrs.contains(NAME) && attrs.contains(TARGET)) {
        parent.setUserObject(new HelpTreeNode(attrs.get(NAME),
                                              attrs.get(TARGET)));
      }
    } else if (TOPIC.equals(tag)) {
      if ((attrs != null) && attrs.contains(NAME) && attrs.contains(TARGET)) {
        key = attrs.get(NAME);
        value = attrs.get(TARGET);
        parent.add(new DefaultMutableTreeNode(new HelpTreeNode(key, value)));
      }
    } else if (TOCITEM.equals(tag) && (attrs != null) &&
               attrs.contains(NAME) && attrs.contains(TARGET)) {
      key = attrs.get(NAME);
      value = attrs.get(TARGET);
      HelpTreeNode userObject = new HelpTreeNode(key, value, true);
      DefaultMutableTreeNode node = new DefaultMutableTreeNode(userObject);
      stack.push(parent);
      parent.add(node);
      parent = node;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void endElement(String tag) throws ParsingException {
    if (TOCITEM.equals(tag)) {
      parent = (DefaultMutableTreeNode)stack.pop();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 21:00:59 Slobodan
   Initial version

*****************************************************************************^*/
