/*
 * @(#)HelpTreeNode.java  1.0.0 / 2008-05-17
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.help;

import net.wrka.util.CommonConstants;

/**
 * SimpleHelp Tree Node.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class HelpTreeNode {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private String name;
  private String target;
  private boolean directory;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new HelpTreeNode object.
   *
   * @param name the node name
   * @param target the target help file
   */
  public HelpTreeNode(String name, String target) {
    this(name, target, false);
  }

  /**
   * Constructs new HelpTreeNode object.
   *
   * @param name the node name
   * @param target the target help file
   * @param directory if <tt>true</tt> node is directory
   */
  public HelpTreeNode(String name, String target, boolean directory) {
    if (name == null) {
      this.name = CommonConstants.STRING_EMPTY;
    } else {
      this.name = name;
    }
    if (target == null) {
      this.target = CommonConstants.STRING_EMPTY;
    } else {
      this.target = target;
    }
    this.directory = directory;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns node name.
   *
   * @return node name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns target help file.
   *
   * @return target help file
   */
  public String getTarget() {
    return target;
  }

  /**
   * Returns <tt>true</tt> if node is directory.
   *
   * @return <tt>true</tt> if node is directory
   */
  public boolean isDirectory() {
    return directory;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String toString() {
    return name;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-17 21:35:15 Slobodan
   Initial version

*****************************************************************************^*/
