/*
 * @(#)TextComponentOutputStream.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.io;

import java.io.IOException;
import java.io.OutputStream;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.JTextComponent;

/**
 * TextComponent OutputStream.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class TextComponentOutputStream extends OutputStream {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  private final Document document;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TextComponentOutputStream object.
   *
   * @param textComponent the JTextComponent
   */
  public TextComponentOutputStream(JTextComponent textComponent) {
    super();
    if (textComponent == null) {
      throw new IllegalArgumentException("Null Text Component"); //$NON-NLS-1$
    }
    document = textComponent.getDocument();
    if (document == null) {
      throw new IllegalArgumentException("Null Text Document"); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Clears this output stream.
   */
  public void clear() {
    try {
      document.remove(0, document.getLength());
    } catch (BadLocationException bae) { // NOPMD
      // empty catch block
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void write(int b) throws IOException {
    try {
      document.insertString(document.getLength(), String.valueOf(b), null);
    } catch (BadLocationException bae) {
      throw new IOException(bae.getMessage()); // NOPMD
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(byte[] b) throws IOException {
    try {
      document.insertString(document.getLength(), new String(b), null);
    } catch (BadLocationException bae) {
      throw new IOException(bae.getMessage()); // NOPMD
    }
  }

  /**
   * {@inheritDoc}
   */
  public void write(byte[] b, int off, int len) throws IOException {
    try {
      document.insertString(document.getLength(), new String(b, off, len),
                            null);
    } catch (BadLocationException bae) {
      throw new IOException(bae.getMessage()); // NOPMD
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 21:47:41 Slobodan
   Initial version

*****************************************************************************^*/
