/*
 * @(#)BorderNonUIResource.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.patch;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.border.Border;

/**
 * Border NON UIResource.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BorderNonUIResource implements Border {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final Border delegate;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new BorderNonUIResource object.
   *
   * @param delegate the border
   */
  public BorderNonUIResource(Border delegate) {
    if (delegate == null) {
      throw new IllegalArgumentException("Null delegate"); //$NON-NLS-1$
    }
    this.delegate = delegate;
  }

/*^**************************************************************************^*/
/*- Border Interface Implementation.                                         -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int width,
                          int height) {
    delegate.paintBorder(c, g, x, y, width, height);
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return delegate.getBorderInsets(c);
  }

  /**
   * {@inheritDoc}
   */
  public boolean isBorderOpaque() {
    return delegate.isBorderOpaque();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 21:18:37 Slobodan
   Initial version

*****************************************************************************^*/
