/*
 * @(#)AlphaColorUIResource.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf;

import java.awt.Color;

import javax.swing.plaf.UIResource;

/**
 * Color UIResource with transparency support. <i> (This class is necessary
 * because javax.swing.plaf.ColorUIResource does not support transparency) </i>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class AlphaColorUIResource extends Color implements UIResource {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 5556388097930034592L;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param rgb the rgb representation of the color
   */
  public AlphaColorUIResource(int rgb) {
    super(rgb);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param rgb the rgb representation of the color
   * @param hasAlpha <tt>true</tt> if the color has alpha component
   */
  public AlphaColorUIResource(int rgb, boolean hasAlpha) {
    super(rgb, hasAlpha);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param r the red component of the color
   * @param g the green component of the color
   * @param b the blue component of the color
   */
  public AlphaColorUIResource(float r, float g, float b) {
    super(r, g, b);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param r the red component of the color
   * @param g the green component of the color
   * @param b the blue component of the color
   * @param a the alpha component of the color
   */
  public AlphaColorUIResource(float r, float g, float b, float a) {
    super(r, g, b, a);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param r the red component of the color
   * @param g the green component of the color
   * @param b the blue component of the color
   */
  public AlphaColorUIResource(int r, int g, int b) {
    super(r, g, b);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param r the red component of the color
   * @param g the green component of the color
   * @param b the blue component of the color
   * @param a the alpha component of the color
   */
  public AlphaColorUIResource(int r, int g, int b, int a) {
    super(r, g, b, a);
  }

  /**
   * Constructs new AlphaColorUIResource object.
   *
   * @param c the color
   */
  public AlphaColorUIResource(Color c) {
    super(c.getRed(), c.getGreen(), c.getBlue(), c.getAlpha());
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 22:03:46 Slobodan
   Initial version

*****************************************************************************^*/
