/*
 * @(#)LookAndFeelPreview.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;

import javax.swing.BorderFactory;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JTree;

import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.tooltips.ToolTipManager;

/**
 * Look and feel demonstration panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class LookAndFeelPreview extends JWxGriddedPanel {

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

  JFileChooser fc;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new LookAndFeelPreview object.
   */
  public LookAndFeelPreview() {
    super();
    JWxGriddedPanel content = new JWxGriddedPanel();
    JTree jtree = new JTree();
    jtree.expandRow(1);
    JScrollPane jsp = new JScrollPane(jtree);
    jsp.setPreferredSize(new Dimension(120, 100));
    content.addComponent(jsp, 0, 0, 1, 4, GridBagConstraints.NORTHWEST,
                         GridBagConstraints.BOTH, 0, 0, 4, 4, 1.0, 1.0);
    FCButton fcb = new FCButton();
    content.addComponent(fcb, 0, 4, 1, 1, GridBagConstraints.NORTH,
                         GridBagConstraints.NONE, 0, 0, 0, 4, 1.0, 1.0);
    content.addComponent(new JTextField("Text Field"), //$NON-NLS-1$
                         1, 0, 1, 1, GridBagConstraints.NORTH,
                         GridBagConstraints.HORIZONTAL, 0, 0, 0, 4, 1.0, 1.0);
    content.addComponent(new JComboBox(new String[] {"one", "two", "three"}),
                         1, 1, 1, 1, GridBagConstraints.NORTH,
                         GridBagConstraints.HORIZONTAL, 0, 0, 0, 4, 1.0, 1.0);
    content.addComponent(new JCheckBox("JCheckBox"), 1, 2, 1, 1, //$NON-NLS-1$
                         GridBagConstraints.NORTHWEST, GridBagConstraints.NONE,
                         0, 0, 0, 4, 1.0, 1.0);
    content.addComponent(new JRadioButton("JRadioButton"), 1, 3, //$NON-NLS-1$
                         1, 1, GridBagConstraints.NORTHWEST,
                         GridBagConstraints.NONE, 0, 0, 0, 4, 1.0, 1.0);
    content.addComponent(new JToggleButton("JToggleButton"), 1, 4, //$NON-NLS-1$
                         1, 1, GridBagConstraints.NORTH,
                         GridBagConstraints.NONE, 0, 0, 0, 4, 1.0, 1.0);
    JTabbedPane tp = new JTabbedPane();
    JPanel jptab1 = new JPanel();
    jptab1.add(new JLabel("Tab: 1")); //$NON-NLS-1$
    JPanel jptab2 = new JPanel();
    jptab2.add(new JLabel("Tab: 2")); //$NON-NLS-1$
    Dimension dim = new Dimension(90, 50);
    jptab1.setPreferredSize(dim);
    jptab2.setPreferredSize(dim);
    tp.addTab("Tab1", jptab1); //$NON-NLS-1$
    tp.addTab("Tab2", jptab2); //$NON-NLS-1$
    content.addComponent(tp, 2, 0, 1, 3, GridBagConstraints.NORTH,
                         GridBagConstraints.NONE, 0, 0, 0, 0, 1.0, 1.0);
    content.addComponent(new JLabel("JLabel"), 2, 3, 1, 1, //$NON-NLS-1$
                         GridBagConstraints.NORTH, GridBagConstraints.NONE,
                         0, 0, 0, 0, 1.0, 1.0);
    JSlider s = new JSlider(JSlider.HORIZONTAL);
    s.setPreferredSize(new Dimension(tp.getPreferredSize().width,
                                     fcb.getPreferredSize().height));
    content.addComponent(s, 2, 4, 1, 1, GridBagConstraints.NORTH,
                         GridBagConstraints.NONE, 0, 0, 0, 0, 0.0, 0.0);
    content.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
    setLayout(new BorderLayout());
    add(content);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void updateUI() {
    super.updateUI();
    if (fc != null) {
      fc.updateUI();
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * FileChooser Button.
   */
  final class FCButton extends JWxButton {
    /**
     * Constructs new FCButton object.
     *
     */
    public FCButton() {
      super("JFileChooser", "Show File Chooser"); //$NON-NLS-1$ //$NON-NLS-2$
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      if (fc == null) {
        fc = new JFileChooser();
        ToolTipManager.registerContainer(fc);
      }
      fc.showDialog(LookAndFeelPreview.this, "OK"); //$NON-NLS-1$
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 22:08:30 Slobodan
   Initial version

*****************************************************************************^*/
