/*
 * @(#)UIUpdater.java  1.0.0 / 2008-05-19
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf;

import java.awt.Component;

import javax.swing.SwingUtilities;

/**
 * Component UI Updater.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class UIUpdater implements Runnable {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final Component component;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Construct new UIUpdater object.
   *
   * @param component the target component
   *
   * @throws IllegalArgumentException if the component is <tt>null</tt>
   */
  public UIUpdater(Component component) {
    if (component == null) {
      throw new IllegalArgumentException("Null component"); //$NON-NLS-1$
    }
    this.component = component;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns target component.
   *
   * @return target component
   */
  public Component getComponent() {
    return component;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void run() {
    SwingUtilities.updateComponentTreeUI(component);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-19 22:05:33 Slobodan
   Initial version

*****************************************************************************^*/
