/*
 * @(#)BasicStatusBarPanelUI.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;

import net.wrka.swing.components.plaf.StatusBarPanelUI;

/**
 * Basic StatusBarPanelUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicStatusBarPanelUI extends StatusBarPanelUI {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String PROPERTY_PREFIX;
  private static final Border DEFAULT_STATUS_PANEL_BORDER;
  private static final BasicStatusBarPanelUI STATUS_PANEL_UI;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PROPERTY_PREFIX = "StatusBarPanelUI."; //$NON-NLS-1$
    STATUS_PANEL_UI = new BasicStatusBarPanelUI();
    DEFAULT_STATUS_PANEL_BORDER = new BasicWxStatusPanelBorder();
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return STATUS_PANEL_UI;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void installDefaults(JComponent c) {
    String pp = getPropertyPrefix();
    Border statusPaneBorder = UIManager.getBorder(pp.concat(
                                                "statusBorder")); //$NON-NLS-1$
    if (statusPaneBorder == null) {
      statusPaneBorder = DEFAULT_STATUS_PANEL_BORDER;
    }
    Border border = c.getBorder();
    if ((border == null) || (border instanceof UIResource)) {
      c.setBorder(statusPaneBorder);
    }
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns property prefix.
   *
   * @return property prefix
   */
  protected String getPropertyPrefix() {
    return PROPERTY_PREFIX;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 21:20:42 Slobodan
   Initial version

*****************************************************************************^*/
