/*
 * @(#)BasicStatusSeparatorUI.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ComponentUI;

import net.wrka.swing.components.plaf.StatusBarSeparatorUI;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Basic divider for JWxFlatSplitPane.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicStatusSeparatorUI extends StatusBarSeparatorUI
                                    implements LookAndFeelChangeListener  {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final BasicStatusSeparatorUI SEPARATOR_UI;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SEPARATOR_UI = new BasicStatusSeparatorUI();
  }

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Highlight color.
   */
  protected Color highlightColor;

  /**
   * Shadow color.
   */
  protected Color shadowColor;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs type specified border.
   */
  public BasicStatusSeparatorUI() {
    super();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return SEPARATOR_UI;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    if (LookAndFeelManager.isWinXPThemed()) {
      Insets insets = c.getInsets();
      int width = c.getWidth();
      int wh2 = width / 2;
      if ((width % 2) == 0) {
        wh2 = wh2 - 1;
      }
      int wh2p1 = wh2 + 1;
      int itp1 = insets.top + 1;
      int hmmbt = c.getHeight() - insets.bottom - insets.top - 2;
      g.setColor(getShadowColor(c));
      g.drawLine(wh2, itp1, wh2, hmmbt);
      g.setColor(getHighlightColor(c));
      g.drawLine(wh2p1, itp1, wh2p1, hmmbt);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns shadow color.
   *
   * @param c the component
   *
   * @return shadow color
   */
  protected Color getShadowColor(Component c) {
    if (shadowColor == null) {
      shadowColor = GraphicUtils.darkerColor(c.getBackground(), 0.2f);
    }
    return shadowColor;
  }

  /**
   * Returns highlightColor color.
   *
   * @param c the component
   *
   * @return highlightColor color
   */
  protected Color getHighlightColor(Component c) {
    if (highlightColor == null) {
      highlightColor = GraphicUtils.brighterColor(c.getBackground(), 0.7f);
    }
    return highlightColor;
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    highlightColor = null;
    shadowColor = null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 20:37:34 Slobodan
   Initial version

*****************************************************************************^*/
