/*
 * @(#)BasicWxStatusBarUI.java  1.0.0 / 2008-05-20
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;

import net.wrka.swing.components.JWxStatusBar;
import net.wrka.swing.components.plaf.WxStatusBarUI;

/**
 * Basic WxStatusBarUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicWxStatusBarUI extends WxStatusBarUI {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String PROPERTY_PREFIX;
  private static final BasicWxStatusBarUI STATUS_BAR_UI;
  private static final Border DEFAULT_STATUS_BAR_BORDER;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PROPERTY_PREFIX = "WxStatusBarUI."; //$NON-NLS-1$
    STATUS_BAR_UI = new BasicWxStatusBarUI();
    DEFAULT_STATUS_BAR_BORDER = new BasicWxStatusBarBorder();
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return STATUS_BAR_UI;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void installDefaults(JWxStatusBar sb) {
    sb.setOpaque(true);
    sb.setBackground(UIManager.getColor("Panel.background")); //$NON-NLS-1$
    Border statusPanelBorder = UIManager.getBorder(getPropertyPrefix() +
                                                  "statusBorder"); //$NON-NLS-1$
    if (statusPanelBorder == null) {
      statusPanelBorder = DEFAULT_STATUS_BAR_BORDER;
    }
    Border border = sb.getBorder();
    if ((border == null) || (border instanceof UIResource)) {
      sb.setBorder(statusPanelBorder);
    }
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns property prefix.
   *
   * @return property prefix
   */
  protected String getPropertyPrefix() {
    return PROPERTY_PREFIX;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-20 22:03:55 Slobodan
   Initial version

*****************************************************************************^*/
