/*
 * @(#)BasicWxStatusPanelBorder.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.LookAndFeel;
import javax.swing.border.AbstractBorder;
import javax.swing.plaf.UIResource;

import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.LookAndFeelChangeListener;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Basic Status Panel Border.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicWxStatusPanelBorder extends AbstractBorder
                                      implements UIResource,
                                                 LookAndFeelChangeListener {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final Insets INSETS = new Insets(3, 3, 3, 3);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Color highlight;
  private Color shadow;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs type specified border.
   */
  public BasicWxStatusPanelBorder() {
    super();
    LookAndFeelManager.addLnFListener(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isBorderOpaque() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c) {
    return (Insets)INSETS.clone();
  }

  /**
   * {@inheritDoc}
   */
  public Insets getBorderInsets(Component c, Insets insets) {
    insets.left = INSETS.left;
    insets.top = INSETS.top;
    insets.bottom = INSETS.bottom;
    insets.right = INSETS.right;
    return insets;
  }

  /**
   * {@inheritDoc}
   */
  public void paintBorder(Component c, Graphics g, int x, int y, int w, int h) {
    if (LookAndFeelManager.isMetalLookAndFeel()) {
      paintMetalBorder(c, g, x, y, w, h);
    } else if (!LookAndFeelManager.isWinXPThemed()) {
      paintBasicBorder(c, g, x, y, w, h);
    }
  }

/*^**************************************************************************^*/
/*- LookAndFeelChangeListener Implementation.                                -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void lookAndFeelChanged(LookAndFeel oldLnF, LookAndFeel newLnF) {
    highlight = null;
    shadow = null;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns shadow color.
   *
   * @param c the component
   *
   * @return shadow color
   */
  protected Color getShadowColor(Component c) {
    if (shadow == null) {
      shadow = c.getBackground().darker();
    }
    return shadow;
  }

  /**
   * Returns highlight color.
   *
   * @param c the component
   *
   * @return highlight color
   */
  protected Color getHighlightColor(Component c) {
    if (highlight == null) {
      highlight = c.getBackground().brighter();
    }
    return highlight;
  }

  /**
   * Paints basic border.
   *
   * @param c the component
   * @param g the graphic
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintBasicBorder(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    GraphicUtils.draw2ColorsRect(g, x, y, w, h, getShadowColor(c),
                                   getHighlightColor(c));
  }

  /**
   * Paints metal border.
   *
   * @param c the component
   * @param g the graphic
   * @param x the x coordinate
   * @param y the y coordinate
   * @param w the width
   * @param h the height
   */
  protected void paintMetalBorder(Component c, Graphics g, int x, int y, int w,
                                  int h) {
    GraphicUtils.draw2ColorsRect(g, x, y, w, h, getShadowColor(c),
                                 getShadowColor(c));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 20:50:32 Slobodan
   Initial version

*****************************************************************************^*/
