/*
 * @(#)BasicWxTitlePaneUI.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.basic;

import java.awt.Color;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.BorderUIResource;
import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;

import net.wrka.swing.borders.ShadowBorder;
import net.wrka.swing.borders.TitleBorder;
import net.wrka.swing.components.JWxGradientPanel;
import net.wrka.swing.components.JWxTitlePane;
import net.wrka.swing.components.plaf.WxTitlePaneUI;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.DarkShadowLineBorderUIResource;
import net.wrka.swing.plaf.LookAndFeelManager;

/**
 * Basic UI for JWxTitlePaneUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class BasicWxTitlePaneUI extends WxTitlePaneUI {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final BasicWxTitlePaneUI SHARED_INSTANCE;

/*^**************************************************************************^*/
/*- Private Final Fields.                                                    -*/
/*^**************************************************************************^*/

  private boolean defaultsInitialized;
  private Color defaultTitleBackground;
  private Color defaultTitleDkBackground;
  private Color defaultContentBackground;
  private Color defaultContentDkBackground;
  private Border defaultShadowBorder;
  private Border defaultLineBorder;
  private Border defaultTitleBorder;
  private Border defaultRaisedTitleBorder;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SHARED_INSTANCE = new BasicWxTitlePaneUI();
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return SHARED_INSTANCE;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void installDefaults(JWxTitlePane tp) { // NOPMD
    String pp = getPropertyPrefix();
    if (!defaultsInitialized) {
      defaultTitleBackground = UIManager.getColor(
                                        pp.concat("background")); //$NON-NLS-1$
      if (defaultTitleBackground == null) {
        defaultTitleBackground = LookAndFeelManager.getControlColor();
        if (defaultTitleBackground == null) {
          defaultTitleBackground = Color.gray;
        }
      }
      defaultTitleBackground = new ColorUIResource(
          GraphicUtils.darkerColor(defaultTitleBackground, 0.1f));
      defaultTitleDkBackground = new ColorUIResource(
          GraphicUtils.darkerColor(defaultTitleBackground, 0.5f));
      defaultShadowBorder = UIManager.getBorder(
                                      pp.concat("shadowBorder")); //$NON-NLS-1$
      if (defaultShadowBorder == null) {
        defaultShadowBorder = ShadowBorder.getUIInstance();
      }
      defaultLineBorder = UIManager.getBorder(
                                        pp.concat("lineBorder")); //$NON-NLS-1$
      if (defaultLineBorder == null) {
        defaultLineBorder = DarkShadowLineBorderUIResource.SHARED_INSTANCE;
      }
      defaultTitleBorder = UIManager.getBorder(
                                       pp.concat("titleBorder")); //$NON-NLS-1$
      if (defaultTitleBorder == null) {
        defaultTitleBorder = new BorderUIResource(new TitleBorder());
      }
      defaultRaisedTitleBorder = UIManager.getBorder(
                                 pp.concat("raisedTitleBorder")); //$NON-NLS-1$
      if (defaultRaisedTitleBorder == null) {
        defaultRaisedTitleBorder = new BorderUIResource(
            new TitleBorder(TitleBorder.RAISED));
      }
      defaultContentBackground = UIManager.getColor("text"); //$NON-NLS-1$
      if (defaultContentBackground == null) {
        defaultContentBackground = new Color(250, 250, 250);
      }
      defaultContentDkBackground = new ColorUIResource(
          GraphicUtils.darkerColor(defaultContentBackground, 0.2f));
      defaultsInitialized = true;
    }
    JComponent titlePanel = tp.getTitlePanel();
    Color cb = titlePanel.getBackground();
    if ((cb == null) || (cb instanceof UIResource)) {
      titlePanel.setBackground(defaultTitleBackground);
    }
    if (titlePanel instanceof JWxTitlePane.TitlePane) {
      JWxTitlePane.TitlePane tpl = (JWxTitlePane.TitlePane)titlePanel;
      Color sb = tpl.getSecondaryBackground();
      if ((sb == null) || (sb instanceof UIResource)) {
        tpl.setSecondaryBackground(defaultTitleDkBackground);
      }
    }
    JComponent contentPanel = tp.getContentPanel();
    cb = contentPanel.getBackground();
    if ((cb == null) || (cb instanceof UIResource)) {
      contentPanel.setBackground(defaultContentDkBackground);
    }
    if (contentPanel instanceof JWxGradientPanel) {
      JWxGradientPanel gp = (JWxGradientPanel)contentPanel;
      cb = gp.getSecondaryBackground();
      if ((cb == null) || (cb instanceof UIResource)) {
        gp.setSecondaryBackground(defaultContentBackground);
      }
      gp.setGradientOrientation(JWxGradientPanel.DIAGONAL_GRADIENT);
      gp.setGradientOrg(0.5f);
      gp.setGradientIntensity(1.5f);
    }
    Border currentBorder = tp.getBorder();
    if ((currentBorder == null) || (currentBorder instanceof UIResource)) {
      int tpType = tp.getType();
      if ((tpType == JWxTitlePane.TYPE_SIMPLE) ||
          (tpType == JWxTitlePane.TYPE_RAISED)) {
        tp.setBorder(defaultLineBorder);
      } else {
        tp.setBorder(defaultShadowBorder);
      }
      if ((tpType == JWxTitlePane.TYPE_SIMPLE) ||
          (tpType == JWxTitlePane.TYPE_SIMPLE_SHADOW)) {
        titlePanel.setBorder(defaultTitleBorder);
      } else {
        titlePanel.setBorder(defaultRaisedTitleBorder);
      }
    }
    if (currentBorder instanceof TitleBorder) {
      Color bc = defaultTitleDkBackground.brighter();
      ((TitleBorder)currentBorder).setHighlightColor(bc);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallDefaults(JWxTitlePane tp) {
    defaultsInitialized = false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 20:17:16 Slobodan
   Initial version

*****************************************************************************^*/
