/*
 * @(#)KhakiMetalTheme.java  1.8 03/01/23
 *
 * Copyright (c) 2003 Sun Microsystems, Inc. All  Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * -Redistributions of source code must retain the above copyright
 *  notice, this list of conditions and the following disclaimer.
 *
 * -Redistribution in binary form must reproduct the above copyright
 *  notice, this list of conditions and the following disclaimer in
 *  the documentation and/or other materials provided with the distribution.
 *
 * Neither the name of Sun Microsystems, Inc. or the names of contributors
 * may be used to endorse or promote products derived from this software
 * without specific prior written permission.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING
 * ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
 * OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT
 * BE LIABLE FOR ANY DAMAGES OR LIABILITIES SUFFERED BY LICENSEE AS A RESULT
 * OF OR RELATING TO USE, MODIFICATION OR DISTRIBUTION OF THE SOFTWARE OR ITS
 * DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS BE LIABLE FOR ANY LOST
 * REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
 * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN
 * IF SUN HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * You acknowledge that Software is not designed, licensed or intended for
 * use in the design, construction, operation or maintenance of any nuclear
 * facility.
 */
package net.wrka.swing.plaf.metal.themes;

import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;

/**
 * This class describes a theme using "khaki" colors.
 *
 * @author Steve Wilson
 * @version 1.8 01/23/03
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class KhakiMetalTheme extends DefaultMetalTheme {

/*^**************************************************************************^*/
/*- Protected Static Fields.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Primary 1 color.
   */
  protected static final ColorUIResource PRIMARY_1;

  /**
   * Primary 2 color.
   */
  protected static final ColorUIResource PRIMARY_2;

  /**
   * Primary 3 color.
   */
  protected static final ColorUIResource PRIMARY_3;

  /**
   * Secondary 1 color.
   */
  protected static final ColorUIResource SECONDARY_1;

  /**
   * Secondary 2 color.
   */
  protected static final ColorUIResource SECONDARY_2;

  /**
   * Secondary 3 color.
   */
  protected static final ColorUIResource SECONDARY_3;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PRIMARY_1 = new ColorUIResource(87,  87,  47);
    PRIMARY_2 = new ColorUIResource(159, 151, 111);
    PRIMARY_3 = new ColorUIResource(199, 183, 143);
    SECONDARY_1 = new ColorUIResource(111, 111, 111);
    SECONDARY_2 = new ColorUIResource(159, 159, 159);
    SECONDARY_3 = new ColorUIResource(231, 215, 183);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Sandstone"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary1() {
    return PRIMARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary2() {
    return PRIMARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary3() {
    return PRIMARY_3;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary1() {
    return SECONDARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary2() {
    return SECONDARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary3() {
    return SECONDARY_3;
  }

}

/*^*****************************************************************************
  File History:

 - Initial version Steve Wilson

 - 2008-05-19 23:55:07 Slobodan
   Adopted version

*****************************************************************************^*/
