/*
 * @(#)DefaultSmoothTheme.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Font;

import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;

import net.wrka.swing.graphics.FontUtils;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.AlphaColorUIResource;

/**
 * Default Smooth Theme.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class DefaultSmoothTheme extends SmoothTheme {

/*^**************************************************************************^*/
/*- Protected Static Fields.                                                 -*/
/*^**************************************************************************^*/

  /**
   * White color.
   */
  protected static final ColorUIResource WHITE;

  /**
   * Black color.
   */
  protected static final ColorUIResource BLACK;

  // Primary Colors. -----------------------------------------------------------

  /**
   * Primary color 1.
   */
  protected static final ColorUIResource PRIMARY_1;

  /**
   * Primary color 2.
   */
  protected static final ColorUIResource PRIMARY_2;

  /**
   * Primary color 3.
   */
  protected static final ColorUIResource PRIMARY_3;

  // Secondary Colors. ---------------------------------------------------------

  /**
   * Secondary color 1.
   */
  protected static final ColorUIResource SECONDARY_1;

  /**
   * Secondary color 2.
   */
  protected static final ColorUIResource SECONDARY_2;

  /**
   * Secondary color 3.
   */
  protected static final ColorUIResource SECONDARY_3;

  // Gradient Colors. ----------------------------------------------------------

  /**
   * Component gradient color.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_COLOR;

  /**
   * Component gradient color reflection.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_REFLECTION;

  /**
   * Component gradient color shadow.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_COLOR_SHADOW;

  /**
   * Component gradient color light.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_COLOR_LIGHT;

  /**
   * Component gradient color reflection light.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_REF_LIGHT;

  /**
   * Component gradient color shadow light.
   */
  protected static final AlphaColorUIResource COMPONENT_GRADIENT_SHADOW_LIGHT;

  // Rollover Colors. ----------------------------------------------------------

  /**
   * Rollover backgourd factor.
   */
  protected static final float ROLLOVER_BACKGROUND_FACTOR;

  /**
   * Rollover backgournd.
   */
  protected static final AlphaColorUIResource ROLLOVER_BACKGROUND;

  /**
   * Rollover foregournd factor.
   */
  protected static final float ROLLOVER_FOREGROUND_FACTOR;

  /**
   * Rollover foregournd.
   */
  protected static final AlphaColorUIResource ROLLOVER_FOREGROUND;

  /**
   * Rollover mask color.
   */
  protected static final AlphaColorUIResource ROLLOVER_MASK_COLOR;

  // Menu Colors. --------------------------------------------------------------

  /**
   * Menu item left backgournd.
   */
  protected static final AlphaColorUIResource MENU_ITEM_LEFT_BACKGROUND;

  /**
   * Menu item right backgournd.
   */
  protected static final AlphaColorUIResource MENU_ITEM_RIGHT_BACKGROUND;

  /**
   * Menu item left background width.
   */
  protected static final int MENU_ITEM_LEFT_BACKGROUND_WIDTH;

  // Internal Frame Colors. ----------------------------------------------------

  /**
   * Internal frame titile pane active background 1.
   */
  protected static final AlphaColorUIResource INTERNAL_FRAME_TITLE_PANE_ABG1;

  /**
   * Internal frame titile pane active background 2.
   */
  protected static final AlphaColorUIResource INTERNAL_FRAME_TITLE_PANE_ABG2;

  /**
   * Active title foreground.
   */
  protected static final AlphaColorUIResource ACTIVE_TITLE_FOREGROUND;

  /**
   * Inactive title color.
   */
  protected static final AlphaColorUIResource INACTIVE_TITLE_FOREGROUND;

  /**
   * Desktop color.
   */
  protected static final ColorUIResource DESKTOP_COLOR;

  // Fonts. --------------------------------------------------------------------

  /**
   * Menu font.
   */
  protected static final FontUIResource MENU_FONT;

  /**
   * Control font.
   */
  protected static final FontUIResource CONTROL_FONT;

  /**
   * Window title font.
   */
  protected static final FontUIResource WINDOW_TITLE_FONT;

  /**
   * Monospaced font.
   */
  protected static final FontUIResource MONOSPACED_FONT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    WHITE = new ColorUIResource(250, 250, 250);
    BLACK = new ColorUIResource(0, 0, 0);
    PRIMARY_1 = new ColorUIResource(22, 22, 54);
    PRIMARY_2 = new ColorUIResource(110, 110, 130);
    PRIMARY_3 = new ColorUIResource(210, 210, 230);
    SECONDARY_1 = new ColorUIResource(100, 100, 100);
    SECONDARY_2 = new ColorUIResource(130, 130, 130);
    SECONDARY_3 = new ColorUIResource(180, 180, 180);
    COMPONENT_GRADIENT_COLOR = new AlphaColorUIResource(250, 250, 250, 40);
    COMPONENT_GRADIENT_REFLECTION = new AlphaColorUIResource(255, 255, 255,
                                                             166);
    COMPONENT_GRADIENT_COLOR_SHADOW = new AlphaColorUIResource(0, 0, 0, 88);
    COMPONENT_GRADIENT_COLOR_LIGHT =
        new AlphaColorUIResource(250, 250, 250, 20);
    COMPONENT_GRADIENT_REF_LIGHT =
        new AlphaColorUIResource(255, 255, 255, 83);
    COMPONENT_GRADIENT_SHADOW_LIGHT = new AlphaColorUIResource(0, 0, 0, 38);
    ROLLOVER_BACKGROUND_FACTOR = 0.15f;
    ROLLOVER_BACKGROUND =
              GraphicUtils.brighterColorUIResource(SECONDARY_3,
                                                   ROLLOVER_BACKGROUND_FACTOR);
    ROLLOVER_FOREGROUND = new AlphaColorUIResource(75, 14, 199);
    ROLLOVER_FOREGROUND_FACTOR = 1f;
    ROLLOVER_MASK_COLOR = new AlphaColorUIResource(20, 40, 250, 33);
    MENU_ITEM_LEFT_BACKGROUND =
                      GraphicUtils.darkerColorUIResource(SECONDARY_3, 0.17f);
    MENU_ITEM_RIGHT_BACKGROUND =
                      GraphicUtils.brighterColorUIResource(SECONDARY_3, 0.3f);
    MENU_ITEM_LEFT_BACKGROUND_WIDTH = 24;
    INTERNAL_FRAME_TITLE_PANE_ABG1 = new AlphaColorUIResource(56, 84, 148);
    INTERNAL_FRAME_TITLE_PANE_ABG2 = new AlphaColorUIResource(105, 135, 206);
    ACTIVE_TITLE_FOREGROUND = new AlphaColorUIResource(255, 255, 255);
    INACTIVE_TITLE_FOREGROUND = new AlphaColorUIResource(170, 170, 170);
    DESKTOP_COLOR = new ColorUIResource(120, 120, 120);
    MENU_FONT = new FontUIResource(FontUtils.DEFAULT_FONT);
    CONTROL_FONT = new FontUIResource(FontUtils.DEFAULT_FONT.
                                                        deriveFont(Font.BOLD));
    WINDOW_TITLE_FONT = CONTROL_FONT;
    MONOSPACED_FONT = new FontUIResource(FontUtils.DEFAULT_FIXED_FONT);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Dark gray"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getWhite() {
    return WHITE;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getBlack() {
    return BLACK;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary1() {
    return PRIMARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary2() {
    return PRIMARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary3() {
    return PRIMARY_3;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary1() {
    return SECONDARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary2() {
    return SECONDARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary3() {
    return SECONDARY_3;
  }

  /**
   * {@inheritDoc}
   */
  public float getRollOverBackgroundFactor() {
    return ROLLOVER_BACKGROUND_FACTOR;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isSupported() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverBackground() {
    return ROLLOVER_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public float getRollOverForegroundFactor() {
    return ROLLOVER_FOREGROUND_FACTOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverForeground() {
    return ROLLOVER_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverMaskColor() {
    return ROLLOVER_MASK_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientReflection() {
    return COMPONENT_GRADIENT_REFLECTION;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientShadow() {
    return COMPONENT_GRADIENT_COLOR_SHADOW;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientColor() {
    return COMPONENT_GRADIENT_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientReflectionLight() {
    return COMPONENT_GRADIENT_REF_LIGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientShadowLight() {
    return COMPONENT_GRADIENT_SHADOW_LIGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientLight() {
    return COMPONENT_GRADIENT_COLOR_LIGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemLeftBackground() {
    return MENU_ITEM_LEFT_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemRightBackground() {
    return MENU_ITEM_RIGHT_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public int getMenuItemLeftBackgroundWidth() {
    return MENU_ITEM_LEFT_BACKGROUND_WIDTH;
  }

  /**
   * {@inheritDoc}
   */
  public ColorUIResource getDesktopColor() {
    return DESKTOP_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInternalFrameTitlePaneABG1() {
    return INTERNAL_FRAME_TITLE_PANE_ABG1;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInternalFrameTitlePaneABG2() {
    return INTERNAL_FRAME_TITLE_PANE_ABG2;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getActiveTitleForeground() {
    return ACTIVE_TITLE_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInactiveTitleForeground() {
    return INACTIVE_TITLE_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getFixedFont() {
    return MONOSPACED_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getControlTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getMenuTextFont() {
    return MENU_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getSystemTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getUserTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getWindowTitleFont() {
    return WINDOW_TITLE_FONT;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 21:24:09 Slobodan
   Initial version

*****************************************************************************^*/
