/*
 * @(#)SmoothComboBoxEditor.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics;
import java.awt.Insets;

import javax.swing.JComboBox;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.AbstractBorder;
import javax.swing.border.Border;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicComboBoxEditor;

import net.wrka.util.CommonConstants;

/**
 * ComboBox Editor.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothComboBoxEditor extends BasicComboBoxEditor
                                  implements UIResource {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothComboBoxEditor object.
   *
   * @param comboBox the combo box.
   */
  public SmoothComboBoxEditor(JComboBox comboBox) {
    super();
    editor = new JTextField(CommonConstants.STRING_EMPTY,
                            UIManager.getInt(
                                      "ComboBox.editorColumns")); //$NON-NLS-1$
    editor.setBorder(getComboBoxEditorBorder(comboBox));
    editor.setEnabled(comboBox.isEnabled());
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns combo box editor border.
   *
   * @param comboBox the combo box
   *
   * @return combo box editor border
   */
  protected Border getComboBoxEditorBorder(JComboBox comboBox) {
    return new ComboBoxEditorBorder(comboBox);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * ComboBoxEditorBorder Class.
   */
  protected static class ComboBoxEditorBorder extends AbstractBorder {

    // Private Static Final Field. ---------------------------------------------

    private static final Insets INSETS = new Insets(2, 4, 2, 1);

    // Private Final Field. ----------------------------------------------------

    private final JComboBox comboBox;

    // Public Constructor. -----------------------------------------------------

    /**
     * Constructs new ComboBoxEditorBorder object.
     *
     * @param comboBox the combo box
     */
    public ComboBoxEditorBorder(JComboBox comboBox) {
      super();
      this.comboBox = comboBox;
    }

    // Overridden Method. ------------------------------------------------------

    /**
     * {@inheritDoc}
     */
    public void paintBorder(Component c, Graphics g,  int x, int y, int w,
                            int h) {
      int wm2 = w - 2;
      int hm1 = h - 1;
      if (comboBox == null) {
        g.setColor(SmoothLookAndFeel.getControl());
      } else {
        Container cp = comboBox.getParent();
        if (cp == null) {
          g.setColor(comboBox.getBackground());
        } else {
          g.setColor(cp.getBackground());
        }
      }
      g.fillRect(0, 0, 1, 1);
      g.fillRect(0, hm1, 1, 1);
      if (c.isEnabled()) {
        g.setColor(SmoothLookAndFeel.getControlDarkShadow());
      } else {
        g.setColor(SmoothLookAndFeel.getControlShadow());
      }
      g.fillRect(2, 0, wm2, 1);
      g.fillRect(0, 2, 1, h - 4);
      g.fillRect(2, hm1, wm2, 1);
      g.setColor(SmoothLookAndFeel.getControlShadow());
      g.fillRect(0, 1, 2, 1);
      g.fillRect(1, 0, 1, 1);
      g.fillRect(0, h - 2, 2, 1);
      g.fillRect(1, hm1, 1, 1);
    }

    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c) {
      return (Insets)INSETS.clone();
    }

    /**
     * {@inheritDoc}
     */
    public Insets getBorderInsets(Component c, Insets insets) {
      insets.top = INSETS.top;
      insets.right = INSETS.right;
      insets.bottom = INSETS.bottom;
      insets.left = INSETS.left;
      return insets;
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 10:02:46 Slobodan
   Initial version

*****************************************************************************^*/
