/*
 * @(#)SmoothComboBoxUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ComboBoxEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.ListCellRenderer;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicComboPopup;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.plaf.metal.MetalComboBoxUI;

/**
 * ComboBoxUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothComboBoxUI extends MetalComboBoxUI {

/*^**************************************************************************^*/
/*- Private Staic Field.                                                     -*/
/*^**************************************************************************^*/

  private static final Insets NULL_INSETS = new Insets(0, 0, 0, 0);

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  int editableButtonWidth;
  ListCellRenderer renderer;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothComboBoxUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize(JComponent c) {
    if (!isMinimumSizeDirty) {
      return (Dimension)cachedMinimumSize.clone();
    }
    Dimension size = null;
    if (!comboBox.isEditable() && (arrowButton != null) &&
        (arrowButton instanceof SmoothComboBoxButton)) {
      SmoothComboBoxButton button = (SmoothComboBoxButton)arrowButton;
      Insets buttonInsets = button.getInsets();
      Insets rendererInsets = getRendererInsets();
      Insets insets = comboBox.getInsets();
      size = getDisplaySize();
      size.width += insets.left + insets.right;
      size.width += buttonInsets.left + buttonInsets.right;
      size.width += rendererInsets.left + rendererInsets.right;
      size.width += button.getComboIcon().getIconWidth() + 6;
      size.height += insets.top + insets.bottom;
      size.height += rendererInsets.top + rendererInsets.bottom;
      if (size.height < 21) {
        size.height = 21;
      }
    } else if (comboBox.isEditable() && (arrowButton != null) &&
               (editor != null)) {
      size = getDisplaySize();
      Insets insets = comboBox.getInsets();
      Insets editorInsets = getEditorInsets();
      int buttonWidth = editableButtonWidth;
      size.width += insets.left + insets.right;
      size.width += editorInsets.left + editorInsets.right;
      size.width += buttonWidth;
      size.height += insets.top + insets.bottom;
    } else {
      size = super.getMinimumSize(c);
    }
    cachedMinimumSize.setSize(size.width, size.height);
    isMinimumSizeDirty = false;
    return (Dimension)cachedMinimumSize.clone();
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    editableButtonWidth = UIManager.getInt("ScrollBar.width") - 1; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected LayoutManager createLayoutManager() {
    return new SmoothComboBoxLayoutManager();
  }

  /**
   * {@inheritDoc}
   */
  protected ComboPopup createPopup() {
    return new SmoothComboPopup(comboBox);
  }

  /**
   * {@inheritDoc}
   */
  public PropertyChangeListener createPropertyChangeListener() {
    return new SmoothPropertyChangeHandler();
  }

  /**
   * {@inheritDoc}
   */
  protected JButton createArrowButton() {
    return new SmoothComboBoxButton(comboBox,
                                    SmoothIconFactory.getComboBoxButtonIcon(),
                                    comboBox.isEditable(), currentValuePane,
                                    listBox);
  }

  /**
   * {@inheritDoc}
   */
  protected ComboBoxEditor createEditor() {
    return new SmoothComboBoxEditor(comboBox);
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns editor insets.
   *
   * @return editor insets
   */
  protected Insets getEditorInsets() {
    if (editor instanceof JComponent) {
      return ((JComponent)editor).getInsets();
    }
    return (Insets)NULL_INSETS.clone();
  }

  /**
   * Returns renderer insets.
   *
   * @return renderer insets
   */
  protected Insets getRendererInsets() {
    if (renderer instanceof JComponent) {
      return ((JComponent)renderer).getInsets();
    }
    return (Insets)NULL_INSETS.clone();
  }

  /**
   * Returns combo box.
   *
   * @return combo box
   */
  protected JComboBox getComboBox() { // optimization
    return comboBox;
  }

  /**
   * Returns arrow button.
   *
   * @return arrow button
   */
  protected JButton getArrowButton() { // optimization
    return arrowButton;
  }

  /**
   * Returns editor.
   *
   * @return editor
   */
  protected Component getEditor() { // optimization
    return editor;
  }

  /**
   * Returns combobox insets.
   *
   * @return combobox insets
   */
  protected final Insets getComboBoxInsets() { // optimization
    return comboBox.getInsets();
  }

  /**
   * Returns rectangle for current value.
   *
   * @return rectangle for current value
   */
  protected final Rectangle getRectangleForCurrentValue() { // optimization
    return rectangleForCurrentValue();
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * ComboBox LayoutManager.
   */
  protected class SmoothComboBoxLayoutManager extends
                                                  MetalComboBoxLayoutManager {
    /**
     * {@inheritDoc}
     */
    public void layoutContainer(Container parent) {
      JComboBox cb = (JComboBox)parent;
      if (!cb.isEditable()) {
        super.layoutContainer(parent);
        return;
      }
      int width = cb.getWidth();
      int height = cb.getHeight();
      Insets insets = getComboBoxInsets();
      int buttonWidth = editableButtonWidth;
      int buttonHeight = height - (insets.top + insets.bottom);
      final JButton arrowButton = getArrowButton();
      if (arrowButton != null) {
        if (cb.getComponentOrientation().isLeftToRight()) {
          arrowButton.setBounds(width - (insets.right + buttonWidth),
                                insets.top, buttonWidth, buttonHeight);
        } else {
          arrowButton.setBounds(insets.left, insets.top, buttonWidth,
                                buttonHeight);
        }
      }
      Component editor = getEditor();
      if (editor != null) {
        editor.setBounds(getRectangleForCurrentValue());
      }
    }
  }

  /**
   * PropertyChange Handler.
   */
  protected class SmoothPropertyChangeHandler extends PropertyChangeHandler {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      super.propertyChange(e);
      String propertyName = e.getPropertyName();
      final JComboBox comboBox = getComboBox();
      final JButton arrowButton = getArrowButton();
      if ("editable".equals(propertyName)) { //$NON-NLS-1$
        SmoothComboBoxButton button = (SmoothComboBoxButton)arrowButton;
        button.setIconOnly(comboBox.isEditable());
        button.setRequestFocusEnabled(!comboBox.isEditable() &&
                                      comboBox.isEnabled());
        comboBox.repaint();
      } else if ("enabled".equals(propertyName)) { //$NON-NLS-1$
        if (arrowButton instanceof SmoothComboBoxButton)  {
          arrowButton.setRequestFocusEnabled(!comboBox.isEditable() &&
                                             comboBox.isEnabled());
          comboBox.repaint();
        }
        ComboBoxEditor editor = comboBox.getEditor();
        if (editor != null) {
          editor.getEditorComponent().setEnabled(comboBox.isEnabled());
          comboBox.repaint();
        }
      } else if ("renderer".equals(propertyName)) { //$NON-NLS-1$
        renderer = comboBox.getRenderer();
        if ((renderer != null) && (renderer instanceof JComponent)) {
          ((JComponent)renderer).setOpaque(false);
        }
      }
    }
  }

  /**
   * ComboBox Popup.
   */
  protected static class SmoothComboPopup extends BasicComboPopup {
    /**
     * Constructs new SmoothComboPopup object.
     *
     * @param combo the combo box
     */
    protected SmoothComboPopup(JComboBox combo) {
      super(combo);
    }
    /**
     * {@inheritDoc}
     */
    protected void configureList() {
      super.configureList();
      list.setForeground(SmoothLookAndFeel.getMenuForeground());
      list.setBackground(SmoothLookAndFeel.getMenuItemRightBackground());
    }
    /**
     * {@inheritDoc}
     */
    protected void configurePopup() {
      super.configurePopup();
      setBorder(SmoothBorders.getPopupMenuBorder());
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 18:48:20 Slobodan
   Initial version

*****************************************************************************^*/
