/*
 * @(#)SmoothDesktopIconUI.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicDesktopIconUI;

/**
 * DesktopIconUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothDesktopIconUI extends BasicDesktopIconUI {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int width;
  private JComponent iconPane;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothDesktopIconUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize(JComponent c) {
    return getMinimumSize(c);
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize(JComponent c) {
    Dimension dim = new Dimension(iconPane.getMinimumSize());
    Border border = frame.getBorder();
    if (border != null) {
      Insets insets = border.getBorderInsets(frame);
      dim.height += insets.bottom + insets.top;
    }
    dim.width = width;
    return dim;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize(JComponent c) {
    return getMinimumSize(c);
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    width = UIManager.getInt("DesktopIcon.width"); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected void installComponents() {
    iconPane = new SmoothInternalFrameTitlePane(frame);
    desktopIcon.setLayout(new BorderLayout());
    desktopIcon.add(iconPane, BorderLayout.CENTER);
    desktopIcon.setBorder(SmoothBorders.getInternalFrameBorder());
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallComponents() {
    desktopIcon.remove(iconPane);
    desktopIcon.setLayout(null);
    iconPane = null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 10:10:34 Slobodan
   Initial version

*****************************************************************************^*/
