/*
 * @(#)SmoothInternalFrameUI.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;

import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicInternalFrameUI;

/**
 * InternalFrameUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothInternalFrameUI extends BasicInternalFrameUI {

/*^**************************************************************************^*/
/*- Pubilc Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothInternalFrameUI object.
   *
   * @param internalFrame the internal frame
   */
  public SmoothInternalFrameUI(JInternalFrame internalFrame) {
    super(internalFrame);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothInternalFrameUI((JInternalFrame)c);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected JComponent createNorthPane(JInternalFrame w) {
    return new SmoothInternalFrameTitlePane(w);
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    JComponent contentPane = (JComponent)frame.getContentPane();
    if (contentPane != null) {
      Color bg = contentPane.getBackground();
      if ((bg == null) || (bg instanceof UIResource)) {
        contentPane.setBackground(SmoothLookAndFeel.getControl());
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 10:13:45 Slobodan
   Initial version

*****************************************************************************^*/
