/*
 * @(#)SmoothMenuUI.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.ActionMap;
import javax.swing.ButtonModel;
import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;
import javax.swing.MenuElement;
import javax.swing.MenuSelectionManager;
import javax.swing.SwingUtilities;
import javax.swing.Timer;
import javax.swing.UIManager;
import javax.swing.event.MenuDragMouseEvent;
import javax.swing.event.MenuDragMouseListener;
import javax.swing.event.MenuEvent;
import javax.swing.event.MenuKeyEvent;
import javax.swing.event.MenuKeyListener;
import javax.swing.event.MenuListener;
import javax.swing.event.MouseInputListener;
import javax.swing.plaf.ComponentUI;

/**
 * MenuUI.
 * <p>
 *  <tt>The code is based on Sun's javax.swing.plaf.basic.BasicMenuUI</tt>
 * </p>
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothMenuUI extends SmoothMenuItemUI {

/*^**************************************************************************^*/
/*- NOTE: The code is not cleaned and optimized.                             -*/
/*^**************************************************************************^*/

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static MenuListener sharedMenuListener;

/*^**************************************************************************^*/
/*- Packge-Private Fields.                                                   -*/
/*^**************************************************************************^*/

  MenuListener menuListener;
  JMenu menu;
  int lastMnemonic;
  int oldDelay;
  boolean oldOpaqueSet;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothMenuUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected String getPropertyPrefix() {
    return "Menu"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    super.installDefaults();
    menu = (JMenu)menuItem;
    if (menu.isOpaque()) {
      if (INSTALL_PROPERTY_METHOD == null) {
        oldOpaqueSet = true;
        menu.setOpaque(false);
      } else {
        installProperty(menuItem, "opaque", Boolean.FALSE); //$NON-NLS-1$
      }
    }
    oldDelay = menu.getDelay();
    menu.setDelay(200);
  }

  /**
   * {@inheritDoc}
   */
  protected void installListeners() {
    super.installListeners();
    if (propertyChangeListener == null) {
      propertyChangeListener = createPropertyChangeListener(menuItem);
    }
    if (propertyChangeListener != null) {
      menuItem.addPropertyChangeListener(propertyChangeListener);
    }
    if (menuListener == null) {
      menuListener = createMenuListener(menuItem);
    }
    if (menuListener != null) {
      menu.addMenuListener(menuListener);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installKeyboardActions() {
    super.installKeyboardActions();
    updateMnemonicBinding();
  }

  /**
   * {@inheritDoc}
   */
  protected ActionMap getActionMap() {
    return createActionMap();
  }

  /**
   * {@inheritDoc}
   */
  protected ActionMap createActionMap() {
    ActionMap am = super.createActionMap();
    if (am != null) {
      am.put("selectMenu", new PostAction(menu, true)); //$NON-NLS-1$
    }
    return am;
  }

  /**
   * {@inheritDoc}
   */
  protected MouseInputListener createMouseInputListener(JComponent c) {
    return new MouseInputHandler();
  }

  /**
   * {@inheritDoc}
   */
  protected void paintBackground(Graphics g, JMenuItem menuItem) {
    ButtonModel model = menuItem.getModel();
    Color oldColor = g.getColor();
    int menuWidth = menuItem.getWidth();
    int menuHeight = menuItem.getHeight();
    if (menu.isTopLevelMenu()) {
      if (model.isArmed() || model.isSelected()) {
        g.setColor(menuItem.getBackground());
        g.fillRect(2, 2, menuWidth - 4, menuHeight - 4);
        g.setColor(oldColor);
      }
    } else {
      g.setColor(SmoothLookAndFeel.getMenuItemRightBackground());
      g.fillRect(menuItemLeftBgdWidth, 0, menuWidth - menuItemLeftBgdWidth,
                 menuHeight);
      g.setColor(SmoothLookAndFeel.getMenuItemLeftBackground());
      g.fillRect(0, 0, menuItemLeftBgdWidth, menuHeight);
      g.setColor(oldColor);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected PropertyChangeListener createPropertyChangeListener(JComponent c) {
    return new PropertyChangeHandler();
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallDefaults() {
    menu.setArmed(false);
    menu.setSelected(false);
    menu.resetKeyboardActions();
    super.uninstallDefaults();
    menu.setDelay(oldDelay);
    if (oldOpaqueSet) {
      menu.setOpaque(true);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallListeners() {
    super.uninstallListeners();
    if (propertyChangeListener != null) {
      menuItem.removePropertyChangeListener(propertyChangeListener);
    }
    if (menuListener != null) {
      menu.removeMenuListener(menuListener);
    }
    propertyChangeListener = null;
    menuListener = null;
  }

  /**
   * {@inheritDoc}
   */
  protected MenuDragMouseListener createMenuDragMouseListener(JComponent c) {
    return new MenuDragMouseHandler();
  }

  /**
   * {@inheritDoc}
   */
  protected MenuKeyListener createMenuKeyListener(JComponent c) {
    return new MenuKeyHandler();
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize(JComponent c) {
    if (menu.isTopLevelMenu()) {
      Dimension d = c.getPreferredSize();
      return new Dimension(d.width, Short.MAX_VALUE);
    }
    return null;
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Updates mnemonic binding.
   */
  protected void updateMnemonicBinding() {
    int mnemonic = menuItem.getModel().getMnemonic();
    int[] shortcutKeys = (int[])UIManager.get(
                                            "Menu.shortcutKeys"); //$NON-NLS-1$
    if ((mnemonic == lastMnemonic) || (shortcutKeys == null)) {
      return;
    }
    if ((lastMnemonic != 0) && (windowInputMap != null)) {
      for (int i = 0; i < shortcutKeys.length; i++) {
        windowInputMap.remove(KeyStroke.getKeyStroke(lastMnemonic,
                                                     shortcutKeys[i], false));
      }
    }
    if (mnemonic != 0) {
      if (windowInputMap == null) {
        windowInputMap = createInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
        SwingUtilities.replaceUIInputMap(menuItem,
                                         JComponent.WHEN_IN_FOCUSED_WINDOW,
                                         windowInputMap);
      }
      for (int i = 0; i < shortcutKeys.length; i++) {
        windowInputMap.put(KeyStroke.getKeyStroke(mnemonic, shortcutKeys[i],
                                                  false),
                                                  "selectMenu"); //$NON-NLS-1$
      }
    }
    lastMnemonic = mnemonic;
  }

  /**
   * Setup post timer.
   *
   * @param menu the menu
   */
  protected void setupPostTimer(JMenu menu) {
    Timer timer = new Timer(menu.getDelay(), new PostAction(menu, false));
    timer.setRepeats(false);
    timer.start();
  }

  /**
   * Creates menu listener.
   *
   * @param c the component
   *
   * @return menu listener
   */
  protected MenuListener createMenuListener(JComponent c) {
    if (sharedMenuListener == null) {
      sharedMenuListener = new MenuHandler();
    }
    return sharedMenuListener;
  }

  /**
   * Append path.
   *
   * @param path the path
   * @param elem the elem
   */
  protected static void appendPath(MenuElement[] path, MenuElement elem) {
    MenuElement[] newPath = new MenuElement[path.length + 1];
    System.arraycopy(path, 0, newPath, 0, path.length);
    newPath[path.length] = elem;
    MenuSelectionManager.defaultManager().setSelectedPath(newPath);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * PropertyChange Handler.
   */
  class PropertyChangeHandler implements PropertyChangeListener {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      String prop = e.getPropertyName();
      if (AbstractButton.MNEMONIC_CHANGED_PROPERTY.equals(prop)) {
        updateMnemonicBinding();
      }
    }
  }

  /**
   * MouseInput Handler.
   */
  protected class MouseInputHandler implements MouseInputListener {
    /**
     * {@inheritDoc}
     */
    public void mouseClicked(MouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void mousePressed(MouseEvent e) {
      if (menu.isEnabled()) {
        MenuSelectionManager manager = MenuSelectionManager.defaultManager();
        if (menu.isTopLevelMenu()) {
          if (menu.isSelected()) {
            manager.clearSelectedPath();
          } else {
            Container cnt = menu.getParent();
            if (cnt instanceof JMenuBar) {
              MenuElement[] me = new MenuElement[2];
              me[0] = (MenuElement)cnt;
              me[1] = menu;
              manager.setSelectedPath(me);
            }
          }
        }
        MenuElement[] selectedPath = manager.getSelectedPath();
        if ((selectedPath.length > 0) &&
            (selectedPath[selectedPath.length - 1] != menu.getPopupMenu())) {
          if (menu.isTopLevelMenu() || (menu.getDelay() == 0)) {
            appendPath(selectedPath, menu.getPopupMenu());
          } else {
            setupPostTimer(menu);
          }
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseReleased(MouseEvent e) {
      if (menu.isEnabled()) {
        MenuSelectionManager manager = MenuSelectionManager.defaultManager();
        manager.processMouseEvent(e);
        if (!e.isConsumed()) {
          manager.clearSelectedPath();
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseEntered(MouseEvent e) {
      if (menu.isEnabled()) {
        MenuSelectionManager manager = MenuSelectionManager.defaultManager();
        MenuElement[] selectedPath = manager.getSelectedPath();
        if (menu.isTopLevelMenu()) {
          if ((selectedPath.length > 0) &&
              (selectedPath[0] == menu.getParent())) {
            MenuElement[] newPath = new MenuElement[3];
            newPath[0] = (MenuElement)menu.getParent();
            newPath[1] = menu;
            newPath[2] = menu.getPopupMenu();
            manager.setSelectedPath(newPath);
          }
        } else {
          if (!((selectedPath.length > 0) &&
                (selectedPath[selectedPath.length - 1] ==
                   menu.getPopupMenu()))) {
            if (menu.getDelay() == 0) {
              appendPath(getPath(), menu.getPopupMenu());
            } else {
              manager.setSelectedPath(getPath());
              setupPostTimer(menu);
            }
          }
        }
        menu.getModel().setRollover(true);
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseExited(MouseEvent e) {
      menu.getModel().setRollover(false);
    }
    /**
     * {@inheritDoc}
     */
    public void mouseDragged(MouseEvent e) {
      if (menu.isEnabled()) {
        MenuSelectionManager.defaultManager().processMouseEvent(e);
      }
    }
    /**
     * {@inheritDoc}
     */
    public void mouseMoved(MouseEvent e) {
      // empty method body
    }
  }

  /**
   * MenuDragMouse Handler.
   */
  protected class MenuDragMouseHandler implements MenuDragMouseListener {
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseEntered(MenuDragMouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseDragged(MenuDragMouseEvent e) {
      if (menuItem.isEnabled()) {
        MenuSelectionManager manager = e.getMenuSelectionManager();
        MenuElement[] path = e.getPath();
        Point p = e.getPoint();
        if ((p.x >= 0) && (p.x < menuItem.getWidth()) && (p.y >= 0) &&
            (p.y < menuItem.getHeight())) {
          MenuElement[] selectedPath = manager.getSelectedPath();
          if (!((selectedPath.length > 0) &&
                (selectedPath[selectedPath.length - 1] ==
                   menu.getPopupMenu()))) {
            if (menu.isTopLevelMenu() || (menu.getDelay() == 0) ||
                (e.getID() == MouseEvent.MOUSE_DRAGGED)) {
              appendPath(path, menu.getPopupMenu());
            } else {
              manager.setSelectedPath(path);
              setupPostTimer(menu);
            }
          }
        } else if (e.getID() == MouseEvent.MOUSE_RELEASED) {
          Component comp = manager.componentForPoint(e.getComponent(),
                                                     e.getPoint());
          if (comp == null) {
            manager.clearSelectedPath();
          }
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseExited(MenuDragMouseEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuDragMouseReleased(MenuDragMouseEvent e) {
      // empty method body
    }
  }

  /**
   * MenuKey Handler.
   */
  protected class MenuKeyHandler implements MenuKeyListener {
    int[] indexes;
    char lastMnemonic;
    int lastIndex;
    int matches;
    /**
     * {@inheritDoc}
     */
    public void menuKeyTyped(MenuKeyEvent e) {
      int key = menuItem.getMnemonic();
      if (key != 0) {
        MenuElement[] path = e.getPath();
        if (Character.toLowerCase((char)key) ==
            Character.toLowerCase(e.getKeyChar())) {
          JPopupMenu popupMenu = menu.getPopupMenu();
          MenuElement[] sub = popupMenu.getSubElements();
          if (sub.length > 0) {
            MenuSelectionManager manager = e.getMenuSelectionManager();
            MenuElement[] newPath = new MenuElement[path.length + 2];
            System.arraycopy(path, 0, newPath, 0, path.length);
            newPath[path.length] = popupMenu;
            newPath[path.length + 1] = sub[0];
            manager.setSelectedPath(newPath);
          }
          e.consume();
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void menuKeyPressed(MenuKeyEvent e) {
      char keyChar = e.getKeyChar();
      if (Character.isLetterOrDigit(keyChar)) {
        MenuSelectionManager manager = e.getMenuSelectionManager();
        MenuElement[] path = e.getPath();
        MenuElement[] selectedPath = manager.getSelectedPath();
        for (int i = selectedPath.length - 1; i >= 0; i--) {
          if (selectedPath[i] == menuItem) {
            JPopupMenu popupMenu = menu.getPopupMenu();
            MenuElement[] items = popupMenu.getSubElements();
            if ((indexes == null) || (lastMnemonic != keyChar)) {
              matches = 0;
              lastIndex = 0;
              indexes = new int[items.length];
              for (int j = 0; j < items.length; j++) {
                int key = ((JMenuItem)items[j]).getMnemonic();
                if (Character.toLowerCase((char)key) ==
                    Character.toLowerCase(keyChar)) {
                  indexes[matches++] = j;
                }
              }
              lastMnemonic = keyChar;
            }
            if (matches == 1) {
              JMenuItem item = (JMenuItem)items[indexes[0]];
              if (!(item instanceof JMenu)) {
                manager.clearSelectedPath();
                item.doClick();
              }
            } else if (matches != 0) {
              if (lastIndex == matches) {
                lastIndex = 0;
              }
              MenuElement menuItem = items[indexes[lastIndex++]];
              MenuElement[] newPath = new MenuElement[path.length + 2];
              System.arraycopy(path, 0, newPath, 0, path.length);
              newPath[path.length] = popupMenu;
              newPath[path.length + 1] = menuItem;
              manager.setSelectedPath(newPath);
            }
            e.consume();
            return;
          }
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public void menuKeyReleased(MenuKeyEvent e) {
      // empty method body
    }
  }

  /**
   * Post Action.
   */
  protected static class PostAction extends AbstractAction {
    JMenu menu;
    boolean force;
    /**
     * Constructs new PostAction object.
     *
     * @param menu the menu
     * @param shouldForce the force flag
     */
    public PostAction(JMenu menu, boolean shouldForce) {
      super();
      this.menu = menu;
      this.force = shouldForce;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      MenuSelectionManager defaultManager = MenuSelectionManager.
                                              defaultManager();
      if (force) {
        Container parent = menu.getParent();
        if ((parent != null) && (parent instanceof JMenuBar)) {
          MenuElement[] me;
          MenuElement[] subElements;
          subElements = menu.getPopupMenu().getSubElements();
          if (subElements.length > 0) {
            me = new MenuElement[4];
            me[0] = (MenuElement)parent;
            me[1] = menu;
            me[2] = menu.getPopupMenu();
            me[3] = subElements[0];
          } else {
            me = new MenuElement[3];
            me[0] = (MenuElement)parent;
            me[1] = menu;
            me[2] = menu.getPopupMenu();
          }
          defaultManager.setSelectedPath(me);
        }
      } else {
        MenuElement[] path = defaultManager.getSelectedPath();
        if ((path.length > 0) && (path[path.length - 1] == menu)) {
          appendPath(path, menu.getPopupMenu());
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public boolean isEnabled() {
      return menu.getModel().isEnabled();
    }
  }

  /**
   * Menu Handler.
   */
  protected static class MenuHandler implements MenuListener {
    /**
     * {@inheritDoc}
     */
    public void menuSelected(MenuEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuDeselected(MenuEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void menuCanceled(MenuEvent e) {
      JMenu m = (JMenu)e.getSource();
      MenuSelectionManager manager = MenuSelectionManager.defaultManager();
      if (manager.isComponentPartOfCurrentMenu(m)) {
        MenuSelectionManager.defaultManager().clearSelectedPath();
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 11:52:37 Slobodan
   Initial version

*****************************************************************************^*/
