/*
 * @(#)SmoothOptionPaneUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicOptionPaneUI;

import net.wrka.awt.AwtTools;

/**
 * OptionPaneUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothOptionPaneUI extends BasicOptionPaneUI {

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothOptionPaneUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected Container createButtonArea() {
    JPanel bottom = new JPanel();
    bottom.setBorder(UIManager.getBorder(
                                 "OptionPane.buttonAreaBorder")); //$NON-NLS-1$
    bottom.setLayout(new SmoothButtonAreaLayout(true, 6));
    addButtonComponents(bottom, getButtons(), getInitialValueIndex());
    return bottom;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Smooth Button Area Layout.
   */
  protected class SmoothButtonAreaLayout extends ButtonAreaLayout {
    /**
     * Constructs new SmoothButtonAreaLayout object.
     *
     * @param syncAllWidths the synchronization width flag
     * @param padding the padding
     */
    public SmoothButtonAreaLayout(boolean syncAllWidths, int padding) {
      super(syncAllWidths, padding);
    }
    /**
     * {@inheritDoc}
     */
    public void layoutContainer(Container container) {
      final Component[] children = container.getComponents();
      if ((children != null) && (children.length > 0)) {
        int numChildren = children.length;
        Dimension[] sizes = new Dimension[numChildren];
        int counter;
        int yLocation = container.getInsets().top;
        if (syncAllWidths) {
          int maxWidth = getMinimumButtonWidth();
          for (counter = 0; counter < numChildren; counter++) {
            sizes[counter] = children[counter].getPreferredSize();
            maxWidth = Math.max(maxWidth, sizes[counter].width);
          }
          int xLocation;
          int xOffset;
          if (getCentersChildren()) {
            xLocation =
                (container.getSize().width -
                  (maxWidth * numChildren + (numChildren - 1) * padding)) >> 1;
            xOffset = padding + maxWidth;
          } else {
            if (numChildren > 1) {
              xLocation = 0;
              xOffset = (container.getSize().width - (maxWidth * numChildren)) /
                        (numChildren - 1) + maxWidth;
            } else {
              xLocation = (container.getSize().width - maxWidth) >> 1;
              xOffset = 0;
            }
          }
          for (counter = 0; counter < numChildren; counter++) {
            children[counter].setBounds(xLocation, yLocation, maxWidth,
                                        sizes[counter].height);
            xLocation += xOffset;
          }
        } else {
          int totalWidth = 0;
          for (counter = 0; counter < numChildren; counter++) {
            sizes[counter] = children[counter].getPreferredSize();
            totalWidth += sizes[counter].width;
          }
          totalWidth += ((numChildren - 1) * padding);
          boolean cc = getCentersChildren();
          int xOffset;
          int xLocation;
          if (cc) {
            xLocation = (container.getSize().width - totalWidth) >> 1;
            xOffset = padding;
          } else {
            if (numChildren > 1) {
              xOffset = (container.getSize().width - totalWidth) /
                        (numChildren - 1);
              xLocation = 0;
            } else {
              xLocation = (container.getSize().width - totalWidth) >> 1;
              xOffset = 0;
            }
          }
          for (counter = 0; counter < numChildren; counter++) {
            children[counter].setBounds(xLocation, yLocation,
                                        sizes[counter].width,
                                        sizes[counter].height);
            xLocation += xOffset + sizes[counter].width;
          }
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public Dimension minimumLayoutSize(Container c) {
      if (c != null) {
        Component[] children = c.getComponents();
        if ((children != null) && (children.length > 0)) {
          Dimension cpSize;
          int numChildren = children.length;
          int height = 0;
          Insets cInsets = c.getInsets();
          int extraHeight = cInsets.top + cInsets.bottom;
          if (syncAllWidths) {
            int maxWidth = getMinimumButtonWidth();
            for (int i = 0; i < numChildren; i++) {
              cpSize = children[i].getPreferredSize();
              height = Math.max(height, cpSize.height);
              maxWidth = Math.max(maxWidth, cpSize.width);
            }
            return new Dimension(
                         maxWidth * numChildren + (numChildren - 1) * padding,
                         extraHeight + height);
          }
          int totalWidth = 0;
          for (int i = 0; i < numChildren; i++) {
            cpSize = children[i].getPreferredSize();
            height = Math.max(height, cpSize.height);
            totalWidth += cpSize.width;
          }
          totalWidth += ((numChildren - 1) * padding);
          return new Dimension(totalWidth, extraHeight + height);
        }
      }
      return new Dimension(0, 0);
    }
    private int getMinimumButtonWidth() {
      if (AwtTools.isLowResolution()) {
        return 60;
      }
      return 80;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 20:08:49 Slobodan
   Initial version

*****************************************************************************^*/
