/*
 * @(#)SmoothPopupMenuSeparatorUI.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JComponent;
import javax.swing.JSeparator;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.metal.MetalSeparatorUI;

import net.wrka.swing.graphics.GraphicUtils;

/**
 * PopupMenuSeparatorUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothPopupMenuSeparatorUI extends MetalSeparatorUI {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final SmoothPopupMenuSeparatorUI SHARED_INSTANCE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SHARED_INSTANCE = new SmoothPopupMenuSeparatorUI();
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean deftsInited;
  private Color menuItemLeftBgd;
  private Color menuItemRightBgd;
  private Color separatorLine;
  private int menuItemLeftBgdWidth;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return SHARED_INSTANCE;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize(JComponent c) {
    return new Dimension(0, 3);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    g.setColor(menuItemRightBgd);
    g.fillRect(menuItemLeftBgdWidth, 0, c.getWidth() - menuItemLeftBgdWidth,
               c.getHeight());
    g.setColor(menuItemLeftBgd);
    g.fillRect(0, 0, menuItemLeftBgdWidth, c.getHeight());
    g.setColor(separatorLine);
    g.drawLine(menuItemLeftBgdWidth + 2, 1, c.getWidth() - 2, 1);
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults(JSeparator s) {
    if (!deftsInited) {
      menuItemLeftBgd = SmoothLookAndFeel.getMenuItemLeftBackground();
      separatorLine = GraphicUtils.darkerColor(menuItemLeftBgd, 0.2f);
      menuItemRightBgd = SmoothLookAndFeel.getMenuItemRightBackground();
      menuItemLeftBgdWidth = SmoothLookAndFeel.getMenuItemLeftBackgroundWidth();
      deftsInited = true;
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void uninstallDefaults(JSeparator s) {
    deftsInited = false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 11:12:43 Slobodan
   Initial version

*****************************************************************************^*/

