/*
 * @(#)SmoothSplitPaneDivider.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JButton;
import javax.swing.JSplitPane;
import javax.swing.border.Border;
import javax.swing.plaf.basic.BasicSplitPaneDivider;
import javax.swing.plaf.basic.BasicSplitPaneUI;

/**
 * SplitPaneDivider.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothSplitPaneDivider extends BasicSplitPaneDivider {

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SmoothSplitPaneDivider object.
   *
   * @param ui the basci split pane ui
   */
  public SmoothSplitPaneDivider(BasicSplitPaneUI ui) {
    super(ui);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected JButton createLeftOneTouchButton() {
    return new OneTouchLeftButton();
  }

  /**
   * {@inheritDoc}
   */
  protected JButton createRightOneTouchButton() {
    return new OneTouchRightButton();
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns split pane.
   *
   * @return split pane
   */
  protected JSplitPane getSplitPane() { // optimization
    return splitPane;
  }

  /**
   * Returns orientation.
   *
   * @return orientation
   */
  protected int getOrientation() { // optimization
    return orientation;
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Left one touch button.
   */
  protected class OneTouchLeftButton extends JButton {
    /**
     * Constructs new OneTouchLeftButton object.
     */
    public OneTouchLeftButton() {
      super();
      setMinimumSize(new Dimension(ONE_TOUCH_SIZE, ONE_TOUCH_SIZE));
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      setFocusPainted(false);
      setBorderPainted(false);
      setRequestFocusEnabled(false);
    }
    /**
     * {@inheritDoc}
     */
    public void setBorder(Border border) {
      // empty method body
    }
    /* @deprecated as of java 1.4 but OK here.
    public boolean isFocusTraversable() {
      return false;
    }*/
    /**
     * {@inheritDoc}
     */
    public boolean isFocusable() {
      return false;
    }
    /**
     * {@inheritDoc}
     */
    public void paint(Graphics g) {
      if (getSplitPane() != null) {
        int[] xs = new int[3];
        int[] ys = new int[3];
        int blockSize;
        // Fill the background first ...
        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());
        // ... then draw the arrow.
        g.setColor(Color.BLACK);
        if (getOrientation() == JSplitPane.VERTICAL_SPLIT) {
          blockSize = Math.min(getHeight(), ONE_TOUCH_SIZE);
          xs[0] = blockSize;
          xs[1] = 0;
          xs[2] = blockSize << 1;
          ys[0] = 0;
          ys[1] = ys[2] = blockSize;
          g.drawPolygon(xs, ys, 3); // Little trick to make the
                                    // arrows of equal size
        } else {
          blockSize = Math.min(getWidth(), ONE_TOUCH_SIZE);
          xs[0] = xs[2] = blockSize;
          xs[1] = 0;
          ys[0] = 0;
          ys[1] = blockSize;
          ys[2] = blockSize << 1;
        }
        g.fillPolygon(xs, ys, 3);
      }
    }
  }

  /**
   * Right one touch button.
   */
  protected class OneTouchRightButton extends JButton {
    /**
     * Constructs new OneTouchRightButton object.
     */
    public OneTouchRightButton() {
      super();
      setMinimumSize(new Dimension(ONE_TOUCH_SIZE, ONE_TOUCH_SIZE));
      setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
      setFocusPainted(false);
      setBorderPainted(false);
      setRequestFocusEnabled(false);
    }
    /**
     * {@inheritDoc}
     */
    public void setBorder(Border border) {
      // empty method body
    }
    /* deprecated as of java 1.4 but OK here.
    public boolean isFocusTraversable() {
      return false;
    }*/
    /**
     * {@inheritDoc}
     */
    public boolean isFocusable() {
      return false;
    }
    /**
     * {@inheritDoc}
     */
    public void paint(Graphics g) {
      if (getSplitPane() != null) {
        int[] xs = new int[3];
        int[] ys = new int[3];
        int blockSize;
        // Fill the background first ...
        g.setColor(getBackground());
        g.fillRect(0, 0, getWidth(), getHeight());
        // ... then draw the arrow.
        if (getOrientation() == JSplitPane.VERTICAL_SPLIT) {
          blockSize = Math.min(getHeight(), ONE_TOUCH_SIZE);
          xs[0] = blockSize;
          xs[1] = blockSize << 1;
          xs[2] = 0;
          ys[0] = blockSize;
          ys[1] = ys[2] = 0;
        } else {
          blockSize = Math.min(getWidth(), ONE_TOUCH_SIZE);
          xs[0] = xs[2] = 0;
          xs[1] = blockSize;
          ys[0] = 0;
          ys[1] = blockSize;
          ys[2] = blockSize << 1;
        }
        g.setColor(Color.BLACK);
        g.fillPolygon(xs, ys, 3);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 13:57:33 Slobodan
   Initial version

*****************************************************************************^*/
