/*
 * @(#)SmoothTheme.java  1.0.0 / 2008-05-24
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import javax.swing.UIDefaults;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.FontUIResource;
import javax.swing.plaf.metal.DefaultMetalTheme;

import net.wrka.swing.plaf.AlphaColorUIResource;
import net.wrka.swing.plaf.RenderingPolicy;

/**
 * Smooth Theme.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class SmoothTheme extends DefaultMetalTheme {

/*^**************************************************************************^*/
/*- Protected Static Field.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Default rendering policy.
   */
  protected static final DefaultSmoothRenderingPolicy DEFAULT_RENDERING_POLICY;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    DEFAULT_RENDERING_POLICY = new DefaultSmoothRenderingPolicy();
  }

/*^**************************************************************************^*/
/*- Public Abstract Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the theme is supported on current platform.
   *
   * @return <tt>true</tt> if the theme is supported on current platform
   */
  public abstract boolean isSupported();

  /**
   * Returns rollover highlight factor for the component's background.
   *
   * @return rollover highlight factor for the component's background
   */
  public abstract float getRollOverBackgroundFactor();

  /**
   * Returns rollover component background.
   *
   * @return rollover component background
   */
  public abstract AlphaColorUIResource getRolloverBackground();

  /**
   * Returns rollover highlight factor for the component's foreground.
   *
   * @return rollover highlight factor for the component's foreground
   */
  public abstract float getRollOverForegroundFactor();

  /**
   * Returns rollover component foreground.
   *
   * @return rollover component foreground
   */
  public abstract AlphaColorUIResource getRolloverForeground();

  /**
   * Returns rollover mask color.
   *
   * @return rollover mask color
   */
  public abstract AlphaColorUIResource getRolloverMaskColor();

/*^************************************************************************^*/
/*- Gradient Colors.                                                       -*/
/*^************************************************************************^*/

  /**
   * Returns component gradient color.
   *
   * @return component gradient color
   */
  public abstract AlphaColorUIResource getComponentGradientColor();

  /**
   * Returns component gradient color reflection.
   *
   * @return component gradient color reflection
   */
  public abstract AlphaColorUIResource getComponentGradientReflection();

  /**
   * Returns component gradient color shadow.
   *
   * @return component gradient color shadow
   */
  public abstract AlphaColorUIResource getComponentGradientShadow();

  /**
   * Returns component gradient color light.
   *
   * @return component gradient color light
   */
  public abstract AlphaColorUIResource getComponentGradientLight();

  /**
   * Returns component gradient color reflection light.
   *
   * @return component gradient color reflection light
   */
  public abstract AlphaColorUIResource getComponentGradientReflectionLight();

  /**
   * Returns component gradient color shadow light.
   *
   * @return component gradient color shadow light
   */
  public abstract AlphaColorUIResource getComponentGradientShadowLight();

/*^************************************************************************^*/
/*- Menu Colors.                                                           -*/
/*^************************************************************************^*/

  /**
   * Returns the left backgournd color of the menu items.
   *
   * @return the left backgournd color of the menu items
   */
  public abstract AlphaColorUIResource getMenuItemLeftBackground();

  /**
   * Returns the right backgournd color of the menu items.
   *
   * @return the right backgournd color of the menu items
   */
  public abstract AlphaColorUIResource getMenuItemRightBackground();

  /**
   * Returns the width of the menu left part.
   *
   * @return the width of the menu left part
   */
  public abstract int getMenuItemLeftBackgroundWidth();

/*^************************************************************************^*/
/*- Internal Frame Colors.                                                 -*/
/*^************************************************************************^*/

  // CHANGE: rename abg1 and abg2 x

  /**
   * Returns internal frame title pane active background 1.
   *
   * @return internal frame title pane active background 1
   */
  public abstract AlphaColorUIResource getInternalFrameTitlePaneABG1();

  /**
   * Returns internal frame title pane active background 2.
   *
   * @return internal frame title pane active background 2
   */
  public abstract AlphaColorUIResource getInternalFrameTitlePaneABG2();

  /**
   * Returns active title foreground.
   *
   * @return active title foreground
   */
  public abstract AlphaColorUIResource getActiveTitleForeground();

  /**
   * Returns inactive title foreground.
   *
   * @return inactive title foreground
   */
  public abstract AlphaColorUIResource getInactiveTitleForeground();

/*^**************************************************************************^*/
/*- Fonts.                                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns fixed font.
   *
   * @return fixed font
   */
  public abstract FontUIResource getFixedFont();

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns rendering policy for component UI.
   *
   * @param cui the ComponentUI
   *
   * @return rendering policy for component UI
   */
  public RenderingPolicy getRenderingPolicy(ComponentUI cui) {
    return DEFAULT_RENDERING_POLICY;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void addCustomEntriesToTable(UIDefaults table) {
    table.put("InternalFrame.activeTitleForeground", //$NON-NLS-1$
              getActiveTitleForeground());
    table.put("InternalFrame.inactiveTitleForeground", //$NON-NLS-1$
              getInactiveTitleForeground());
    table.put("InternalFrame.titleFont", //$NON-NLS-1$
              getInactiveTitleForeground());
    table.put("TextArea.font", getFixedFont()); //$NON-NLS-1$
    table.put("TextPane.font", getFixedFont()); //$NON-NLS-1$
    table.put("EditorPane.font", getFixedFont()); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-24 11:17:55 Slobodan
   Initial version

*****************************************************************************^*/
