/*
 * @(#)SmoothToolBarSeparatorUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JComponent;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;
import javax.swing.UIDefaults;
import javax.swing.UIManager;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicToolBarSeparatorUI;

/**
 * SmoothToolBarSeparatorUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothToolBarSeparatorUI extends BasicToolBarSeparatorUI {

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothToolBarSeparatorUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getPreferredSize(JComponent c) {
    Dimension size = ((JToolBar.Separator)c).getSeparatorSize();
    if (size == null) {
      size = new Dimension(5, 5);
      if (((JSeparator)c).getOrientation() == SwingConstants.VERTICAL) {
        size.height = 0;
      } else {
        size.width = 0;
      }
    } else {
      size = size.getSize();
    }
    return size;
  }

  /**
   * {@inheritDoc}
   */
  public Dimension getMaximumSize(JComponent c) {
    Dimension pref = getPreferredSize(c);
    if (((JSeparator)c).getOrientation() == SwingConstants.VERTICAL) {
      return new Dimension(pref.width, Short.MAX_VALUE);
    }
    return new Dimension(Short.MAX_VALUE, pref.height);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    Color temp = g.getColor();
    UIDefaults table = UIManager.getLookAndFeelDefaults();
    Color shadow = table.getColor("ToolBar.shadow"); //$NON-NLS-1$
    if (((JSeparator)c).getOrientation() == SwingConstants.VERTICAL) {
      int x = (c.getWidth() >> 1);
      g.setColor(shadow);
      g.drawLine(x, 2, x, c.getHeight() - 3);
    } else {
      int y = (c.getHeight() >> 1);
      g.setColor(shadow);
      g.drawLine(2, y, c.getWidth() - 3, y);
    }
    g.setColor(temp);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 17:13:13 Slobodan
   Initial version

*****************************************************************************^*/
