/*
 * @(#)SmoothToolBarUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;

import javax.swing.AbstractButton;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.LookAndFeel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.IconUIResource;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.basic.BasicToolBarUI;

import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.graphics.ImageFactory;

/**
 * ToolBarUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothToolBarUI extends BasicToolBarUI {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final String OLD_CONTENT_AREA_FIELD;
  private static final String OLD_FOCUS_PAINTED;
  private static final String OLD_BORDER;
  private static final String OLD_ROLLOVER;
  private static final String REMOVE_ROLLOVER_ICON;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    OLD_CONTENT_AREA_FIELD = "stbOldContentAreaFilled"; //$NON-NLS-1$
    OLD_FOCUS_PAINTED = "stbOldFocusPainted"; //$NON-NLS-1$
    OLD_BORDER = "stbOldBorder"; //$NON-NLS-1$
    OLD_ROLLOVER = "stbOldRollover"; //$NON-NLS-1$
    REMOVE_ROLLOVER_ICON = "stbRemoveRolloverIcon"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothToolBarUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    if (c.isOpaque() && !isFlat(c)) {
      SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
      if (((JToolBar)c).getOrientation() == SwingConstants.HORIZONTAL) {
        SmoothUtils.drawFast3DEffectVerticalLight(g, c);
      } else {
        SmoothUtils.drawFast3DEffectHorizontalLight(g, c);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void installDefaults() {
    LookAndFeel.installBorder(toolBar, "ToolBar.border"); //$NON-NLS-1$
    LookAndFeel.installColorsAndFont(toolBar,
                                     "ToolBar.background", //$NON-NLS-1$
                                     "ToolBar.foreground", //$NON-NLS-1$
                                     "ToolBar.font"); //$NON-NLS-1$
    // Toolbar specific defaults
    if ((dockingColor == null) || (dockingColor instanceof UIResource)) {
      dockingColor = UIManager.getColor(
                                    "ToolBar.dockingBackground"); //$NON-NLS-1$
    }
    if ((floatingColor == null) || (floatingColor instanceof UIResource)) {
      floatingColor = UIManager.getColor(
                                   "ToolBar.floatingBackground"); //$NON-NLS-1$
    }
    if ((dockingBorderColor == null) ||
        (dockingBorderColor instanceof UIResource)) {
      dockingBorderColor = UIManager.getColor(
                                    "ToolBar.dockingForeground"); //$NON-NLS-1$
    }
    if ((floatingBorderColor == null) ||
        (floatingBorderColor instanceof UIResource)) {
      floatingBorderColor = UIManager.getColor(
                                   "ToolBar.floatingForeground"); //$NON-NLS-1$
    }
    setRolloverBorders(true);
  }

  /**
   * {@inheritDoc}
   */
  protected Border createRolloverBorder() {
    return null;
  }

  /**
   * {@inheritDoc}
   */
  protected Border createNonRolloverBorder() {
    return null;
  }

  /**
   * {@inheritDoc}
   */
  protected void setBorderToRollover(Component c) {
    if (c instanceof JButton) {
      JButton b = (JButton)c;
      setButtonRollover(b);
      b.setBorder(SmoothBorders.getButtonRolloverBorder());
    } else if (c instanceof JToggleButton) {
      JToggleButton b = (JToggleButton)c;
      setButtonRollover(b);
      b.setBorder(SmoothBorders.getToggleButtonRolloverBorder());
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void setBorderToNormal(Component c) {
    if ((c instanceof JButton) || (c instanceof JToggleButton)) {
      AbstractButton b = (AbstractButton)c;
      Boolean bul = (Boolean)b.getClientProperty(OLD_CONTENT_AREA_FIELD);
      if (bul != null) {
        b.setContentAreaFilled(bul.booleanValue());
        b.putClientProperty(OLD_CONTENT_AREA_FIELD, null);
      }
      bul = (Boolean)b.getClientProperty(OLD_FOCUS_PAINTED);
      if (bul != null) {
        b.setFocusPainted(bul.booleanValue());
        b.putClientProperty(OLD_FOCUS_PAINTED, null);
      }
      bul = (Boolean)b.getClientProperty(OLD_ROLLOVER);
      if (bul != null) {
        b.setRolloverEnabled(bul.booleanValue());
        b.putClientProperty(OLD_ROLLOVER, null);
      }
      bul = (Boolean)b.getClientProperty(REMOVE_ROLLOVER_ICON);
      if (bul != null) {
        b.setRolloverIcon(null);
        b.putClientProperty(REMOVE_ROLLOVER_ICON, null);
      }
      Border oldBorder = (Border)b.getClientProperty(OLD_BORDER);
      b.setBorder(oldBorder);
      b.putClientProperty(OLD_BORDER, null);
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Sets button rollover state.
   *
   * @param b the button
   */
  protected void setButtonRollover(AbstractButton b) {
    b.putClientProperty(OLD_CONTENT_AREA_FIELD,
                        Boolean.valueOf(b.isContentAreaFilled()));
    b.putClientProperty(OLD_FOCUS_PAINTED,
                        Boolean.valueOf(b.isFocusPainted()));
    b.putClientProperty(OLD_ROLLOVER,
                        Boolean.valueOf(b.isRolloverEnabled()));
    b.setContentAreaFilled(false);
    b.setFocusPainted(false);
    b.setRolloverEnabled(true);
    if (b.getRolloverIcon() == null) {
      Icon icon = b.getIcon();
      if (icon != null) {
        b.setRolloverIcon(new IconUIResource(
                                ImageFactory.createShadowedImageIcon(icon, b)));
        b.putClientProperty(REMOVE_ROLLOVER_ICON, Boolean.TRUE);
      }
    }
    b.putClientProperty(OLD_BORDER, b.getBorder());
  }

  /**
   * Returns <tt>true</tt> if the specified toolbar is flat.
   *
   * @param c the toolbar
   *
   * @return  <tt>true</tt> if the specified toolbar is flat
   */
  protected boolean isFlat(JComponent c) {
    Object flat = c.getClientProperty(JWxToolBar.IS_FLAT);
    if (flat instanceof Boolean) {
      return ((Boolean)flat).booleanValue();
    }
    return false;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 16:36:11 Slobodan
   Initial version

*****************************************************************************^*/
