/*
 * @(#)SmoothTreeUI.java  1.0.0 / 2008-05-25
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JComponent;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTreeUI;

/**
 * TreeUI.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SmoothTreeUI extends BasicTreeUI {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final String LINE_STYLE = "JTree.lineStyle"; //$NON-NLS-1$
  private static final String NO_STYLE_STRING = "None"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Packge-Protected Fields.                                                 -*/
/*^**************************************************************************^*/

  boolean drawLines;
  PropertyChangeListener lineStyleListener;

/*^**************************************************************************^*/
/*- Public Static Method.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Factory method. Return ui for the specified component.
   *
   * @param c the component
   *
   * @return ui for the specified component
   */
  public static ComponentUI createUI(JComponent c) {
    return new SmoothTreeUI();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void installUI(JComponent c) {
    super.installUI(c);
    drawLines = !(NO_STYLE_STRING.equals(c.getClientProperty(LINE_STYLE)));
    lineStyleListener = new LineListener();
    c.addPropertyChangeListener(lineStyleListener);
  }

  /**
   * {@inheritDoc}
   */
  public void uninstallUI(JComponent c) {
    c.removePropertyChangeListener(lineStyleListener);
    super.uninstallUI(c);
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g, JComponent c) {
    SmoothLookAndFeel.getRenderingPolicy(this).apply((Graphics2D)g);
    super.paint(g, c);
  }

  /**
   * {@inheritDoc}
   */
  protected void paintVerticalLine(Graphics g, JComponent c, int x, int top,
                                  int bottom) {
    if (drawLines) {
      for (int dy = bottom - 1; dy >= top; dy -= 3) {
        g.fillRect(x, dy, 1, 2);
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void paintHorizontalLine(Graphics g, JComponent c, int y, int left,
                                     int right) {
    if (drawLines) {
      for (int dx = right - 1; dx >= left; dx -= 3) {
        g.fillRect(dx, y, 2, 1);
      }
    }
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Line Listener.
   */
  protected class LineListener implements PropertyChangeListener {
    /**
     * {@inheritDoc}
     */
    public void propertyChange(PropertyChangeEvent e) {
      String name = e.getPropertyName();
      if (LINE_STYLE.equals(name)) {
        drawLines = !(NO_STYLE_STRING.equals(e.getNewValue()));
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-25 16:33:41 Slobodan
   Initial version

*****************************************************************************^*/
