/*
 * @(#)LightGrayTheme.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth.themes;

import javax.swing.plaf.ColorUIResource;

import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.AlphaColorUIResource;
import net.wrka.swing.plaf.smooth.DefaultSmoothTheme;

/**
 * Light Gray Theme.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class LightGrayTheme extends DefaultSmoothTheme {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  // primary colors
  private static final ColorUIResource PRIMARY_1;
  private static final ColorUIResource PRIMARY_2;
  private static final ColorUIResource PRIMARY_3;

  // secondary colors
  private static final ColorUIResource SECONDARY_1;
  private static final ColorUIResource SECONDARY_2;
  private static final ColorUIResource SECONDARY_3;

  // rollover colors
  private static final AlphaColorUIResource ROLLOVER_FOREGROUND;

  // menu colors
  private static final AlphaColorUIResource MENU_ITEM_LEFT_BACKGROUND;
  private static final AlphaColorUIResource MENU_ITEM_RIGHT_BACKGROUND;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    PRIMARY_1 = new ColorUIResource(42, 42, 74);
    PRIMARY_2 = new ColorUIResource(130, 140, 170);
    PRIMARY_3 = new ColorUIResource(220, 220, 240);
    SECONDARY_1 = new ColorUIResource(130, 130, 130);
    SECONDARY_2 = new ColorUIResource(160, 160, 160);
    SECONDARY_3 = new ColorUIResource(210, 210, 210);
    ROLLOVER_FOREGROUND = new AlphaColorUIResource(95, 54, 199);
    MENU_ITEM_LEFT_BACKGROUND =
                      GraphicUtils.darkerColorUIResource(SECONDARY_3, 0.14f);
    MENU_ITEM_RIGHT_BACKGROUND =
                      GraphicUtils.brighterColorUIResource(SECONDARY_3, 0.2f);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Light Gray"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary1() {
    return PRIMARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary2() {
    return PRIMARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary3() {
    return PRIMARY_3;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary1() {
    return SECONDARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary2() {
    return SECONDARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary3() {
    return SECONDARY_3;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverForeground() {
    return ROLLOVER_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemLeftBackground() {
    return MENU_ITEM_LEFT_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemRightBackground() {
    return MENU_ITEM_RIGHT_BACKGROUND;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 21:46:02 Slobodan
   Initial version

*****************************************************************************^*/
