/*
 * @(#)NativeTheme.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth.themes;

import java.awt.Color;
import java.awt.Font;
import java.awt.SystemColor;
import java.awt.Toolkit;

import javax.swing.plaf.ColorUIResource;
import javax.swing.plaf.FontUIResource;

import net.wrka.swing.graphics.FontUtils;
import net.wrka.swing.graphics.GraphicUtils;
import net.wrka.swing.plaf.AlphaColorUIResource;
import net.wrka.swing.plaf.smooth.SmoothTheme;

/**
 * Native Theme.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class NativeTheme extends SmoothTheme {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final ColorUIResource WHITE;
  private static final ColorUIResource BLACK;

  // primary colors
  private static final ColorUIResource PRIMARY_1;
  private static final ColorUIResource PRIMARY_2;
  private static final ColorUIResource PRIMARY_3;

  // secondary colors
  private static final ColorUIResource SECONDARY_1;
  private static final ColorUIResource SECONDARY_2;
  private static final ColorUIResource SECONDARY_3;

  // gradient colors
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR;
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR_REFLECTION;
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR_SHADOW;
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR_LIGHT;
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR_REF_LGHT;
  private static final AlphaColorUIResource CMP_GRADIENT_COLOR_SHADOW_LGHT;

  // rollover colors
  private static final float ROLLOVER_BACKGROUND_FACTOR;
  private static final AlphaColorUIResource ROLLOVER_BACKGROUND;
  private static final float ROLLOVER_FOREGROUND_FACTOR;
  private static final AlphaColorUIResource ROLLOVER_FOREGROUND;
  private static final AlphaColorUIResource ROLLOVER_MASK_COLOR;

  // menu colors
  private static final AlphaColorUIResource MENU_ITEM_LEFT_BACKGROUND;
  private static final AlphaColorUIResource MENU_ITEM_RIGHT_BACKGROUND;
  private static final int MENU_ITEM_LEFT_BGD_WIDTH;

  // internal frame colors
  private static final AlphaColorUIResource INTERNAL_FRAME_TITLE_PANE_ABG1;
  private static final AlphaColorUIResource INTERNAL_FRAME_TITLE_PANE_ABG2;
  private static final AlphaColorUIResource ACTIVE_TITLE_FOREGROUND;
  private static final AlphaColorUIResource INACTIVE_TITLE_FOREGROUND;
  private static final ColorUIResource DESKTOP_COLOR;

  // fonts
  private static final FontUIResource MENU_FONT;
  private static final FontUIResource CONTROL_FONT;
  private static final FontUIResource WINDOW_TITLE_FONT;
  private static final FontUIResource MONOSPACED_FONT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    Toolkit tk = Toolkit.getDefaultToolkit();
    WHITE = new ColorUIResource(SystemColor.text);
    BLACK = new ColorUIResource(SystemColor.textText);
    PRIMARY_1 = new ColorUIResource(Color.darkGray);
    PRIMARY_2 = new ColorUIResource(SystemColor.scrollbar);
    PRIMARY_3 = new ColorUIResource(SystemColor.info);
    SECONDARY_1 = new ColorUIResource(GraphicUtils.brighterColor(
                                      SystemColor.controlDkShadow, 0.3f));
    SECONDARY_2 = new ColorUIResource(GraphicUtils.brighterColor(
                                      SystemColor.controlShadow, 0.3f));
    SECONDARY_3 = new ColorUIResource(SystemColor.control);
    ROLLOVER_FOREGROUND_FACTOR = 1f;
    ROLLOVER_FOREGROUND = new AlphaColorUIResource(BLACK);
    ROLLOVER_BACKGROUND_FACTOR = 0.1f;
    ROLLOVER_BACKGROUND =
            GraphicUtils.brighterColorUIResource(SECONDARY_3,
                                                 ROLLOVER_BACKGROUND_FACTOR);
    MENU_ITEM_LEFT_BGD_WIDTH = 24;
    MENU_ITEM_RIGHT_BACKGROUND = new AlphaColorUIResource(SystemColor.window);
    MENU_ITEM_LEFT_BACKGROUND =
        new AlphaColorUIResource(
                        GraphicUtils.brighterColor(SystemColor.control, 0.1f));
    CMP_GRADIENT_COLOR_LIGHT = new AlphaColorUIResource(250, 250, 250, 20);
    CMP_GRADIENT_COLOR_REF_LGHT = new AlphaColorUIResource(255, 255, 255, 83);
    CMP_GRADIENT_COLOR_SHADOW_LGHT = new AlphaColorUIResource(0, 0, 0, 28);
    CMP_GRADIENT_COLOR = CMP_GRADIENT_COLOR_LIGHT;
    CMP_GRADIENT_COLOR_REFLECTION = CMP_GRADIENT_COLOR_REF_LGHT;
    CMP_GRADIENT_COLOR_SHADOW = CMP_GRADIENT_COLOR_SHADOW_LGHT;
    ROLLOVER_MASK_COLOR = new AlphaColorUIResource(0, 20, 200, 33);
    INTERNAL_FRAME_TITLE_PANE_ABG1 =
        new AlphaColorUIResource(SystemColor.activeCaption);
    INTERNAL_FRAME_TITLE_PANE_ABG2 = new AlphaColorUIResource(
        GraphicUtils.brighterColor(SystemColor.activeCaption, 0.6f));
    ACTIVE_TITLE_FOREGROUND =
        new AlphaColorUIResource(SystemColor.activeCaptionText);
    INACTIVE_TITLE_FOREGROUND =
        new AlphaColorUIResource(SystemColor.inactiveCaptionText);
    DESKTOP_COLOR = new ColorUIResource(SystemColor.desktop);
    Font mf = (Font)tk.getDesktopProperty("win.menu.font"); //$NON-NLS-1$
    if (mf == null) {
      MENU_FONT = new FontUIResource(FontUtils.DEFAULT_FONT);
    } else {
      MENU_FONT = new FontUIResource(mf);
    }
    CONTROL_FONT = MENU_FONT;
    Font wtf = (Font)tk.getDesktopProperty(
                                        "win.frame.captionFont"); //$NON-NLS-1$
    if (wtf == null) {
      WINDOW_TITLE_FONT = new FontUIResource(FontUtils.DEFAULT_FONT);
    } else {
      WINDOW_TITLE_FONT = new FontUIResource(wtf);
    }
    MONOSPACED_FONT = new FontUIResource(FontUtils.DEFAULT_FIXED_FONT);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Native"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getWhite() {
    return WHITE;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getBlack() {
    return BLACK;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary1() {
    return PRIMARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary2() {
    return PRIMARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getPrimary3() {
    return PRIMARY_3;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary1() {
    return SECONDARY_1;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary2() {
    return SECONDARY_2;
  }

  /**
   * {@inheritDoc}
   */
  protected ColorUIResource getSecondary3() {
    return SECONDARY_3;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isSupported() {
    return true;
  }

  /**
   * {@inheritDoc}
   */
  public float getRollOverBackgroundFactor() {
    return ROLLOVER_BACKGROUND_FACTOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverBackground() {
    return ROLLOVER_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public float getRollOverForegroundFactor() {
    return ROLLOVER_FOREGROUND_FACTOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverForeground() {
    return ROLLOVER_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getRolloverMaskColor() {
    return ROLLOVER_MASK_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientReflection() {
    return CMP_GRADIENT_COLOR_REFLECTION;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientShadow() {
    return CMP_GRADIENT_COLOR_SHADOW;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientColor() {
    return CMP_GRADIENT_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientReflectionLight() {
    return CMP_GRADIENT_COLOR_REF_LGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientShadowLight() {
    return CMP_GRADIENT_COLOR_SHADOW_LGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getComponentGradientLight() {
    return CMP_GRADIENT_COLOR_LIGHT;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemLeftBackground() {
    return MENU_ITEM_LEFT_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getMenuItemRightBackground() {
    return MENU_ITEM_RIGHT_BACKGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public int getMenuItemLeftBackgroundWidth() {
    return MENU_ITEM_LEFT_BGD_WIDTH;
  }

  /**
   * {@inheritDoc}
   */
  public ColorUIResource getDesktopColor() {
    return DESKTOP_COLOR;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInternalFrameTitlePaneABG1() {
    return INTERNAL_FRAME_TITLE_PANE_ABG1;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInternalFrameTitlePaneABG2() {
    return INTERNAL_FRAME_TITLE_PANE_ABG2;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getActiveTitleForeground() {
    return ACTIVE_TITLE_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public AlphaColorUIResource getInactiveTitleForeground() {
    return INACTIVE_TITLE_FOREGROUND;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getFixedFont() {
    return MONOSPACED_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getControlTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getMenuTextFont() {
    return MENU_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getSystemTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getUserTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getWindowTitleFont() {
    return WINDOW_TITLE_FONT;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 21:46:27 Slobodan
   Initial version

*****************************************************************************^*/
