/*
 * @(#)RedmondLargeTheme.java  1.0.0 / 2008-05-22
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.plaf.smooth.themes;

import java.awt.Font;

import javax.swing.plaf.FontUIResource;

import net.wrka.swing.graphics.FontUtils;

/**
 * Large Redmond Theme.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class RedmondLargeTheme extends RedmondTheme {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final FontUIResource MENU_FONT;
  private static final FontUIResource CONTROL_FONT;
  private static final FontUIResource WINDOW_TITLE_FONT;
  private static final FontUIResource MONOSPACED_FONT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    CONTROL_FONT = new FontUIResource(
                         FontUtils.DEFAULT_FONT.deriveFont(Font.BOLD, 13));
    MENU_FONT = CONTROL_FONT;
    WINDOW_TITLE_FONT = CONTROL_FONT;
    MONOSPACED_FONT = new FontUIResource(
                        FontUtils.DEFAULT_FIXED_FONT.deriveFont(Font.BOLD, 15));
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getName() {
    return "Redmond Large"; //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getControlTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getMenuTextFont() {
    return MENU_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getSystemTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getUserTextFont() {
    return CONTROL_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getWindowTitleFont() {
    return WINDOW_TITLE_FONT;
  }

  /**
   * {@inheritDoc}
   */
  public FontUIResource getFixedFont() {
    return MONOSPACED_FONT;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-22 21:46:42 Slobodan
   Initial version

*****************************************************************************^*/
