/*
 * @(#)GeneralPreferences.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.util.Locale;

import net.wrka.LibraryInfo;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.util.CommonConstants;
import net.wrka.util.GPTools;

/**
 * General preferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class GeneralPreferences extends AbstractPreferencesGroup {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * General preferences group name string key.
   */
  public static final String RK_GROUP_NAME = "GPGroupName"; //$NON-NLS-1$

  /**
   * Store preferences string key.
   */
  public static final String STORE_PREFERENCES =
                                              "storepreferences"; //$NON-NLS-1$

  /**
   * Show splash string key.
   */
  public static final String SHOW_SPLASH = "showsplash"; //$NON-NLS-1$

  /**
   * Locale string key.
   */
  public static final String LOCALE = "locale"; //$NON-NLS-1$

/*^**************************************************************************^*/
/*- Protected Constructors.                                                -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GeneralPreferences object.
   */
  protected GeneralPreferences() {
    super(CommonConstants.STRING_EMPTY,
          BuiltInResources.INSTANCE.getResourceBundle(
                     PersistentPreferences.RESOURCES).getString(RK_GROUP_NAME));
  }

  /**
   * Constructs new GeneralPreferences object.
   *
   * @param nodeName the name of the group node
   */
  protected GeneralPreferences(String nodeName) {
    super(nodeName,
          BuiltInResources.INSTANCE.getResourceBundle(
                     PersistentPreferences.RESOURCES).getString(RK_GROUP_NAME));
  }

  /**
   * Constructs new GeneralPreferences object.
   *
   * @param nodeName the name of the group node
   * @param groupName the name of the group
   */
  protected GeneralPreferences(String nodeName, String groupName) {
    super(nodeName, groupName);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Installs this preferences.
   *
   * @return <tt>false</tt> if failed
   */
  public boolean install() {
    PersistentPreferences.installShutdownHook();
    boolean res = super.install();
    if (res) {
      String locale = preferences.get(LOCALE, null);
      if (locale != null) {
        Locale.setDefault(GPTools.parseLocale(locale));
      }
    }
    return res;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns default preferences.
   *
   * @param n the number of additional preferences
   *
   * @return default preferences
   */
  protected String[] getDefaultPreferences(int n) {
    String[] prefs = new String[n + 6];
    prefs[0] = STORE_PREFERENCES;
    prefs[1] = CommonConstants.STRING_FALSE;
    prefs[2] = LOCALE;
    prefs[3] = LibraryInfo.LOCALE_DEFAULT.toString();
    prefs[4] = SHOW_SPLASH;
    prefs[5] = CommonConstants.STRING_TRUE;
    return prefs;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:38:30 Slobodan
   Initial version

*****************************************************************************^*/
