/*
 * @(#)GeneralPreferencesPanel.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;
import java.util.prefs.Preferences;

import javax.swing.JCheckBox;

import net.wrka.swing.resources.BuiltInResources;

/**
 * General preferences panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class GeneralPreferencesPanel extends PreferencesPanel
                                              implements ActionListener {

/*^**************************************************************************^*/
/*- Protected Fields.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Store preferences check box.
   */
  protected JCheckBox jcbStorepreferences;

  /**
   * Show splash check box.
   */
  protected JCheckBox jcbShowSplash;

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new GeneralPreferencesPanel object.
   *
   * @param preferences the preferences
   */
  protected GeneralPreferencesPanel(Preferences preferences) {
    super(preferences);
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                               PersistentPreferences.RESOURCES);
    jcbStorepreferences = new JCheckBox(rb.getString(
                                    PersistentPreferences.RK_REMEMBER_CHANGES));
    jcbStorepreferences.addActionListener(this);
    jcbShowSplash = new JCheckBox(rb.getString(
                                     PersistentPreferences.RK_SHOW_SPLASH));
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Updates this panel.
   */
  public void update() {
    boolean sp = preferences.getBoolean(GeneralPreferences.STORE_PREFERENCES,
                                        false);
    jcbStorepreferences.setSelected(sp);
    boolean tos = preferences.getBoolean(GeneralPreferences.SHOW_SPLASH,
                                         true);
    jcbShowSplash.setSelected(tos);
    jcbShowSplash.setEnabled(!tos || sp);
  }

  /**
   * Applies changes.
   */
  public void applyChanges() {
    preferences.putBoolean(GeneralPreferences.STORE_PREFERENCES,
                           jcbStorepreferences.isSelected());
    preferences.putBoolean(GeneralPreferences.SHOW_SPLASH,
                           jcbShowSplash.isSelected());
  }

  /**
   * Cancels changes.
   */
  public void cancelChanges() { // NOPMD
    // empty method body.
  }

/*^**************************************************************************^*/
/*- ActionListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    if (jcbStorepreferences == e.getSource()) {
      jcbShowSplash.setEnabled(jcbStorepreferences.isSelected());
    }
  }
}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:28:14 Slobodan
   Initial version

*****************************************************************************^*/
