/*
 * @(#)PlafPreferencesPanel.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Insets;
import java.util.prefs.Preferences;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;
import javax.swing.plaf.metal.MetalLookAndFeel;
import javax.swing.plaf.metal.MetalTheme;

import net.wrka.swing.borders.ThinBevelBorder;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.components.JWxPlafComboBox;
import net.wrka.swing.components.JWxPlafThemeComboBox;
import net.wrka.swing.plaf.LookAndFeelManager;
import net.wrka.swing.plaf.LookAndFeelPreview;
import net.wrka.swing.plaf.metal.MetalThemeManager;
import net.wrka.swing.plaf.smooth.SmoothLookAndFeel;
import net.wrka.swing.plaf.smooth.SmoothTheme;

/**
 * Look and Feel Preferences Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class PlafPreferencesPanel extends PreferencesPanel {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final JWxPlafThemeComboBox jcbThemes;
  private transient LookAndFeel oldLnF;
  private transient MetalTheme oldMetalTheme;
  private transient SmoothTheme oldSmoothTheme;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PlafPreferencesPanel object.
   *
   * @param parent the parent component
   * @param preferences the preferences
   */
  public PlafPreferencesPanel(JComponent parent, Preferences preferences) {
    super(preferences);
    JWxPlafComboBox jcbLnFs = new JWxPlafComboBox(parent);
    jcbThemes = new JWxPlafThemeComboBox(parent);
    addComponent(new JLabel("Look and Feel"), 0, 0, 1, 1, //$NON-NLS-1$
                 JWxGriddedPanel.GBC_SOUTHWEST, JWxGriddedPanel.GBC_HORIZONTAL,
                 new Insets(1, 5, 0, 5), 0.9, 0.0);
    addComponent(jcbLnFs, 0, 1, 1, 1, JWxGriddedPanel.GBC_NORTHWEST,
                 JWxGriddedPanel.GBC_HORIZONTAL, new Insets(1, 2, 0, 5), 1.0,
                 0.0);
    addComponent(new JLabel("Theme"), 1, 0, 1, 1, //$NON-NLS-1$
                 JWxGriddedPanel.GBC_SOUTHWEST, JWxGriddedPanel.GBC_HORIZONTAL,
                 new Insets(1, 5, 0, 2), 1.1, 0.0);
    addComponent(jcbThemes, 1, 1, 1, 1, JWxGriddedPanel.GBC_NORTHWEST,
                 JWxGriddedPanel.GBC_HORIZONTAL, new Insets(1, 2, 0, 2), 1.0,
                 0.0);
    addComponent(new JLabel("Preview"), 0, 2, 1, 1, //$NON-NLS-1$
                 JWxGriddedPanel.GBC_SOUTHWEST, JWxGriddedPanel.GBC_NONE,
                 new Insets(6, 5, 0, 3), 1.0, 0.0);
    JPanel jpLnFCont = new JPanel(new BorderLayout());
    jpLnFCont.setBorder(BorderFactory.createCompoundBorder(
                              new ThinBevelBorder(ThinBevelBorder.TYPE_LOWERED),
                              new EmptyBorder(3, 3, 3, 3)));
    jpLnFCont.setPreferredSize(new Dimension(0, 140));
    jpLnFCont.setMaximumSize(getPreferredSize());
    jpLnFCont.add(new JScrollPane(new LookAndFeelPreview()));
    addComponent(jpLnFCont, 0, 3, 2, 1, JWxGriddedPanel.GBC_CENTER,
                 JWxGriddedPanel.GBC_BOTH, new Insets(2, 3, 3, 3), 1.0, 1.0);
    update();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Updates this panel.
   */
  public void update() {
    PlafPreferences.instance.setCanUpdateLnFTargets(false);
    oldLnF = UIManager.getLookAndFeel();
    if (oldLnF instanceof SmoothLookAndFeel) {
      oldSmoothTheme = SmoothLookAndFeel.getCurrentSmoothTheme();
    } else if (oldLnF instanceof MetalLookAndFeel) {
      oldMetalTheme = MetalThemeManager.getCurrentMetalTheme();
    }
  }

  /**
   * Applies changes.
   */
  public void applyChanges() {
    LookAndFeel lnf = UIManager.getLookAndFeel();
    preferences.put(PlafPreferences.PLAF_CLASS_NAME, lnf.getClass().getName());
    int lnfTheme;
    if (jcbThemes.isEnabled()) {
      lnfTheme = jcbThemes.getSelectedIndex();
    } else {
      lnfTheme = -1;
    }
    preferences.putInt(PlafPreferences.PLAF_THEME, lnfTheme);
    preferences.put(PlafPreferences.PLAF_NAME,
                    lnf.getName().concat(String.valueOf(lnfTheme)));
    PlafPreferences.instance.setCanUpdateLnFTargets(true);
    PlafPreferences.instance.updateTargets();
    oldLnF = null;
    oldMetalTheme = null;
    oldSmoothTheme = null;
  }

  /**
   * Cancels changes.
   */
  public void cancelChanges() {
    LookAndFeel lnf = UIManager.getLookAndFeel();
    if (oldLnF.equals(lnf)) {
      if ((oldSmoothTheme != null) && // NOPMD
          (oldSmoothTheme != SmoothLookAndFeel.getCurrentSmoothTheme())) {
        SmoothLookAndFeel.setCurrentTheme(oldSmoothTheme);
        PlafPreferences.instance.setCanUpdateLnFTargets(true);
        PlafPreferences.instance.updateTargets();
      } else if ((oldMetalTheme != null) &&
                 (oldMetalTheme != MetalThemeManager.getCurrentMetalTheme())) {
        MetalLookAndFeel.setCurrentTheme(oldMetalTheme);
        PlafPreferences.instance.setCanUpdateLnFTargets(true);
        PlafPreferences.instance.updateTargets();
      }
    } else {
      LookAndFeelManager.setLookAndFeel(oldLnF);
      PlafPreferences.instance.setCanUpdateLnFTargets(true);
      PlafPreferences.instance.updateTargets();
    }
    oldLnF = null;
    oldMetalTheme = null;
    oldSmoothTheme = null;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:22:24 Slobodan
   Initial version

*****************************************************************************^*/
