/*
 * @(#)PreferencesDialog.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.ResourceBundle;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.wrka.swing.components.JWxButton;
import net.wrka.swing.components.JWxDialog;
import net.wrka.swing.components.JWxGriddedPanel;
import net.wrka.swing.components.JWxLogoPanel;
import net.wrka.swing.components.JWxSeparator;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.resources.BuiltInResources;
import net.wrka.swing.tooltips.ToolTipManager;
import net.wrka.util.SimpleArrayList;

/**
 * Preferences Dialog.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class PreferencesDialog extends JWxDialog implements ChangeListener {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int DIALOG_WIDTH = 420;
  private static final int DIALOG_HEIGHT = 430;

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private JTabbedPane tabPane;
  private JLabel groupTitle;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PreferencesDialog object.
   *
   * @param owner the dialog owner
   * @param name the dialog title
   */
  public PreferencesDialog(Frame owner, String name) {
    super(owner, name, true);
    initDialog();
  }

  /**
   * Constructs new PreferencesDialog object.
   *
   * @param owner the dialog owner
   * @param name the dialog title
   */
  public PreferencesDialog(Dialog owner, String name) {
    super(owner, name, true);
    initDialog();
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates panels.
   *
   */
  public void updatePanels() {
    JTabbedPane tabPane = this.tabPane;
    int size = tabPane.getTabCount();
    for (int i = 0; i < size; i++) {
      PreferencesPanel pp = (PreferencesPanel)tabPane.getComponentAt(i);
      pp.update();
    }
    if (size > 0) {
      tabPane.setSelectedIndex(0);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    cancelChanges();
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void initDialog() {
    ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                               PersistentPreferences.RESOURCES);
    setDefaultCloseOperation(DISPOSE_ON_CLOSE);
    setSize(DIALOG_WIDTH, DIALOG_HEIGHT);
    setResizable(false);
    JWxGriddedPanel cp = new JWxGriddedPanel(new Insets(0, 0, 0, 0));
    JWxLogoPanel lp = new JWxLogoPanel();
    groupTitle = new JLabel();
    groupTitle.setVerticalAlignment(JLabel.TOP);
    groupTitle.setBorder(new EmptyBorder(12, 8, 12, 8));
    lp.add(groupTitle);
    JLabel prefIcon = new JLabel(BuiltInResources.INSTANCE.
                                 getImageIcon("Preferences.png")); //$NON-NLS-1$
    prefIcon.setBorder(new EmptyBorder(0, 0, 0, 17));
    lp.add(prefIcon, BorderLayout.EAST);
    Dimension lpd = new Dimension(1, 70);
    lp.setPreferredSize(lpd);
    lp.setMinimumSize(lpd);
    cp.addComponent(lp, 0, 0, 1, 1, JWxGriddedPanel.GBC_NORTH,
                    JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    cp.addComponent(new JWxSeparator(new Dimension(1, 3),
                                     new Insets(0, 1, 1, 2), JWxSeparator.TOP),
                    0, 1, 1, 1, JWxGriddedPanel.GBC_NORTH,
                    JWxGriddedPanel.GBC_HORIZONTAL, null, 1.0, 0.0);
    tabPane = new JTabbedPane();
    SimpleArrayList prefGroups = PersistentPreferences.prefGroups;
    int size = prefGroups.size();
    for (int i = 0; i < size; i++) {
      PreferencesGroup pg = (PreferencesGroup)prefGroups.get(i);
      tabPane.addTab(pg.getGroupName(), pg.getGroupPanel(cp));
    }
    tabPane.addChangeListener(this);
    stateChanged(null);
    cp.addComponent(tabPane, 0, 2, 1, 1, JWxGriddedPanel.GBC_CENTER,
                    JWxGriddedPanel.GBC_BOTH, new Insets(13, 6, 6, 6), 1.0,
                    1.0);
    JPanel tmp = new JPanel(new GridLayout(1, 3, 3, 2));
    tmp.add(new ApproveButton(rb.getString(
                                    PersistentPreferences.RK_APPLY_BUTTON_TEXT),
                              rb.getString(
                               PersistentPreferences.RK_APPLY_BUTTON_TOOLTIP)));
    tmp.add(new CancelButton(rb.getString(
                                   PersistentPreferences.RK_CANCEL_BUTTON_TEXT),
                             rb.getString(
                              PersistentPreferences.RK_CANCEL_BUTTON_TOOLTIP)));
    tmp.add(new HelpButton(rb.getString(
                                     PersistentPreferences.RK_HELP_BUTTON_TEXT),
                           rb.getString(
                                PersistentPreferences.RK_HELP_BUTTON_TOOLTIP)));
    cp.addComponent(tmp, 0, 3, 1, 1, JWxGriddedPanel.GBC_EAST,
                    JWxGriddedPanel.GBC_NONE, new Insets(0, 4, 6, 8),
                    1.0, 0.0, 3, 0);
    ToolTipManager.registerContainer(cp);
    setContentPane(cp);
  }

/*^**************************************************************************^*/
/*- ChangeListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void stateChanged(ChangeEvent e) {
    int seltab = tabPane.getSelectedIndex();
    PreferencesGroup pg =
                 (PreferencesGroup)PersistentPreferences.prefGroups.get(seltab);
    groupTitle.setText(pg.getGroupTitle());
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void applyChanges() {
    JTabbedPane tabPane = this.tabPane;
    int size = tabPane.getTabCount();
    for (int i = 0; i < size; i++) {
      PreferencesPanel pp = (PreferencesPanel)tabPane.getComponentAt(i);
      pp.applyChanges();
    }
  }

  void cancelChanges() {
    JTabbedPane tabPane = this.tabPane;
    int size = tabPane.getTabCount();
    for (int i = size - 1; i >= 0; i--) {
      PreferencesPanel pp = (PreferencesPanel)tabPane.getComponentAt(i);
      pp.cancelChanges();
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Approve Button.
   */
  final class ApproveButton extends JWxButton {
    /**
     * Constructs new ApproveButton object.
     *
     * @param label the label
     * @param tooltip the tool tip
     */
    public ApproveButton(String label, String tooltip) {
      super(label, tooltip);
      setDefaultCapable(false);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      applyChanges();
      dispose();
    }
  }

  /**
   * Cancel Button.
   */
  final class CancelButton extends JWxButton {
    /**
     * Constructs new CancelButton object.
     *
     * @param label the label
     * @param tooltip the tool tip
     */
    public CancelButton(String label, String tooltip) {
      super(label, tooltip);
      setDefaultCapable(false);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      cancelChanges();
      dispose();
    }
  }

  /**
   * SimpleHelp Button.
   */
  static final class HelpButton extends JWxButton {
    private final String helpTopic;
    /**
     * Constructs new HelpButton object.
     *
     * @param label the label
     * @param tooltip the tool tip
     */
    public HelpButton(String label, String tooltip) {
      super(label, tooltip);
      setDefaultCapable(false);
      ResourceBundle rb = BuiltInResources.INSTANCE.getResourceBundle(
                                               PersistentPreferences.RESOURCES);
      helpTopic = rb.getString(PersistentPreferences.RK_HELP_TOPIC);
    }
    /**
     * {@inheritDoc}
     */
    protected void onAction(ActionEvent e) {
      SimpleHelp.showHelp(helpTopic);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:31:18 Slobodan
   Initial version

*****************************************************************************^*/
