/*
 * @(#)PreferencesGroup.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.util.prefs.Preferences;

import javax.swing.JComponent;

/**
 * Preferences Group.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface PreferencesGroup {

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the name of the group.
   *
   * @return the name of the group
   */
  public String getGroupName();

  /**
   * Returns preferences.
   *
   * @return preferences
   */
  public Preferences getPreferences();

  /**
   * Installs preferences.
   *
   * @return <tt>true</tt> if succeed
   */
  public boolean install();

  /**
   * Uninstalls preferences.
   *
   * @return <tt>true</tt> if succeed
   */
  public boolean uninstall();

  /**
   * Returns group title.
   *
   * @return group title
   */
  public String getGroupTitle();

  /**
   * Returns new group panel.
   *
   * @param parent the panel parent
   *
   * @return group panel
   */
  public PreferencesPanel getGroupPanel(JComponent parent);

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 17:23:03 Slobodan
   Initial version

*****************************************************************************^*/
