/*
 * @(#)PreferencesPanel.java  1.0.0 / 2008-05-18
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.preferences;

import java.awt.Insets;
import java.util.prefs.Preferences;

import javax.swing.border.EmptyBorder;

import net.wrka.swing.components.JWxGriddedPanel;

/**
 * Preferences Panel.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class PreferencesPanel extends JWxGriddedPanel {

/*^**************************************************************************^*/
/*- Protected Static Field.                                                  -*/
/*^**************************************************************************^*/

  /**
   * Shared empty border.
   */
  protected static final EmptyBorder EMPTY_BORDER = new EmptyBorder(4, 4, 4, 3);

/*^**************************************************************************^*/
/*- Protected Field.                                                         -*/
/*^**************************************************************************^*/

  /**
   * Preferences.
   */
  protected final Preferences preferences;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new PreferencesPanel object.
   *
   * @param preferences Preferences
   */
  public PreferencesPanel(Preferences preferences) {
    super();
    if (preferences == null) {
      throw new IllegalArgumentException("Null preferences node"); //$NON-NLS-1$
    }
    this.preferences = preferences;
    setBorder(EMPTY_BORDER);
  }

  /**
   * Constructs new PreferencesPanel object.
   *
   * @param insets insets
   * @param preferences Preferences
   */
  public PreferencesPanel(Insets insets, Preferences preferences) {
    super(insets);
    if (preferences == null) {
      throw new IllegalArgumentException("Null preferences node"); //$NON-NLS-1$
    }
    this.preferences = preferences;
    setBorder(EMPTY_BORDER);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Updates this panel.
   */
  public abstract void update();

  /**
   * Applies changes.
   */
  public abstract void applyChanges();

  /**
   * Cancels changes.
   */
  public abstract void cancelChanges();

}

/*^*****************************************************************************
  File History:

 - 2008-05-18 21:19:25 Slobodan
   Initial version

*****************************************************************************^*/
