/*
 * @(#)ActionResources.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.resources.bundles;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ListResourceBundle;

import javax.swing.KeyStroke;

import net.wrka.swing.actions.AboutAction;
import net.wrka.swing.actions.ClearAction;
import net.wrka.swing.actions.CloseAction;
import net.wrka.swing.actions.CopyAction;
import net.wrka.swing.actions.CutAction;
import net.wrka.swing.actions.ExitAction;
import net.wrka.swing.actions.FindAction;
import net.wrka.swing.actions.FindNextAction;
import net.wrka.swing.actions.HelpAction;
import net.wrka.swing.actions.NewAction;
import net.wrka.swing.actions.OpenAction;
import net.wrka.swing.actions.PasteAction;
import net.wrka.swing.actions.PreferencesAction;
import net.wrka.swing.actions.RedoAction;
import net.wrka.swing.actions.ReplaceAction;
import net.wrka.swing.actions.SaveAction;
import net.wrka.swing.actions.SaveAsAction;
import net.wrka.swing.actions.SelectAllAction;
import net.wrka.swing.actions.UndoAction;
import net.wrka.swing.graphics.EmptyIcon;
import net.wrka.swing.resources.BuiltInResources;

/**
 * Action Resources.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ActionResources extends ListResourceBundle {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static final Object[][] CONTENTS = {
    //CS:OFF
    // Action New
    { NewAction.RK_NAME, "New..." }, //$NON-NLS-1$
    { NewAction.RK_SHORT_DESCRIPTION, "New" }, //$NON-NLS-1$
    { NewAction.RK_LONG_DESCRIPTION, "New File" }, //$NON-NLS-1$
    { NewAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("New.png")}, //$NON-NLS-1$
    { NewAction.RK_ACTION_COMMAND_KEY, "new" }, //$NON-NLS-1$
    { NewAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK)},
    { NewAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_N) }, // NOPMD
    // Action Open
    { OpenAction.RK_NAME, "Open..." }, //$NON-NLS-1$
    { OpenAction.RK_SHORT_DESCRIPTION, "Open" }, //$NON-NLS-1$
    { OpenAction.RK_LONG_DESCRIPTION, "Open File" }, //$NON-NLS-1$
    { OpenAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Open.png")}, //$NON-NLS-1$
    { OpenAction.RK_ACTION_COMMAND_KEY, "open" }, //$NON-NLS-1$
    { OpenAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK)},
    { OpenAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_O) }, // NOPMD
    // Action Save
    { SaveAction.RK_NAME, "Save" }, //$NON-NLS-1$
    { SaveAction.RK_SHORT_DESCRIPTION, "Save" }, //$NON-NLS-1$
    { SaveAction.RK_LONG_DESCRIPTION, "Save File" }, //$NON-NLS-1$
    { SaveAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Save.png")}, //$NON-NLS-1$
    { SaveAction.RK_ACTION_COMMAND_KEY, "save" }, //$NON-NLS-1$
    { SaveAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK)},
    { SaveAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_S) }, // NOPMD
    // Action Save As
    { SaveAsAction.RK_NAME, "Save As..." }, //$NON-NLS-1$
    { SaveAsAction.RK_SHORT_DESCRIPTION, "Save As" }, //$NON-NLS-1$
    { SaveAsAction.RK_LONG_DESCRIPTION, "Save File As" }, //$NON-NLS-1$
    { SaveAsAction.RK_SMALL_ICON, EmptyIcon.ICON_16},
    { SaveAsAction.RK_ACTION_COMMAND_KEY, "save as" }, //$NON-NLS-1$
    { SaveAsAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_A) }, // NOPMD
    // Action Preferences
    { PreferencesAction.RK_NAME, "Preferences..." }, //$NON-NLS-1$
    { PreferencesAction.RK_SHORT_DESCRIPTION, "Preferences" }, //$NON-NLS-1$
    { PreferencesAction.RK_LONG_DESCRIPTION,
                                           "Setup Preferences" }, //$NON-NLS-1$
    { PreferencesAction.RK_SMALL_ICON, EmptyIcon.ICON_16 },
    { PreferencesAction.RK_ACTION_COMMAND_KEY, "preferences" }, //$NON-NLS-1$
    { PreferencesAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_P) }, // NOPMD
    // Action Close
    { CloseAction.RK_NAME, "Close" }, //$NON-NLS-1$
    { CloseAction.RK_SHORT_DESCRIPTION, "Close" }, //$NON-NLS-1$
    { CloseAction.RK_LONG_DESCRIPTION, "Close File" }, //$NON-NLS-1$
    { CloseAction.RK_SMALL_ICON, EmptyIcon.ICON_16 },
    { CloseAction.RK_ACTION_COMMAND_KEY, "close" }, //$NON-NLS-1$
    { CloseAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.CTRL_MASK)},
    { CloseAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_L) }, // NOPMD
    // Action Exit
    { ExitAction.RK_NAME, "Exit" }, //$NON-NLS-1$
    { ExitAction.RK_SHORT_DESCRIPTION, "Exit" }, //$NON-NLS-1$
    { ExitAction.RK_LONG_DESCRIPTION, "Exit Program" }, //$NON-NLS-1$
    { ExitAction.RK_SMALL_ICON, EmptyIcon.ICON_16 },
    { ExitAction.RK_ACTION_COMMAND_KEY, "exit" }, //$NON-NLS-1$
    { ExitAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_X) }, // NOPMD
    // Action Undo
    { UndoAction.RK_NAME, "Undo" }, //$NON-NLS-1$
    { UndoAction.RK_SHORT_DESCRIPTION, "Undo" }, //$NON-NLS-1$
    { UndoAction.RK_LONG_DESCRIPTION, "Undo Last Action" }, //$NON-NLS-1$
    { UndoAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Undo.png") }, //$NON-NLS-1$
    { UndoAction.RK_ACTION_COMMAND_KEY, "undo" }, //$NON-NLS-1$
    { UndoAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_Z, InputEvent.CTRL_MASK)},
    { UndoAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_U) }, // NOPMD
    // Action Redo
    { RedoAction.RK_NAME, "Redo" }, //$NON-NLS-1$
    { RedoAction.RK_SHORT_DESCRIPTION, "Redo" }, //$NON-NLS-1$
    { RedoAction.RK_LONG_DESCRIPTION, "Redo Last Action" }, //$NON-NLS-1$
    { RedoAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Redo.png") }, //$NON-NLS-1$
    { RedoAction.RK_ACTION_COMMAND_KEY, "redo" }, //$NON-NLS-1$
    { RedoAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_Y, InputEvent.CTRL_MASK)},
    { RedoAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_R) }, // NOPMD
    // Action Cut
    { CutAction.RK_NAME, "Cut" }, //$NON-NLS-1$
    { CutAction.RK_SHORT_DESCRIPTION, "Cut" }, //$NON-NLS-1$
    { CutAction.RK_LONG_DESCRIPTION, "Cut Selected Text" }, //$NON-NLS-1$
    { CutAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Cut.png") }, //$NON-NLS-1$
    { CutAction.RK_ACTION_COMMAND_KEY, "cut" }, //$NON-NLS-1$
    { CutAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK)},
    { CutAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_T) }, // NOPMD
    // Action Copy
    { CopyAction.RK_NAME, "Copy" }, //$NON-NLS-1$
    { CopyAction.RK_SHORT_DESCRIPTION, "Copy" }, //$NON-NLS-1$
    { CopyAction.RK_LONG_DESCRIPTION, "Copy Selected Text" }, //$NON-NLS-1$
    { CopyAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Copy.png") }, //$NON-NLS-1$
    { CopyAction.RK_ACTION_COMMAND_KEY, "copy" }, //$NON-NLS-1$
    { CopyAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_C, InputEvent.CTRL_MASK)},
    { CopyAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_C) }, // NOPMD
    // Action Paste
    { PasteAction.RK_NAME, "Paste" }, //$NON-NLS-1$
    { PasteAction.RK_SHORT_DESCRIPTION, "Paste" }, //$NON-NLS-1$
    { PasteAction.RK_LONG_DESCRIPTION, "Paste Copied Text" }, //$NON-NLS-1$
    { PasteAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Paste.png") }, //$NON-NLS-1$
    { PasteAction.RK_ACTION_COMMAND_KEY, "paste" }, //$NON-NLS-1$
    { PasteAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_V, InputEvent.CTRL_MASK)},
    { PasteAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_P) }, // NOPMD
    // Action Clear
    { ClearAction.RK_NAME, "Clear" }, //$NON-NLS-1$
    { ClearAction.RK_SHORT_DESCRIPTION, "Clear" }, //$NON-NLS-1$
    { ClearAction.RK_LONG_DESCRIPTION, "Clear Selected Text" }, //$NON-NLS-1$
    { ClearAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Clear.png") }, //$NON-NLS-1$
    { ClearAction.RK_ACTION_COMMAND_KEY, "clear" }, //$NON-NLS-1$
    { ClearAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0)},
    { ClearAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_L) }, // NOPMD
    // Action SelectAll
    { SelectAllAction.RK_NAME, "Select All" }, //$NON-NLS-1$
    { SelectAllAction.RK_SHORT_DESCRIPTION, "Select All" }, //$NON-NLS-1$
    { SelectAllAction.RK_LONG_DESCRIPTION, "Select All Text" }, //$NON-NLS-1$
    { SelectAllAction.RK_SMALL_ICON, EmptyIcon.ICON_16 },
    { SelectAllAction.RK_ACTION_COMMAND_KEY, "selectall" }, //$NON-NLS-1$
    { SelectAllAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK)},
    { SelectAllAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_A) }, // NOPMD
    // Action Find
    { FindAction.RK_NAME, "Find..." }, //$NON-NLS-1$
    { FindAction.RK_SHORT_DESCRIPTION, "Find" }, //$NON-NLS-1$
    { FindAction.RK_LONG_DESCRIPTION, "Find Text" }, //$NON-NLS-1$
    { FindAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Find.png")}, //$NON-NLS-1$
    { FindAction.RK_ACTION_COMMAND_KEY, "find" }, //$NON-NLS-1$
    { FindAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_F, InputEvent.CTRL_MASK)},
    { FindAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_F) }, // NOPMD
    // Action Find Next
    { FindNextAction.RK_NAME, "Find Next" }, //$NON-NLS-1$
    { FindNextAction.RK_SHORT_DESCRIPTION, "Find Next" }, //$NON-NLS-1$
    { FindNextAction.RK_LONG_DESCRIPTION,
      "Find Next Occurrence" }, //$NON-NLS-1$
    { FindNextAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.
                             getCachedImageIcon("FindNext.png")}, //$NON-NLS-1$
    { FindNextAction.RK_ACTION_COMMAND_KEY, "findnext" }, //$NON-NLS-1$
    { FindNextAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_F3, 0)},
    { FindNextAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_N) }, // NOPMD
    // Action Replace
    { ReplaceAction.RK_NAME, "Replace..." }, //$NON-NLS-1$
    { ReplaceAction.RK_SHORT_DESCRIPTION, "Replace" }, //$NON-NLS-1$
    { ReplaceAction.RK_LONG_DESCRIPTION, "Replace Text" }, //$NON-NLS-1$
    { ReplaceAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.
                              getCachedImageIcon("Replace.png")}, //$NON-NLS-1$
    { ReplaceAction.RK_ACTION_COMMAND_KEY, "replace" }, //$NON-NLS-1$
    { ReplaceAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_H, InputEvent.CTRL_MASK)},
    { ReplaceAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_R) }, // NOPMD
    // Action SimpleHelp
    { HelpAction.RK_NAME, "Help..." }, //$NON-NLS-1$
    { HelpAction.RK_SHORT_DESCRIPTION, "Help" }, //$NON-NLS-1$
    { HelpAction.RK_LONG_DESCRIPTION, "Show Help" }, //$NON-NLS-1$
    { HelpAction.RK_SMALL_ICON,
      BuiltInResources.INSTANCE.getCachedImageIcon("Help.png")}, //$NON-NLS-1$
    { HelpAction.RK_ACTION_COMMAND_KEY, "help" }, //$NON-NLS-1$
    { HelpAction.RK_ACCELERATOR_KEY,
      KeyStroke.getKeyStroke(KeyEvent.VK_F1, 0)},
    { HelpAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_H) }, // NOPMD
    // Action About
    { AboutAction.RK_NAME_KEY, "About..." }, //$NON-NLS-1$
    { AboutAction.RK_SHORT_DESCRIPTION, "About" }, //$NON-NLS-1$
    { AboutAction.RK_SMALL_ICON, EmptyIcon.ICON_16 },
    { AboutAction.RK_ACTION_COMMAND_KEY, "about" }, //$NON-NLS-1$
    { AboutAction.RK_MNEMONIC_KEY, new Integer(KeyEvent.VK_A) } // NOPMD
    // CS:ON
  };

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected Object[][] getContents() {
    return CONTENTS; // NOPMD
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 10:58:12 Slobodan
   Initial version

*****************************************************************************^*/
