/*
 * @(#)SplashWindow.java  1.0.0 / 2008-05-15
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.swing.splash;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.ImageIcon;
import javax.swing.JWindow;

import net.wrka.awt.AwtTools;

/**
 * Splash Window.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class SplashWindow extends JWindow implements Runnable, MouseListener {

/*^**************************************************************************^*/
/*- Private Static Final Fields.                                             -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 8729207466264436819L;

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final SplashImage spalshImage;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs splash window.
   *
   * @param imageIcon the splash image icon
   * @param steps the number of the splash steps
   * @param startmessage the start message
   */
  public SplashWindow(ImageIcon imageIcon, int steps, String startmessage) {
    super();
    spalshImage = new SplashImage(imageIcon, steps, startmessage);
    getContentPane().add(spalshImage);
    addMouseListener(this);
    setSize(spalshImage.getSize()); // pack() does'n work correct
    AwtTools.screenCenterWindow(this);
    setVisible(true);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Updates splash.
   */
  public void updateSplash() {
    spalshImage.updateSplash();
  }

  /**
   * Updates splash.
   *
   * @param message the step message
   */
  public void updateSplash(String message) {
    spalshImage.updateSplash(message);
  }

  /**
   * Closes splash and eliminates blinking during splash - main window exchange.
   */
  public void close() {
    new Thread(this).start();
  }

/*^**************************************************************************^*/
/*- Runnable Interface Implementation.                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void run() {
    try {
      Thread.sleep(700);
    } catch (InterruptedException ie) { // NOPMD
      // empty catch block
    }
    spalshImage.dispose();
    dispose();
  }

/*^**************************************************************************^*/
/*- MouseListener Implementation.                                            -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void mouseClicked(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mousePressed(MouseEvent e) {
    Splash.hide();
  }

  /**
   * {@inheritDoc}
   */
  public void mouseReleased(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mouseEntered(MouseEvent e) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void mouseExited(MouseEvent e) {
    // empty method body
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-15 20:40:32 Slobodan
   Initial version

*****************************************************************************^*/

