/*
 * @(#)EmptyIterator.java  1.0.0 / 2008-05-05
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.lang.ref.SoftReference;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Empty iterator.<br>
 * Based on Sun's java.util.Hashtable.EmptyIterator.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class EmptyIterator implements Iterator {

/*^**************************************************************************^*/
/*- Private Static Field.                                                    -*/
/*^**************************************************************************^*/

  private static SoftReference cachedInstance;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    cachedInstance = new SoftReference(new EmptyIterator());
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new EmptyIterator object.
   */
  private EmptyIterator() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns cached instance of this class.
   *
   * @return EmptyIterator
   */
  public static EmptyIterator getInstance() {
    // NOTE: no need for synchronization.
    EmptyIterator ei = (EmptyIterator)cachedInstance.get();
    if (ei == null) {
      ei = new EmptyIterator();
      cachedInstance = new SoftReference(ei);
    }
    return ei;
  }

/*^**************************************************************************^*/
/*- Iterator Implementation.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Iterator's hasNext method.
   *
   * @return <tt>false</tt>
   */
  public boolean hasNext() {
    return false;
  }

  /**
   * Iterator's next method.
   *
   * @return <tt>null</tt>
   *
   * @throws NoSuchElementException
   */
  public Object next() {
    throw new NoSuchElementException();
  }

  /**
   * Iterator's remove method.
   *
   * @throws IllegalStateException
   */
  public void remove() {
    throw new IllegalStateException();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-05 11:59:16 Slobodan
   Initial version

*****************************************************************************^*/
