/*
 * @(#)ObjectContainer.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

/**
 * Object wrapper for lists, trees ...
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ObjectContainer {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private Object object;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ObjectContainer object.
   */
  public ObjectContainer() {
    // empty constructor
  }

  /**
   * Constructs new ObjectContainer object.
   *
   * @param object the object
   */
  public ObjectContainer(Object object) {
    this.object = object;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Sets object.
   *
   * @param object the object
   */
  public void setObject(Object object) {
    this.object = object;
  }

  /**
   * Returns object.
   *
   * @return object
   */
  public Object getObject() {
    return object;
  }

  /**
   * Tests if this container contains an object.
   *
   * @return <tt>true</tt> if contains an object
   */
  public boolean isEmpty() {
    return (object == null);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object obj) {
    if (object == null) {
      return super.equals(obj);
    }
    return object.equals(obj);
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    if (object == null) {
      return super.hashCode();
    }
    return object.hashCode();
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    if (object == null) {
      return super.toString();
    }
    return object.toString();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 20:25:14 Slobodan
   Initial version

*****************************************************************************^*/
