/*
 * @(#)SyncMonitor.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

/**
 * Synchronization support.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class SyncMonitor {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean signaled;
  private int waiters;
  private final Object lock;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new monitor.
   */
  public SyncMonitor() {
    lock = new Object();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Clears this monitor.
   */
  public void clear() {
    synchronized (lock) {
      signaled = false;
    }
  }

  /**
   * Signals the monitor.
   */
  public void signal() {
    synchronized (lock) {
      if (waiters > 0) {
        signaled = true;
        lock.notifyAll();
      }
    }
  }

  /**
   * Waits for the monitor.
   */
  public void waitFor() {
    synchronized (lock) {
      waiters++;
      try {
        while (!signaled) {
          lock.wait();
        }
      } catch (InterruptedException e) {
        // empty catch block
      }
      waiters--;
    }
  }

  /**
   * Waits for and clears the monitor.
   */
  public void waitForAndClear() {
    synchronized (lock) {
      waiters++;
      try {
        while (!signaled) {
          lock.wait();
        }
        signaled = false;
      } catch (InterruptedException e) {
        // empty catch block
      }
      waiters--;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 20:54:05 Slobodan
   Initial version

*****************************************************************************^*/
