/*
 * @(#)IntQueue.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

/**
 * Simple integer queue.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class IntQueue {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int front;
  private int rear;
  private int size;
  private int queueSize;
  private int capacity;
  private int[] queue;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new IntQueue object.
   */
  public IntQueue() {
    this(11);
  }

  /**
   * Constructs new IntQueue object.
   *
   * @param capacity the initial queue size
   */
  public IntQueue(int capacity) {
    if (capacity <= 0) {
      throw new IllegalArgumentException("Invalid capacity"); //$NON-NLS-1$
    }
    this.capacity = capacity;
    queueSize = capacity + 1;
    front = capacity;
    rear = capacity;
    queue = new int[queueSize];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> is the queue is empty.
   *
   * @return <tt>true</tt> is the queue is empty
   */
  public boolean isEmpty() {
    return (front == rear);
  }

  /**
   * Returns the size of the queue.
   *
   * @return the size of the queue
   */
  public int size() {
    return size;
  }

  /**
   * Returns the capacity of the queue.
   *
   * @return the capacity of the queue
   */
  public int capacity() {
    return capacity;
  }

  /**
   * Clears the queue.
   */
  public void clear() {
    size = 0;
    front = capacity;
    rear = capacity;
  }

  /**
   * Removes the first element in the queue.
   *
   * @return the first element in the queue
   */
  public int remove() {
    if (front == rear) {
      throw new QueueException("Queue is empty"); //$NON-NLS-1$
    }
    size--;
    front = ++front % queueSize;
    return queue[front];
  }

  /**
   * Inserts new integer into this queue.
   *
   * @param i the integer
   */
  public void insert(int i) {
    rear = ++rear % queueSize;
    if (front == rear) {
      int[] oldQueue = queue;
      int rl = capacity - front;
      queueSize = ((3 * queueSize) >> 1) + 1;
      queue = new int[queueSize];
      capacity = queueSize - 1;
      System.arraycopy(oldQueue, 0, queue, 0, rear);
      if (rl > 0) {
        front = capacity - rl;
        System.arraycopy(oldQueue, rear + 1, queue, front + 1, rl);
      } else {
        front = capacity;
      }
    }
    size++;
    queue[rear] = i;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 18:57:54 Slobodan
   Initial version

*****************************************************************************^*/
