/*
 * @(#)ObjectQueue.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

/**
 * Simple object queue.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class ObjectQueue {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int front;
  private int rear;
  private int size;
  private int queueSize;
  private int capacity;
  private Object[] queue;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ObjectQueue object.
   */
  public ObjectQueue() {
    this(11);
  }

  /**
   * Constructs new ObjectQueue object.
   *
   * @param capacity initial queue size
   */
  public ObjectQueue(int capacity) {
    if (capacity <= 0) {
      throw new IllegalArgumentException("Invalid capacity"); //$NON-NLS-1$
    }
    this.capacity = capacity;
    queueSize = capacity + 1;
    front = capacity;
    rear = capacity;
    queue = new Object[queueSize];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> is this queue is empty.
   *
   * @return <tt>true</tt> is this queue is empty
   */
  public boolean isEmpty() {
    return (front == rear);
  }

  /**
   * Returns size of this stack.
   *
   * @return size of this stack
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of this queue.
   *
   * @return capacity of this queue
   */
  public int capacity() {
    return capacity;
  }

  /**
   * Removes all of the objects from this list.
   */
  public void clear() {
    for (int i = size - 1; i >= 0; i--) {
      queue[i] = null;
    }
    size = 0;
    front = capacity;
    rear = capacity;
  }

  /**
   * Returns the first element in this queue.
   *
   * @return the first element in this queue
   */
  public Object remove() {
    if (front == rear) {
      throw new QueueException("Queue is empty"); //$NON-NLS-1$
    }
    size--;
    front = ++front % queueSize;
    Object result = queue[front];
    queue[front] = null;
    return result;
  }

  /**
   * Inserts new object into this queue.
   *
   * @param o the object
   */
  public void insert(Object o) {
    rear = ++rear % queueSize;
    if (front == rear) {
      Object[] oldQueue = queue;
      int rl = capacity - front;
      queueSize = ((3 * queueSize) >> 1) + 1;
      queue = new Object[queueSize];
      capacity = queueSize - 1;
      System.arraycopy(oldQueue, 0, queue, 0, rear);
      if (rl > 0) {
        front = capacity - rl;
        System.arraycopy(oldQueue, rear + 1, queue, front + 1, rl);
      } else {
        front = capacity;
      }
    }
    size++;
    queue[rear] = o;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 19:02:26 Slobodan
   Initial version

*****************************************************************************^*/
