/*
 * @(#)StringQueue.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

/**
 * Simple string queue.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StringQueue {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int front;
  private int rear;
  private int size;
  private int queueSize;
  private int capacity;
  private String[] queue;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StringQueue object.
   *
   * @param capacity initial queue size
   */
  public StringQueue(int capacity) {
    if (capacity <= 0) {
      throw new IllegalArgumentException("Invalid capacity"); //$NON-NLS-1$
    }
    this.capacity = capacity;
    queueSize = capacity + 1;
    front = capacity;
    rear = capacity;
    queue = new String[queueSize];
  }

  /**
   * Constructs new StringQueue object.
   */
  public StringQueue() {
    this(11);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> is the queue is empty.
   *
   * @return <tt>true</tt> is the queue is empty
   */
  public boolean isEmpty() {
    return (front == rear);
  }

  /**
   * Returns size of the queue.
   *
   * @return size of the queue
   */
  public int size() {
    return size;
  }

  /**
   * Returns capacity of the queue.
   *
   * @return capacity of the queue
   */
  public int capacity() {
    return capacity;
  }

  /**
   * Clears the queue.
   */
  public void clear() {
    for (int i = size - 1; i >= 0; i--) {
      queue[i] = null;
    }
    size = 0;
    front = capacity;
    rear = capacity;
  }

  /**
   * Returns the first element in the queue.
   *
   * @return the first element in the queue
   */
  public String remove() {
    if (front == rear) {
      throw new QueueException("Queue is empty"); //$NON-NLS-1$
    }
    size--;
    front = ++front % queueSize;
    String result = queue[front];
    queue[front] = null;
    return result;
  }

  /**
   * Inserts new string into the queue.
   *
   * @param s the string
   */
  public void insert(String s) {
    rear = ++rear % queueSize;
    if (front == rear) {
      String[] oldQueue = queue;
      int rl = capacity - front;
      queueSize = ((3 * queueSize) >> 1) + 1;
      queue = new String[queueSize];
      capacity = queueSize - 1;
      System.arraycopy(oldQueue, 0, queue, 0, rear);
      if (rl > 0) {
        front = capacity - rl;
        System.arraycopy(oldQueue, rear + 1, queue, front + 1, rl);
      } else {
        front = capacity;
      }
    }
    size++;
    queue[rear] = s;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 19:08:42 Slobodan
   Initial version

*****************************************************************************^*/
