/*
 * @(#)IntStack.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

/**
 * Simple integer stack.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class IntStack {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int top;
  private int[] stack;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new IntStack object.
   */
  public IntStack() {
    this(10);
  }

  /**
   * Constructs new IntStack object.
   *
   * @param depth the estimated depth
   *
   * @throws IllegalArgumentException if depth <= 0
   */
  public IntStack(int depth) {
    if (depth <= 0) {
      throw new IllegalArgumentException("Invalid depth"); //$NON-NLS-1$
    }
    stack = new int[depth];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the stack is empty.
   *
   * @return <tt>true</tt> if the stack is empty
   */
  public boolean isEmpty() {
    return (top == 0);
  }

  /**
   * Returns stack size.
   *
   * @return stack size
   */
  public int size() {
    return top;
  }

  /**
   * Pushes an int onto the top of the stack.
   *
   * @param i the int to be pushed
   */
  public void push(int i) {
    int oldSize = stack.length;
    int minSize = top + 1;
    if (minSize > oldSize) {
      int[] oldStack = stack;
      int newSize = ((3 * oldSize) >> 1) + 1;
      if (newSize < minSize) {
        newSize = minSize;
      }
      stack = new int[newSize];
      System.arraycopy(oldStack, 0, stack, 0, top);
    }
    stack[top++] = i;
  }

  /**
   * Removes int from the top of the stack and returns it.
   *
   * @return int from the top of the stack and returns it
   *
   * @throws StackException if stack is empty
   */
  public int pop() {
    if (top > 0) {
      return stack[--top];
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   * Returns int from the top of the stack.
   *
   * @return int from the top of the stack
   *
   * @throws StackException if stack is empty
   */
  public int top() {
    if (top > 0) {
      return stack[top - 1];
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   *  Removes all ints from the stack.
   */
  public void clear() {
    top = 0;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:39:41 Slobodan
   Initial version

*****************************************************************************^*/
