/*
 * @(#)StringStack.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

/**
 * Simple string stack.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StringStack {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int top;
  private String[] stack;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StringStack object.
   */
  public StringStack() {
    this(10);
  }

  /**
   * Constructs new StringStack object.
   *
   * @param depth the estimated depth
   *
   * @throws IllegalArgumentException if depth <= 0
   */
  public StringStack(int depth) {
    if (depth <= 0) {
      throw new IllegalArgumentException("Invalid depth"); //$NON-NLS-1$
    }
    stack = new String[depth];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

 /**
   * Returns <tt>true</tt> if the stack is empty.
   *
   * @return <tt>true</tt> if the stack is empty
   */
  public boolean isEmpty() {
    return (top == 0);
  }

  /**
   * Returns stack size.
   *
   * @return stack siz
   */
  public int size() {
    return top;
  }

  /**
   * Pushes an string onto the top of the stack.
   *
   * @param s the string to be pushed
   */
  public void push(String s) {
    int oldSize = stack.length;
    int minSize = top + 1;
    if (minSize > oldSize) {
      String[] oldStack = stack;
      int newSize = ((3 * oldSize) >> 1) + 1;
      if (newSize < minSize) {
        newSize = minSize;
      }
      stack = new String[newSize];
      System.arraycopy(oldStack, 0, stack, 0, top);
    }
    stack[top++] = s;
  }

  /**
   * Removes string from the top of the stack and returns it.
   *
   * @return string from the top of the stack and returns it
   *
   * @throws StackException if stack is empty
   */
  public String pop() {
    if (top > 0) {
      String s = stack[--top];
      stack[top] = null;
      return s;
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   * Returns string from the top of the stack.
   *
   * @return string from the top of the stack
   *
   * @throws StackException if stack is empty
   */
  public String top() {
    if (top > 0) {
      return stack[top - 1];
    }
    throw new StackException("Stack empty"); //$NON-NLS-1$
  }

  /**
   * Removes all strings from the stack.
   */
  public void clear() {
    String[] s = stack;
    for (int i = top - 1; i >= 0; i--) {
      s[i] = null;
    }
    top = 0;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:44:22 Slobodan
   Initial version

*****************************************************************************^*/
