/*
 * @(#)XMLPreferencesHandler.java  1.0.0 / 2008-05-13
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.preferences;

import java.util.prefs.BackingStoreException;

import net.wrka.xml.simple.XMLAttributeNotExists;
import net.wrka.xml.simple.XMLAttributes;
import net.wrka.xml.simple.sax.DefaultDocumentHandler;
import net.wrka.xml.simple.sax.ParsingException;

/**
 * XML Preferences Handler.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class XMLPreferencesHandler extends DefaultDocumentHandler
                                  implements XMLPreferencesConstants {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final int STATE_INIT = 0;
  private static final int STATE_PREF = 1;
  private static final int STATE_ROOT = 2;
  private static final int STATE_MAP = 3;
  private static final int STATE_NODE = 4;
  private static final int STATE_END = 5;
  private static final int STATE_STOP = 6;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private XMLPreferences cnode;
  private int state;
  boolean systemPreferences;

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  XMLPreferencesHandler(boolean systemPreferences) {
    super();
    cnode = null;
    state = STATE_INIT;
    this.systemPreferences = systemPreferences;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void startElement(String tag,
                           XMLAttributes attrs) throws ParsingException {
    switch (state) {
      case STATE_INIT:
        if (PRERERENCES.equals(tag)) {
          state = STATE_PREF;
        } else {
          throwInvalidDocumentException();
        }
        break;
      case STATE_PREF:
        if (ROOT.equals(tag)) {
          try {
            if (USER.equals(attrs.getString(TYPE))) {
              if (systemPreferences) {
                throwInvalidDocumentException();
              }
              cnode = XMLPreferences.USER_ROOT;
            } else {
              if (!systemPreferences) {
                throwInvalidDocumentException();
              }
              cnode = XMLPreferences.SYSTEM_ROOT;
            }
            cnode.clear();
          } catch (XMLAttributeNotExists e) {
            throwInvalidDocumentException();
          } catch (BackingStoreException bse) {
            throw new ParsingException(// NOPMD
                                  "Could not clear preferences"); //$NON-NLS-1$
          }
          state = STATE_ROOT;
        } else {
          throwInvalidDocumentException();
        }
        break;
      case STATE_ROOT:
        if (MAP.equals(tag)) {
          state = STATE_MAP;
        } else {
          throwInvalidDocumentException();
        }
        break;
      case STATE_MAP:
        if (ENTRY.equals(tag)) {
          try {
            cnode.put(attrs.getString(KEY), attrs.getString(VALUE));
          } catch (XMLAttributeNotExists e) {
            throwInvalidDocumentException();
          }
        } else {
          throwInvalidDocumentException();
        }
        break;
      case STATE_NODE:
        if (NODE.equals(tag)) {
          try {
            cnode = (XMLPreferences)cnode.node(attrs.getString(NAME));
          } catch (XMLAttributeNotExists e) {
            throwInvalidDocumentException();
          }
          state = STATE_ROOT;
        } else {
          throwInvalidDocumentException();
        }
        break;
      default:
        throwInvalidDocumentException();
    }
  }

  /**
   * {@inheritDoc}
   */
  public void endElement(String tag) throws ParsingException {
    switch (state) { // NOPMD
      case STATE_MAP:
        if (MAP.equals(tag)) {
          state = STATE_NODE;
        }
        break;
      case STATE_NODE:
        if (ROOT.equals(tag)) {
          state = STATE_END;
        } else {
          cnode = (XMLPreferences)cnode.parent();
        }
        break;
      case STATE_END:
        if (PRERERENCES.equals(tag)) {
          state = STATE_STOP;
        } else {
          throwInvalidDocumentException();
        }
        break;
      case STATE_STOP:
        throwInvalidDocumentException();
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void throwInvalidDocumentException() throws ParsingException {
    throw new ParsingException("Invalid preferences document"); //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-13 14:11:33 Slobodan
   Initial version

*****************************************************************************^*/
