/*
 * @(#)XHTMLPrintServer.java  1.0.0 / 2008-04-29
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.io.Writer;

/**
 * Simple XHTML Print Server.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XHTMLPrintServer extends XMLPrintServer {

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param writer the writer
   */
  public XHTMLPrintServer(Writer writer) {
    super(writer);
  }

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param outputStream the output stream
   */
  public XHTMLPrintServer(OutputStream outputStream) {
    super(outputStream);
  }

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the character set
   *
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XHTMLPrintServer(OutputStream outputStream,
                          String charsetName) throws
                                                 UnsupportedEncodingException {
    super(outputStream, charsetName);
  }

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param closeStream close the stream
   */
  public XHTMLPrintServer(OutputStream outputStream, boolean closeStream) {
    super(outputStream, closeStream);
  }

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param outputStream the output stream
   * @param charsetName the name of the character set
   * @param closeStream close the stream
   *
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XHTMLPrintServer(OutputStream outputStream, String charsetName,
                          boolean closeStream) throws
                                                 UnsupportedEncodingException {
    super(outputStream, charsetName, closeStream);
  }

  /**
   * Constructs new XHTMLPrintServer object.
   *
   * @param file the output file
   *
   * @throws FileNotFoundException if the file not exists
   */
  public XHTMLPrintServer(File file) throws FileNotFoundException {
    super(file);
  }

  /**
   * Constructs new XMLPrintServer object.
   *
   * @param file the output file
   * @param charsetName the name of the character set
   *
   * @throws FileNotFoundException if the file not exists
   * @throws UnsupportedEncodingException if the encoding is not supported
   */
  public XHTMLPrintServer(File file,
                          String charsetName) throws FileNotFoundException,
                                                  UnsupportedEncodingException {
    super(file, charsetName);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Begins XHTML document.
   *
   * @param doctype the XHTML document type
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginXHTMLDocument(String doctype) throws IOException {
    super.beginDocument(null, doctype, null, null);
  }

  /**
   * Begins Transitional XHTML document.
   *
   * @throws IOException if an I/O error occurred
   */
  public void beginTransitionalDocument() throws IOException {
    beginXHTMLDocument(XMLConstants.TRANSITIONAL_XHTML_HEADER);
  }

  /**
   * Breaks current XHTML line.
   *
   * @throws IOException if an I/O error occurred
   */
  public void breakLine() throws IOException  {
    beginEmptyElement(XMLConstants.STRING_BR);
    endElement();
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void beginDocument(String header, String doctype, String comment,
                            String warning) throws IOException {
    super.beginDocument(null, doctype, comment, warning);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-29 18:02:56 Slobodan
   Initial version

*****************************************************************************^*/
