/*
 * @(#)XMLLoadingException.java  1.0.0 / 2008-04-29
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.IOException;

/**
 * Indicates XML Loading Errors.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLLoadingException extends IOException {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = -1182817284719694679L;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new XMLLoadingException object.
   *
   * @param message the error message
   */
  public XMLLoadingException(String message) {
    super(message);
  }

  /**
   * Constructs new XMLLoadingException object.
   */
  public XMLLoadingException() {
    super("XML loading failed"); //$NON-NLS-1$
  }

  /**
   * Constructs new XMLLoadingException object.
   *
   * @param ioe the IOException
   */
  public XMLLoadingException(IOException ioe) {
    super(ioe.getMessage());
  }

  /**
   * Constructs new XMLLoadingException object.
   *
   * @param message the error message
   * @param line the line number
   * @param position the position
   */
  public XMLLoadingException(String message, int line, int position) {
    super(message + " line: " + line + " column: " + // NON-NLS-2
          position);
  }

  /**
   * Constructs new XMLLoadingException object.
   *
   * @param line the line number
   * @param position the position
   */
  public XMLLoadingException(int line, int position) {
    super("Error at line: " + line + " column: " //$NON-NLS-1$ //$NON-NLS-2$
          + position);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-29 17:55:02 Slobodan
   Initial version

*****************************************************************************^*/
