/*
 * @(#)DocumentHandler.java  1.0.0 / 2008-04-26
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple.sax;

import net.wrka.xml.simple.XMLAttributes;

/**
 * XML Document Handler.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface DocumentHandler {

  /**
   * Notifies handler of the beginning of a document.
   *
   * @throws ParsingException on document error which stops parsing
   */
  public void startDocument() throws ParsingException;

  /**
   * Notifies handler of the end of a document.
   *
   * @throws ParsingException on document error which stops parsing
   */
  public void endDocument() throws ParsingException;

  /**
   * Notifies handler of the beginning of an element.
   *
   * @param tag the element tag name.
   * @param attrs the attributes attached to the element.
   *
   * @throws ParsingException on document error which stops parsing
   */
  public void startElement(String tag,
                           XMLAttributes attrs) throws ParsingException;

  /**
   * Notifies handler of the end of an element.
   *
   * @param tag the element tag name.
   *
   * @throws ParsingException on document error which stops parsing
   */
  public void endElement(String tag) throws ParsingException;

  /**
   * Notifies handler of content of the element.
   *
   * @param data the character data.
   *
   * @throws ParsingException on document error which stops parsing
   */
  public void characters(String data) throws ParsingException;

}

/*^*****************************************************************************
  File History:

 - 2008-04-26 11:52:01 Slobodan
   Initial version

*****************************************************************************^*/
