/*
 * @(#)ParsingException.java  1.0.0 / 2008-04-26
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple.sax;

/**
 * Indicates XML parsing errors.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ParsingException extends Exception {

/*^**************************************************************************^*/
/*- Private Static Final Field.                                              -*/
/*^**************************************************************************^*/

  private static final long serialVersionUID = 7285761730641282304L;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int line;
  private int position;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ParsingException object.
   */
  public ParsingException() {
    this("XML parsing failed"); //$NON-NLS-1$
  }

  /**
   * Constructs new ParsingException object.
   *
   * @param message the error message
   */
  public ParsingException(String message) {
    super(message);
    line = -1;
    position = -1;
  }

  /**
   * Constructs new ParsingException object.
   *
   * @param line the line number
   * @param position the position
   */
  public ParsingException(int line, int position) {
    super("Error at line: " + line + " position: " + //$NON-NLS-1$ $NON-NLS-2$
          position);
    this.line = line;
    this.position = position;
  }

  /**
   * Constructs new ParsingException object.
   *
   * @param message the error message
   * @param line the line number
   * @param position the position
   */
  public ParsingException(String message, int line, int position) {
    super(message + " at line: " + line + //$NON-NLS-1$
          " position: " + position); //$NON-NLS-1$
    this.line = line;
    this.position = position;
  }

  /**
   * Constructs new ParsingException object.
   *
   * @param exception the wrapped exception
   */
  public ParsingException(Exception exception) {
    super(null, exception);
    line = -1;
    position = -1;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the line number of the error.
   *
   * @return the line number of the error or -1 if none is available.
   */
  public int getLine() {
    return line;
  }

  /**
   * Sets line.
   *
   * @param line the line
   */
  public void setLine(int line) {
    this.line = line;
  }

  /**
   * Returns the position of the error.
   *
   * @return the position of the error or -1 if none is available.
   */
  public int getPosition() {
    return position;
  }

  /**
   * Sets position.
   *
   * @param position the position
   */
  public void setPosition(int position) {
    this.position = position;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-26 11:52:33 Slobodan
   Initial version

*****************************************************************************^*/
