/*
 * @(#)MJAsmError.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * Indicates MJ Assembler Error.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJAsmError {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final String errorDecs;
  private final String stringRep;
  private final int errorCode;
  private final int pos;
  private final int len;
  private final int line;
  private final int col;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmError object.
   *
   * @param errorPref error prefix
   * @param errorDecs error description
   * @param errorCode error code
   * @param pos error start position
   * @param len error length
   * @param line error line
   * @param col error column
   */
  public MJAsmError(String errorPref, String errorDecs, int errorCode,
                    int pos, int len, int line, int col) {
    if (errorPref == null) {
      throw new IllegalArgumentException("Null errorPref"); //$NON-NLS-1$
    }
    this.errorCode = errorCode;
    if (errorDecs == null) {
      this.errorDecs = getErrorDecs(errorCode);
    } else {
      this.errorDecs = errorDecs;
    }
    this.pos = pos;
    this.len = len;
    this.line = line;
    this.col = col;
    StringBuffer sb = new StringBuffer(errorPref);
    sb.append(" at line: "); //$NON-NLS-1$
    sb.append(line);
    sb.append(", column: "); //$NON-NLS-1$
    sb.append(col);
    if (this.errorDecs != null) {
      sb.append("; "); //$NON-NLS-1$
      sb.append(this.errorDecs);
    }
    stringRep = sb.toString();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns error code.
   *
   * @return error code
   */
  public int getErrorCode() {
    return errorCode;
  }

  /**
   * Returns error prefix.
   *
   * @return error prefix
   */
  public String getPrefix() {
    return errorDecs;
  }

  /**
   * Returns error description.
   *
   * @return error description
   */
  public String getDescription() {
    return errorDecs;
  }

  /**
   * Returns error position.
   *
   * @return error position
   */
  public int getPosition() {
    return pos;
  }

  /**
   * Returns error length.
   *
   * @return error length
   */
  public int getLength() {
    return len;
  }

  /**
   * Returns error line.
   *
   * @return error line
   */
  public int getLine() {
    return line;
  }

  /**
   * Returns error column.
   *
   * @return error column
   */
  public int getColumn() {
    return col;
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Returns string representation of this error.
   *
   * @return string representation of this error
   */
  public String toString() {
    return stringRep;
  }

/*^**************************************************************************^*/
/*- Protected Method.                                                        -*/
/*^**************************************************************************^*/

  /**
   * Returns error description.
   *
   * @param errorCode the rror code
   *
   * @return  error description
   */
  protected String getErrorDecs(int errorCode) {
    return "Illegal error code"; //$NON-NLS-1$
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 19:59:24 Slobodan
   Initial version

*****************************************************************************^*/
