/*
 * @(#)MJAsmErrorList.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import java.io.PrintStream;
import java.util.ArrayList;

/**
 * MJAsmError List.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmErrorList {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final ArrayList errors;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmErrorList object.
   */
  public MJAsmErrorList() {
    errors = new ArrayList();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns the number of the errors in this list.
   *
   * @return the number of the errors in this list
   */
  public int getErrorCount() {
    return errors.size();
  }

  /**
   * Returns <tt>true</tt> if the list is empty.
   *
   * @return <tt>true</tt> if the list is empty
   */
  public boolean isEmpty() {
    return errors.isEmpty();
  }

  /**
   * Returns error at specified position.
   *
   * @param index the error position
   *
   * @return error at specified position
   */
  public MJAsmError getErrorAt(int index) {
    return (MJAsmError)errors.get(index);
  }

  /**
   * Prints this error list to the specified print stream.
   *
   * @param ps the PrintStream
   */
  public void list(PrintStream ps) {
    if (ps == null) {
      throw new IllegalArgumentException("Null PrintStream"); //$NON-NLS-1$
    }
    ArrayList al = errors;
    int el = al.size();
    for (int i = 0; i < el; i++) {
      ps.println(al.get(i));
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Adds specified error to the list.
   *
   * @param mjase the MJAsmError
   */
  void addError(MJAsmError mjase) {
    if (mjase == null) {
      throw new IllegalArgumentException("Null MJAsmError"); //$NON-NLS-1$
    }
    errors.add(mjase);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 19:58:27 Slobodan
   Initial version

*****************************************************************************^*/
