/*
 * @(#)MJAsmLabel.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * MJ Assembler Label.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmLabel {

/*^**************************************************************************^*/
/*- Fields.                                                                  -*/
/*^**************************************************************************^*/

  private final String name;
  private final int address;

/*^**************************************************************************^*/
/*- Constructor.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmLabel object.
   *
   * @param name the label name
   * @param address the label address
   */
  public MJAsmLabel(String name, int address) {
    if (name == null) {
      throw new IllegalArgumentException("Null label name"); //$NON-NLS-1$
    }
    this.name = name;
    this.address = address;
  }

/*^**************************************************************************^*/
/*- Methods.                                                                 -*/
/*^**************************************************************************^*/

  /**
   * Returns label name.
   *
   * @return label name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns label address.
   *
   * @return label address
   */
  public int getAddress() {
    return address;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object o) {
    if (o == this) { // NOPMD
      return true;
    }
    if (o instanceof MJAsmLabel) {
      MJAsmLabel mjal = (MJAsmLabel)o;
      return (name.equals(mjal.name) && (address == mjal.address));
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    return name.hashCode();
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    return name;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 20:52:04 Slobodan
   Initial version

*****************************************************************************^*/
