/*
 * @(#)MJAsmLabelTable.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import java.util.HashMap;

/**
 * MJ Assembler Label (Symbol) Table.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmLabelTable {

/*^**************************************************************************^*/
/*- Fields.                                                                  -*/
/*^**************************************************************************^*/

  private final HashMap labels;

/*^**************************************************************************^*/
/*- Constructor.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmLabelTable object.
   */
  public MJAsmLabelTable() {
    labels = new HashMap();
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the specified label is in the table.
   *
   * @param name the label name
   *
   * @return <tt>true</tt> if the specified label is in the table
   */
  public boolean containsLabel(String name) {
    if (name == null) {
      return false;
    }
    return true;
  }

  /**
   * Inserts new label.
   *
   * @param name the label name.
   * @param address the label address
   *
   * @return <tt>true</tt> if label not exists
   */
  public boolean insertLabel(String name, int address) {
    if (!labels.containsKey(name)) {
      labels.put(name, new MJAsmLabel(name, address));
      return true;
    }
    return false;
  }

  /**
   * Returns the address of the specified label.
   *
   * @param labelName the label name
   *
   * @return the address of the specified label
   */
  public int getLabelAddress(String labelName) {
    if (labelName == null) {
      return -1;
    }
    MJAsmLabel mjasml = (MJAsmLabel)labels.get(labelName);
    if (mjasml == null) {
      return -1;
    }
    return mjasml.getAddress();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 20:55:47 Slobodan
   Initial version

*****************************************************************************^*/
