/*
 * @(#)MJAsmLineBreaks.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * MJ Assembler Line Breaks Manager.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmLineBreaks {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int[] lineBreaks;
  private int size;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmLineBreaks object.
   *
   */
  public MJAsmLineBreaks() {
    this(100);
  }

  /**
   * Constructs new MJAsmLineBreaks object.
   *
   * @param lineBreaksCount expected number of line breaks
   */
  public MJAsmLineBreaks(int lineBreaksCount) {
    if (lineBreaksCount <= 0) {
      throw new IllegalArgumentException("Illegal maxLineBreaks"); //$NON-NLS-1$
    }
    lineBreaks = new int[lineBreaksCount];
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Clears line breaks.
   */
  void clear() {
    size = 0;
  }

  /**
   * Breaks line at specified position.
   *
   * @param position the position
   */
  void breakLineAt(int position) {
    if ((position < 0) || ((size > 0) && (position < lineBreaks[size - 1]))) {
      throw new IllegalArgumentException("Invalid line break"); //$NON-NLS-1$
    }
    if (size >= lineBreaks.length) {
      int[] oldLineBreaks = lineBreaks;
      lineBreaks = new int[((3 * size) >> 1) + 1];
      System.arraycopy(oldLineBreaks, 0, lineBreaks, 0, size - 1);
    }
    lineBreaks[size++] = position;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns line number for specified position.
   *
   * @param position the position
   *
   * @return line number for specified position
   */
  public int getLine(int position) {
    if (position < 0) {
      return 0;
    }
    int i = 0;
    while ((i < size) && (position > lineBreaks[i])) {
      i++;
    }
    return i;
  }

  /**
   * Returns column number for specified position.
   *
   * @param position the position
   *
   * @return column number for specified position
   */
  public int getColumn(int position) {
    int ln = getLine(position);
    if (ln > 0) {
      return position - lineBreaks[ln - 1];
    }
    return position;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 20:58:09 Slobodan
   Initial version

*****************************************************************************^*/
