/*
 * @(#)MJAsmSyntaxError.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * Z# syntax error.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmSyntaxError extends MJAsmError {

/*^**************************************************************************^*/
/*- Package-Private Static Final Field.                                      -*/
/*^**************************************************************************^*/

  static final String[] ERRORS = {
    /* 0 */  null, //Undefined
    /* 1 */  "Invalid character constant", //$NON-NLS-1$
    /* 2 */  "Missing operand", //$NON-NLS-1$
    /* 3 */  "Illegal struct declaration", //$NON-NLS-1$
    /* 4 */  "Illegal method declaration", //$NON-NLS-1$
    /* 5 */  "Invalid type", //$NON-NLS-1$
    /* 6 */  "Invalid assignment", //$NON-NLS-1$
    /* 7 */  "Invalid method call", //$NON-NLS-1$
    /* 8 */  "Invalid condition", //$NON-NLS-1$
    /* 9 */  "Invalid block", //$NON-NLS-1$
    /* 10 */ "Invalid return statement", //$NON-NLS-1$
    /* 11 */ "Invalid statement", //$NON-NLS-1$
    /* 12 */ "Invalid array index", //$NON-NLS-1$
    /* 13 */ "Missing ;", //$NON-NLS-1$
    /* 14 */ "Empty structure declaration" //$NON-NLS-1$
  };

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmSyntaxError object.
   *
   * @param errorCode error code
   * @param errorDecs error description
   * @param pos error start position
   * @param len error length
   * @param line error line
   * @param col error column
   */
  public MJAsmSyntaxError(int errorCode, String errorDecs, int pos, int len,
                          int line, int col) {
    super("Syntax error", errorDecs, errorCode, pos, len, line, //$NON-NLS-1$
          col);
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected String getErrorDecs(int errorCode) {
    if ((errorCode >= 0) && (errorCode < ERRORS.length)) {
      return ERRORS[errorCode];
    }
    return super.getErrorDecs(errorCode);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 09:34:51 Slobodan
   Initial version

*****************************************************************************^*/
