/*
 * @(#)MJAsmUnresolvedLabelQueue.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * MJ Assembler Queue of Unresolved Label Usages.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmUnresolvedLabelQueue {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private int[] address;
  private String[] label;
  private int[] position;
  private int size;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmUnresolvedLabelQueue object.
   */
  public MJAsmUnresolvedLabelQueue() {
    address = new int[10];
    label = new String[10];
    position = new int[10];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns size of the queue.
   *
   * @return size of the queue
   */
  public int getSize() {
    return size;
  }

  /**
   * Returns <tt>true</tt> if the queue is empty.
   *
   * @return <tt>true</tt> if the queue is empty
   */
  public boolean isEmpty() {
    return (size == 0);
  }

  /**
   * Returns the address at the specified index.
   *
   * @param index the index
   *
   * @return the address at the specified index
   */
  public int getAddressAt(int index) {
    if ((index < 0) || (index >= size)) {
      throw new IllegalArgumentException("Invalid index"); //$NON-NLS-1$
    }
    return address[index];
  }

  /**
   * Returns the label at the specified index.
   *
   * @param index the index
   *
   * @return the label at the specified index
   */
  public String getLabelAt(int index) {
    if ((index < 0) || (index >= size)) {
      throw new IllegalArgumentException("Invalid index"); //$NON-NLS-1$
    }
    return label[index];
  }

  /**
   * Returns the label position at the specified index.
   *
   * @param index the index
   *
   * @return the label position at the specified index
   */
  public int getLabelPositionAt(int index) {
    if ((index < 0) || (index >= size)) {
      throw new IllegalArgumentException("Invalid index"); //$NON-NLS-1$
    }
    return position[index];
  }

  /**
   * Inserts new unresolved label use.
   *
   * @param address the address of label
   * @param name the name of label
   * @param position the position of label
   */
  public void insert(int address, String name, int position) {
    ensureCapacity(size + 1);
    this.address[size] = address;
    label[size] = name;
    this.position[size++] = position;
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void ensureCapacity(int minCapacity) {
    int oldCapacity = address.length;
    if (minCapacity > oldCapacity) {
      int[] oldAddress = address;
      String[] oldLabel = label;
      int[] oldPositions = position;
      int newCapacity = ((3 * oldCapacity) >> 1) + 1;
      if (newCapacity < minCapacity) {
        newCapacity = minCapacity;
      }
      address = new int[newCapacity];
      label = new String[newCapacity];
      position = new int[newCapacity];
      System.arraycopy(oldAddress, 0, address, 0, size);
      System.arraycopy(oldLabel, 0, label, 0, size);
      System.arraycopy(oldPositions, 0, position, 0, size);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:38:53 Slobodan
   Initial version

*****************************************************************************^*/
