/*
 * @(#)MJVMInstructionDescriptor.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

/**
 * MJVM Instruction Descriptor.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMInstructionDescriptor {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Operand type <i>NONE</i>.
   */
  public static final int OP_TYPE_NONE = 0;

  /**
   * Operand type <i>BYTE</i>.
   */
  public static final int OP_TYPE_BYTE = 1;

  /**
   * Operand type <i>SHORT</i>.
   */
  public static final int OP_TYPE_SHORT = 2;

  /**
   * Operand type <i>WORD</i>.
   */
  public static final int OP_TYPE_WORD = 4;

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final int opcode;
  private final String name;
  private final int opcnt;
  private final int optype;
  private final int length;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMInstructionDescriptor object.
   *
   * @param opcode instruction code
   * @param name instruction name
   */
  public MJVMInstructionDescriptor(int opcode, String name) {
    this(opcode, name, 0, OP_TYPE_NONE);
  }

  /**
   * Constructs new MJVMInstructionDescriptor object.
   *
   * @param opcode instruction code
   * @param name instruction name
   * @param opcnt the number of operands
   * @param optype the operand type
   */
  public MJVMInstructionDescriptor(int opcode, String name, int opcnt,
                                   int optype) {
    if (name == null) {
      throw new IllegalArgumentException("Null instruction name"); //$NON-NLS-1$
    }
    this.name = name;
    this.opcode = opcode;
    this.opcnt = opcnt;
    this.optype = optype;
    length = 1 + opcnt * optype;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns instruction name.
   *
   * @return instruction name
   */
  public String getName() {
    return name;
  }

  /**
   * Returns instruction code.
   *
   * @return instruction code
   */
  public int getOpcode() {
    return opcode;
  }

  /**
   * Returns the number of operands.
   *
   * @return the number of operands
   */
  public int getOperandCount() {
    return opcnt;
  }

  /**
   * Returns type of the operands.
   *
   * @return type of the operands
   */
  public int getOperandType() {
    return optype;
  }

  /**
   * Returns instruction length in bytes.
   *
   * @return instruction length in bytes
   */
  public int getLength() {
    return length;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean equals(Object o) {
    if (o == this) { // NOPMD
      return true;
    }
    if (o instanceof MJVMInstructionDescriptor) {
      MJVMInstructionDescriptor mjvmid = (MJVMInstructionDescriptor)o;
      return (opcode == mjvmid.opcode);
    }
    return false;
  }

  /**
   * {@inheritDoc}
   */
  public int hashCode() {
    return name.hashCode();
  }

  /**
   * {@inheritDoc}
   */
  public String toString() {
    return name;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:59:35 Slobodan
   Initial version

*****************************************************************************^*/
