/*
 * @(#)MJVMInstructions.java  1.0.0 / 2008-05-28
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMInstructionCodes;

/**
 * MJVM Instructions.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMInstructions {

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Instruction descriptions.
   */
  private static final MJVMInstructionDescriptor[] INSTRUCTIONS;

  /**
   * The number of instructions.
   */
  private static final int INSTRUCTION_COUNT;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    INSTRUCTIONS = new MJVMInstructionDescriptor[] {
      new MJVMInstructionDescriptor(0, // Illegal/unknown instruction
                                    "???"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.LOAD,
                                    "load", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_BYTE),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.LOAD_0,
                                    "load_0"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.LOAD_1,
                                    "load_1"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.LOAD_2,
                                    "load_2"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.LOAD_3,
                                    "load_3"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.STORE,
                                    "store", 1, //$NON-NLS-1$
                                    MJVMInstructionDescriptor.OP_TYPE_BYTE),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.STORE_0,
                                    "store_0"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.STORE_1,
                                    "store_1"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.STORE_2,
                                    "store_2"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.STORE_3,
                                    "store_3"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.GETSTATIC,
                                    "getstatic", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.PUTSTATIC,
                                    "putstatic", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.GETFIELD,
                                    "getfield", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.PUTFIELD,
                                    "putfield", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_0,
                                    "const_0"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_1,
                                    "const_1"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_2,
                                    "const_2"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_3,
                                    "const_3"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_4,
                                    "const_4"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_5,
                                    "const_5"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST_M1,
                                    "const_m1"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CONST,
                                    "const", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_WORD),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.ADD,
                                    "add"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.SUB,
                                    "sub"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.MUL,
                                    "mul"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.DIV,
                                    "div"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.REM,
                                    "rem"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.NEG,
                                    "neg"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.SHL,
                                    "shl"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.SHR,
                                    "shr"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.INC,
                                    "inc", //$NON-NLS-1$
                                    2, MJVMInstructionDescriptor.OP_TYPE_BYTE),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.NEW,
                                    "new", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.NEWARRAY,
                                    "newarray", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_BYTE),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.ALOAD,
                                    "aload"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.ASTORE,
                                    "astore"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.BALOAD,
                                    "baload"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.BASTORE,
                                    "bastore"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.ARRAYLENGTH,
                                    "arraylength"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.POP,
                                    "pop"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.DUP,
                                    "dup"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.DUP2,
                                    "dup2"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JMP,
                                    "jmp", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JEQ,
                                    "jeq", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JNE,
                                    "jne", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JLT,
                                    "jlt", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JLE,
                                    "jle", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JGT,
                                    "jgt", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.JGE,
                                    "jge", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.CALL,
                                    "call", //$NON-NLS-1$
                                    1, MJVMInstructionDescriptor.OP_TYPE_SHORT),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.RETURN,
                                    "return"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.ENTER,
                                    "enter", //$NON-NLS-1$
                                    2,  MJVMInstructionDescriptor.OP_TYPE_BYTE),
      new MJVMInstructionDescriptor(MJVMInstructionCodes.EXIT,
                                    "exit"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.READ,
                                    "read"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.PRINT,
                                    "print"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.BREAD,
                                    "bread"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.BPRINT,
                                    "bprint"), //$NON-NLS-1$
      new MJVMInstructionDescriptor(MJVMInstructionCodes.TRAP,
                                    "trap", 1, //$NON-NLS-1$
                                    MJVMInstructionDescriptor.OP_TYPE_BYTE)
    };
    INSTRUCTION_COUNT = INSTRUCTIONS.length;
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Class has no instances.
   */
  private MJVMInstructions() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Static Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns MJVMInstructionDescriptor for specified instruction code.
   *
   * @param opcode instruction code
   *
   * @return MJVMInstructionDescriptor for specified instruction code
   */
  public static MJVMInstructionDescriptor getInstruction(int opcode) {
    if ((opcode <= 0) || (opcode >= INSTRUCTION_COUNT)) {
      opcode = 0;
    }
    return INSTRUCTIONS[opcode];
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-28 08:47:48 Slobodan
   Initial version

*****************************************************************************^*/
