/*
 * @(#)MJAsmAbstractOutput.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import net.wrka.util.CommonConstants;

/**
 * Assembler Abstract Output.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class MJAsmAbstractOutput implements MJAsmOutput {

/*^**************************************************************************^*/
/*- Abstract string write methods.                                           -*/
/*^**************************************************************************^*/

  /**
   * Writes the specified text.
   *
   * @param text the text
   */
  protected abstract void write(String text);

  /**
   * Writes new line.
   */
  protected abstract void writeln();

  /**
   * Writes the specified text and new line.
   *
   * @param text the text
   */
  protected abstract void writeln(String text);

/*^**************************************************************************^*/
/*- MJAsmOutput Implementation.                                              -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void print(boolean b) {
    if (b) {
      write(CommonConstants.STRING_TRUE);
    } else {
      write(CommonConstants.STRING_FALSE);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void print(char c) {
    write(String.valueOf(c));
  }

  /**
   * {@inheritDoc}
   */
  public void print(int i) {
    write(String.valueOf(i));
  }

  /**
   * {@inheritDoc}
   */
  public void print(long l) {
    write(String.valueOf(l));
  }

  /**
   * {@inheritDoc}
   */
  public void print(float f) {
    write(String.valueOf(f));
  }

  /**
   * {@inheritDoc}
   */
  public void print(double d) {
    write(String.valueOf(d));
  }

  /**
   * {@inheritDoc}
   */
  public void print(byte[] ba) {
    write(new String(ba));
  }

  /**
   * {@inheritDoc}
   */
  public void print(byte[] ba, int offset, int count) {
    write(new String(ba, offset, count));
  }

  /**
   * {@inheritDoc}
   */
  public void print(char[] ca) {
    write(String.valueOf(ca));
  }

  /**
   * {@inheritDoc}
   */
  public void print(char[] ca, int offset, int count) {
    write(String.valueOf(ca, offset, count));
  }

  /**
   * {@inheritDoc}
   */
  public void print(String s) {
    write(String.valueOf(s));
  }

  /**
   * {@inheritDoc}
   */
  public void print(Object o) {
    write(String.valueOf(o));
  }

  /**
   * {@inheritDoc}
   */
  public void printError(boolean b) {
    setPrintErrors(true);
    if (b) {
      write(CommonConstants.STRING_TRUE);
    } else {
      write(CommonConstants.STRING_FALSE);
    }
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(char c) {
    setPrintErrors(true);
    write(String.valueOf(c));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(int i) {
    setPrintErrors(true);
    write(String.valueOf(i));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(long l) {
    setPrintErrors(true);
    write(String.valueOf(l));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(float f) {
    setPrintErrors(true);
    write(String.valueOf(f));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(double d) {
    setPrintErrors(true);
    write(String.valueOf(d));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(byte[] ba) {
    setPrintErrors(true);
    write(new String(ba));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(byte[] ba, int offset, int count) {
    setPrintErrors(true);
    write(new String(ba, offset, count));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(char[] ca) {
    setPrintErrors(true);
    write(String.valueOf(ca));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(char[] ca, int offset, int count) {
    setPrintErrors(true);
    write(String.valueOf(ca, offset, count));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(String s) {
    setPrintErrors(true);
    write(String.valueOf(s));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printError(Object o) {
    setPrintErrors(true);
    write(String.valueOf(o));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void println() {
    writeln();
  }

  /**
   * {@inheritDoc}
   */
  public void println(boolean b) {
    if (b) {
      writeln(CommonConstants.STRING_TRUE);
    } else {
      writeln(CommonConstants.STRING_FALSE);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void println(char c) {
    writeln(String.valueOf(c));
  }

  /**
   * {@inheritDoc}
   */
  public void println(int i) {
    writeln(String.valueOf(i));
  }

  /**
   * {@inheritDoc}
   */
  public void println(long l) {
    writeln(String.valueOf(l));
  }

  /**
   * {@inheritDoc}
   */
  public void println(float f) {
    writeln(String.valueOf(f));
  }

  /**
   * {@inheritDoc}
   */
  public void println(double d) {
    writeln(String.valueOf(d));
  }

  /**
   * {@inheritDoc}
   */
  public void println(char[] ca) {
    writeln(String.valueOf(ca));
  }

  /**
   * {@inheritDoc}
   */
  public void println(String s) {
    writeln(String.valueOf(s));
  }

  /**
   * {@inheritDoc}
   */
  public void println(Object o) {
    writeln(String.valueOf(o));
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError() {
    setPrintErrors(true);
    writeln();
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(boolean b) {
    setPrintErrors(true);
    if (b) {
      writeln(CommonConstants.STRING_TRUE);
    } else {
      writeln(CommonConstants.STRING_FALSE);
    }
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(char c) {
    setPrintErrors(true);
    writeln(String.valueOf(c));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(int i) {
    setPrintErrors(true);
    writeln(String.valueOf(i));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(long l) {
    setPrintErrors(true);
    writeln(String.valueOf(l));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(float f) {
    setPrintErrors(true);
    writeln(String.valueOf(f));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(double d) {
    setPrintErrors(true);
    writeln(String.valueOf(d));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(char[] ca) {
    setPrintErrors(true);
    writeln(String.valueOf(ca));
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(String s) {
    setPrintErrors(true);
    writeln(String.valueOf(s));
    setPrintErrors(false);
  }

  /**
   * {@inheritDoc}
   */
  public void printlnError(Object o) {
    setPrintErrors(true);
    writeln(String.valueOf(o));
    setPrintErrors(false);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 16:40:08 Slobodan
   Initial version

*****************************************************************************^*/
