/*
 * @(#)MJAsmCodeEditor.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

import javax.swing.border.EmptyBorder;
import javax.swing.text.AbstractDocument;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;

import net.wrka.swing.components.text.code.JWxCodeEditor;

/**
 * MJ Assembler Editor.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJAsmCodeEditor extends JWxCodeEditor {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private String prevTooltip;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmCodeEditor object.
   */
  public MJAsmCodeEditor() {
    super(new MJAsmDocument.Kit());
    setSyntaxHighlight(true);
    setFont(new Font("DialogInput", Font.BOLD, 12)); //$NON-NLS-1$
    setBorder(new EmptyBorder(3, 3, 3, 3));
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Marks error.
   *
   * @param position the error position.
   * @param length the error length
   */
  public void markError(int position, int length) {
    Document doc = getDocument();
    if (doc instanceof MJAsmDocument) {
      ((MJAsmDocument)doc).markError(position, length);
    }
  }

  /**
   * Unhighlihts the editor.
   */
  public void unHighlight() {
    Document doc = getDocument();
    if (doc instanceof MJAsmDocument) {
      ((MJAsmDocument)doc).unHighlight();
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public String getToolTipText(MouseEvent me) {
    String tooltip = null;
    Point p = me.getPoint();
    int ind = viewToModel(p);
    if (ind < 0) {
      return null;
    }
    try {
      Rectangle vr = modelToView(ind);
      p.x = p.x + 3;
      p.y = p.y + 1;
      vr.width = 6;
      if (!vr.contains(p)) {
        return null;
      }
    } catch (BadLocationException ble) {
      return null;
    }
    Document doc = getDocument();
    if (doc instanceof DefaultStyledDocument) {
      DefaultStyledDocument dsd = (DefaultStyledDocument)doc;
      Element elem = dsd.getCharacterElement(ind);
      if (elem instanceof AbstractDocument.AbstractElement) {
        AbstractDocument.AbstractElement ae =
                                         (AbstractDocument.AbstractElement)elem;
        tooltip = ae.getAttribute(StyleConstants.NameAttribute).toString();
        if ("default".equals(tooltip)) { //$NON-NLS-1$
          tooltip = null;
        }
      }
    }
    if ((tooltip != null) && (tooltip.equals(prevTooltip))) {
      tooltip = tooltip.concat(" "); //$NON-NLS-1$
    }
    prevTooltip = tooltip;
    return tooltip;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 16:44:52 Slobodan
   Initial version

*****************************************************************************^*/
