/*
 * @(#)MJAsmDocument.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;
import java.io.StringReader;

import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BoxView;
import javax.swing.text.ComponentView;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.IconView;
import javax.swing.text.LabelView;
import javax.swing.text.ParagraphView;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;

import net.wrka.swing.components.text.code.AbstractCodeDocument;
import net.wrka.swing.components.text.code.AbstractCodeEditorKit;
import yu.ac.bg.etf.ri4pp.mjtools.asm.MJAsmScanner;
import yu.ac.bg.etf.ri4pp.mjtools.asm.MJAsmToken;

/**
 * MJ Assembler Document.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmDocument extends AbstractCodeDocument {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final long serialVersionUID = 5554778556235458307L;

/*^**************************************************************************^*/
/*- Private Static Fields.                                                   -*/
/*^**************************************************************************^*/

  private static final String SN_DEFAULT;
  private static final String SN_LABEL;
  //private static final Color CL_LABEL;
  private static final String SN_ERROR;
  private static final Color CL_ERROR;
  private static final String SN_COMMENT;
  private static final Color CL_COMMENT;
  private static final String SN_INSTRUCTION;
  private static final Color CL_INSTRUCTION;
  private static final String SN_LITERAL;
  private static final Color CL_LITERAL;
  private static final String SN_DIRECTIVE;
  private static final Color CL_DIRECTIVE;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    SN_DEFAULT = "default"; //$NON-NLS-1$
    SN_LABEL = "Label"; //$NON-NLS-1$
    //CL_LABEL = new Color(120, 40, 40); //$NON-NLS-1$
    SN_ERROR = "Error"; //$NON-NLS-1$
    CL_ERROR = new Color(128, 128, 128); //$NON-NLS-1$
    SN_COMMENT = "Comment"; //$NON-NLS-1$
    CL_COMMENT = new Color(0, 122, 0); //$NON-NLS-1$
    SN_INSTRUCTION = "Instruction"; //$NON-NLS-1$
    CL_INSTRUCTION = new Color(0, 0, 128); //$NON-NLS-1$
    SN_LITERAL = "Literal"; //$NON-NLS-1$
    CL_LITERAL = new Color(0, 0, 223); //$NON-NLS-1$
    SN_DIRECTIVE = "Directive"; //$NON-NLS-1$
    CL_DIRECTIVE = new Color(100, 100, 100); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  Style defaultStyle;
  Style errorStyle;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmDocument object.
   */
  public MJAsmDocument() {
    super(true, "DialogInput", 12); //$NON-NLS-1$
    defaultStyle = getStyle(SN_DEFAULT);
    addStyle(SN_LABEL, defaultStyle);
    errorStyle = addStyle(SN_ERROR, defaultStyle);
    StyleConstants.setForeground(errorStyle, CL_ERROR);
    Style literalStyle = addStyle(SN_LITERAL, defaultStyle);
    StyleConstants.setForeground(literalStyle, CL_LITERAL);
    Style commentStyle = addStyle(SN_COMMENT, defaultStyle);
    StyleConstants.setForeground(commentStyle, CL_COMMENT);
    Style instructionStyle = addStyle(SN_INSTRUCTION, defaultStyle);
    StyleConstants.setBold(instructionStyle, true);
    StyleConstants.setForeground(instructionStyle, CL_INSTRUCTION);
    Style directiveStyle = addStyle(SN_DIRECTIVE, defaultStyle);
    StyleConstants.setBold(directiveStyle, true);
    StyleConstants.setForeground(directiveStyle, CL_DIRECTIVE);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Marks error.
   *
   * @param position the position
   * @param length the length
   */
  public void markError(int position, int length) {
    setCharacterAttributes(position, length, errorStyle, true);
  }

  /**
   * Unhighlights the document.
   */
  public void unHighlight() {
    setCharacterAttributes(0, getLength(), getStyle(SN_DEFAULT), true);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void highlightAll() {
    try {
      String text = getText(0, getLength());
      highlighting = true;
      setCharacterAttributes(0, getLength(), getStyle(SN_DEFAULT), true);
      MJAsmScanner mjAsmScanner = new MJAsmScanner(new StringReader(text),
                                                   true);
      MJAsmToken mjat;
      while ((mjat = mjAsmScanner.nextToken()) != null) {
        switch (mjat.getType()) { // NOPMD
          case MJAsmToken.error:
            highlightToken(mjat, getStyle(SN_ERROR));
            break;
          case MJAsmToken.ERROR:
            highlightToken(mjat, getStyle(SN_ERROR));
            break;
          case MJAsmToken.LABEL:
            highlightToken(mjat, getStyle(SN_LABEL));
            break;
          case MJAsmToken.INSTRUCTION:
            highlightToken(mjat, getStyle(SN_INSTRUCTION));
            break;
          case MJAsmToken.DIRECTIVE:
            highlightToken(mjat, getStyle(SN_DIRECTIVE));
            break;
          case MJAsmToken.NUMBER:
            highlightToken(mjat, getStyle(SN_LITERAL));
            break;
          case MJAsmToken.CHAR:
            highlightToken(mjat, getStyle(SN_LITERAL));
            break;
          case MJAsmToken.COMMENT:
            highlightToken(mjat, getStyle(SN_COMMENT));
            break;
          case MJAsmToken.EOF:
            return;
        }
      }
    } catch (Throwable t) { // NOPMD
      // empty method body
    } finally {
      highlighting = false;
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void highlightToken(MJAsmToken mjat, Style s) {
    setCharacterAttributes(mjat.left, mjat.getLength(), s, false);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Editor Kit.
   */
  static final class Kit extends AbstractCodeEditorKit {
    private static final ViewFactory DEFAULT_FACTORY = new MJAViewFactory();
    /**
     * {@inheritDoc}
     */
    public Document createDefaultDocument() {
      return new MJAsmDocument();
    }
    /**
     * {@inheritDoc}
     */
    public ViewFactory getViewFactory() {
      return DEFAULT_FACTORY;
    }
    /**
     * MJA Label View.
     */
    static final class MJALabelView extends LabelView {
      AttributeSet attrs;
      MJAsmDocument mjdoc;
      /**
       * Constructs new MJALabelView object.
       *
       * @param elem the element
       */
      public MJALabelView(Element elem) {
        super(elem);
        attrs = elem.getAttributes();
        mjdoc = (MJAsmDocument)elem.getDocument();
      }
      /**
       * {@inheritDoc}
       */
      public void paint(Graphics g, Shape a) {
        super.paint(g, a);
        if (attrs.containsAttributes(mjdoc.errorStyle)) {
          g.setColor(Color.RED);
          Rectangle r = a.getBounds();
          int x1 = r.x + 1;
          int y1 = r.y + r.height - 2;
          int x2 = r.x + 3;
          int y2 = r.y + r.height;
          int lx = r.x + r.width;
          while (x2 < lx) {
            g.drawLine(x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
            x2 += 2;
            y2 -= 2;
            g.drawLine(x1, y1, x2, y2);
            x1 = x2;
            y1 = y2;
            x2 += 2;
            y2 += 2;
          }
        }
      }
    }
    /**
     * MJA View Factory.
     */
    static final class MJAViewFactory implements ViewFactory {
      /**
       * {@inheritDoc}
       */
      public View create(Element elem) {
        String kind = elem.getName();
        if (kind != null) {
          if (kind.equals(AbstractDocument.ContentElementName)) {
            return new MJALabelView(elem);
          } else if (kind.equals(AbstractDocument.ParagraphElementName)) {
            return new ParagraphView(elem);
          } else if (kind.equals(AbstractDocument.SectionElementName)) {
            return new BoxView(elem, View.Y_AXIS);
          } else if (kind.equals(StyleConstants.ComponentElementName)) {
            return new ComponentView(elem);
          } else if (kind.equals(StyleConstants.IconElementName)) {
            return new IconView(elem);
          }
        }
        return new MJALabelView(elem);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 17:50:03 Slobodan
   Initial version

*****************************************************************************^*/
