/*
 * @(#)MJAsmEditor.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.awt.Window;

import javax.swing.ImageIcon;

import net.wrka.swing.appcontainer.AppContainer;
import net.wrka.swing.appcontainer.GUIApplication;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.preferences.PersistentPreferences;
import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJIOManager;

/**
 * MJ Assembler Editor.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmEditor extends GUIApplication {

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private MJAsmEditor(String[] args) {
    super("MJAsm", "MicroJava Assembler", //$NON-NLS-1$
          "1.0.0 (2008-06-02)", //$NON-NLS-1$
          "ETF (www.etf.bg.ac.yu)", MJResourceManager.INSTANCE, //$NON-NLS-1$
          args);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected Window createMainWindow() {
    return new MJAsmEditorFrame();
  }

  /**
   * {@inheritDoc}
   */
  protected ImageIcon getSplashImage() {
    return guiResourceManager.getImageIcon("MJAsmSplash.png"); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected int getSplashUpdatesCount() {
    return 2;
  }

  /**
   * {@inheritDoc}
   */
  protected void addPreferenceGroups() {
    PersistentPreferences.addPreferenceGroup(MJAsmGeneralPreferences.INSTANCE);
    PersistentPreferences.addPreferenceGroup(PlafPreferences.getInstance());
  }

  /**
   * {@inheritDoc}
   */
  protected void initHelp() {
    SimpleHelp.setHelp("Help", //$NON-NLS-1$
                       guiResourceManager.getHelpURL(
                                            "MJAsmHelpTOC.xml")); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected boolean warmUp() {
    return super.warmUp() && MJIOManager.warmUp();
  }

/*^**************************************************************************^*/
/*- Main Method.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Starts application.
   *
   * @param args the application arguments.
   */
  public static void main(String[] args) {
    AppContainer.run(new MJAsmEditor(args));
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 18:03:13 Slobodan
   Initial version

*****************************************************************************^*/
