/*
 * @(#)MJAsmEditorFrame.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;
import java.io.StringReader;

import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.BadLocationException;

import net.wrka.LibraryConfig;
import net.wrka.io.FileUtils;
import net.wrka.natives.ExternalLauncher;
import net.wrka.swing.actions.AboutAction;
import net.wrka.swing.actions.CommonAction;
import net.wrka.swing.actions.ExitAction;
import net.wrka.swing.actions.FindAction;
import net.wrka.swing.actions.FindNextAction;
import net.wrka.swing.actions.NewAction;
import net.wrka.swing.actions.OpenAction;
import net.wrka.swing.actions.ReplaceAction;
import net.wrka.swing.actions.SaveAction;
import net.wrka.swing.actions.SaveAsAction;
import net.wrka.swing.actions.SelectableAction;
import net.wrka.swing.borders.ThinBevelBorder;
import net.wrka.swing.components.JWxFlatSplitPane;
import net.wrka.swing.components.JWxFrame;
import net.wrka.swing.components.JWxMenu;
import net.wrka.swing.components.JWxStatusBar;
import net.wrka.swing.components.JWxTitlePane;
import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.components.about.AboutBox;
import net.wrka.swing.components.about.HtmlAboutGroup;
import net.wrka.swing.components.about.SystemAboutGroup;
import net.wrka.swing.components.text.LineCounterPanel;
import net.wrka.swing.components.text.TextSearchDialog;
import net.wrka.swing.graphics.EmptyIcon;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.preferences.PersistentPreferences;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.splash.Splash;
import net.wrka.swing.tooltips.ToolTipManager;

import yu.ac.bg.etf.ri4pp.mjtools.asm.MJAssembler;
import yu.ac.bg.etf.ri4pp.mjtools.asm.MJBytecodeBuffer;
import yu.ac.bg.etf.ri4pp.mjtools.bce.MJBytecodePanel;
import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchETF;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchRI4PP;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJByteIterator;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJIOManager;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJBytecodeFormatException;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;

/**
 * MJ Assembler Editor Frame.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmEditorFrame extends JWxFrame {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  static final String DEFAULT_FILE_NAME;
  static final String DEFAULT_SOURCE_FILE_NAME;
  static final String DEFAULT_BYTECODE_FILE_NAME;
  static final String STATUS_EDIT;
  static final String STATUS_LOADING;
  static final String STATUS_STORING;
  static final String STATUS_ENCODING;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    DEFAULT_FILE_NAME = "MJAsmCode"; //$NON-NLS-1$
    DEFAULT_SOURCE_FILE_NAME = DEFAULT_FILE_NAME + ".mja"; //$NON-NLS-1$
    DEFAULT_BYTECODE_FILE_NAME = DEFAULT_FILE_NAME + ".mjb"; //$NON-NLS-1$
    STATUS_EDIT = "Edit"; //$NON-NLS-1$
    STATUS_LOADING = "Loading"; //$NON-NLS-1$
    STATUS_STORING = "Storing"; //$NON-NLS-1$
    STATUS_ENCODING = "Encoding"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Action actionNew;
  private Action actionOpen;
  private Action actionSave;
  private Action actionSaveAs;
  private Action actionSaveBytecode;
  private Action actionPreferences;
  private Action actionExit;
  private Action actionUndo;
  private Action actionRedo;
  private Action actionCut;
  private Action actionCopy;
  private Action actionPaste;
  private Action actionClear;
  private Action actionSelectAll;
  private Action actionFind;
  private Action actionFindNext;
  private Action actionReplace;
  private Action actionViewToolbar;
  private Action actionViewStatusBar;
  private Action actionEncode;
  private Action actionEncodeAndRun;
  private Action actionEncodeAndRunNext;
  private Action actionRun;
  private Action actionRunNext;
  private Action actionHelp;
  private Action actionAbout;
  private final Dimension minDimension;
  private JMenuBar menuBar;
  private final JWxToolBar toolBar;
  private final JWxStatusBar statusBar;
  private final JLabel jlStatusMessage;
  private final JLabel jlCaretIndicator;
  private final JLabel jlState;
  private final TextSearchDialog tsd;
  private final MJAsmErrorPrintStream mjaErr;
  private PrintStream oldErr;
  private final MJAsmStandardPrintStream mjaOut;
  private PrintStream oldOut;
  private final JWxTitlePane messegesPane;
  private final JTabbedPane tabPane;
  private MJVM mjvm;
  private File currentSourceFile;
  private MJEncodeRunner encodeRunner;
  private MJEncodeAndRunRunner encodeAndRunRunner;

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

  ActionViewMessages actionViewMessages;
  MJAsmCodeEditor mjaCodeEditor;
  MJBytecodePanel mjBytecodePanel;
  MJAsmTextPaneOutput messegesOutput;
  MJAsmAboutBox aboutBox;
  MJBytecodeBuffer bytecodeBuffer;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJAsmEditor object.
   */
  public MJAsmEditorFrame() {
    super("MicroJava Assembler", true); //$NON-NLS-1$
    Splash.update("Initializing editor"); //$NON-NLS-1$
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    setIconImage(MJResourceManager.
                   INSTANCE.getImage("MJAsmIcon.png")); //$NON-NLS-1$
    Container cp = getContentPane();
    minDimension = new Dimension(720, 560);
    setSize(minDimension);
    mjaCodeEditor = new MJAsmCodeEditor();
    tsd = new TextSearchDialog(mjaCodeEditor);
    createActions();
    menuBar = createMenuBar();
    setJMenuBar(menuBar);
    toolBar = createToolBar();
    cp.add(toolBar, BorderLayout.NORTH);
    statusBar = new JWxStatusBar(20);
    jlStatusMessage = new JLabel("File not saved"); //$NON-NLS-1$
    jlCaretIndicator = new JLabel("1:1"); //$NON-NLS-1$
    jlCaretIndicator.setHorizontalAlignment(JLabel.CENTER);
    jlState = new JLabel(STATUS_EDIT);
    statusBar.addStatusPanel(jlStatusMessage);
    statusBar.addStatusPanel(jlCaretIndicator, 50);
    statusBar.addStatusPanel(jlState, 130);
    setJWxStatusBar(statusBar);
    mjaCodeEditor.setInsertTabSize(2);
    MJAsmEditorListener listener = new MJAsmEditorListener();
    mjaCodeEditor.addCaretListener(listener);
    String currentSourceFileName = DEFAULT_SOURCE_FILE_NAME;
    tabPane = new JTabbedPane();
    JScrollPane sp = new JScrollPane(new LineCounterPanel(mjaCodeEditor, 1));
    tabPane.addTab(currentSourceFileName, sp);
    mjBytecodePanel = new MJBytecodePanel();
    mjBytecodePanel.setEditable(false);
    String currentBytecodeFileName = DEFAULT_BYTECODE_FILE_NAME;
    tabPane.addTab(currentBytecodeFileName, mjBytecodePanel);
    tabPane.setIconAt(0,
                      MJResourceManager.
                          INSTANCE.getImageIcon("MJAsmTab.png")); //$NON-NLS-1$
    tabPane.setIconAt(1,
                      MJResourceManager.
                        INSTANCE.getImageIcon("MJBytecode.png")); //$NON-NLS-1$
    tabPane.addChangeListener(new MJAsmTabChangeListener());
    messegesOutput = new MJAsmTextPaneOutput();
    mjaErr = new MJAsmErrorPrintStream(messegesOutput);
    mjaOut = new MJAsmStandardPrintStream(messegesOutput);
    messegesPane = messegesOutput.getJWxTitlePane();
    messegesPane.addComponentListener(new MJAsmMessegesPaneListener());
    JPanel tabPaneContainer = new JPanel(new BorderLayout());
    tabPaneContainer.add(tabPane);
    ThinBevelBorder wxtbb = new ThinBevelBorder(ThinBevelBorder.TYPE_LOWERED);
    tabPaneContainer.setBorder(wxtbb);
    JWxFlatSplitPane splitPane =
        new JWxFlatSplitPane(JWxFlatSplitPane.VERTICAL_SPLIT, true,
                             tabPaneContainer, messegesPane);
    splitPane.setBorder(BorderFactory.createEmptyBorder(1, 3, 1, 3));
    splitPane.setResizeWeight(0.85);
    cp.add(splitPane, BorderLayout.CENTER);
    PlafPreferences wxlnfp = PlafPreferences.getInstance();
    wxlnfp.addLnFChangeComponent(tsd);
    ToolTipManager.registerFrame(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return minDimension;
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    performExit();
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  boolean performClose() {
    StringBuffer sb = new StringBuffer("Save changes to \""); //$NON-NLS-1$
    sb.append(tabPane.getTitleAt(0));
    sb.append('"');
    int cdr = JOptionPane.showConfirmDialog(this, sb.toString(), getTitle(),
                                            JOptionPane.YES_NO_CANCEL_OPTION);
    switch (cdr) {
      case JOptionPane.CANCEL_OPTION:
        return false;
      case JOptionPane.YES_OPTION:
        return performSave();
      default:
        return true;
    }
  }

  void performNew() {
    if (mjaCodeEditor.isDocumentModified() && !performClose()) {
      return;
    }
    mjaCodeEditor.clearDocument();
    mjBytecodePanel.clear();
    tabPane.setTitleAt(0, DEFAULT_SOURCE_FILE_NAME);
    tabPane.setTitleAt(1, DEFAULT_BYTECODE_FILE_NAME);
    currentSourceFile = null;
    bytecodeBuffer = null;
    messegesOutput.clear();
    jlStatusMessage.setText("File not saved"); //$NON-NLS-1$
  }

  void performLoadExample(String name) {
    if (mjaCodeEditor.isDocumentModified() && !performClose()) {
      return;
    }
    InputStream is = MJResourceManager.INSTANCE.getExampleStream(name);
    InputStreamReader isr = null;
    try {
      // "Cp1252" example!
      isr = new InputStreamReader(is, "Cp1252"); //$NON-NLS-1$
      mjaCodeEditor.read(isr, null);
      tabPane.setTitleAt(0, name);
      tabPane.setTitleAt(1, FileUtils.changeFileExt(name, "mjb")); //$NON-NLS-1$
      jlStatusMessage.setText("Example loaded"); //$NON-NLS-1$
      currentSourceFile = null;
    } catch (IOException ioe) {
      showError(
           "Could not load example.\n".concat(ioe.getMessage())); //$NON-NLS-1$
    } finally {
      mjaCodeEditor.setDocumentModified(false);
      mjaCodeEditor.resetUndoManager();
      if (isr != null) { // NOPMD
        try {
          isr.close();
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      } else if (is != null) {
        try {
          is.close();
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      }
    }
  }

  void performOpen() {
    if (mjaCodeEditor.isDocumentModified() && !performClose()) {
      return;
    }
    JFileChooser fc = MJIOManager.getOpenMJAssemblerFileChooser();
    if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      if (f.exists() && f.canRead()) {
        currentSourceFile = fc.getSelectedFile();
        loadAssembler();
      }
    }
  }

  boolean performSave() {
    if (currentSourceFile != null) {
      if (mjaCodeEditor.isDocumentModified()) {
        return storeAssembler();
      }
      return true;
    }
    return performSaveAs();
  }

  boolean performSaveAs() {
    JFileChooser fc = MJIOManager.getSaveMJAssemblerFileChooser();
    File sf = new File(fc.getCurrentDirectory(), tabPane.getTitleAt(0));
    fc.setSelectedFile(sf);
    if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
      currentSourceFile = fc.getSelectedFile();
      return storeAssembler();
    }
    return false;
  }

  void performSaveBytecode() {
    if (bytecodeBuffer != null) {
      JFileChooser fc = MJIOManager.getSaveMJBytecodeFileChooser();
      File sf = new File(fc.getCurrentDirectory(), tabPane.getTitleAt(1));
      fc.setSelectedFile(sf);
      if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
        BufferedOutputStream bos = null;
        FileOutputStream fos = null;
        try {
          fos = new FileOutputStream(fc.getSelectedFile());
          bos = new BufferedOutputStream(fos);
          MJByteIterator mjbi = bytecodeBuffer.getByteIterator();
          while (mjbi.hasNext()) {
            bos.write(mjbi.nextByte());
          }
        } catch (IOException ioe) {
          showError(
             "Error storing file!\n".concat(ioe.getMessage())); //$NON-NLS-1$
        } finally {
          if (bos == null) {
            if (fos != null) {
              try {
                fos.close();
              } catch (IOException ioe) { // NOPMD
                // empty catch block
              }
            }
          } else {
            try {
              bos.close();
            } catch (IOException ioe) { // NOPMD
              // empty catch block
            }
          }
        }
      }
    }
  }

  void performExit() {
    if (mjaCodeEditor.isDocumentModified() && !performClose()) {
      return;
    }
    dispose();
  }

  void performFind() {
    tsd.find(mjaCodeEditor.getSelectedText());
  }

  void performFindNext() {
    tsd.findNext();
  }

  void performReplace() {
    tsd.replace(mjaCodeEditor.getSelectedText());
  }

  void performViewToolbar(boolean show) {
    toolBar.setVisible(show);
  }

  void performViewMessages(boolean show) {
    messegesPane.setVisible(show);
  }

  void performViewStatusBar(boolean show) {
    statusBar.setVisible(show);
  }

  void performEncode() {
    if (encodeRunner == null) {
      encodeRunner = new MJEncodeRunner();
    }
    SwingUtilities.invokeLater(encodeRunner);
  }

  void performEncodeAndRun(boolean debug) {
    if (encodeAndRunRunner == null) {
      encodeAndRunRunner = new MJEncodeAndRunRunner();
    }
    encodeAndRunRunner.debug = debug;
    SwingUtilities.invokeLater(encodeAndRunRunner);
  }

  void performRun(boolean debug) {
    if (bytecodeBuffer != null) {
      mjvm = new MJVM();
      try {
        mjvm.load(bytecodeBuffer.getByteIterator());
        if (debug) {
          mjvm.runNext();
        } else {
          mjvm.run();
        }
      } catch (MJBytecodeFormatException mjbcfe) {
        showError(mjbcfe.getMessage());
      }
    }
  }

  void performTabChange() {
    if (tabPane.getSelectedIndex() == 0) {
      mjaCodeEditor.updateEditActions();
      mjaCodeEditor.updateUndoActions();
      tsd.setTextComponent(mjaCodeEditor);
      actionReplace.setEnabled(true);
    } else {
      actionUndo.setEnabled(false);
      actionRedo.setEnabled(false);
      actionCut.setEnabled(false);
      actionCopy.setEnabled(false);
      actionPaste.setEnabled(false);
      actionClear.setEnabled(false);
      actionSelectAll.setEnabled(false);
      actionReplace.setEnabled(false);
      tsd.setTextComponent(mjBytecodePanel.getBytecodeTextComponent());
    }
  }

  void updateCaretIndicator(int dot) {
    try {
      int ln = mjaCodeEditor.getLineOfOffset(dot);
      int ls = mjaCodeEditor.getLineStartOffset(ln);
      StringBuffer sb = new StringBuffer(5);
      sb.append(ln + 1); // NOPMD
      sb.append(':');
      sb.append(dot - ls + 1);
      jlCaretIndicator.setText(sb.toString());
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
  }

  void performEncodeRun() {
    messegesOutput.clear();
    String text = mjaCodeEditor.getText();
    if (text.length() == 0) {
      messegesOutput.println("Document is empty"); //$NON-NLS-1$
      return;
    }
    setStatus(STATUS_ENCODING);
    setMJAsmPrintStreams();
    MJAssembler mjasm = new MJAssembler();
    mjasm.setVerbose(MJAsmGeneralPreferences.INSTANCE.getPreferences().
                      getBoolean(MJAsmGeneralPreferences.MJASM_VERBOSE, false));
    try {
      bytecodeBuffer = mjasm.encodeBytecodeBuffer(new StringReader(text));
      if (mjasm.hasErrors()) {
        bytecodeBuffer = null;
        mjBytecodePanel.clear();
        System.err.println(); // NOPMD
        System.err.println("Error report: "); //$NON-NLS-1$ NOPMD
        mjasm.getErrors().list(System.err);
      } else {
        mjBytecodePanel.loadBytecode(bytecodeBuffer.getByteIterator());
      }
    } catch (Throwable t) { // NOPMD
      if (LibraryConfig.DEBUG) {
        setDefaultPrintStreams();
        Throwable tc = t.getCause();
        if (tc == null) {
          t.printStackTrace(); // NOPMD
        } else {
          tc.printStackTrace(); // NOPMD
        }
      } else {
        messegesOutput.println();
        messegesOutput.printlnError(t.getMessage());
      }
    } finally {
      setDefaultPrintStreams();
      setStatus(STATUS_EDIT);
    }
  }

  void performEncodeAndRunRun(boolean debug) {
    performEncodeRun();
    if (bytecodeBuffer != null) {
      performRun(debug);
    }
  }

  void setMJAsmPrintStreams() {
    oldOut = System.out;
    System.setOut(mjaOut);
    oldErr = System.err;
    System.setErr(mjaErr);
  }

  void setDefaultPrintStreams() {
    System.setErr(oldErr);
    System.setOut(oldOut);
  }

  void setStatus(String status) {
    jlState.setText(status);
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void createActions() {
    actionNew = new ActionNew();
    actionOpen = new ActionOpen();
    actionSave = new ActionSave();
    actionSaveAs = new ActionSaveAs();
    actionSaveBytecode = new ActionSaveBytecode();
    actionExit = new ActionExit();
    actionUndo = mjaCodeEditor.getUndoAction();
    actionRedo = mjaCodeEditor.getRedoAction();
    actionCut = mjaCodeEditor.getCutAction();
    actionCopy = mjaCodeEditor.getCopyAction();
    actionPaste = mjaCodeEditor.getPasteAction();
    actionClear = mjaCodeEditor.getClearAction();
    actionSelectAll = mjaCodeEditor.getSelectAllAction();
    actionFind = new ActionFind();
    actionFindNext = new ActionFindNext();
    actionReplace = new ActionReplace();
    actionViewToolbar = new ActionViewToolbar();
    actionViewMessages = new ActionViewMessages();
    actionViewStatusBar = new ActionViewStatusBar();
    actionEncode = new ActionEncode();
    actionEncodeAndRun = new ActionEncodeAndRun();
    actionEncodeAndRunNext = new ActionEncodeAndRunNext();
    actionRun = new ActionRun();
    actionRunNext = new ActionRunNext();
    actionPreferences = PersistentPreferences.getPreferencesAction();
    actionHelp = SimpleHelp.getHelpAction("MJ Assembler"); //$NON-NLS-1$
    actionAbout = new ActionAbout();
  }

  private JMenuBar createMenuBar() {
    menuBar = new JMenuBar();
    JMenu menu = new JMenu("File"); //$NON-NLS-1$
    menu.setMnemonic('F');
    menu.add(actionNew);
    menu.add(actionOpen);
    menu.add(actionSave);
    menu.add(actionSaveAs).setDisplayedMnemonicIndex(5);
    menu.add(actionSaveBytecode);
    menu.addSeparator();
    menu.add(actionPreferences);
    menu.addSeparator();
    menu.add(actionExit);
    menuBar.add(menu);
    menu = new JMenu("Edit"); //$NON-NLS-1$
    menu.setMnemonic('E');
    menu.add(actionUndo);
    menu.add(actionRedo);
    menu.addSeparator();
    menu.add(actionCut);
    menu.add(actionCopy);
    menu.add(actionPaste);
    menu.add(actionClear);
    menu.addSeparator();
    menu.add(actionSelectAll);
    menuBar.add(menu);
    menu = new JMenu("Search"); //$NON-NLS-1$
    menu.setMnemonic('S');
    menu.add(actionFind);
    menu.add(actionFindNext);
    menu.add(actionReplace);
    menuBar.add(menu);
    menu = new JWxMenu("View"); //$NON-NLS-1$
    menu.setMnemonic('V');
    menu.add(actionViewToolbar);
    menu.add(actionViewMessages);
    menu.add(actionViewStatusBar);
    menuBar.add(menu);
    menu = new JMenu("Assembler"); //$NON-NLS-1$
    menu.setMnemonic('A');
    menu.add(actionEncode);
    menu.addSeparator();
    menu.add(actionEncodeAndRun);
    menu.add(actionEncodeAndRunNext);
    menu.addSeparator();
    menu.add(actionRun);
    menu.add(actionRunNext);
    menuBar.add(menu);
    menu = new JMenu("Help"); //$NON-NLS-1$
    menu.setMnemonic('H');
    menu.add(actionHelp);
    menu.addSeparator();
    JMenu menuExmples = new JMenu("Examples"); //$NON-NLS-1$
    menuExmples.setMnemonic('E');
    menuExmples.setIcon(EmptyIcon.ICON_16);
    menuExmples.add(new ActionLoadExample("HelloWorld.mja")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("ByteEcho.mja")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("IntEcho.mja")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("JumpLabels.mja")); //$NON-NLS-1$
    menu.add(menuExmples);
    if (ExternalLauncher.AVAILABLE) {
      menu.addSeparator();
      menu.add(new MJActionLaunchRI4PP());
      menu.add(new MJActionLaunchETF());
    }
    menu.addSeparator();
    menu.add(actionAbout);
    menuBar.add(menu);
    return menuBar;
  }

  private JWxToolBar createToolBar() {
    JWxToolBar toolBar = new JWxToolBar("Tools", //$NON-NLS-1$
                                        JWxToolBar.HORIZONTAL, false);
    toolBar.addSeparator();
    toolBar.addToolButton(actionNew);
    toolBar.addToolButton(actionOpen);
    toolBar.addToolButton(actionSave);
    toolBar.addSeparator();
    toolBar.addToolButton(actionCut);
    toolBar.addToolButton(actionCopy);
    toolBar.addToolButton(actionPaste);
    toolBar.addToolButton(actionClear);
    toolBar.addSeparator();
    toolBar.addToolButton(actionUndo);
    toolBar.addToolButton(actionRedo);
    toolBar.addSeparator();
    toolBar.addToolButton(actionFind);
    toolBar.addToolButton(actionFindNext);
    toolBar.addToolButton(actionReplace);
    toolBar.addSeparator();
    toolBar.addToolButton(actionEncode);
    toolBar.addToolButton(actionEncodeAndRun);
    toolBar.addToolButton(actionEncodeAndRunNext);
    toolBar.addToolButton(actionRun);
    toolBar.addToolButton(actionRunNext);
    toolBar.addSeparator();
    toolBar.add(Box.createHorizontalGlue());
    toolBar.addSeparator();
    toolBar.addToolButton(actionHelp);
    toolBar.addSeparator();
    return toolBar;
  }

  private void loadAssembler() {
    setStatus(STATUS_LOADING);
    FileReader fr = null;
    try {
      fr = new FileReader(currentSourceFile);
      mjaCodeEditor.read(fr, null);
      String fn = currentSourceFile.getName();
      tabPane.setTitleAt(0, fn);
      tabPane.setTitleAt(1, FileUtils.changeFileExt(fn, "mjb")); //$NON-NLS-1$
      jlStatusMessage.setText(currentSourceFile.getAbsolutePath());
    } catch (IOException ioe) {
      showError("Error loading file!\n".concat(ioe.getMessage())); //$NON-NLS-1$
    } finally {
      mjaCodeEditor.setDocumentModified(false);
      mjaCodeEditor.resetUndoManager();
      if (fr != null) {
        try {
          fr.close();
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      }
      setStatus(STATUS_EDIT);
    }
  }

  private boolean storeAssembler() {
    setStatus(STATUS_STORING);
    FileWriter fw = null;
    try {
      fw = new FileWriter(currentSourceFile);
      mjaCodeEditor.write(fw);
      mjaCodeEditor.setDocumentModified(false);
      jlStatusMessage.setText("File saved"); //$NON-NLS-1$
      String fn = currentSourceFile.getName();
      tabPane.setTitleAt(0, fn);
      tabPane.setTitleAt(1, FileUtils.changeFileExt(fn, "mjb")); //$NON-NLS-1$
      return true;
    } catch (IOException ioe) {
      showError("Error storing file!\n".concat(ioe.getMessage()));
      return false;
    } finally {
      if (fw != null) {
        try {
          fw.close();
        } catch (IOException ioe) { // NOPMD
          // empty catch block
        }
      }
      setStatus(STATUS_EDIT);
    }
  }

  private void showError(String message) {
    showError(getTitle(), message);
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * New Action.
   */
  final class ActionNew extends NewAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performNew();
    }
  }

  /**
   * Open Action.
   */
  final class ActionOpen extends OpenAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performOpen();
    }
  }

  /**
   * Save Action.
   */
  final class ActionSave extends SaveAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSave();
    }
  }

  /**
   * Save As Action.
   */
  final class ActionSaveAs extends SaveAsAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSaveAs();
    }
  }

  /**
   * Save Bytecode Action.
   */
  final class ActionSaveBytecode extends CommonAction {
    /**
     * Constructs new ActionSaveBytecode object.
     */
    public ActionSaveBytecode() {
      super("Save Bytecode...", EmptyIcon.ICON_16, KeyEvent.VK_B, //$NON-NLS-1$
            "Save Bytecode Code..."); //$NON-NLS-1$
      //enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSaveBytecode();
    }
  }

  /**
   * Exit Action.
   */
  final class ActionExit extends ExitAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performExit();
    }
  }

  /**
   * Find Action.
   */
  final class ActionFind extends FindAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performFind();
    }
  }

  /**
   * Find Next Action.
   */
  final class ActionFindNext extends FindNextAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performFindNext();
    }
  }

  /**
   * Replace Action.
   */
  final class ActionReplace extends ReplaceAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performReplace();
    }
  }

  /**
   * View Toolbar Action.
   */
  final class ActionViewToolbar extends SelectableAction {
    /**
     * Constructs new ActionViewToolbar object.
     *
     */
    public ActionViewToolbar() {
      super("Toolbar", null, KeyEvent.VK_T, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_T, InputEvent.CTRL_MASK |
                                                  InputEvent.ALT_MASK),
            "View Toolbar"); //$NON-NLS-1$
      setSelected(true);
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewToolbar(select);
    }
  }

  /**
   * View Messages Action.
   */
  final class ActionViewMessages extends SelectableAction {
    /**
     * Constructs new ActionViewMessages object.
     *
     */
    public ActionViewMessages() {
      super("Messages", null, KeyEvent.VK_M, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK |
                                                    InputEvent.ALT_MASK),
            "View Messages"); //$NON-NLS-1$
      setSelected(true);
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewMessages(select);
    }
  }

  /**
   * View Status Bar Action.
   */
  final class ActionViewStatusBar extends SelectableAction {
    /**
     * Constructs new ActionViewStatusBar object.
     */
    public ActionViewStatusBar() {
      super("Status Bar", null, KeyEvent.VK_S, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK |
                                                  InputEvent.ALT_MASK),
            "View Status Bar"); //$NON-NLS-1$
      setSelected(true);
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewStatusBar(select);
    }
  }

  /**
   * Encode Action.
   */
  final class ActionEncode extends CommonAction {
    /**
     * Constructs new ActionEncode object.
     */
    public ActionEncode() {
      super("Encode", //$NON-NLS-1$
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("Encode.png"), //$NON-NLS-1$
            KeyEvent.VK_E, KeyStroke.getKeyStroke(KeyEvent.VK_F7, 0),
            "Encode"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performEncode();
    }
  }

  /**
   * Run Action.
   */
  final class ActionRun extends CommonAction {
    /**
     * Constructs new ActionRun object.
     */
    public ActionRun() {
      super("Run...",
            MJResourceManager.INSTANCE.
                                   getCachedImageIcon("Run.png"), //$NON-NLS-1$
            KeyEvent.VK_R, KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0),
            "Run"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRun(false);
    }
  }

  /**
   * Encode And Run Action.
   */
  final class ActionEncodeAndRun extends CommonAction {
    /**
     * Constructs new ActionEncodeAndRun object.
     */
    public ActionEncodeAndRun() {
      super("Encode And Run...", //$NON-NLS-1$
            MJResourceManager.
                INSTANCE.getCachedImageIcon("EncodeAndRun.png"), //$NON-NLS-1$
            KeyEvent.VK_A, KeyStroke.getKeyStroke(KeyEvent.VK_F8, 0),
            "Encode And Run"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performEncodeAndRun(false);
    }
  }

  /**
   * Encode And RunNext Action.
   */
  final class ActionEncodeAndRunNext extends CommonAction {
    /**
     * Constructs new ActionEncodeAndRun object.
     */
    public ActionEncodeAndRunNext() {
      super("Encode And Run Next...", //$NON-NLS-1$
            MJResourceManager.
              INSTANCE.getCachedImageIcon("EncodeAndRunNext.png"), //$NON-NLS-1$
            KeyEvent.VK_N, KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0),
            "Encode And RunNext"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performEncodeAndRun(true);
    }
  }

  /**
   * Run Next Action.
   */
  final class ActionRunNext extends CommonAction {
    /**
     * Constructs new ActionRunNext object.
     */
    public ActionRunNext() {
      super("Run Next...", //$NON-NLS-1$
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("RunNext.png"), //$NON-NLS-1$
            KeyEvent.VK_N, KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0),
            "Run Next Instruction"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRun(true);
    }
  }

  /**
   * About Action.
   */
  final class ActionAbout extends AboutAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      if (aboutBox == null) {
        aboutBox = new MJAsmAboutBox(MJAsmEditorFrame.this);
      }
      SwingUtilities.invokeLater(aboutBox);
    }
  }

  /**
   * Load Example Action.
   */
  final class ActionLoadExample extends CommonAction {
    /**
     * Constructs new ActionLoadExample object.
     *
     * @param name the action name
     */
    public ActionLoadExample(String name) {
      super(name);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performLoadExample(((JMenuItem)e.getSource()).getText());
    }
  }

  /**
   * MJAsmAboutBox.
   */
  static final class MJAsmAboutBox implements Runnable {
    private final AboutBox ab;
    /**
     * Constructs new MJAsmAboutBox object.
     *
     * @param mjasmef
     */
    public MJAsmAboutBox(MJAsmEditorFrame mjasmef) {
      ab = new AboutBox(mjasmef,
                        MJResourceManager.INSTANCE.
                            getImageIcon("MJAsmAboutLogo.png")); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Program", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                                getAboutURL("MJAsmAbout.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("License", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                              getAboutURL("License.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Legal", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                                     getAboutURL("Legal.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new SystemAboutGroup());
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      ab.show();
    }
  }

  /**
   * MJAsmEditor Listener.
   */
  final class MJAsmEditorListener implements CaretListener {
    /**
     * {@inheritDoc}
     */
    public void caretUpdate(CaretEvent e) {
      updateCaretIndicator(e.getDot());
    }
  }

  /**
   * MJAsmTabChange Listener.
   */
  final class MJAsmTabChangeListener implements ChangeListener {
    /**
     * {@inheritDoc}
     */
    public void stateChanged(ChangeEvent e) {
      performTabChange();
    }
  }

  /**
   * MJAsmMessegesPane Listener.
   */
  final class MJAsmMessegesPaneListener implements ComponentListener {
    /**
     * {@inheritDoc}
     */
    public void componentResized(ComponentEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void componentMoved(ComponentEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void componentShown(ComponentEvent e) {
      // empty method body
    }
    /**
     * {@inheritDoc}
     */
    public void componentHidden(ComponentEvent e) {
      actionViewMessages.setSelected(false);
    }
  }

  /**
   * MJEncodeRunner.
   */
  final class MJEncodeRunner implements Runnable {
    /**
     * {@inheritDoc}
     */
    public void run() {
      performEncodeRun();
    }
  }

  /**
   * MJEncodeAndRunRunner.
   */
  final class MJEncodeAndRunRunner implements Runnable {
    boolean debug;
    /**
     * {@inheritDoc}
     */
    public void run() {
      performEncodeAndRunRun(debug);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 15:12:07 Slobodan
   Initial version

*****************************************************************************^*/
