/*
 * @(#)MJAsmGeneralPreferences.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import javax.swing.JComponent;

import net.wrka.swing.preferences.GeneralPreferences;
import net.wrka.swing.preferences.PreferencesPanel;
import net.wrka.util.CommonConstants;

import yu.ac.bg.etf.ri4pp.mjtools.vm.gui.MJVMGeneralPreferences;

/**
 * MJ Assembler General Preferences.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmGeneralPreferences extends GeneralPreferences {

/*^**************************************************************************^*/
/*- Public Static Fields.                                                    -*/
/*^**************************************************************************^*/

  /**
   * Singleton instance.
   */
  public static final MJAsmGeneralPreferences INSTANCE;

  /**
   * Assembler verbose property.
   */
  public static final String MJASM_VERBOSE;

  /**
   * Always create new mjvm property.
   */
  public static final String MJASM_ALWAYS_NEW_MJVM;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    INSTANCE = new MJAsmGeneralPreferences();
    MJASM_VERBOSE = "AssemblerVerbose"; //$NON-NLS-1$
    MJASM_ALWAYS_NEW_MJVM = "AlwaysCreateNewMJVM"; //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

 /**
  * Class has no instances.
  */
  private MJAsmGeneralPreferences() {
    super();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Returns default preferences.
   *
   * @return default preferences
   */
  public String[] getDefaultPreferences() {
    String[] prefs = super.getDefaultPreferences(6);
    int si = prefs.length - 6;
    prefs[si] = MJASM_VERBOSE;
    prefs[si + 1] = CommonConstants.STRING_FALSE;
    prefs[si + 2] = MJVMGeneralPreferences.MJVM_CONSOLE_ECHO;
    prefs[si + 3] = CommonConstants.STRING_FALSE;
    prefs[si + 4] = MJASM_ALWAYS_NEW_MJVM;
    prefs[si + 5] = CommonConstants.STRING_TRUE;
    return prefs;
  }

  /**
   * Returns group title.
   *
   * @return group title
   */
  public String getGroupTitle() {
    return "General Preferences"; //$NON-NLS-1$
  }

  /**
   * Creates and returns PreferencesPanel.
   *
   * @param parent the panel's parent.
   *
   * @return PreferencesPanel
   */
  public PreferencesPanel getGroupPanel(JComponent parent) {
    return new MJAsmGeneralPreferencesPanel(preferences);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 14:04:59 Slobodan
   Initial version

*****************************************************************************^*/
