/*
 * @(#)MJAsmOutput.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

/**
 * MJ Assembler Output.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface MJAsmOutput {

  /**
   * Returns <tt>true</tt> if output is enabled.
   *
   * @return <tt>true</tt> if output is enabled
   */
  public boolean isEnabled();

  /**
   * Enables/disabled output.
   *
   * @param enabled if <tt>true</tt> output is enabled
   */
  public void setEnabled(boolean enabled);

  /**
   * Returns <tt>true</tt> if output prints errors.
   *
   * @return <tt>true</tt> if output prints errors
   */
  public boolean isPrintingErrors();

  /**
   * Sets error printing status.
   *
   * @param printErrors if <tt>true</tt> prints errors.
   */
  public void setPrintErrors(boolean printErrors);

  /**
   * Clears output.
   */
  public void clear();

  // Methods that do not terminate lines. --------------------------------------

  /**
   * Prints boolean.
   *
   * @param b the boolean
   */
  public void print(boolean b);

  /**
   * Prints char.
   *
   * @param c the char
   */
  public void print(char c);

  /**
   * Prints int.
   *
   * @param i the int
   */
  public void print(int i);

  /**
   * Prints long.
   *
   * @param l the long
   */
  public void print(long l);

  /**
   * Prints float.
   *
   * @param f the float
   */
  public void print(float f);

  /**
   * Prints double.
   *
   * @param d the double
   */
  public void print(double d);

  /**
   * Prints byte array.
   *
   * @param ba the byte array
   */
  public void print(byte[] ba);

  /**
   * Prints byte array.
   *
   * @param ba the byte array
   * @param offset the array offset
   * @param count the count
   */
  public void print(byte[] ba, int offset, int count);

  /**
   * Prints char array.
   *
   * @param ca the char array
   */
  public void print(char[] ca);

  /**
   * Prints char array.
   *
   * @param ca the char array
   * @param offset the array offset
   * @param count the count
   */
  public void print(char[] ca, int offset, int count);

  /**
   * Prints string.
   *
   * @param s the string
   */
  public void print(String s);

  /**
   * Prints object.
   *
   * @param o the object
   */
  public void print(Object o);

  /**
   * Prints error boolean.
   *
   * @param b the boolean
   */
  public void printError(boolean b);

  /**
   * Prints error char.
   *
   * @param c the char
   */
  public void printError(char c);

  /**
   * Prints error int.
   *
   * @param i the int
   */
  public void printError(int i);

  /**
   * Prints error long.
   *
   * @param l the long
   */
  public void printError(long l);

  /**
   * Prints error float.
   *
   * @param f the float
   */
  public void printError(float f);

  /**
   * Prints error double.
   *
   * @param d the double
   */
  public void printError(double d);

  /**
   * Prints error byte array.
   *
   * @param ba the byte array
   */
  public void printError(byte[] ba);

  /**
   * Prints error byte array.
   *
   * @param ba the byte array
   * @param offset the array offset
   * @param count the count
   */
  public void printError(byte[] ba, int offset, int count);

  /**
   * Prints error char array.
   *
   * @param ca the char array
   */
  public void printError(char[] ca);

  /**
   * Prints error char array.
   *
   * @param ca the char array
   * @param offset the array offset
   * @param count the count
   */
  public void printError(char[] ca, int offset, int count);

  /**
   * Prints error string.
   *
   * @param s the string
   */
  public void printError(String s);

  /**
   * Prints error object.
   *
   * @param o the object
   */
  public void printError(Object o);

  // Methods that do terminate lines. ------------------------------------------

  /**
   * Prints new line.
   */
  public void println();

  /**
   * Prints boolean and new line.
   *
   * @param b the boolean
   */
  public void println(boolean b);

  /**
   * Prints char and new line.
   *
   * @param c the boolean
   */
  public void println(char c);

  /**
   * Prints int and new line.
   *
   * @param i the int
   */
  public void println(int i);

  /**
   * Prints long and new line.
   *
   * @param l the long
   */
  public void println(long l);

  /**
   * Prints float and new line.
   *
   * @param f the float
   */
  public void println(float f);

  /**
   * Prints double and new line.
   *
   * @param d the double
   */
  public void println(double d);

  /**
   * Prints char array and new line.
   *
   * @param ca the double
   */
  public void println(char[] ca);

  /**
   * Prints string and new line.
   *
   * @param s the string
   */
  public void println(String s);

  /**
   * Prints object and new line.
   *
   * @param o the object
   */
  public void println(Object o);

  /**
   * Prints error new line.
   */
  public void printlnError();

  /**
   * Prints error boolean and new line.
   *
   * @param b the boolean
   */
  public void printlnError(boolean b);

  /**
   * Prints error char and new line.
   *
   * @param c the char
   */
  public void printlnError(char c);

  /**
   * Prints error int and new line.
   *
   * @param i the int
   */
  public void printlnError(int i);

  /**
   * Prints error long and new line.
   *
   * @param l the long
   */
  public void printlnError(long l);

  /**
   * Prints error float and new line.
   *
   * @param f the float
   */
  public void printlnError(float f);

  /**
   * Prints error double and new line.
   *
   * @param d the double
   */
  public void printlnError(double d);

  /**
   * Prints error char array and new line.
   *
   * @param ca the char array
   */
  public void printlnError(char[] ca);

  /**
   * Prints error string and new line.
   *
   * @param s the string
   */
  public void printlnError(String s);

  /**
   * Prints error object and new line.
   *
   * @param o the object
   */
  public void printlnError(Object o);

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 16:20:27 Slobodan
   Initial version

*****************************************************************************^*/
