/*
 * @(#)MJAsmStandardOutput.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.io.OutputStream;
import java.io.PrintStream;

/**
 * Assembler Standard Output.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJAsmStandardOutput extends MJAsmAbstractOutput {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean enabled;
  private boolean printErrors;
  private PrintStream ps;
  private PrintStream errorps;
  private PrintStream normalps;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new LCSStandardSimulationOutput object.
   */
  public MJAsmStandardOutput() {
    this(System.out, System.err);
  }

  /**
   * Constructs new LCSStandardSimulationOutput object.
   *
   * @param ps PrintStream normal print stream
   * @param errorps PrintStream error print stream
   */
  public MJAsmStandardOutput(PrintStream ps, PrintStream errorps) {
    super();
    if (ps == null) {
      throw new IllegalArgumentException("Invalid PrintStream"); //$NON-NLS-1$
    }
    if (errorps == null) {
      throw new IllegalArgumentException(
                                    "Invalid ErrorPrintStream"); //$NON-NLS-1$
    }
    enabled = true;
    this.ps = ps;
    normalps = ps;
    this.errorps = errorps;
  }

  /**
   * Constructs new LCSStandardSimulationOutput object.
   *
   * @param os messages OutputStream
   * @param eos error OutputStream
   */
  public MJAsmStandardOutput(OutputStream os, OutputStream eos) {
    this(new PrintStream(os), new PrintStream(eos));
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void write(String text) {
    if (enabled) {
      ps.print(text);
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void writeln() {
    if (enabled) {
      ps.println();
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void writeln(String text) {
    if (enabled) {
      ps.println(text);
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isEnabled() {
    return enabled;
  }

  /**
   * {@inheritDoc}
   */
  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isPrintingErrors() {
    return printErrors;
  }

  /**
   * {@inheritDoc}
   */
  public void setPrintErrors(boolean printErrors) {
    this.printErrors = printErrors;
    if (printErrors) {
      ps = errorps;
    } else {
      ps = normalps;
    }
  }

  /**
   * {@inheritDoc}
   */
  public void clear() {
    if (enabled) {
      ps = normalps;
      ps.println();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 18:07:13 Slobodan
   Initial version

*****************************************************************************^*/
