/*
 * @(#)MJAsmTextPaneOutput.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.asm.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import net.wrka.LibraryInfo;
import net.wrka.swing.components.JWxTitlePane;
import net.wrka.swing.components.TitlePaneFactory;
import net.wrka.swing.graphics.FontUtils;

/**
 * Assembler TextPane Output.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJAsmTextPaneOutput extends MJAsmAbstractOutput {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean enabled;
  private boolean printErrors;
  private final JWxTitlePane outputPanel;
  private final Document document;
  private final SimpleAttributeSet sasNormal;
  private final SimpleAttributeSet sasError;
  private SimpleAttributeSet sasCurrent;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new LCSTextAreaSimulationOutput object.
   */
  public MJAsmTextPaneOutput() {
    super();
    enabled = true;
    sasNormal = new SimpleAttributeSet();
    StyleConstants.setFontFamily(sasNormal,
                                 FontUtils.DEFAULT_FIXED_FONT.getName());
    StyleConstants.setFontSize(sasNormal,
                               FontUtils.DEFAULT_FIXED_FONT.getSize());
    sasError = new SimpleAttributeSet();
    StyleConstants.setForeground(sasError, Color.RED);
    StyleConstants.setFontFamily(sasError,
                                 FontUtils.DEFAULT_FIXED_FONT.getName());
    StyleConstants.setFontSize(sasError,
                               FontUtils.DEFAULT_FIXED_FONT.getSize());
    StyleConstants.setFirstLineIndent(sasError, 6f);
    sasCurrent = sasNormal;
    JTextPane jtpOutput = new JTextPane();
    document = jtpOutput.getDocument();
    jtpOutput.setBackground(new Color(250, 250, 250));
    jtpOutput.setEditable(false);
    Font messagePaneFont = new Font("Verdana", Font.BOLD, 11); //$NON-NLS-1$
    outputPanel = TitlePaneFactory.createClosePane(JWxTitlePane.TYPE_RAISED,
                                                   "Messages", //$NON-NLS-1$
                                                   "Close",
                                                   messagePaneFont,
                                                   Color.WHITE);
    JComponent cp = outputPanel.getContentPanel();
    cp.setPreferredSize(new Dimension(200, 70));
    cp.setMaximumSize(new Dimension(Short.MAX_VALUE, 90));
    cp.setLayout(new BorderLayout());
    cp.add(new JScrollPane(jtpOutput));
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns title pane.
   *
   * @return title pane
   */
  public JWxTitlePane getJWxTitlePane() {
    return outputPanel;
  }

/*^**************************************************************************^*/
/*- MJAsmOutput Implementation.                                              -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean isEnabled() {
    return enabled;
  }

  /**
   * {@inheritDoc}
   */
  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  /**
   * {@inheritDoc}
   */
  public boolean isPrintingErrors() {
    return printErrors;
  }

  /**
   * {@inheritDoc}
   */
  public void setPrintErrors(boolean printErrors) {
    this.printErrors = printErrors;
    synchronized (this) {
      if (printErrors) {
        sasCurrent = sasError;
      } else {
        sasCurrent = sasNormal;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public void clear() {
    try {
      document.remove(0, document.getLength());
    } catch (BadLocationException bae) { // NOPMD
      // empty catch block
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void write(String text) {
    if (enabled) {
      synchronized (this) {
        try {
          document.insertString(document.getLength(), text, sasCurrent);
        } catch (BadLocationException bae) { // NOPMD
          // empty catch block
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void writeln() {
    if (enabled) {
      synchronized (this) {
        try {
          document.insertString(document.getLength(),
                                LibraryInfo.LINE_SEPARATOR, null);
        } catch (BadLocationException bae) { // NOPMD
          // empty catch block
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void writeln(String text) {
    if (enabled) {
      synchronized (this) {
        try {
          document.insertString(document.getLength(), text, sasCurrent);
          document.insertString(document.getLength(),
                                LibraryInfo.LINE_SEPARATOR, null);
        } catch (BadLocationException bae) { // NOPMD
          // empty catch block
        }
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 18:12:00 Slobodan
   Initial version

*****************************************************************************^*/
