/*
 * @(#)MJBytecodeDocument.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.bce;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Element;
import javax.swing.text.PlainDocument;

import net.wrka.util.CommonConstants;

/**
 * MicroJava Bytecode Document.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJBytecodeDocument extends PlainDocument {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean filterBytes;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJBytecodeDocument object.
   */
  public MJBytecodeDocument() {
    super();
    filterBytes = true;
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the document filtering valid mj bytes.
   *
   * @return <tt>true</tt> if the document filtering valid mj bytes
   */
  public boolean isFilteringBytes() {
    return filterBytes;
  }

  /**
   * Sets bytes filtering state.
   *
   * @param filter if <tt>true</tt> the document filters valid bytes.
   */
  public void setFilterBytes(boolean filter) {
    this.filterBytes = filter;
  }

  /**
   * Returns new bytecode segment.
   *
   * @return new bytecode segment
   */
  public MJBytecodeSegment getBytecodeSegment() {
    MJBytecodeSegment seg = new MJBytecodeSegment();
    try {
      Content c = getContent();
      c.getChars(0, c.length(), seg);
      seg.last = seg.count - seg.offset;
    } catch (BadLocationException ble) { // NOPMD
      // empty catch block
    }
    return seg;
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void insertString(int offs, String str,
                           AttributeSet a) throws BadLocationException {
    if (!filterBytes ||
        CommonConstants.STRING_DEFAULT_LINE_SEPARATOR.equals(str)) {
      super.insertString(offs, str, a);
    } else {
      try {
        Element map = getDefaultRootElement();
        map = map.getElement(map.getElementIndex(offs));
        int startOffset = map.getStartOffset();
        int endOffset = map.getEndOffset() - 1;
        String text = getText(startOffset, offs - startOffset);
        StringBuffer sb = new StringBuffer(text);
        sb.append(str);
        if (offs < endOffset) {
          text = getText(offs, endOffset - offs);
          sb.append(text);
        }
        text = sb.toString();
        String[] textln = text.split(
                                 CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
        for (int index = textln.length - 1; index >= 0; index--) {
          int i = Integer.parseInt(textln[index]);
          if ((i < 0) || (i >= 256)) {
            return;
          }
        }
        super.insertString(offs, str, a);
      } catch (NumberFormatException nfe) { // NOPMD
        // empty catch block
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 13:06:13 Slobodan
   Initial version

*****************************************************************************^*/
