/*
 * @(#)MJBytecodeEditorFrame.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.bce;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.util.prefs.Preferences;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import net.wrka.io.FileUtils;
import net.wrka.natives.ExternalLauncher;
import net.wrka.swing.actions.AboutAction;
import net.wrka.swing.actions.ActionGroup;
import net.wrka.swing.actions.CloseAction;
import net.wrka.swing.actions.CommonAction;
import net.wrka.swing.actions.ExitAction;
import net.wrka.swing.actions.FindAction;
import net.wrka.swing.actions.FindNextAction;
import net.wrka.swing.actions.NewAction;
import net.wrka.swing.actions.OpenAction;
import net.wrka.swing.actions.ReplaceAction;
import net.wrka.swing.actions.SaveAction;
import net.wrka.swing.actions.SaveAsAction;
import net.wrka.swing.borders.ThinBevelBorder;
import net.wrka.swing.components.JWxFrame;
import net.wrka.swing.components.JWxStatusBar;
import net.wrka.swing.components.JWxTabbedPane;
import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.components.about.AboutBox;
import net.wrka.swing.components.about.HtmlAboutGroup;
import net.wrka.swing.components.about.SystemAboutGroup;
import net.wrka.swing.components.text.TextSearchDialog;
import net.wrka.swing.graphics.EmptyIcon;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.preferences.PersistentPreferences;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.splash.Splash;
import net.wrka.swing.tooltips.ToolTipManager;
import net.wrka.util.CommonConstants;

import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchETF;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchRI4PP;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJIOManager;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJBytecodeFormatException;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;
import yu.ac.bg.etf.ri4pp.mjtools.vm.gui.MJVMGeneralPreferences;

/**
 * MJ Bytecode Editor Frame.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJBytecodeEditorFrame extends JWxFrame {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static int untitledMJBIndex;

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final byte[] MJ_BYTECODE_HEADER;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    MJ_BYTECODE_HEADER = new byte[] {
      'M', 'J', 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    };
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Action actionNew;
  private Action actionOpen;
  private Action actionSave;
  private Action actionSaveAs;
  private Action actionSaveDisassembled;
  private Action actionPreferences;
  private Action actionExit;
  private Action actionFind;
  private Action actionFindNext;
  private Action actionReplace;
  private Action actionRun;
  private Action actionRunAndSave;
  private Action actionRunNext;
  private Action actionHelp;
  private Action actionAbout;
  private final Dimension minDimension;
  private final JLabel jlStatus;

/*^**************************************************************************^*/
/*- Package-Private Field.                                                   -*/
/*^**************************************************************************^*/

  ActionGroup actionUndo;
  ActionGroup actionRedo;
  ActionGroup actionCut;
  ActionGroup actionCopy;
  ActionGroup actionPaste;
  ActionGroup actionClear;
  ActionGroup actionSelectAll;
  Action actionClose;
  MJBCEAboutBox aboutBox;
  MJBCETabbedPane tabPane;
  TextSearchDialog tsd;
  MJVM mjvm;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJBytecodeEditorFrame object.
   */
  public MJBytecodeEditorFrame() {
    super("MicroJava Bytecode Editor", true); //$NON-NLS-1$
    Splash.update("Initializing editor"); //$NON-NLS-1$
    setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    setIconImage(MJResourceManager.
                             INSTANCE.getImage("MJBCEIcon.png")); //$NON-NLS-1$
    Container cp = getContentPane();
    minDimension = new Dimension(720, 560);
    setSize(minDimension);
    tabPane = new MJBCETabbedPane();
    tsd = new TextSearchDialog(tabPane.getCurrentBytecodeTextComponent());
    createActions();
    JMenuBar menuBar = createMenuBar();
    setJMenuBar(menuBar);
    JWxToolBar toolBar = createToolBar();
    cp.add(toolBar, BorderLayout.NORTH);
    JWxStatusBar statusBar = new JWxStatusBar(20);
    jlStatus = new JLabel("File not saved"); //$NON-NLS-1$
    statusBar.addStatusPanel(jlStatus);
    setJWxStatusBar(statusBar);
    cp.add(tabPane);
    tabPane.addChangeListener(new MJBCETabbedPaneChangeListener());
    PlafPreferences wxlnfp = PlafPreferences.getInstance();
    wxlnfp.addLnFChangeComponent(tsd);
    ToolTipManager.registerFrame(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return minDimension;
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    performExit();
  }

/*^**************************************************************************^*/
/*- Pacakge-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  void performNew() {
    MJBytecodePanel mjbcp = new MJBytecodePanel();
    try {
      mjbcp.loadBytecode(new ByteArrayInputStream(MJ_BYTECODE_HEADER));
    } catch (IOException ioe) {
      showError(ioe.getMessage());
    }
    StringBuffer sb = new StringBuffer("Untitled"); //$NON-NLS-1$
    sb.append(++untitledMJBIndex);
    sb.append(".mjb"); //$NON-NLS-1$
    tabPane.addTab(sb.toString(), mjbcp);
  }

  void performOpen() {
    JFileChooser fc = MJIOManager.getOpenMJBytecodeFileChooser();
    if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      File f = fc.getSelectedFile();
      if (f.exists() && f.canRead()) {
        for (int i = tabPane.getTabCount() - 1; i >= 0; i--) {
          if (f.equals(tabPane.getBytecodePanel(i).getBytecodeFile())) {
            tabPane.setSelectedIndex(i);
            return;
          }
        }
        try {
          MJBytecodePanel mjbcp = new MJBytecodePanel(fc.getSelectedFile());
          tabPane.addTab(f.getName(), mjbcp);
        } catch (IOException ioe) {
          showError("Error opening file: ".//$NON-NLS-1$
                                           concat(ioe.getMessage()));
        }
      }
    }
  }

  boolean performSave() {
    MJBytecodePanel mjbp = tabPane.getCurrentBytecodePanel();
    File bcf = mjbp.getBytecodeFile();
    if (bcf != null) {
      try {
        mjbp.storeBytecode(bcf);
        return true;
      } catch (IOException ioe) {
        showError("Error saving file: ".//$NON-NLS-1$
                                        concat(ioe.getMessage()));
        return false;
      }
    }
    return performSaveAs();
  }

  boolean performSaveAs() {
    JFileChooser fc = MJIOManager.getSaveMJBytecodeFileChooser();
    int ind = tabPane.getSelectedIndex();
    File sf = new File(fc.getCurrentDirectory(), tabPane.getTitleAt(ind));
    fc.setSelectedFile(sf);
    if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
      try {
        MJBytecodePanel mjbp = tabPane.getBytecodePanel(ind);
        File file = fc.getSelectedFile();
        mjbp.storeBytecode(file);
        tabPane.setTitleAt(tabPane.getSelectedIndex(), file.getName());
        return true;
      } catch (IOException ioe) {
        showError("Error saving file: ".//$NON-NLS-1$
                                        concat(ioe.getMessage()));
      }
    }
    return false;
  }

  void performSaveDisassembled() {
    JFileChooser fc = MJIOManager.getSaveMJAssemblerFileChooser();
    int ind = tabPane.getSelectedIndex();
    String dsn = FileUtils.changeFileExt(tabPane.getTitleAt(ind),
                                         "mja"); //$NON-NLS-1$
    File sf = new File(fc.getCurrentDirectory(), dsn);
    fc.setSelectedFile(sf);
    if (fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
      MJBytecodePanel mjbp = tabPane.getCurrentBytecodePanel();
      try {
        mjbp.storeDisassembled(fc.getSelectedFile());
      } catch (IOException ioe) {
        showError("Error saving file: ".//$NON-NLS-1$
                                        concat(ioe.getMessage()));
      }
    }
  }

  void performClose() {
    int index = tabPane.getSelectedIndex();
    if (index >= 0) {
      tabPane.closeTabAt(index);
    }
  }

  void performExit() {
    dispose();
  }

  void performFind() {
    MJBytecodeTextArea mjbctc = tabPane.getCurrentBytecodeTextComponent();
    String seltext = mjbctc.getSelectedText();
    if (seltext != null) {
      int nli = seltext.indexOf(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
      if (nli >= 0) {
        seltext = seltext.substring(0, nli);
      }
    }
    tsd.find(seltext);
  }

  void performFindNext() {
    tsd.findNext();
  }

  void performReplace() {
    MJBytecodeTextArea mjbctc = tabPane.getCurrentBytecodeTextComponent();
    String seltext = mjbctc.getSelectedText();
    if (seltext != null) {
      int nli = seltext.indexOf(CommonConstants.STRING_DEFAULT_LINE_SEPARATOR);
      if (nli >= 0) {
        seltext = seltext.substring(0, nli);
      }
    }
    tsd.replace(seltext);
  }

  void performRun(boolean debug) {
    Preferences p = MJBytecodeGeneralPreferences.INSTANCE.getPreferences();
    if (mjvm == null) {
      mjvm = new MJVM();
    } else {
      if (p.getBoolean(MJBytecodeGeneralPreferences.MJBCE_ALWAYS_NEW_MJVM,
                       true)) {
        mjvm = new MJVM();
      } else {
        mjvm.stop();
      }
    }
    try {
      MJBytecodePanel mjbcp = tabPane.getCurrentBytecodePanel();
      mjvm.setEcho(p.getBoolean(MJVMGeneralPreferences.MJVM_CONSOLE_ECHO,
                                false));
      mjvm.load(mjbcp.getByteIterator());
      if (debug) {
        mjvm.runNext();
      } else {
        mjvm.run();
      }
    } catch (MJBytecodeFormatException mjbcfe) {
      showError(mjbcfe.getMessage());
    }
  }

  void performRunAndSave() {
    if (performSave()) {
      performRun(false);
    }
  }

  void loadExample(String name) {
    try {
      MJBytecodePanel mjbcp = new MJBytecodePanel(MJResourceManager.INSTANCE.
                                                    getExampleStream(name));
      tabPane.addTab(name, mjbcp);
    } catch (IOException ioe) {
      StringBuffer sb = new StringBuffer(80);
      sb.append("Error loading example: "); //$NON-NLS-1$
      sb.append(name);
      sb.append(" - "); //$NON-NLS-1$
      sb.append(ioe.getMessage());
      setStatus(CommonConstants.STRING_EMPTY);
      showError(sb.toString());
    }
  }

  void selectGroupAction(int index) {
    if (index < actionUndo.getActionCount()) {
      actionUndo.selectCurrentAction(index);
      actionRedo.selectCurrentAction(index);
      actionCut.selectCurrentAction(index);
      actionCopy.selectCurrentAction(index);
      actionPaste.selectCurrentAction(index);
      actionClear.selectCurrentAction(index);
      actionSelectAll.selectCurrentAction(index);
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void setStatus(String statusText) {
    jlStatus.setText(statusText);
  }

  private void createActions() {
    actionNew = new ActionNew();
    actionOpen = new ActionOpen();
    actionSave = new ActionSave();
    actionSaveAs = new ActionSaveAs();
    actionSaveDisassembled = new ActionSaveDisassembled();
    actionClose = new ActionClose();
    actionExit = new ActionExit();
    MJBytecodePanel mjbcp = tabPane.getCurrentBytecodePanel();
    MJBytecodeTextArea mjbctc = mjbcp.getBytecodeTextComponent();
    actionUndo = new ActionGroup();
    actionUndo.addAction(mjbctc.getUndoAction());
    actionRedo = new ActionGroup();
    actionRedo.addAction(mjbctc.getRedoAction());
    actionCut = new ActionGroup();
    actionCut.addAction(mjbctc.getCutAction());
    actionCopy = new ActionGroup();
    actionCopy.addAction(mjbctc.getCopyAction());
    actionPaste = new ActionGroup();
    actionPaste.addAction(mjbctc.getPasteAction());
    actionClear = new ActionGroup();
    actionClear.addAction(mjbctc.getClearAction());
    actionSelectAll = new ActionGroup();
    actionSelectAll.addAction(mjbctc.getSelectAllAction());
    selectGroupAction(0);
    actionFind = new ActionFind();
    actionFindNext = new ActionFindNext();
    actionReplace = new ActionReplace();
    actionRun = new ActionRun();
    actionRunAndSave = new ActionRunAndSave();
    actionRunNext = new ActionRunNext();
    actionPreferences = PersistentPreferences.getPreferencesAction();
    actionHelp = SimpleHelp.getHelpAction("MJ Bytecode Editor"); //$NON-NLS-1$
    actionAbout = new ActionAbout();
  }

  private JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    JMenu menu = new JMenu("File"); //$NON-NLS-1$
    menu.setMnemonic('F');
    menu.add(actionNew);
    menu.add(actionOpen);
    menu.add(actionSave);
    menu.add(actionSaveAs).setDisplayedMnemonicIndex(5);
    menu.add(actionSaveDisassembled);
    menu.add(actionClose);
    menu.addSeparator();
    menu.add(actionPreferences);
    menu.addSeparator();
    menu.add(actionExit);
    menuBar.add(menu);
    menu = new JMenu("Edit"); //$NON-NLS-1$
    menu.setMnemonic('E');
    menu.add(actionUndo);
    menu.add(actionRedo);
    menu.addSeparator();
    menu.add(actionCut);
    menu.add(actionCopy);
    menu.add(actionPaste);
    menu.add(actionClear);
    menu.addSeparator();
    menu.add(actionSelectAll);
    menuBar.add(menu);
    menu = new JMenu("Search"); //$NON-NLS-1$
    menu.setMnemonic('S');
    menu.add(actionFind);
    menu.add(actionFindNext);
    menu.add(actionReplace);
    menuBar.add(menu);
    menu = new JMenu("Bytecode"); //$NON-NLS-1$
    menu.setMnemonic('B');
    menu.add(actionRun);
    menu.add(actionRunAndSave);
    menu.add(actionRunNext);
    menuBar.add(menu);
    menu = new JMenu("Help"); //$NON-NLS-1$
    menu.setMnemonic('H');
    menu.add(actionHelp);
    menu.addSeparator();
    JMenu menuExmples = new JMenu("Examples"); //$NON-NLS-1$
    menuExmples.setMnemonic('E');
    menuExmples.setIcon(EmptyIcon.ICON_16);
    menuExmples.add(new ActionLoadExample("HelloWorld.mjb")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("ByteEcho.mjb")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("IntEcho.mjb")); //$NON-NLS-1$
    menu.add(menuExmples);
    if (ExternalLauncher.AVAILABLE) {
      menu.addSeparator();
      menu.add(new MJActionLaunchRI4PP());
      menu.add(new MJActionLaunchETF());
    }
    menu.addSeparator();
    menu.add(actionAbout);
    menuBar.add(menu);
    return menuBar;
  }

  private JWxToolBar createToolBar() {
    JWxToolBar toolBar = new JWxToolBar("Tools", //$NON-NLS-1$
                                        JWxToolBar.HORIZONTAL, false);
    toolBar.addSeparator();
    toolBar.addToolButton(actionNew);
    toolBar.addToolButton(actionOpen);
    toolBar.addToolButton(actionSave);
    toolBar.addSeparator();
    toolBar.addToolButton(actionCut);
    toolBar.addToolButton(actionCopy);
    toolBar.addToolButton(actionPaste);
    toolBar.addToolButton(actionClear);
    toolBar.addSeparator();
    toolBar.addToolButton(actionUndo);
    toolBar.addToolButton(actionRedo);
    toolBar.addSeparator();
    toolBar.addToolButton(actionFind);
    toolBar.addToolButton(actionFindNext);
    toolBar.addToolButton(actionReplace);
    toolBar.addSeparator();
    toolBar.addToolButton(actionRun);
    toolBar.addToolButton(actionRunAndSave);
    toolBar.addToolButton(actionRunNext);
    toolBar.addSeparator();
    toolBar.add(Box.createHorizontalGlue());
    toolBar.addSeparator();
    toolBar.addToolButton(actionHelp);
    toolBar.addSeparator();
    return toolBar;
  }

  private void showError(Object message) {
    showError("Error", message); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * New Action.
   */
  final class ActionNew extends NewAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performNew();
    }
  }

  /**
   * Open Action.
   */
  final class ActionOpen extends OpenAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performOpen();
    }
  }

  /**
   * Save Action.
   */
  final class ActionSave extends SaveAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSave();
    }
  }

  /**
   * Save As Action.
   */
  final class ActionSaveAs extends SaveAsAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSaveAs();
    }
  }

  /**
   * Save Disassembled Action.
   */
  final class ActionSaveDisassembled extends CommonAction {
    /**
     * Constructs new ActionSaveDisassembled object.
     *
     */
    public ActionSaveDisassembled() {
      super("Save Disassembled...", EmptyIcon.ICON_16, //$NON-NLS-1$
            KeyEvent.VK_D, "Save Disassembled"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSaveDisassembled();
    }
  }

  /**
   * Close Action.
   */
  final class ActionClose extends CloseAction {
    /**
     * Constructs new ActionClose object.
     */
    public ActionClose() {
      super();
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performClose();
    }
  }

  /**
   * Exit Action.
   */
  final class ActionExit extends ExitAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performExit();
    }
  }

  /**
   * Find Action.
   */
  final class ActionFind extends FindAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performFind();
    }
  }

  /**
   * Find Next Action.
   */
  final class ActionFindNext extends FindNextAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performFindNext();
    }
  }

  /**
   * Replace Action.
   */
  final class ActionReplace extends ReplaceAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performReplace();
    }
  }

  /**
   * Run Action.
   */
  final class ActionRun extends CommonAction {
    /**
     * Constructs new ActionRun object.
     *
     */
    public ActionRun() {
      super("Run...", //$NON-NLS-1$
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("Run.png"), //$NON-NLS-1$
            KeyEvent.VK_R, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0),
            "Run"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRun(false);
    }
  }

  /**
   * Run And Save Action.
   */
  final class ActionRunAndSave extends CommonAction {
    /**
     * Constructs new ActionRunAndSave object.
     */
    public ActionRunAndSave() {
      super("Run And Save...", //$NON-NLS-1$
            MJResourceManager.
                INSTANCE.getCachedImageIcon("RunAndSave.png"), //$NON-NLS-1$
            KeyEvent.VK_S, KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0),
            "Run And Save"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRunAndSave();
    }
  }

  /**
   * Run Next Action.
   */
  final class ActionRunNext extends CommonAction {
    /**
     * Constructs new ActionRunNext object.
     */
    public ActionRunNext() {
      super("Run Next...", //$NON-NLS-1$
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("RunNext.png"), //$NON-NLS-1$
            KeyEvent.VK_N, KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0),
            "Run Next Instruction"); //$NON-NLS-1$
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRun(true);
    }
  }

  /**
   * Load Example Action.
   */
  final class ActionLoadExample extends CommonAction {
    /**
     * Constructs new ActionLoadExample object.
     *
     * @param name the action name
     */
    public ActionLoadExample(String name) {
      super(name);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      JMenuItem jmi = (JMenuItem)e.getSource();
      loadExample(jmi.getText());
    }
  }

  /**
   * About Action.
   */
  final class ActionAbout extends AboutAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      if (aboutBox == null) {
        aboutBox = new MJBCEAboutBox(MJBytecodeEditorFrame.this);
      }
      SwingUtilities.invokeLater(aboutBox);
    }
  }

  /**
   * MJBCEAboutBox.
   */
  static final class MJBCEAboutBox implements Runnable {
    private final AboutBox ab;
    /**
     * Constructs new MJBCEAboutBox object.
     *
     * @param mjbcef the editor frame
     */
    public MJBCEAboutBox(MJBytecodeEditorFrame mjbcef) {
      ab = new AboutBox(mjbcef,
                        MJResourceManager.INSTANCE.
                          getImageIcon("MJBCEAboutLogo.png")); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Program", //$NON-NLS-1$
            MJResourceManager.
                     INSTANCE.getAboutURL("MJBCEAbout.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("License", //$NON-NLS-1$
             MJResourceManager.
                     INSTANCE.getAboutURL("License.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Legal", //$NON-NLS-1$
             MJResourceManager.
                     INSTANCE.getAboutURL("Legal.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new SystemAboutGroup());
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      ab.show();
    }
  }

  /**
   * MJBCETabbedPane.
   */
  final class MJBCETabbedPane extends JWxTabbedPane {
    /**
     * Constructs new MJBCETabbedPane object.
     *
     */
    public MJBCETabbedPane() {
      super(false);
      setBorder(new ThinBevelBorder(ThinBevelBorder.TYPE_LOWERED));
      //setFocusable(false);
      MJBytecodePanel mjbcp = new MJBytecodePanel();
      try {
        mjbcp.loadBytecode(new ByteArrayInputStream(MJ_BYTECODE_HEADER));
      } catch (IOException ioe) {
        showError("Error", ioe); //$NON-NLS-1$
      }
      StringBuffer sb = new StringBuffer("Untitled"); //$NON-NLS-1$
      sb.append(++untitledMJBIndex);
      sb.append(".mjb"); //$NON-NLS-1$
      MJBytecodeTextArea mjbcta = mjbcp.getBytecodeTextComponent();
      mjbcta.resetUndoManager();
      super.addTab(sb.toString(), mjbcp);
    }
    /**
     * Returns current bytecode panel.
     *
     * @return current bytecode panel
     */
    public MJBytecodePanel getCurrentBytecodePanel() {
      return (MJBytecodePanel)getComponent(getSelectedIndex());
    }
    /**
     * Returns the specified bytecode panel.
     *
     * @param index the index
     *
     * @return the specified bytecode panel
     */
    public MJBytecodePanel getBytecodePanel(int index) {
      return (MJBytecodePanel)getComponent(index);
    }
    /**
     * Returns current bytecode text component.
     *
     * @return current bytecode text component
     */
    public MJBytecodeTextArea getCurrentBytecodeTextComponent() {
      int index = getSelectedIndex();
      MJBytecodePanel mjbcp = (MJBytecodePanel)getComponent(index);
      return mjbcp.getBytecodeTextComponent();
    }
    /**
     * Returns bytecode text component.
     *
     * @param index the index
     *
     * @return  bytecode text component
     */
    public MJBytecodeTextArea getBytecodeTextComponent(int index) {
      MJBytecodePanel mjbcp = (MJBytecodePanel)getComponent(index);
      return mjbcp.getBytecodeTextComponent();
    }
    /**
     * Adds tab.
     *
     * @param title the title
     * @param mjbcp the bytecode panel
     */
    public void addTab(String title, MJBytecodePanel mjbcp) {
      int tc = getTabCount();
      if (tc == 1) {
        setTabCloseable(true);
        actionClose.setEnabled(true);
      }
      super.addTab(title, mjbcp);
      MJBytecodeTextArea mjbctc = mjbcp.getBytecodeTextComponent();
      mjbctc.resetUndoManager();
      actionUndo.addAction(mjbctc.getUndoAction());
      actionRedo.addAction(mjbctc.getRedoAction());
      actionCut.addAction(mjbctc.getCutAction());
      actionCopy.addAction(mjbctc.getCopyAction());
      actionPaste.addAction(mjbctc.getPasteAction());
      actionClear.addAction(mjbctc.getClearAction());
      actionSelectAll.addAction(mjbctc.getSelectAllAction());
      setSelectedIndex(tc);
    }
    /**
     * {@inheritDoc}
     */
    public void insertTab(String title, Icon icon, Component component,
                          String tip, int index) {
      super.insertTab(title,
                      MJResourceManager.INSTANCE.
                        getCachedImageIcon("MJBytecode.png"), //$NON-NLS-1$
                      component, tip, index);
    }
    /**
     * {@inheritDoc}
     */
    public void closeTabAt(int index) {
      if (canCloseTab(index)) {
        MJBytecodePanel mjbp = getBytecodePanel(index);
        if (mjbp.isModified()) {
          StringBuffer sb = new StringBuffer(80);
          sb.append("Save changes to \""); //$NON-NLS-1$
          sb.append(tabPane.getTitleAt(index));
          sb.append('"');
          switch (JOptionPane.showConfirmDialog(this, sb.toString(), // NOPMD
                                                getTitle(), JOptionPane.
                                                        YES_NO_CANCEL_OPTION)) {
            case JOptionPane.CANCEL_OPTION:
              return;
            case JOptionPane.YES_OPTION:
              if (!performSave()) {
                return;
              }
              break;
          }
        }
        removeTabAt(index);
        if (getTabCount() == 1) {
          setTabCloseable(false);
          actionClose.setEnabled(false);
        }
      }
    }
    /**
     * {@inheritDoc}
     */
    public boolean canCloseTab(int tabIndex) {
      return (getTabCount() > 1);
    }
  }

  /**
   * MJBCETabbedPane Change Listener.
   */
  final class MJBCETabbedPaneChangeListener implements ChangeListener {
    /**
     * {@inheritDoc}
     */
    public void stateChanged(ChangeEvent e) {
      int index = tabPane.getSelectedIndex();
      selectGroupAction(index);
      MJBytecodePanel mjbcp = tabPane.getBytecodePanel(index);
      MJBytecodeTextArea mjbctc = mjbcp.getBytecodeTextComponent();
      mjbctc.updateActions();
      tsd.setTextComponent(mjbctc);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 14:00:57 Slobodan
   Initial version

*****************************************************************************^*/
