/*
 * @(#)MJBytecodeScrollListener.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.bce;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JViewport;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * MicroJava Bytecode Scroll Listener.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJBytecodeScrollListener implements ChangeListener, ActionListener,
                                                Runnable {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private boolean enabled;
  private boolean blocked;
  private final JViewport jvLeft;
  private final JViewport jvRight;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJBytecodeScrollListener object.
   *
   * @param jvLeft the left view port
   * @param jvRight the right view port
   */
  public MJBytecodeScrollListener(JViewport jvLeft, JViewport jvRight) {
    this.jvLeft = jvLeft;
    this.jvRight = jvRight;
    jvLeft.addChangeListener(this);
    jvRight.addChangeListener(this);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if this listener is enabled.
   *
   * @return <tt>true</tt> if this listener is enabled
   */
  public boolean isEnabled() {
    return enabled;
  }

  /**
   * Enabled/disable this listener.
   *
   * @param enabled if <tt>true</tt> the listener is enabled
   */
  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  /**
   * Returns <tt>true</tt> if the listener is blocked.
   *
   * @return <tt>true</tt> if the listener is blocked
   */
  public boolean isBlocked() {
    return blocked;
  }

  /**
   * Blocks/releases listener.
   *
   * @param blocked if <tt>true</tt> the listener is blocked
   */
  public void setBlocked(boolean blocked) {
    this.blocked = blocked;
  }

  /**
   * Aligns right view.
   */
  public void alignRight() {
    boolean oldBlocked = blocked;
    blocked = true;
    jvRight.setViewPosition(jvLeft.getViewPosition());
    blocked = oldBlocked;
  }

/*^**************************************************************************^*/
/*- ChangeListener Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void stateChanged(ChangeEvent e) {
    if (enabled && !blocked) {
      JViewport view = (JViewport)e.getSource();
      boolean oldBlocked = blocked;
      blocked = true;
      if (view == jvLeft) { // NOPMD
        jvRight.setViewPosition(jvLeft.getViewPosition());
      } else {
        jvLeft.setViewPosition(jvRight.getViewPosition());
      }
      blocked = oldBlocked;
    }
  }

/*^**************************************************************************^*/
/*- ActionEListener Implementation.                                          -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void actionPerformed(ActionEvent e) {
    JCheckBox jcb = (JCheckBox)e.getSource();
    if (jcb.isSelected()) {
      alignRight();
      setEnabled(true);
    } else {
      setEnabled(false);
    }
  }

/*^**************************************************************************^*/
/*- Runnable Implementation.                                                 -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void run() {
    alignRight();
    if (isEnabled()) {
      setBlocked(false);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 14:08:22 Slobodan
   Initial version

*****************************************************************************^*/
