/*
 * @(#)MJByteArrayByteIterator.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.utils;

import java.util.NoSuchElementException;

/**
 * Byte Array MJByteIterator.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJByteArrayByteIterator implements MJByteIterator {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final byte[] bytes;
  private int index;
  private int last;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJByteArrayByteIterator object.
   *
   * @param bytes the byte array
   *
   * @throws IllegalArgumentException if byte array is <tt>null</tt>
   */
  public MJByteArrayByteIterator(byte[] bytes) {
    this(bytes, 0, bytes.length);
  }

  /**
   * Constructs new MJByteArrayByteIterator object.
   *
   * @param bytes the byte array
   * @param length the length
   *
   * @throws IllegalArgumentException if byte array is <tt>null</tt>
   */
  public MJByteArrayByteIterator(byte[] bytes, int length) {
    this(bytes, 0, length);
  }

  /**
   * Constructs new MJByteArrayByteIterator object.
   *
   * @param bytes the byte array
   * @param offset the start offset
   * @param length the length
   *
   * @throws IllegalArgumentException if byte array is <tt>null</tt>
   */
  public MJByteArrayByteIterator(byte[] bytes, int offset, int length) {
    if (bytes == null) {
      throw new IllegalArgumentException("Null byte array"); //$NON-NLS-1$
    }
    this.bytes = (byte[])bytes.clone();
    index = offset;
    last = offset + length;
  }

/*^**************************************************************************^*/
/*- MJByteIterator Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public boolean hasNext() {
    return index < last;
  }

  /**
   * {@inheritDoc}
   */
  public byte nextByte() {
    if (index < last) {
      return bytes[index++];
    }
    throw new NoSuchElementException();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 19:47:35 Slobodan
   Initial version

*****************************************************************************^*/
