/*
 * @(#)MJInputStreamByteIterator.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.NoSuchElementException;

/**
 * Input Stream MJByteIterator.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJInputStreamByteIterator implements MJByteIterator {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final InputStream is;
  private byte nextByte;
  private boolean hasNext;
  private boolean eos;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJInputStreamByteIterator object.
   *
   * @param is the InputStream
   *
   * @throws IllegalArgumentException if input stream is null
   */
  public MJInputStreamByteIterator(InputStream is) {
    if (is == null) {
      throw new IllegalArgumentException("Null input stream"); //$NON-NLS-1$
    }
    this.is = is;
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Closes input stream.
   */
  public void closeStream() {
    try {
      is.close();
    } catch (IOException e) { // NOPMD
      // empty method body
    }
  }

/*^**************************************************************************^*/
/*- MJByteIterator Implementation.                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the iteration has more elements.
   *
   * @return <tt>true</tt> if the iterator has more elements.
   */
  public boolean hasNext() {
    if (!hasNext && !eos) {
      try {
        nextByte = (byte)is.read();
        if (nextByte < 0) {
          eos = true;
        } else {
          hasNext = true;
        }
      } catch (IOException e) {
        eos = true;
      }
    }
    return hasNext;
  }

  /**
   * Returns the next byte in the iteration.
   *
   * @return the next byte in the iteration
   *
   * @exception NoSuchElementException iteration has no more elements.
   */
  public byte nextByte() {
    if (!eos) {
      if (hasNext) {
        hasNext = false;
        return nextByte;
      }
      try {
        nextByte = (byte)is.read();
        if (nextByte < 0) {
          eos = true;
        } else {
          return nextByte;
        }
      } catch (IOException e) {
        eos = true;
      }
    }
    throw new NoSuchElementException();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 19:48:29 Slobodan
   Initial version

*****************************************************************************^*/
