/*
 * @(#)MJVMBreakPoints.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

import java.util.HashSet;

/**
 * MJVM Break Points.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJVMBreakPoints {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final HashSet breakPoints;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMBreakPoints object.
   */
  public MJVMBreakPoints() {
    breakPoints = new HashSet();
  }

  /**
   * Constructs new MJVMBreakPoints object.
   *
   * @param breakPointsArray the break points array
   */
  public MJVMBreakPoints(int[] breakPointsArray) {
    breakPoints = new HashSet(2 * breakPointsArray.length);
    addBreakPoints(breakPointsArray);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if there is break point at specified address.
   *
   * @param address the break point address
   *
   * @return <tt>true</tt> if there is break point at specified address
   */
  public boolean isBreakPoint(int address) {
    return breakPoints.contains(new Integer(address));
  }

  /**
   * Adds break point.
   *
   * @param address the break point address
   */
  public void addBreakPoint(int address) {
    breakPoints.add(new Integer(address));
  }

  /**
   * Adds break points.
   *
   * @param breakPointsArray the break points array
   */
  public void addBreakPoints(int[] breakPointsArray) {
    if (breakPointsArray != null) {
      for (int i = breakPointsArray.length - 1; i >= 0; i--) {
        breakPoints.add(new Integer(breakPointsArray[i]));
      }
    }
  }

  /**
   * Adds break points.
   *
   * @param breakPoints the break poitns
   */
  public void addBreakPoints(MJVMBreakPoints breakPoints) {
    if (breakPoints != null) {
      this.breakPoints.addAll(breakPoints.breakPoints);
    }
  }

  /**
   * Removes break point.
   *
   * @param address the break point address
   */
  public void removeBreakPoint(int address) {
    breakPoints.remove(new Integer(address));
  }

  /**
   * Removes all break points.
   */
  public void removeAllBreakPoints() {
    breakPoints.clear();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:31:03 Slobodan
   Initial version

*****************************************************************************^*/
