/*
 * @(#)MJVMEStack.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Expression Stack.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMEStack extends MJVMWordMemory {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private int esp;  // Expression stack pointer

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMEStack object.
   *
   * @param size the expression stack size
   *
   * @throws IllegalArgumentException if size <= 0
   */
  MJVMEStack(int size) {
    super(size);
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns expression stack pointer value.
   *
   * @return expression stack pointer value
   */
  public int getESP() {
    return esp;
  }

  /**
   * Sets new value of the expression stack pointer.
   *
   * @param value the new expression stack pointer value
   *
   * @throws IllegalArgumentException if value is out of stack range
   */
  public void setESP(int value) {
    if ((value < 0) || (value >= size)) {
      StringBuffer sb = new StringBuffer("ESP out of range: "); //$NON-NLS-1$
      sb.append(value);
      throw new IllegalArgumentException(sb.toString());
    }
    int oldESP = esp;
    esp = value;
    fireESPChanged(oldESP, esp);
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Resets the stack.
   */
  void reset() {
    super.reset();
    int oldESP = esp;
    esp = 0;
    fireESPChanged(oldESP, esp);
  }

  /**
   * Returns and removes value from the top of the stack.
   *
   * @throws MJVMRuntimeError if the stack is empty
   *
   * @return value from the top of the stack
   */
  int pop() throws MJVMRuntimeError {
    if (esp == 0) {
      throw new MJVMRuntimeError("Expression stack underflow"); //$NON-NLS-1$
    }
    int oldESP = esp;
    esp = esp - 1;
    fireESPChanged(oldESP, esp);
    return memory[esp];
  }

  /**
   * Pushes new value onto the stack.
   *
   * @param value the new value to be pushed
   *
   * @throws MJVMRuntimeError if the stack is full
   */
  void push(int value) throws MJVMRuntimeError {
    if (esp == size) {
      throw new MJVMRuntimeError("Expression stack overflow"); //$NON-NLS-1$
    }
    int oldESP = esp;
    esp = esp + 1;
    fireESPChanged(oldESP, esp);
    memory[oldESP] = value;
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void fireESPChanged(int oldValue, int newValue) {
    MJVMMemoryListener[] listeners = memoryChageListeners.listeners;
    for (int i = memoryChageListeners.size - 1; i >= 0; i--) {
      if (listeners[i] instanceof MJVMEStackListener) {
        ((MJVMEStackListener)listeners[i]).estackESPChanged(oldValue, newValue);
      }
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:37:16 Slobodan
   Initial version

*****************************************************************************^*/
