/*
 * @(#)MJVMInputOutput.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Input/Output Interface.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public interface MJVMInputOutput {

/*^**************************************************************************^*/
/*- Input Interface                                                          -*/
/*^**************************************************************************^*/

  /**
   * Reads byte.
   *
   * @return read byte
   */
  public byte readByte();

  /**
   * Reads integer.
   *
   * @return read integer
   */
  public int readInt();

  /**
   * Aborts current input.
   */
  public void abortInput();

/*^**************************************************************************^*/
/*- Output Interface                                                         -*/
/*^**************************************************************************^*/

  /**
   * Clears the output.
   */
  public void clear();

  /**
   * Prints character.
   *
   * @param c the character
   * @param w the minimum width
   */
  public void print(char c, int w);

  /**
   * Prints integer.
   *
   * @param i the integer
   * @param w the minimum width
   */
  public void print(int i, int w);

  /**
   * Terminates current line.
   */
  public void println();

  /**
   * Prints error integer.
   *
   * @param i the error integer
   */
  public void printError(int i);

  /**
   * Prints error string.
   *
   * @param e the error string
   */
  public void printError(String e);

  /**
   * Prints error string and terminates line.
   *
   * @param e the error string
   */
  public void printlnError(String e);

/*^**************************************************************************^*/
/*- Input/Output Interface                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns echo status.
   *
   * @return echo status
   */
  public boolean getEcho(); // NOPMD

  /**
   * Sets echo status of input.
   *
   * @param echo if <tt>true</tt> input will be echoed
   */
  public void setEcho(boolean echo);

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:43:45 Slobodan
   Initial version

*****************************************************************************^*/
