/*
 * @(#)MJVMLauncher.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

import java.io.File;
import java.io.IOException;

import net.wrka.swing.appcontainer.AppContainer;
import net.wrka.swing.appcontainer.Application;
import net.wrka.swing.preferences.ApplicationPreferences;
import yu.ac.bg.etf.ri4pp.mjtools.vm.gui.MJVMConsoleLauncher;

/**
 * MicroJava Virtual Machine Launcher.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMLauncher extends Application {

/*^**************************************************************************^*/
/*- Private Constructors.                                                    -*/
/*^**************************************************************************^*/

  private MJVMLauncher(String[] args) {
    super("MJVM", "MicroJava Virtual Machine", //$NON-NLS-1$ //$NON-NLS-2$
          "1.0.0 (2008-06-02)", //$NON-NLS-1$
          "ETF (www.etf.bg.ac.yu)", args); //$NON-NLS-1$
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void exec() {
    if (ApplicationPreferences.contains("help") || //$NON-NLS-1$
        ApplicationPreferences.contains("h")) { //$NON-NLS-1$
      printUsage();
    } else if (ApplicationPreferences.contains("version")) { //$NON-NLS-1$
      printInfo();
    } else {
      File bytecodeFile = new File(args[0]);
      if (bytecodeFile.exists() && bytecodeFile.canRead()) {
        MJVM mjvm = new MJVM(new MJVMStandardInputOutput());
        try {
          mjvm.load(bytecodeFile);
          mjvm.run();
          mjvm.waitFor();
        } catch (IOException ioe) {
          System.err.println(ioe.getMessage()); // NOPMD
        } catch (Throwable t) { // NOPMD
          System.err.print("Unexpected error: "); //$NON-NLS-1$ NOPMD
          t.printStackTrace(); // NOPMD
        }
      } else {
        System.err.println("Invalid option or file name"); //$NON-NLS-1$ NOPMD
      }
    }
    shutdown();
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private void printUsage() {
    System.out.println("Usage: mjvm [options] <file>"); //$NON-NLS-1$ NOPMD
    System.out.println("Options: "); //$NON-NLS-1$ NOPMD
    System.out.println(// NOPMD
        "  --help, -h  Display this information"); //$NON-NLS-1$ NOPMD
    System.out.println(// NOPMD
        "  --version   Print program version and exit"); //$NON-NLS-1$ NOPMD
    System.out.println("  --gui       Start GUI"); //$NON-NLS-1$ NOPMD
    System.out.println("File: "); //$NON-NLS-1$ NOPMD
    System.out.println("    MicroJava bytecode file"); //$NON-NLS-1$ NOPMD
  }

/*^**************************************************************************^*/
/*- Main Method.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Starts application.
   *
   * @param args the application argument
   */
  public static void main(String[] args) {
    if ((args.length == 0) || ("--gui".equals(args[0]))) { //$NON-NLS-1$
      AppContainer.run(new MJVMConsoleLauncher(args));
    } else {
      AppContainer.run(new MJVMLauncher(args));
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:53:24 Slobodan
   Initial version

*****************************************************************************^*/
