/*
 * @(#)MJVMMemoryListenerList.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

/**
 * MJVM Memory Listeners List.
 *
 * @version 1.0.0
 * @author Slobodan Vrkacevic (e-mail: slobodan.vrkacevic@gmail.com)
 */
final class MJVMMemoryListenerList {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  MJVMMemoryListener[] listeners;
  int size;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMMemoryListenerList object.
   */
  public MJVMMemoryListenerList() {
    this(5);
  }

  /**
   * Constructs new MJVMMemoryListenerList object.
   *
   * @param capacity initial capacity of the list
   */
  public MJVMMemoryListenerList(int capacity) {
    if (capacity < 0) {
      throw new IllegalArgumentException(
                                    "Negative initial capacity"); //$NON-NLS-1$
    }
    listeners = new MJVMMemoryListener[capacity];
  }

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Adds new MJVMMemoryListener listener to the list.
   *
   * @param mjvmml the MJVMMemoryListener to be added
   *
   * @throws IllegalArgumentException if listener is <tt>null</tt>
   */
  public void addMemoryListener(MJVMMemoryListener mjvmml) {
    if (mjvmml == null) {
      throw new IllegalArgumentException("Null memory listener"); //$NON-NLS-1$
    }
    if (size >= listeners.length) {
      MJVMMemoryListener[] oldListeners = listeners;
      listeners = new MJVMMemoryListener[((3 * size) >> 1) + 1];
      System.arraycopy(oldListeners, 0, listeners, 0, size - 1);
    }
    listeners[size++] = mjvmml;
  }

  /**
   * Removes specified MJVMMemoryListener listener from the list.
   *
   * @param mjvmml the MJVMMemoryListener to be removed
   */
  public void removeMemoryListener(MJVMMemoryListener mjvmml) {
    if (mjvmml != null) {
      int index = listeners.length - 1;
      while (index >= 0) {
        if (listeners[index] == mjvmml) {
          int numMoved = size - index - 1;
          if (numMoved > 0) {
            System.arraycopy(listeners, index + 1, listeners, index, numMoved);
          }
          listeners[--size] = null;
          break;
        }
        index = index - 1;
      }
    }
  }

  /**
   * Removes all listeners.
   */
  public void removeAllListeners() {
    size = 0;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 20:58:37 Slobodan
   Initial version

*****************************************************************************^*/
