/*
 * @(#)MJVMStandardInputOutput.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

/**
 * MJVM standard input/output.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJVMStandardInputOutput implements MJVMInputOutput {

/*^**************************************************************************^*/
/*- Input Interface Implementation                                           -*/
/*^**************************************************************************^*/

  /**
   * Reads byte from the standard input (System.in).
   *
   * @return read byte or -1 if an IO error has occurred
   */
  public byte readByte() {
    try {
      BufferedReader brin = new BufferedReader(new InputStreamReader(
                                                                   System.in));
      String line = brin.readLine();
      if ((line != null) && (line.length() > 0)) {
        return (byte)line.charAt(0);
      }
    } catch (IOException ioe) { // NOPMD
      // empty catch block
    }
    return -1;
  }

  /**
   * Reads byte from the standard input (System.in).
   *
   * @return read integer or Integer.MIN_VALUE if an IO error has occurred
   */
  public int readInt() {
    try {
      BufferedReader brin = new BufferedReader(new InputStreamReader(
                                                                   System.in));
      String line = brin.readLine();
      if ((line != null) && (line.length() > 0)) {
        return Integer.parseInt(line);
      }
    } catch (Throwable t) { // NOPMD
      // empty catch block
    }
    return Integer.MIN_VALUE;
  }

  /**
   * Aborts current input.
   */
  public void abortInput() {
    // empty method body
  }

/*^**************************************************************************^*/
/*- Output Interface Implementation                                          -*/
/*^**************************************************************************^*/

  /**
   * Standard output could not be cleared. Thus this method is empty.
   */
  public void clear() {
    // empty method body
  }

  /**
   * Prints integer on standard output (System.out).
   *
   * @param i the integer
   * @param w the minimum width
   */
  public void print(int i, int w) {
    String intstr = Integer.toString(i);
    w = w - intstr.length();
    if (w > 0) {
      char[] spc = new char[w];
      for (int j = spc.length - 1; j >= 0; j--) {
        spc[j] = ' ';
      }
      System.out.print(spc); // NOPMD
    }
    System.out.print(intstr); // NOPMD
  }

  /**
   * Prints character on standard output (System.out).
   *
   * @param c the character
   * @param w the minimum width
   */
  public void print(char c, int w) {
    if (w > 1) {
      char[] spc = new char[w - 1];
      for (int i = spc.length - 1; i >= 0; i--) {
        spc[i] = ' ';
      }
      System.out.print(spc); // NOPMD
    }
    System.out.print(c); // NOPMD
  }

  /**
   * Terminates line on standard error output (System.err).
   */
  public void println() {
    System.out.println(); // NOPMD
  }

  /**
   * Prints error integer on standard error output (System.err).
   *
   * @param i the error integer
   */
  public void printError(int i) {
    System.err.print(i); // NOPMD
  }

  /**
   * Prints error string on standard error output (System.err).
   *
   * @param e the error string
   */
  public void printError(String e) {
    System.err.print(e); // NOPMD
  }

  /**
   * Prints error string and terminates line
   * on standard error output (System.err).
   *
   * @param e the error string
   */
  public void printlnError(String e) {
    System.err.println(e); // NOPMD
  }

/*^**************************************************************************^*/
/*- Input/Output Interface Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * Sets echo status of input. This method does nothing.
   *
   * @param echo if <tt>true</tt> input will be echoed
   */
  public void setEcho(boolean echo) {
    // empty method body
  }

  /**
   * Standard input/output always echoes input.
   *
   * @return <tt>true</tt>
   */
  public boolean getEcho() { // NOPMD
    return true;
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 21:31:49 Slobodan
   Initial version

*****************************************************************************^*/
