/*
 * @(#)MJVMCodeBrowser.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;

import javax.swing.JScrollPane;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumnModel;

import net.wrka.awt.AwtTools;
import net.wrka.swing.components.JWxDialog;
import net.wrka.swing.preferences.PlafPreferences;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMBreakPoints;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMCodeMemory;

/**
 * MJVM Code Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMCodeBrowser extends JWxDialog {

/*^**************************************************************************^*/
/*- Static Fields.                                                           -*/
/*^**************************************************************************^*/

  static final Dimension MIN_DIMENSION;
  static final Dimension PREF_DIMENSION;
  static final Dimension DIMENSION;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    MIN_DIMENSION = new Dimension(210, 210);
    PREF_DIMENSION = new Dimension(258, 700);
    int maxheight = AwtTools.getMaximumWindowHeight();
    int height = PREF_DIMENSION.height;
    if (maxheight < height) {
      height = maxheight;
    }
    DIMENSION = new Dimension(PREF_DIMENSION.width, height);
  }

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final MJVMCodeMemory codeMemory;
  private final MJVMConsoleFrame mjvmcf;
  private final MJVMCodeTable codeTable;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMCodeBrowser object.
   *
   * @param mjvmcf the parent frame
   */
  public MJVMCodeBrowser(MJVMConsoleFrame mjvmcf) {
    super(mjvmcf, "MJVM Code Memory", false); //$NON-NLS-1$
    this.mjvmcf = mjvmcf;
    setDialogPosition(POSITION_CUSTOM);
    setBounds(calcBrowserRectangle());
    MJVM mjvm = mjvmcf.getMJVM();
    Container cp = getContentPane();
    codeMemory = mjvm.getCodeMemory();
    MJVMBreakPoints bps = mjvm.getBreakPoints();
    MJVMCodeTableModel codeTableModel = new MJVMCodeTableModel(codeMemory, bps);
    codeTable = new MJVMCodeTable(codeTableModel);
    cp.add(new JScrollPane(codeTable));
    update();
    PlafPreferences.getInstance().addLnFChangeComponent(this);
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates browser.
   */
  public void update() {
    int pc = codeMemory.getPC();
    if (!codeTable.isCellSelected(pc, 0)) {
      codeTable.changeSelection(pc, 0, false, false);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return (Dimension)MIN_DIMENSION.clone();
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    mjvmcf.hideCodeBrowser();
  }

  /**
   * {@inheritDoc}
   */
  protected void onDeactivated(WindowEvent we) {
    TableCellEditor cellEditor = codeTable.getCellEditor();
    if (cellEditor != null) {
      cellEditor.cancelCellEditing();
    }
  }

/*^**************************************************************************^*/
/*- Private Methods.                                                         -*/
/*^**************************************************************************^*/

  private Rectangle calcBrowserRectangle() {
    int x = mjvmcf.getX() - DIMENSION.width - 1;
    int y = mjvmcf.getY() + (mjvmcf.getHeight() - DIMENSION.height) / 2 - 10;
    if (y < 2) {
      y = 2;
    }
    int w = DIMENSION.width;
    int h = DIMENSION.height;
    int mwh = AwtTools.getMaximumWindowHeight();
    if (y + h > mwh) {
      y = (mwh - h) / 2;
    }
    return new Rectangle(x, y, w, h);
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * MJVM Code Table.
   */
  static class MJVMCodeTable extends MJVMMemoryTable {
    /**
     * Constructs new MJVMCodeTable object.
     *
     * @param mjvmctm
     */
    public MJVMCodeTable(MJVMCodeTableModel mjvmctm) {
      super(mjvmctm, false);
      TableColumnModel colmod = getColumnModel();
      colmod.getColumn(0).setCellRenderer(
                              new MJVMPointerCellRenderer("PC")); //$NON-NLS-1$
      setDefaultRenderer(Boolean.class, new MJVMBreakPointsRenderer());
      setDefaultEditor(Boolean.class, new MJVMBreakPointsEditor());
      colmod.getColumn(0).setPreferredWidth(40);
      colmod.getColumn(1).setPreferredWidth(37);
      colmod.getColumn(2).setPreferredWidth(38);
      colmod.getColumn(4).setPreferredWidth(30);
      MJVMTableHighlighter.install(this);
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 21:43:29 Slobodan
   Initial version

*****************************************************************************^*/
