/*
 * @(#)MJVMConsole.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;

import net.wrka.natives.ExternalLauncher;
import net.wrka.swing.actions.AboutAction;
import net.wrka.swing.actions.CommonAction;
import net.wrka.swing.actions.ExitAction;
import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.components.about.AboutBox;
import net.wrka.swing.components.about.HtmlAboutGroup;
import net.wrka.swing.components.about.SystemAboutGroup;
import net.wrka.swing.graphics.EmptyIcon;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.preferences.PersistentPreferences;
import net.wrka.util.CommonConstants;

import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchETF;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJActionLaunchRI4PP;

/**
 * MJVM Console.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMConsole extends MJVMConsoleFrame {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  JMenu menuExmples;
  Action actionHelp;
  MJVMAboutBox aboutBox;

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void interpretationStarted() {
    super.interpretationStarted();
    menuExmples.setEnabled(false);
  }

  /**
   * {@inheritDoc}
   */
  public void interpretationFinished() {
    super.interpretationFinished();
    menuExmples.setEnabled(true);
  }

  /**
   * {@inheritDoc}
   */
  public void interpretationStopped() {
    super.interpretationStopped();
    menuExmples.setEnabled(true);
  }

  /**
   * {@inheritDoc}
   */
  public void debuggingStarted() {
    super.debuggingStarted();
    menuExmples.setEnabled(false);
  }

  /**
   * {@inheritDoc}
   */
  public void debuggingFinished() {
    super.debuggingFinished();
    menuExmples.setEnabled(true);
  }

  /**
   * {@inheritDoc}
   */
  public void debuggingStopped() {
    super.debuggingStopped();
    menuExmples.setEnabled(true);
  }

  /**
   * {@inheritDoc}
   */
  public void machineShutdown() {
    dispose();
  }

/*^************************************************************************^*/
/*- Frame Construction.                                                    -*/
/*^************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void createActions() {
    super.createActions();
    actionHelp = SimpleHelp.getHelpAction("MJVM"); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected void createCloseAction() {
    actionClose = new ActionExit();
  }

  /**
   * {@inheritDoc}
   */
  protected JMenuBar createMenuBar() {
    JMenuBar menuBar = super.createMenuBar();
    JMenu menu = createHelpMenu();
    menuBar.add(menu);
    return menuBar;
  }

  /**
   * {@inheritDoc}
   */
  protected JMenu createFileMenu() {
    JMenu menu = new JMenu("File"); //$NON-NLS-1$
    menu.setMnemonic('F');
    menu.add(actionLoad);
    menu.add(actionSave);
    menu.addSeparator();
    menu.add(PersistentPreferences.getPreferencesAction());
    menu.addSeparator();
    menu.add(actionClose);
    return menu;
  }

  /**
   * {@inheritDoc}
   */
  protected JWxToolBar createToolBar() {
    JWxToolBar toolBar = super.createToolBar();
    toolBar.add(Box.createHorizontalGlue());
    toolBar.addSeparator();
    toolBar.addToolButton(actionHelp);
    toolBar.addSeparator();
    return toolBar;
  }

/*^**************************************************************************^*/
/*- Package-Private Methods.                                                 -*/
/*^**************************************************************************^*/

  JMenu createHelpMenu() {
    JMenu menu = new JMenu("Help"); //$NON-NLS-1$
    menu.setMnemonic('H');
    menu.add(actionHelp);
    menu.addSeparator();
    menuExmples = new JMenu("Examples"); //$NON-NLS-1$
    menuExmples.setMnemonic('E');
    menuExmples.setIcon(EmptyIcon.ICON_16);
    menuExmples.add(new ActionLoadExample("HelloWorld.mjb")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("ByteEcho.mjb")); //$NON-NLS-1$
    menuExmples.add(new ActionLoadExample("IntEcho.mjb")); //$NON-NLS-1$
    menu.add(menuExmples);
    if (ExternalLauncher.AVAILABLE) {
      menu.addSeparator();
      menu.add(new MJActionLaunchRI4PP());
      menu.add(new MJActionLaunchETF());
    }
    menu.addSeparator();
    menu.add(new ActionAbout());
    return menu;
  }

  void loadExample(String name) {
    try {
      clear();
      mjvm.load(MJResourceManager.INSTANCE.getExampleStream(name), true);
      updateActionsAndBrowsers(true, true, true, false, false, true);
      StringBuffer sb = new StringBuffer("Example: \""); //$NON-NLS-1$
      sb.append(name);
      sb.append('"');
      setStatus(sb.toString());
    } catch (IOException ioe) {
      StringBuffer sb = new StringBuffer(
                                      "Error loading example: "); //$NON-NLS-1$
      sb.append(name);
      sb.append(" - ");
      sb.append(ioe.getMessage());
      setStatus(CommonConstants.STRING_EMPTY);
      showError("Error", sb.toString()); //$NON-NLS-1$
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Exit Action.
   */
  final class ActionExit extends ExitAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performClose();
    }
  }

  /**
   * Load Example Action.
   */
  final class ActionLoadExample extends CommonAction {
    /**
     * Constructs new ActionLoadExample object.
     *
     * @param name the action name
     */
    public ActionLoadExample(String name) {
      super(name);
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      JMenuItem jmi = (JMenuItem)e.getSource();
      loadExample(jmi.getText());
    }
  }

  /**
   * About Action.
   */
  final class ActionAbout extends AboutAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      if (aboutBox == null) {
        aboutBox = new MJVMAboutBox(MJVMConsole.this);
      }
      SwingUtilities.invokeLater(aboutBox);
    }
  }

  /**
   * MJVM About Box.
   */
  static final class MJVMAboutBox implements Runnable {
    private final AboutBox ab;
    /**
     * Constructs new MJVMAboutBox object.
     *
     * @param mjvmc the MJVM console
     */
    public MJVMAboutBox(MJVMConsole mjvmc) {
      ab = new AboutBox(mjvmc,
                        MJResourceManager.INSTANCE.
                          getImageIcon("MJVMAboutLogo.png")); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Program", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                         getAboutURL("MJVMAbout.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("License", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                         getAboutURL("License.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new HtmlAboutGroup("Legal", //$NON-NLS-1$
                       MJResourceManager.INSTANCE.
                         getAboutURL("Legal.html"))); //$NON-NLS-1$
      ab.addAboutGroup(new SystemAboutGroup());
    }
    /**
     * {@inheritDoc}
     */
    public void run() {
      ab.show();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 21:50:01 Slobodan
   Initial version

*****************************************************************************^*/
