/*
 * @(#)MJVMConsoleFrame.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowEvent;
import java.io.FileWriter;
import java.io.IOException;
import java.util.prefs.PreferenceChangeEvent;
import java.util.prefs.PreferenceChangeListener;

import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;

import net.wrka.swing.actions.CommonAction;
import net.wrka.swing.actions.SelectableAction;
import net.wrka.swing.actions.CloseAction;
import net.wrka.swing.actions.OpenAction;
import net.wrka.swing.actions.SaveAction;
import net.wrka.swing.components.JWxFrame;
import net.wrka.swing.components.JWxMenu;
import net.wrka.swing.components.JWxStatusBar;
import net.wrka.swing.components.JWxToolBar;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.preferences.PersistentPreferences;
import net.wrka.swing.splash.Splash;
import net.wrka.swing.tooltips.ToolTipManager;
import net.wrka.util.CommonConstants;
import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJIOManager;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMInputOutput;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMListener;

/**
 * MJVM Console Frame.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class MJVMConsoleFrame extends JWxFrame
                              implements MJVMInputOutput, MJVMListener,
                                         PreferenceChangeListener {

/*^**************************************************************************^*/
/*- Package-Private Static Fields.                                           -*/
/*^**************************************************************************^*/

  static final Dimension MIN_DIMENSION;
  static final Dimension DEF_DIMENSION;
  static final int CTRL_ALT_MASK;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    MIN_DIMENSION = new Dimension(375, 300);
    DEF_DIMENSION = new Dimension(500, 400);
    CTRL_ALT_MASK = InputEvent.CTRL_MASK | InputEvent.ALT_MASK;
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  Action actionLoad;
  Action actionSave;
  Action actionClose;
  Action actionCut;
  Action actionCopy;
  Action actionPaste;
  Action actionClear;
  Action actionSelectAll;
  ActionViewCode actionViewCode;
  ActionViewData actionViewData;
  ActionViewHeap actionViewHeap;
  ActionViewPStack actionViewPStack;
  ActionViewEStack actionViewEStack;
  Action actionViewShowAll;
  Action actionViewHideAll;
  Action actionRun;
  Action actionRunNext;
  Action actionDebug;
  Action actionPause;
  Action actionStop;
  MJVM mjvm;
  MJVMCodeBrowser mjvmCodeBrowser;
  MJVMDataBrowser mjvmDataBrowser;
  MJVMHeapBrowser mjvmHeapBrowser;
  MJVMEStackBrowser mjvmEStackBrowser;
  MJVMPStackBrowser mjvmPStackBrowser;
  MJVMMemoryBrowsersUpdater browsersUpdater;
  JMenuBar menuBar;
  JWxToolBar toolBar;
  MJVMConsoleTextPane mjvmcTextPane;
  JWxStatusBar statusBar;
  JLabel jlStatus;
  JLabel jlElapsedTime;

/*^**************************************************************************^*/
/*- Public Constructors.                                                     -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMConsoleFrame object.
   */
  public MJVMConsoleFrame() {
    super();
    initFrame();
    mjvm = new MJVM(this);
    mjvm.addMJVMListener(this);
  }

  /**
   * Constructs new MJVMConsole object.
   *
   * @param mjvm the MJVM
   */
  public MJVMConsoleFrame(MJVM mjvm) {
    super();
    initFrame();
    if (mjvm == null) {
      throw new IllegalArgumentException("Null MJVM"); //$NON-NLS-1$
    }
    this.mjvm = mjvm;
  }

  /**
   * Constructs new MJVMConsoleFrame object.
   *
   * @param fileName the bytecode file name
   *
   * @throws IOException on any IO error
   */
  public MJVMConsoleFrame(String fileName) throws IOException {
    this();
    mjvm.load(fileName);
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Returns MJVM.
   *
   * @return MJVM
   */
  public MJVM getMJVM() {
    return mjvm;
  }

/*^**************************************************************************^*/
/*- MJVMInputOutput Implementation.                                          -*/
/*^**************************************************************************^*/

  // Input Interface. ----------------------------------------------------------

  /**
   * Reads byte.
   *
   * @return read byte
   */
  public byte readByte() {
    String status = jlStatus.getText();
    setStatus("Reading input"); //$NON-NLS-1$
    byte rb = mjvmcTextPane.readByte();
    setStatus(status);
    return rb;
  }

  /**
   * Reads byte.
   *
   * @return read integer
   */
  public int readInt() {
    String status = jlStatus.getText();
    setStatus("Reading input"); //$NON-NLS-1$
    int ri = mjvmcTextPane.readInt();
    setStatus(status);
    return ri;
  }

  /**
   * Aborts current input.
   */
  public void abortInput() {
    mjvmcTextPane.cancelInput();
  }

  /*^************************************************************************^*/
  /*- Output Interface                                                       -*/
  /*^************************************************************************^*/

  /**
   * Clears console.
   */
  public void clear() {
    mjvmcTextPane.clearAll();
    setStatus(CommonConstants.STRING_EMPTY);
    jlElapsedTime.setText(CommonConstants.STRING_EMPTY);
    actionSave.setEnabled(false);
  }

  /**
   * Prints integer.
   *
   * @param i the integer
   * @param w the minimum width
   */
  public void print(int i, int w) {
    mjvmcTextPane.print(i, w);
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /**
   * Prints character.
   *
   * @param c the character
   * @param w the minimum width
   */
  public void print(char c, int w) {
    mjvmcTextPane.print(c, w);
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /**
   * Terminates line.
   */
  public void println() {
    mjvmcTextPane.println();
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /**
   * Prints error integer.
   *
   * @param i the error integer
   */
  public void printError(int i) {
    mjvmcTextPane.printError(i);
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /**
   * Prints error string.
   *
   * @param e the error string
   */
  public void printError(String e) {
    mjvmcTextPane.printError(e);
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /**
   * Prints error string and terminates line.
   *
   * @param e the error string
   */
  public void printlnError(String e) {
    mjvmcTextPane.printlnError(e);
    if (!actionSave.isEnabled()) {
      actionSave.setEnabled(true);
    }
  }

  /*^************************************************************************^*/
  /*- Input/Output Interface.                                                -*/
  /*^************************************************************************^*/

  /**
   * Returns echo mode.
   *
   * @return echo mode
   */
  public boolean getEcho() { // NOPMD
    return mjvmcTextPane.getEcho();
  }

  /**
   * Sets echo status of input.
   *
   * @param echo if <tt>true</tt> input will be echoed
   */
  public void setEcho(boolean echo) {
    mjvmcTextPane.setEcho(echo);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return (Dimension)MIN_DIMENSION.clone();
  }

  /**
   * {@inheritDoc}
   */
  public void setVisible(boolean visible) {
    super.setVisible(visible);
    mjvmcTextPane.updateEditActionStates();
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    performClose();
  }

/*^**************************************************************************^*/
/*- MJVMListener Implementation.                                             -*/
/*^**************************************************************************^*/

  /**
   * Notifies listener that bytecode has loaded.
   */
  public void bytecodeLoaded() {
    if (!actionViewCode.isEnabled()) {
      actionViewCode.setEnabled(true);
      actionViewData.setEnabled(true);
      actionViewHeap.setEnabled(true);
      actionViewPStack.setEnabled(true);
      actionViewEStack.setEnabled(true);
      actionViewShowAll.setEnabled(true);
      actionViewHideAll.setEnabled(true);
    }
    updateActionsAndBrowsers(true, true, true, false, false, true);
    setStatus("Bytecode loaded"); //$NON-NLS-1$
  }

  /**
   * Notifies listener that bytecode interpretation has started.
   */
  public void interpretationStarted() {
    if (!isVisible()) {
      setVisible(true);
    }
    toFront();
    mjvmcTextPane.requestFocus();
    actionLoad.setEnabled(false);
    updateActionsAndBrowsers(false, false, false, false, true, false);
    setStatus("Interpreting..."); //$NON-NLS-1$
  }

  /**
   * Notifies listener that bytecode interpretation has finished.
   */
  public void interpretationFinished() {
    actionLoad.setEnabled(true);
    updateActionsAndBrowsers(true, true, true, false, false, true);
    setStatus("Finished"); //$NON-NLS-1$
    setElapsedTime(mjvm.getElapsedTime());
  }

  /**
   * Notifies listener that bytecode interpretation has stopped.
   */
  public void interpretationStopped() {
    actionLoad.setEnabled(true);
    updateActionsAndBrowsers(true, true, true, false, false, true);
    setStatus("Stopped"); //$NON-NLS-1$
    setElapsedTime(mjvm.getElapsedTime());
  }

  /**
   * Notifies listener that bytecode debugging has started.
   */
  public void debuggingStarted() {
    if (!isVisible()) {
      setVisible(true);
    }
    toFront();
    performViewShowAll();
    mjvmcTextPane.requestFocus();
    actionLoad.setEnabled(false);
    updateActionsAndBrowsers(false, false, false, true, true, false);
    setStatus("Debugging..."); //$NON-NLS-1$
  }

  /**
   * Notifies listener that bytecode debugging has finished.
   */
  public void debuggingFinished() {
    actionLoad.setEnabled(true);
    updateActionsAndBrowsers(true, true, true, false, false, true);
    setStatus("Finished"); //$NON-NLS-1$
    setElapsedTime(mjvm.getElapsedTime());
  }

  /**
   * Notifies listener that bytecode debugging has stopped.
   */
  public void debuggingStopped() {
    actionLoad.setEnabled(true);
    updateActionsAndBrowsers(true, true, true, false, false, true);
    setStatus("Stopped"); //$NON-NLS-1$
    setElapsedTime(mjvm.getElapsedTime());
  }

  /**
   * Notifies listener that bytecode debugging has paused.
   *
   * @param address current PC value
   */
  public void debuggingPaused(int address) {
    updateActionsAndBrowsers(false, true, true, false, true, true);
    setStatus("Paused"); //$NON-NLS-1$
  }

  /**
   * Notifies listener that bytecode debugging has resumed.
   */
  public void debuggingResumed() {
    updateActionsAndBrowsers(false, true, true, true, true, false);
    setStatus("Debugging..."); //$NON-NLS-1$
  }

  /**
   * Notifies listener that MJVM has shutdown.
   */
  public void machineShutdown() {
    dispose();
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates actions.
   */
  protected void createActions() {
    createFileActions();
    createCloseAction();
    createEditActions();
    createViewActions();
    createRunActions();
  }

  /**
   * Creates file actions.
   */
  protected void createFileActions() {
    actionLoad = new ActionLoad();
    actionSave = new ActionSave();
  }

  /**
   * Creates close actions.
   */
  protected void createCloseAction() {
    actionClose = new ActionClose();
  }

  /**
   * Creates edit action.
   */
  protected void createEditActions() {
    actionCut = mjvmcTextPane.getCutAction();
    actionCopy = mjvmcTextPane.getCopyAction();
    actionPaste = mjvmcTextPane.getPasteAction();
    actionClear = mjvmcTextPane.getClearAction();
    actionSelectAll = mjvmcTextPane.getSelectAll();
  }

  /**
   * Creates view actions.
   */
  protected void createViewActions() {
    actionViewCode = new ActionViewCode();
    actionViewData = new ActionViewData();
    actionViewHeap = new ActionViewHeap();
    actionViewPStack = new ActionViewPStack();
    actionViewEStack = new ActionViewEStack();
    actionViewShowAll = new ActionViewShowAll();
    actionViewHideAll = new ActionViewHideAll();
  }

  /**
   * Creates run actions.
   */
  protected void createRunActions() {
    actionRun = new ActionRun();
    actionRunNext = new ActionRunNext();
    actionDebug = new ActionDebug();
    actionPause = new ActionPause();
    actionStop = new ActionStop();
  }

  /**
   * Creates menu bar.
   *
   * @return menu bar
   */
  protected JMenuBar createMenuBar() {
    JMenuBar menuBar = new JMenuBar();
    JMenu menu = createFileMenu();
    menuBar.add(menu);
    menu = createEditMenu();
    menuBar.add(menu);
    menu = createViewMenu();
    menuBar.add(menu);
    menu = createRunMenu();
    menuBar.add(menu);
    return menuBar;
  }

  /**
   * Creates file menu.
   *
   * @return file menu
   */
  protected JMenu createFileMenu() {
    JMenu menu = new JMenu("File"); //$NON-NLS-1$
    menu.setMnemonic('F');
    menu.add(actionLoad);
    menu.add(actionSave);
    menu.addSeparator();
    menu.add(actionClose);
    return menu;
  }

  /**
   * Creates edit menu.
   *
   * @return edit menu
   */
  protected JMenu createEditMenu() {
    JMenu menu = new JMenu("Edit"); //$NON-NLS-1$
    menu.setMnemonic('E');
    menu.add(actionCut);
    menu.add(actionCopy);
    menu.add(actionPaste);
    menu.add(actionClear);
    menu.addSeparator();
    menu.add(actionSelectAll);
    return menu;
  }

  /**
   * Creates view menu.
   *
   * @return view menu
   */
  protected JMenu createViewMenu() {
    JMenu menu = new JWxMenu("View"); //$NON-NLS-1$
    menu.setMnemonic('V');
    menu.add(actionViewCode);
    menu.add(actionViewData);
    menu.add(actionViewHeap);
    menu.add(actionViewPStack);
    menu.add(actionViewEStack);
    menu.addSeparator();
    menu.add(actionViewShowAll);
    menu.add(actionViewHideAll);
    return menu;
  }

  /**
   * Creates run menu.
   *
   * @return run menu
   */
  protected JMenu createRunMenu() {
    JMenu menu = new JMenu("Run"); //$NON-NLS-1$
    menu.setMnemonic('R');
    menu.add(actionRun);
    menu.add(actionRunNext).setDisplayedMnemonicIndex(4);
    menu.add(actionDebug);
    menu.add(actionPause);
    menu.add(actionStop);
    return menu;
  }

  /**
   * Creates tool bar.
   *
   * @return tool bar
   */
  protected JWxToolBar createToolBar() {
    JWxToolBar toolBar = new JWxToolBar("Tools", //$NON-NLS-1$
                                        JToolBar.HORIZONTAL, false);
    toolBar.addSeparator();
    toolBar.addToolButton(actionLoad);
    toolBar.addToolButton(actionSave);
    toolBar.addSeparator();
    toolBar.addToolButton(actionCut);
    toolBar.addToolButton(actionCopy);
    toolBar.addToolButton(actionPaste);
    toolBar.addToolButton(actionClear);
    toolBar.addSeparator();
    toolBar.addToolButton(actionRun);
    toolBar.addToolButton(actionRunNext);
    toolBar.addToolButton(actionDebug);
    toolBar.addToolButton(actionPause);
    toolBar.addToolButton(actionStop);
    toolBar.addSeparator();
    return toolBar;
  }

  /**
   * Sets status bar.
   *
   * @param statusText the status bar
   */
  protected void setStatus(String statusText) {
    jlStatus.setText(statusText);
  }

  /**
   * Sets elapsed time.
   *
   * @param elapsedTime the elapsed time
   */
  protected void setElapsedTime(long elapsedTime) {
    StringBuffer sb = new StringBuffer("Elapsed time: "); //$NON-NLS-1$
    if (elapsedTime > 60000) {
      sb.append(elapsedTime / 60000);
      sb.append(" min"); //$NON-NLS-1$
    } else if (elapsedTime > 1000) {
      sb.append(elapsedTime / 1000);
      sb.append(" s"); //$NON-NLS-1$
    } else {
      sb.append(elapsedTime);
      sb.append(" ms"); //$NON-NLS-1$
    }
    jlElapsedTime.setText(sb.toString());
  }

  /**
   * Updates actions and browsers.
   *
   * @param enableRun the enable run flag
   * @param enableRunNext the enable run next flag
   * @param enableDebug the enable debug flag
   * @param enablePause the enable pause flag
   * @param enableStop the enable stop flag
   * @param updateBrowsers the update browsers flag
   */
  protected void updateActionsAndBrowsers(boolean enableRun,
                                          boolean enableRunNext,
                                          boolean enableDebug,
                                          boolean enablePause,
                                          boolean enableStop,
                                          boolean updateBrowsers) {
    if (actionRun.isEnabled() != enableRun) {
      actionRun.setEnabled(enableRun);
    }
    if (actionRunNext.isEnabled() != enableRunNext) {
      actionRunNext.setEnabled(enableRunNext);
    }
    if (actionDebug.isEnabled() != enableDebug) {
      actionDebug.setEnabled(enableDebug);
    }
    if (actionPause.isEnabled() != enablePause) {
      actionPause.setEnabled(enablePause);
    }
    if (actionStop.isEnabled() != enableStop) {
      actionStop.setEnabled(enableStop);
    }
    if (updateBrowsers) {
      if (browsersUpdater == null) {
        browsersUpdater = new MJVMMemoryBrowsersUpdater();
      }
      SwingUtilities.invokeLater(browsersUpdater);
    }
  }

  /**
   * Updates browsers.
   */
  protected void updateBrowsers() {
    if (mjvmCodeBrowser != null) {
      mjvmCodeBrowser.update();
    }
    if (mjvmHeapBrowser != null) {
      mjvmHeapBrowser.update();
    }
    if (mjvmPStackBrowser != null) {
      mjvmPStackBrowser.update();
    }
    if (mjvmEStackBrowser != null) {
      mjvmEStackBrowser.update();
    }
  }

  // Action Methods. -----------------------------------------------------------

  /**
   * Performs load action.
   */
  protected void performLoad() {
    JFileChooser fc = MJIOManager.getLoadMJBytecodeFileChooser();
    if (fc.showDialog(this, null) == JFileChooser.APPROVE_OPTION) {
      try {
        mjvmcTextPane.clearAll();
        jlElapsedTime.setText(CommonConstants.STRING_EMPTY);
        setStatus("Loading file..."); //$NON-NLS-1$
        mjvm.load(fc.getSelectedFile());
        updateActionsAndBrowsers(true, true, true, false, false, true);
        StringBuffer sb = new StringBuffer("Loaded file \""); //$NON-NLS-1$
        sb.append(fc.getSelectedFile().getName());
        sb.append('"');
        setStatus(sb.toString());
      } catch (IOException ioe) {
        StringBuffer sb = new StringBuffer(
                                         "Error loading file: "); //$NON-NLS-1$
        sb.append(fc.getSelectedFile());
        sb.append(" - "); //$NON-NLS-1$
        sb.append(ioe.getMessage());
        setStatus(CommonConstants.STRING_EMPTY);
        showError("Error", sb.toString()); //$NON-NLS-1$
      }
    }
  }

  /**
   * Performs save action.
   */
  protected void performSave() {
    JFileChooser fc = MJIOManager.getSaveMJVMConsoleOutputFileChooser();
    if (fc.showDialog(this, null) == JFileChooser.APPROVE_OPTION) {
      FileWriter fw = null;
      try {
        fw = new FileWriter(fc.getSelectedFile());
        mjvmcTextPane.write(fw);
      } catch (IOException ioe) {
        printlnError("Error saving file: "); //$NON-NLS-1$
        printlnError(ioe.getMessage());
      } finally {
        if (fw != null) {
          try {
            fw.close();
          } catch (IOException ioe) { // NOPMD
            // empty catch block
          }
        }
      }
    }
  }

  /**
   * Performs close action.
   */
  protected void performClose() {
    mjvm.shutdown();
  }

  /**
   * Performs view code action.
   *
   * @param show the show flag
   */
  protected void performViewCode(boolean show) {
    if (mjvmCodeBrowser == null) {
      if (getX() < MJVMCodeBrowser.DIMENSION.width + 4) {
        setLocation(MJVMCodeBrowser.DIMENSION.width + 4, getY());
      }
      mjvmCodeBrowser = new MJVMCodeBrowser(this);
    }
    mjvmCodeBrowser.setVisible(show);
  }

  /**
   * Performs view data action.
   *
   * @param show the show flag
   */
  protected void performViewData(boolean show) {
    if (mjvmDataBrowser == null) {
      mjvmDataBrowser = new MJVMDataBrowser(this);
    }
    mjvmDataBrowser.setVisible(show);
  }

  /**
   * Performs view heap action.
   *
   * @param show the show flag
   */
  protected void performViewHeap(boolean show) {
    if (mjvmHeapBrowser == null) {
      mjvmHeapBrowser = new MJVMHeapBrowser(this);
    }
    mjvmHeapBrowser.setVisible(show);
  }

  /**
   * Performs view pstack action.
   *
   * @param show the show flag
   */
  protected void performViewPStack(boolean show) {
    if (mjvmPStackBrowser == null) {
      mjvmPStackBrowser = new MJVMPStackBrowser(this);
    }
    mjvmPStackBrowser.setVisible(show);
  }

  /**
   * Performs view estack action.
   *
   * @param show the show flag
   */
  protected void performViewEStack(boolean show) {
    if (mjvmEStackBrowser == null) {
      mjvmEStackBrowser = new MJVMEStackBrowser(this);
    }
    mjvmEStackBrowser.setVisible(show);
  }

  /**
   * Performs show all views action.
   */
  protected void performViewShowAll() {
    actionViewCode.actionSelectPerformed(null, true);
    actionViewData.actionSelectPerformed(null, true);
    actionViewHeap.actionSelectPerformed(null, true);
    actionViewPStack.actionSelectPerformed(null, true);
    actionViewEStack.actionSelectPerformed(null, true);
    toFront();
  }

  /**
   * Performs hide all views action.
   */
  protected void performViewHideAll() {
    actionViewCode.actionSelectPerformed(null, false);
    actionViewData.actionSelectPerformed(null, false);
    actionViewHeap.actionSelectPerformed(null, false);
    actionViewPStack.actionSelectPerformed(null, false);
    actionViewEStack.actionSelectPerformed(null, false);
  }

  /**
   * Performs run action.
   */
  protected void performRun() {
    mjvm.run();
  }

  /**
   * Performs run next action.
   */
  protected void performRunNext() {
    mjvm.runNext();
  }

  /**
   * Performs debug action.
   */
  protected void performDebug() {
    mjvm.debug();
  }

  /**
   * Performs pause action.
   */
  protected void performPause() {
    mjvm.pause();
  }

  /**
   * Performs stop action.
   */
  protected void performStop() {
    mjvm.stop();
  }

  // Memory Browser Methods. ---------------------------------------------------

  /**
   * Hides code browser.
   */
  protected void hideCodeBrowser() {
    actionViewCode.setSelected(false);
  }

  /**
   * Hides data browser.
   */
  protected void hideDataBrowser() {
    actionViewData.setSelected(false);
  }

  /**
   * Hides heap browser.
   */
  protected void hideHeapBrowser() {
    actionViewHeap.setSelected(false);
  }

  /**
   * Hides pstack browser.
   */
  protected void hidePStackBrowser() {
    actionViewPStack.setSelected(false);
  }

  /**
   * Hides estack browser.
   */
  protected void hideEStackBrowser() {
    actionViewEStack.setSelected(false);
  }

/*^**************************************************************************^*/
/*- Private Method.                                                          -*/
/*^**************************************************************************^*/

  private void initFrame() {
    Splash.update("Initializing console"); //$NON-NLS-1$
    setTitle("MJVM Console"); //$NON-NLS-1$
    setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    setIconImage(MJResourceManager.
                          INSTANCE.getImage("MicroJavaCup.png")); //$NON-NLS-1$
    setSize(DEF_DIMENSION);
    Container cp = getContentPane();
    mjvmcTextPane = new MJVMConsoleTextPane();
    createActions();
    menuBar = createMenuBar();
    setJMenuBar(menuBar);
    toolBar = createToolBar();
    cp.add(toolBar, BorderLayout.NORTH);
    mjvmcTextPane.setLinuxColors();
    JPanel tmp = new JPanel(new BorderLayout());
    tmp.setBorder(new EmptyBorder(3, 2, 0, 2));
    JScrollPane scroller = new JScrollPane(mjvmcTextPane);
    tmp.add(scroller, BorderLayout.CENTER);
    cp.add(tmp, BorderLayout.CENTER);
    statusBar = new JWxStatusBar(20);
    jlStatus = new JLabel();
    statusBar.addStatusPanel(jlStatus);
    jlElapsedTime = new JLabel();
    statusBar.addStatusPanel(jlElapsedTime, 150);
    setJWxStatusBar(statusBar);
    ToolTipManager.registerFrame(this);
    PlafPreferences.getInstance().addLnFChangeComponent(this);
    PersistentPreferences.getUserPreferences().
                                             addPreferenceChangeListener(this);
    setEcho(MJVMGeneralPreferences.INSTANCE.getPreferences().
                  getBoolean(MJVMGeneralPreferences.MJVM_CONSOLE_ECHO, false));
  }

/*^**************************************************************************^*/
/*- PreferenceChangeListener Implementation.                                 -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void preferenceChange(PreferenceChangeEvent evt) {
    if (MJVMGeneralPreferences.MJVM_CONSOLE_ECHO.equals(evt.getKey())) {
      setEcho(Boolean.valueOf(evt.getNewValue()).booleanValue());
    }
  }

/*^**************************************************************************^*/
/*- Inner Classes.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Load Action.
   */
  protected class ActionLoad extends OpenAction {
    /**
     * Constructs new ActionLoad object.
     */
    public ActionLoad() {
      super("Load ...", "Load MJ Bytecode"); //$NON-NLS-1$ //$NON-NLS-2$
      putValue(Action.MNEMONIC_KEY, new Integer(KeyEvent.VK_L));
      putValue(Action.ACCELERATOR_KEY,
               KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performLoad();
    }
  }

  /**
   * Save Action.
   */
  protected class ActionSave extends SaveAction {
    /**
     * Constructs new ActionSave object.
     *
     */
    public ActionSave() {
      super("Save Output...", //$NON-NLS-1$
            "Save Console Output"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performSave();
    }
  }

  /**
   * Close Action.
   */
  protected class ActionClose extends CloseAction {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performClose();
    }
  }

  /**
   * ViewCode Action.
   */
  protected class ActionViewCode extends SelectableAction {
    /**
     * Constructs new ActionViewCode object.
     */
    public ActionViewCode() {
      super("Code Memory", null, KeyEvent.VK_C, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_C, CTRL_ALT_MASK),
            "View Code Memory"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewCode(select);
    }
  }

  /**
   * ViewData Action.
   */
  protected class ActionViewData extends SelectableAction {
    /**
     * Constructs new ActionViewData object.
     */
    public ActionViewData() {
      super("Data Memory", null, KeyEvent.VK_D, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_D, CTRL_ALT_MASK),
            "View Data Memory"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewData(select);
    }
  }

  /**
   * ViewData Heap.
   */
  protected class ActionViewHeap extends SelectableAction {
    /**
     * Constructs new ActionViewHeap object.
     */
    public ActionViewHeap() {
      super("Heap Memory", null, KeyEvent.VK_H, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_H, CTRL_ALT_MASK),
            "View Heap Memory"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewHeap(select);
    }
  }

  /**
   * ViewEStack Action.
   */
  protected class ActionViewEStack extends SelectableAction {
    /**
     * Constructs new ActionViewEStack object.
     */
    public ActionViewEStack() {
      super("Exspression Stack", null, KeyEvent.VK_E, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_E, CTRL_ALT_MASK),
            "View Exspression Stack"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewEStack(select);
    }
  }

  /**
   * ViewPStack Action.
   */
  protected class ActionViewPStack extends SelectableAction {
    /**
     * Constructs new ActionViewPStack object.
     */
    public ActionViewPStack() {
      super("Procedure Stack", null, KeyEvent.VK_P, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_P, CTRL_ALT_MASK),
            "View Procedure Stack"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void selectActionPerformed(ActionEvent e, boolean select) {
      performViewPStack(select);
    }
  }

  /**
   * Show All Views Action.
   */
  protected class ActionViewShowAll extends CommonAction {
    /**
     * Constructs new ActionViewShowAll object.
     */
    public ActionViewShowAll() {
      super("Show All", null, KeyEvent.VK_S, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_S, CTRL_ALT_MASK),
            "Show All"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performViewShowAll();
    }
  }

  /**
   * Hide All Action.
   */
  protected class ActionViewHideAll extends CommonAction {
    /**
     * Constructs new ActionViewHideAll object.
     */
    public ActionViewHideAll() {
      super("Hide All", null, KeyEvent.VK_H, //$NON-NLS-1$
            KeyStroke.getKeyStroke(KeyEvent.VK_H, CTRL_ALT_MASK),
            "Hide All"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performViewHideAll();
    }
  }

  /**
   * Run Action.
   */
  protected class ActionRun extends CommonAction {
    /**
     * Constructs new ActionRun object.
     */
    public ActionRun() {
      super("Run", MJResourceManager.//$NON-NLS-1$
                       INSTANCE.getCachedImageIcon("Run.png"), //$NON-NLS-1$
            KeyEvent.VK_R, KeyStroke.getKeyStroke(KeyEvent.VK_F5, 0),
            "Run MJ Bytecode"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRun();
    }
  }

  /**
   * Run Next Action.
   */
  protected class ActionRunNext extends CommonAction {
    /**
     * Constructs new ActionRunNext object.
     */
    public ActionRunNext() {
      super("Run Next", //$NON-NLS-1$
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("RunNext.png"), //$NON-NLS-1$
            KeyEvent.VK_N, KeyStroke.getKeyStroke(KeyEvent.VK_F6, 0),
            "Run Next Instruction"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performRunNext();
    }
  }

  /**
   * Debug Action.
   */
  protected class ActionDebug extends CommonAction {
    /**
     * Constructs new ActionDebug object.
     */
    public ActionDebug() {
      super("Debug",
            MJResourceManager.INSTANCE.
                                getCachedImageIcon("Debug.png"), //$NON-NLS-1$
            KeyEvent.VK_D, KeyStroke.getKeyStroke(KeyEvent.VK_F7, 0),
            "Debug MJ Bytecode"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performDebug();
    }
  }

  /**
   * Pause Action.
   */
  protected class ActionPause extends CommonAction {
    /**
     * Constructs new ActionPause object.
     */
    public ActionPause() {
      super("Pause", //$NON-NLS-1$
            MJResourceManager.
                INSTANCE.getCachedImageIcon("Pause.png"), //$NON-NLS-1$
            KeyEvent.VK_P, KeyStroke.getKeyStroke(KeyEvent.VK_F8, 0),
            "Pause"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performPause();
    }
  }

  /**
   * Stop Action.
   */
  protected class ActionStop extends CommonAction {
    /**
     * Constructs new ActionStop object.
     */
    public ActionStop() {
      super("Stop", //$NON-NLS-1$
            MJResourceManager.
                INSTANCE.getCachedImageIcon("Stop.png"), //$NON-NLS-1$
            KeyEvent.VK_S, KeyStroke.getKeyStroke(KeyEvent.VK_F9, 0),
            "Stop"); //$NON-NLS-1$
      enabled = false;
    }
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      performStop();
    }
  }

  /**
   * MJVMMemoryBrowsers Updater.
   */
  protected class MJVMMemoryBrowsersUpdater implements Runnable {
    /**
     * {@inheritDoc}
     */
    public void run() {
      updateBrowsers();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 11:26:01 Slobodan
   Initial version

*****************************************************************************^*/
