/*
 * @(#)MJVMConsoleHorizontalCaret.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Timer;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.JTextComponent;

/**
 * MJVMConsole Horizontal Caret.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMConsoleHorizontalCaret extends DefaultCaret {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final Timer caretFlasher; // NOPMD

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMConsoleCaret object.
   */
  public MJVMConsoleHorizontalCaret() {
    super();
    super.setBlinkRate(0);
    caretFlasher = new Timer(500, new CaretFlasher());
    caretFlasher.start();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void setBlinkRate(int blinkRate) {
    // empty method body
  }

  /**
   * {@inheritDoc}
   */
  public void paint(Graphics g) {
    if (isVisible()) {
      JTextComponent comp = getComponent();
      if (comp != null) {
        int dot = getDot();
        Rectangle r = null;
        char dotChar;
        try {
          r = comp.modelToView(dot);
          if (r == null) {
            return;
          }
          String ct = comp.getText(dot, 1);
          if (ct == null) {
            return;
          }
          dotChar = ct.charAt(0);
        } catch (BadLocationException ble) {
          return;
        }
        g.setColor(comp.getForeground());
        g.setXORMode(comp.getBackground());
        width = g.getFontMetrics().charWidth(dotChar);
        if (width <= 0) {
          width = 7;
        }
        if (isVisible()) {
          g.fillRect(r.x, r.y + r.height - 4, width, 3);
        }
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  protected void damage(Rectangle r) {
    synchronized (this) {
      if (r != null) {
        x = r.x;
        y = r.y + r.height - 4;
        height = 4;
        if (width <= 0) {
          width = getComponent().getWidth();
        }
        repaint();
      }
    }
  }

/*^**************************************************************************^*/
/*- Package-Private Method.                                                  -*/
/*^**************************************************************************^*/

  void repaintCaret() { // optimization
    repaint();
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Caret Flasher.
   */
  final class CaretFlasher implements ActionListener {
    /**
     * {@inheritDoc}
     */
    public void actionPerformed(ActionEvent e) {
      setVisible(!isVisible());
      repaintCaret();
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 22:08:01 Slobodan
   Initial version

*****************************************************************************^*/
