/*
 * @(#)MJVMConsoleInputLock.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

/**
 * MJVMConsole Input Lock.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMConsoleInputLock {

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private Thread t;
  private boolean waiting;
  private boolean lock;

/*^**************************************************************************^*/
/*- Public Methods.                                                          -*/
/*^**************************************************************************^*/

  /**
   * Returns <tt>true</tt> if the lock has waiter.
   *
   * @return <tt>true</tt> if the lock has waiter
   */
  public boolean hasWaiter() {
    synchronized (this) {
      return waiting;
    }
  }

  /**
   * Unlocks waiter.
   */
  public void unlock() {
    synchronized (this) {
      if (t != null) {
        t.interrupt();
        lock = false;
        t = null;
      }
    }
  }

  /**
   * Signals the lock.
   */
  public void _signal() { // NOPMD
    synchronized (this) {
      waiting = false;
      lock = true;
      notify(); // NOPMD, FB: OK, only one thread can wait.
    }
  }

  /**
   * Waits for signal.
   *
   * @throws InterruptedException
   */
  public void _wait() throws InterruptedException { // NOPMD
    synchronized (this) {
      if (!lock) {
        waiting = true;
      }
      t = Thread.currentThread();
      while (!lock) {
        wait();
      }
      lock = false;
      t = null;
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 22:08:30 Slobodan
   Initial version

*****************************************************************************^*/
