/*
 * @(#)MJVMConsoleLauncher.java  1.0.0 / 2008-05-26
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Window;
import java.io.IOException;

import javax.swing.ImageIcon;

import net.wrka.swing.appcontainer.GUIApplication;
import net.wrka.swing.help.SimpleHelp;
import net.wrka.swing.preferences.PlafPreferences;
import net.wrka.swing.preferences.PersistentPreferences;
import yu.ac.bg.etf.ri4pp.mjtools.res.MJResourceManager;
import yu.ac.bg.etf.ri4pp.mjtools.utils.MJIOManager;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;

/**
 * MicroJava Virtual Machine Launcher.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public final class MJVMConsoleLauncher extends GUIApplication {

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMConsoleLauncher object.
   *
   * @param args the arguments
   */
  public MJVMConsoleLauncher(String[] args) {
    super("MJVM", "MicroJava Virtula Machine", //$NON-NLS-1$ //$NON-NLS-2$
          "1.0.0 (2008-06-02)", //$NON-NLS-1$
          "ETF (www.etf.bg.ac.yu)", MJResourceManager.INSTANCE, //$NON-NLS-1$
          args);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void exec() {
    MJVMConsole mjvmc = (MJVMConsole)mainWindow;
    if (args.length > 0) {
      try {
        MJVM mjvm = mjvmc.getMJVM();
        mjvm.load(args[args.length - 1]);
        mjvm.run();
      } catch (IOException ioe) {
        mjvmc.printlnError("Error loading file: "); //$NON-NLS-1$
        mjvmc.printlnError(ioe.getMessage());
      }
    }
    mjvmc.setVisible(true);
    mjvmc.toFront();
  }

  /**
   * {@inheritDoc}
   */
  protected Window createMainWindow() {
    return new MJVMConsole();
  }

  /**
   * {@inheritDoc}
   */
  protected ImageIcon getSplashImage() {
    return guiResourceManager.getImageIcon("MJVMSplash.png"); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected int getSplashUpdatesCount() {
    return 2;
  }

  /**
   * {@inheritDoc}
   */
  protected void addPreferenceGroups() {
    PersistentPreferences.addPreferenceGroup(MJVMGeneralPreferences.INSTANCE);
    PersistentPreferences.addPreferenceGroup(PlafPreferences.getInstance());
  }

  /**
   * {@inheritDoc}
   */
  protected void initHelp() {
    SimpleHelp.setHelp("Help", //$NON-NLS-1$
                       guiResourceManager.
                           getHelpURL("MJVM/MJVMHelpTOC.xml")); //$NON-NLS-1$
  }

  /**
   * {@inheritDoc}
   */
  protected boolean warmUp() {
    return super.warmUp() && MJIOManager.warmUp();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-26 22:10:37 Slobodan
   Initial version

*****************************************************************************^*/
