/*
 * @(#)MJVMDataTableModel.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import javax.swing.table.AbstractTableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMDataMemory;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMWordMemoryListener;

/**
 * MJVM Data Memory Table Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMDataTableModel extends AbstractTableModel
                               implements MJVMWordMemoryListener {

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static final long serialVersionUID = 4313642906080893464L;

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMDataMemory mjvmData;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMDataTableModel object.
   *
   * @param mjvmData the MJVM Data Memory
   */
  public MJVMDataTableModel(MJVMDataMemory mjvmData) {
    super();
    this.mjvmData = mjvmData;
    mjvmData.addMemoryListener(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int getColumnCount() {
    return 2;
  }

  /**
   * {@inheritDoc}
   */
  public String getColumnName(int column) {
    switch (column) {
      case 0:
        return "Address"; //$NON-NLS-1$
      case 1:
        return "Value*"; //$NON-NLS-1$
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public Class getColumnClass(int column) {
    switch (column) {
      case 0:
        return Integer.class; // FB: OK
      case 1:
        return Integer.class; // FB: OK
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getRowCount() {
    return mjvmData.getSize();
  }

  /**
   * {@inheritDoc}
   */
  public Object getValueAt(int row, int col) {
    switch (col) {
      case 0:
        return new Integer(row);
      case 1:
        return new Integer(mjvmData.getValueAt(row));
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isCellEditable(int row, int column) {
    return (column == 1);
  }

  /**
   * {@inheritDoc}
   */
  public void setValueAt(Object value, int row, int col) {
    if (col == 1) {
      mjvmData.setValueAt(row, Integer.parseInt(value.toString()));
    }
  }

/*^**************************************************************************^*/
/*- MJVMDataMemoryListener Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void wordMemoryLocationChanged(int address, int oldValue,
                                        int newValue) {
    fireTableRowsUpdated(address, address);
  }

  /**
   * {@inheritDoc}
   */
  public void memoryChanged() {
    fireTableDataChanged();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:40:38 Slobodan
   Initial version

*****************************************************************************^*/
