/*
 * @(#)MJVMEStackBrowser.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.event.WindowEvent;

import javax.swing.table.TableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMEStack;

/**
 * MJVM EStack Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMEStackBrowser extends MJVMMemoryBrowser {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMEStack eStack;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMEStackBrowser object.
   *
   * @param mjvmcf the console frame
   */
  public MJVMEStackBrowser(MJVMConsoleFrame mjvmcf) {
    super(mjvmcf, "MJVM Expression Stack", 3); //$NON-NLS-1$
    eStack = mjvm.getEStackMemory();
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates browser.
   */
  public void update() {
    int esp = eStack.getESP();
    if (!memoryTable.isCellSelected(esp, 0)) {
      memoryTable.changeSelection(esp, 0, false, false);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected TableModel createMemoryTableModel() {
    return new MJVMEStackTableModel(mjvm.getEStackMemory());
  }

  /**
   * {@inheritDoc}
   */
  protected MJVMMemoryTable createMemoryTable() {
    MJVMMemoryTable eStackTable = new MJVMMemoryTable(memoryTableModel, false);
    eStackTable.
        setDefaultRenderer(Boolean.class,
                           new MJVMPointerCellRenderer("ESP")); //$NON-NLS-1$
    MJVMTableHighlighter.install(eStackTable);
    return eStackTable;
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    mjvmcf.hideEStackBrowser();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:41:43 Slobodan
   Initial version

*****************************************************************************^*/
