/*
 * @(#)MJVMHeapBrowser.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.event.WindowEvent;

import javax.swing.table.TableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMHeapMemory;

/**
 * MJVM Heap Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMHeapBrowser extends MJVMMemoryBrowser {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMHeapMemory heap;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMHeapBrowser object.
   *
   * @param mjvmcf the console frame
   */
  public MJVMHeapBrowser(MJVMConsoleFrame mjvmcf) {
    super(mjvmcf, "MJVM Heap Memory", 1); //$NON-NLS-1$
    heap = mjvm.getHeapMemory();
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates browser.
   */
  public void update() {
    int free = heap.getFree();
    if (!memoryTable.isCellSelected(free, 0)) {
      memoryTable.changeSelection(free, 0, false, false);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected TableModel createMemoryTableModel() {
    return new MJVMHeapTableModel(mjvm.getHeapMemory());
  }

  /**
   * {@inheritDoc}
   */
  protected MJVMMemoryTable createMemoryTable() {
    MJVMMemoryTable heapTable = new MJVMMemoryTable(memoryTableModel, false);
    heapTable.
        setDefaultRenderer(Boolean.class,
                           new MJVMPointerCellRenderer("Free")); //$NON-NLS-1$
    MJVMTableHighlighter.install(heapTable);
    return heapTable;
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    mjvmcf.hideHeapBrowser();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:47:12 Slobodan
   Initial version

*****************************************************************************^*/
