/*
 * @(#)MJVMHeapTableModel.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import javax.swing.table.AbstractTableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMHeapMemory;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMHeapMemoryListener;

/**
 * MJVM Heap Table Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMHeapTableModel extends AbstractTableModel
                               implements MJVMHeapMemoryListener {

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static final long serialVersionUID = -9157087818568636851L;

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMHeapMemory mjvmHeap;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMHeapTableModel object.
   *
   * @param mjvmHeap the MJVM Heap
   */
  public MJVMHeapTableModel(MJVMHeapMemory mjvmHeap) {
    super();
    this.mjvmHeap = mjvmHeap;
    mjvmHeap.addMemoryListener(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int getColumnCount() {
    return 3;
  }

  /**
   * {@inheritDoc}
   */
  public String getColumnName(int column) {
    switch (column) {
      case 0:
        return "#"; //$NON-NLS-1$
      case 1:
        return "Address"; //$NON-NLS-1$
      case 2:
        return "Value*"; //$NON-NLS-1$
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public Class getColumnClass(int column) {
    switch (column) {
      case 0:
        return Boolean.class;
      case 1:
        return Integer.class; // FB: OK
      case 2:
        return Integer.class; // FB: OK
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getRowCount() {
    return mjvmHeap.getFree() + 1;
  }

  /**
   * {@inheritDoc}
   */
  public Object getValueAt(int row, int col) {
    switch (col) {
      case 0:
        return Boolean.valueOf(row == mjvmHeap.getFree());
      case 1:
        return new Integer(row);
      case 2:
        return new Integer(mjvmHeap.getValueAt(row));
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isCellEditable(int row, int column) {
    return (column == 2);
  }

  /**
   * {@inheritDoc}
   */
  public void setValueAt(Object value, int row, int col) {
    if (col == 2) {
      mjvmHeap.setValueAt(row, Integer.parseInt(value.toString()));
    }
  }

/*^**************************************************************************^*/
/*- MJVMHeapMemoryListener Implementation.                                   -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void heapFreeChanged(int oldValue, int newValue) {
    if (newValue > oldValue) {
      fireTableRowsInserted(oldValue + 1, newValue);
    } else {
      fireTableRowsDeleted(newValue + 1, oldValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void wordMemoryLocationChanged(int address, int oldValue,
                                        int newValue) {
    fireTableRowsUpdated(address, address);
  }

  /**
   * {@inheritDoc}
   */
  public void memoryChanged() {
    fireTableDataChanged();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:49:07 Slobodan
   Initial version

*****************************************************************************^*/
