/*
 * @(#)MJVMMemoryBrowser.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableModel;

import net.wrka.awt.AwtTools;
import net.wrka.swing.components.JWxDialog;
import net.wrka.swing.preferences.PlafPreferences;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVM;

/**
 * MJVM Memory Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
abstract class MJVMMemoryBrowser extends JWxDialog {

/*^**************************************************************************^*/
/*- Static Fields.                                                           -*/
/*^**************************************************************************^*/

  static final Dimension MIN_DIMENSION;
  static final Dimension PREF_DIMENSION;
  static final Dimension DIMENSION;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    MIN_DIMENSION = new Dimension(170, 130);
    PREF_DIMENSION = new Dimension(230, 180);
    int cbheight = MJVMCodeBrowser.DIMENSION.height / 4;
    int height = PREF_DIMENSION.height;
    if (cbheight < height) {
      height = cbheight;
    }
    DIMENSION = new Dimension(PREF_DIMENSION.width, height);
  }

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  MJVM mjvm;
  MJVMConsoleFrame mjvmcf;
  TableModel memoryTableModel;
  MJVMMemoryTable memoryTable;

/*^**************************************************************************^*/
/*- Package-Private Constructor.                                             -*/
/*^**************************************************************************^*/

  MJVMMemoryBrowser(MJVMConsoleFrame mjvmcf, String title, int posnum) {
    super(mjvmcf, title, false);
    this.mjvmcf = mjvmcf;
    mjvm = mjvmcf.getMJVM();
    setDialogPosition(POSITION_CUSTOM);
    setBounds(calcBounds(posnum));
    JPanel cp = (JPanel)getContentPane(); // FB: OK
    cp.setBorder(BorderFactory.createEtchedBorder());
    PlafPreferences.getInstance().addLnFChangeComponent(this);
    memoryTableModel = createMemoryTableModel();
    memoryTable = createMemoryTable();
    JScrollPane sp = new JScrollPane(memoryTable);
    cp.add(sp);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Dimension getMinimumSize() {
    return (Dimension)MIN_DIMENSION.clone();
  }

  /**
   * {@inheritDoc}
   */
  protected void onDeactivated(WindowEvent we) {
    TableCellEditor cellEditor = memoryTable.getCellEditor();
    if (cellEditor != null) {
      cellEditor.cancelCellEditing();
    }
  }

/*^**************************************************************************^*/
/*- Protected Methods.                                                       -*/
/*^**************************************************************************^*/

  /**
   * Creates memory table model.
   *
   * @return memory table model
   */
  protected abstract TableModel createMemoryTableModel();

  /**
   * Creates memory table model.
   *
   * @return memory table model
   */
  protected MJVMMemoryTable createMemoryTable() {
    return new MJVMMemoryTable(memoryTableModel, true);
  }

  /**
   * Calculates bounds.
   *
   * @param posnum the position number
   *
   * @return bounds
   */
  protected Rectangle calcBounds(int posnum) {
    int x = mjvmcf.getX() + mjvmcf.getWidth() + 1;
    int y = mjvmcf.getY() + (mjvmcf.getHeight() -
                             MJVMCodeBrowser.DIMENSION.height) / 2 - 10;
    if (y < 2) {
      y = 2;
    }
    int cbh = MJVMCodeBrowser.DIMENSION.height;
    int mwh = AwtTools.getMaximumWindowHeight();
    if (y + cbh > mwh) {
      y = (mwh - cbh) / 2;
    }
    return new Rectangle(x, y + posnum * DIMENSION.height, DIMENSION.width,
                         DIMENSION.height);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:52:54 Slobodan
   Initial version

*****************************************************************************^*/
