/*
 * @(#)MJVMPStackBrowser.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.event.WindowEvent;

import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMPStack;

/**
 * MJVM PStack Browser.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMPStackBrowser extends MJVMMemoryBrowser {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMPStack pStack;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMPStackBrowser object.
   *
   * @param mjvmcf the console frame
   */
  public MJVMPStackBrowser(MJVMConsoleFrame mjvmcf) {
    super(mjvmcf, "MJVM Procedure Stack", 2); //$NON-NLS-1$
    pStack = mjvm.getPStackMemory();
  }

/*^**************************************************************************^*/
/*- Public Method.                                                           -*/
/*^**************************************************************************^*/

  /**
   * Updates browser.
   */
  public void update() {
    int psp = pStack.getPSP();
    if (!memoryTable.isCellSelected(psp, 0)) {
      memoryTable.changeSelection(psp, 0, false, false);
    }
  }

/*^**************************************************************************^*/
/*- Overridden Method.                                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected TableModel createMemoryTableModel() {
    return new MJVMPStackTableModel(mjvm.getPStackMemory());
  }

  /**
   * {@inheritDoc}
   */
  protected MJVMMemoryTable createMemoryTable() {
    MJVMMemoryTable pStackTable = new MJVMMemoryTable(memoryTableModel, false);
    TableColumn col = pStackTable.getColumnModel().getColumn(0);
    col.setCellRenderer(new MJVMPStackPointerCellRenderer());
    MJVMTableHighlighter.install(pStackTable);
    return pStackTable;
  }

  /**
   * {@inheritDoc}
   */
  protected void onClosing(WindowEvent we) {
    mjvmcf.hidePStackBrowser();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:55:51 Slobodan
   Initial version

*****************************************************************************^*/
