/*
 * @(#)MJVMPStackTableModel.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import javax.swing.table.AbstractTableModel;

import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMPStack;
import yu.ac.bg.etf.ri4pp.mjtools.vm.MJVMPStackListener;

/**
 * MJVM PStack Table Model.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMPStackTableModel extends AbstractTableModel
                                 implements MJVMPStackListener {

/*^**************************************************************************^*/
/*- Package-Private Static Field.                                            -*/
/*^**************************************************************************^*/

  static final long serialVersionUID = 6455127385106549612L;

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private final MJVMPStack mjvmPStack;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new MJVMPStackTableModel object.
   *
   * @param mjvmPStack the MJVM PStack
   */
  public MJVMPStackTableModel(MJVMPStack mjvmPStack) {
    super();
    this.mjvmPStack = mjvmPStack;
    mjvmPStack.addMemoryListener(this);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public int getColumnCount() {
    return 3;
  }

  /**
   * {@inheritDoc}
   */
  public String getColumnName(int column) {
    switch (column) {
      case 0:
        return "#"; //$NON-NLS-1$
      case 1:
        return "Address"; //$NON-NLS-1$
      case 2:
        return "Value*"; //$NON-NLS-1$
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public Class getColumnClass(int column) {
    switch (column) {
      case 0:
        return Integer.class; // FB: OK
      case 1:
        return Integer.class; // FB: OK
      case 2:
        return Integer.class; // FB: OK
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public int getRowCount() {
    return mjvmPStack.getPSP() + 1;
  }

  /**
   * {@inheritDoc}
   */
  public Object getValueAt(int row, int col) {
    switch (col) {
      case 0:
        int val = 0;
        if (row == mjvmPStack.getFP()) {
          val = val + 1;
        }
        if (row == mjvmPStack.getPSP()) {
          val = val + 2;
        }
        return new Integer(val);
      case 1:
        return new Integer(row);
      case 2:
        return new Integer(mjvmPStack.getValueAt(row));
      default:
        return null;
    }
  }

  /**
   * {@inheritDoc}
   */
  public boolean isCellEditable(int row, int column) {
    return (column == 2);
  }

  /**
   * {@inheritDoc}
   */
  public void setValueAt(Object value, int row, int col) {
    if (col == 2) {
      mjvmPStack.setValueAt(row, Integer.parseInt(value.toString()));
    }
  }

/*^**************************************************************************^*/
/*- MJVMPStackListener Implementation.                                       -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public void pstackPSPChanged(int oldValue, int newValue) {
    if (newValue > oldValue) {
      fireTableRowsInserted(oldValue + 1, newValue);
    } else {
      fireTableRowsDeleted(newValue + 1, oldValue);
    }
  }

  /**
   * {@inheritDoc}
   */
  public void pstackFPChanged(int oldValue, int newValue) {
    fireTableCellUpdated(oldValue, 0);
    fireTableCellUpdated(newValue, 0);
  }

  /**
   * {@inheritDoc}
   */
  public void wordMemoryLocationChanged(int address, int oldValue,
                                        int newValue) {
    fireTableRowsUpdated(address, address);
  }

  /**
   * {@inheritDoc}
   */
  public void memoryChanged() {
    fireTableDataChanged();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:57:29 Slobodan
   Initial version

*****************************************************************************^*/
