/*
 * @(#)MJVMTableHighlighter.java  1.0.0 / 2008-05-27
 *
 * Copyright (c) 2008, ETF and contributors. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package yu.ac.bg.etf.ri4pp.mjtools.vm.gui;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * MJVM Table Highlighter.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
final class MJVMTableHighlighter implements TableCellRenderer {

/*^**************************************************************************^*/
/*- Package-Private Fields.                                                  -*/
/*^**************************************************************************^*/

  static final Color ODD_ROW_COLOR = new Color(236, 236, 236);

/*^**************************************************************************^*/
/*- Private Fields.                                                          -*/
/*^**************************************************************************^*/

  private final TableCellRenderer targetRenderer;
  private final Color evenBackground;
  private final Color evenForeground;
  private final Color oddBackground;
  private final Color oddForeground;

/*^**************************************************************************^*/
/*- Private Constructor.                                                     -*/
/*^**************************************************************************^*/

  private MJVMTableHighlighter(TableCellRenderer targetRenderer,
                               Color evenBackground, Color evenForeground,
                               Color oddBackground, Color oddForeground) {
    this.targetRenderer = targetRenderer;
    this.evenBackground = evenBackground;
    this.evenForeground = evenForeground;
    this.oddBackground = oddBackground;
    this.oddForeground = oddForeground;
  }

/*^**************************************************************************^*/
/*- TableCellRenderer Implementation.                                        -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  public Component getTableCellRendererComponent(JTable table, Object value,
                                                 boolean isSelected,
                                                 boolean hasFocus, int row,
                                                 int column) {
    TableCellRenderer renderer = targetRenderer;
    if (renderer == null) {
      renderer = table.getDefaultRenderer(table.getColumnClass(column));
    }
    Component comp = renderer.getTableCellRendererComponent(table, value,
                                isSelected, hasFocus, row, column);
    if ((row & 1) == 0) {
      if (evenBackground == null) {
        comp.setBackground(table.getBackground());
      } else {
        comp.setBackground(evenBackground);
      }
      if (evenForeground == null) {
        comp.setForeground(table.getForeground());
      } else {
        comp.setForeground(evenForeground);
      }
    } else {
      if (oddBackground == null) {
        comp.setBackground(table.getBackground());
      } else {
        comp.setBackground(oddBackground);
      }
      if (oddForeground == null) {
        comp.setForeground(table.getForeground());
      } else {
        comp.setForeground(oddForeground);
      }
    }
    return comp;
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Applies renderer to single column.
   *
   * @param table the table
   * @param columnIndex the column index
   * @param ebg the even background
   * @param efg the even foreground
   * @param obg the odd background
   * @param ofg the odd foreground
   */
  public static void install(JTable table, int columnIndex, Color ebg,
                             Color efg, Color obg, Color ofg) {
    TableColumn tc = table.getColumnModel().getColumn(columnIndex);
    TableCellRenderer targetRenderer = tc.getCellRenderer();
    tc.setCellRenderer(new MJVMTableHighlighter(targetRenderer, ebg, efg, obg,
                                                ofg));
  }

  /**
   * Applies renderer to an entire table.
   *
   * @param table the table
   * @param ebg the even background
   * @param efg the even foreground
   * @param obg the odd background
   * @param ofg the odd foreground
   */
  public static void install(JTable table, Color ebg, Color efg, Color obg,
                             Color ofg) {
    MJVMTableHighlighter sharedHighlighter = null;
    for (int i = table.getColumnCount() - 1; i >= 0; i--) {
      TableColumn tc = table.getColumnModel().getColumn(i);
      TableCellRenderer targetRenderer = tc.getCellRenderer();
      if (targetRenderer == null) {
        if (sharedHighlighter == null) {
          sharedHighlighter = new MJVMTableHighlighter(null, ebg, efg, obg,
                                                       ofg);
        }
        tc.setCellRenderer(sharedHighlighter);
      } else {
        tc.setCellRenderer(new MJVMTableHighlighter(targetRenderer, ebg, efg,
                                                    obg, ofg));
      }
    }
  }

  /**
   * Applies renderer to an entire table.
   *
   * @param table the table
   */
  public static void install(JTable table) {
    install(table, null, null, ODD_ROW_COLOR, null);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-27 10:58:48 Slobodan
   Initial version

*****************************************************************************^*/
