/*
 * @(#)ExtendedTestCase.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.junit;

import java.util.Arrays;
import java.util.Random;

import junit.framework.TestCase;

/**
 * Extended Test Case.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public abstract class ExtendedTestCase extends TestCase {

/*^**************************************************************************^*/
/*- Protected Static Fields.                                                 -*/
/*^**************************************************************************^*/

  /**
   * Common Random class.
   */
  protected static final Random RANDOM;

  /**
   * Common pause time (the value is {@value}).
   */
  protected static final int PAUSE_TIME;

/*^**************************************************************************^*/
/*- Static Initialization.                                                   -*/
/*^**************************************************************************^*/

  static {
    RANDOM = new Random();
    PAUSE_TIME = 1000;
  }

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ExtendedTestCase object.
   *
   * @param name the test case name
   */
  protected ExtendedTestCase(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Pauses the current thread for PAUSE_TIME time.
   */
  public static void pause() {
    try {
      Thread.sleep(PAUSE_TIME);
    } catch (InterruptedException ie) {
      // empty catch block
    }
  }

  /**
   * Asserts that two integers are same.
   *
   * @param expected the expected value
   * @param actual the actual value
   */
  public static void assertSame(int expected, int actual) {
    assertSame(null, expected, actual);
  }

  /**
   * Asserts that two integers are same.
   *
   * @param message the error message
   * @param expected the expected value
   * @param actual the actual value
   */
  public static void assertSame(String message, int expected, int actual) {
    if (expected != actual) {
      failNotSame(message, String.valueOf(expected), String.valueOf(actual));
    }
  }

  /**
   * Asserts that two arrays are equal.
   *
   * @param message the message
   * @param expected the expected array
   * @param actual the actual array
   */
  public static void assertEquals(String message, Object[] expected,
                                  Object[] actual) {
    if (!Arrays.equals(expected, actual)) {
      String formatted = ""; //$NON-NLS-1$
      if (message != null) {
        formatted = message + " "; //$NON-NLS-1$
      }
      assertNotNull(formatted +
                    "expected array: <not null> but was <null>", //$NON-NLS-1$
                    expected);
      assertNotNull(formatted +
                    "expected array: <null> but was <not null>", //$NON-NLS-1$
                    actual);
      assertEquals(formatted + "[array length] ", //$NON-NLS-1$
                   expected.length, actual.length);
      for (int i = 0; i < expected.length; i++) {
        assertEquals(formatted + "[position " + i + //$NON-NLS-1$
                     "]", expected[i], actual[i]); //$NON-NLS-1$
      }
    }
  }

  /**
   * Asserts that two arrays are equal.
   *
   * @param expected the expected array
   * @param actual the actual array
   */
  public static void assertEquals(Object[] expected, Object[] actual) {
    assertEquals(null, expected, actual);
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 18:21:34 Slobodan
   Initial version

*****************************************************************************^*/
