/*
 * @(#)TestPackage.java  1.0.0 / 2008-04-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.junit; // NOPMD

import junit.framework.TestSuite;

/**
 * Package Test Helper.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class TestPackage {

/*^**************************************************************************^*/
/*- Protected Constructor.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Constructs new TestPackage object.
   */
  protected TestPackage() {
    // empty constructor body
  }

/*^**************************************************************************^*/
/*- Public Static Methods.                                                   -*/
/*^**************************************************************************^*/

  /**
   * Returns package name for the specified class.
   *
   * @param c the class
   *
   * @return package name for the specified class
   *
   * @throws IllegalArgumentException if the class is <tt>null</tt>
   */
  public static String getPackageName(Class c) {
    if (c == null) {
      throw new IllegalArgumentException("Null class"); //$NON-NLS-1$
    }
    String cn = c.getName();
    return cn.substring(0, cn.lastIndexOf('.'));
  }

  /**
   * Creates test suite.
   *
   * @param name the test suite name
   *
   * @return test suite
   */
  public static TestSuite createTestSuite(String name) {
    return new TestSuite(name);
  }

  /**
   * Creates test suite for the specified package test class.
   *
   * @param c the class
   *
   * @return test suite for the specified package test class
   *
   * @throws IllegalArgumentException if the class is <tt>null</tt>
   */
  public static TestSuite createTestSuite(Class c) {
    return new TestSuite(getPackageName(c));
  }

  /**
   * Starts text test runner.
   *
   * @param c the test class
   * @param args the main method arguments
   */
  public static void startTextTestRunner(Class c, String[] args) {
    if (c == null) {
      System.err.println("Null test class"); //$NON-NLS-1$ NOPMD
    } else {
      junit.textui.TestRunner.main(new String[] {c.getName()});
    }
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-23 19:57:17 Slobodan
   Initial version

*****************************************************************************^*/
