/*
 * @(#)SimpleArrayListTest.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.Enumeration;
import java.util.Iterator;

import net.wrka.junit.ExtendedTestCase;

/**
 * SimpleArrayListTest Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SimpleArrayListTest extends ExtendedTestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private SimpleArrayList list = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleArrayListTest object.
   *
   * @param name the test name
   */
  public SimpleArrayListTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#trimToSize()}.
   */
  public final void testTrimToSize() {
    assertEquals(7, list.capacity());
    list.add(new Object());
    list.trimToSize();
    assertEquals(1, list.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#ensureCapacity(int)}.
   */
  public final void testEnsureCapacity() {
    assertEquals(7, list.capacity());
    list.ensureCapacity(100);
    assertEquals(100, list.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#size()}.
   */
  public final void testSize() {
    assertEquals(0, list.size());
    list.add(new Object());
    assertEquals(1, list.size());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#capacity()}.
   */
  public final void testCapacity() {
    assertEquals(7, list.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(list.isEmpty());
    list.add(new Object());
    assertFalse(list.isEmpty());
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#indexOf(java.lang.Object)}.
   */
  public final void testIndexOf() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertEquals(1, list.indexOf(o));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#lastIndexOf(java.lang.Object)}.
   */
  public final void testLastIndexOf() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertEquals(3, list.lastIndexOf(o));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#contains(java.lang.Object)}.
   */
  public final void testContains() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertTrue(list.contains(o));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#get(int)}.
   */
  public final void testGet() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertSame(o, list.get(1));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#toArray(java.lang.Object[])}.
   */
  public final void testToArrayObjectArray() {
    list.add(new Object());
    list.add(new Object());
    list.add(new Object());
    Object[] expected = new Object[3];
    list.toArray(expected);
    assertEquals(expected, list.toArray());
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#add(java.lang.Object)}.
   */
  public final void testAdd() {
    Object o = new Object();
    list.add(o);
    assertSame(o, list.get(0));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#set(int, java.lang.Object)}.
   */
  public final void testSet() {
    Object o = new Object();
    list.add(new Object());
    list.add(new Object());
    list.add(new Object());
    list.set(1, o);
    assertSame(o, list.get(1));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#insert(int, java.lang.Object)}.
   */
  public final void testInsert() {
    Object o = new Object();
    list.add(new Object());
    list.add(new Object());
    list.add(new Object());
    list.insert(2, o);
    assertSame(o, list.get(2));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#remove(int)}.
   */
  public final void testRemoveInt() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertSame(o, list.remove(1));
    assertEquals(2, list.size());
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleArrayList#remove(java.lang.Object)}.
   */
  public final void testRemoveObject() {
    Object o = new Object();
    list.add(new Object());
    list.add(o);
    list.add(new Object());
    assertTrue(list.remove(o));
    assertEquals(2, list.size());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#clear()}.
   */
  public final void testClear() {
    assertTrue(list.isEmpty());
    list.add(new Object());
    list.add(new Object());
    assertFalse(list.isEmpty());
    list.clear();
    assertTrue(list.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#sort()}.
   */
  public final void testSort() {
    Object o1 = "o1"; //$NON-NLS-1$
    Object o2 = "o1"; //$NON-NLS-1$
    Object o3 = "o1"; //$NON-NLS-1$
    list.add(o3); //$NON-NLS-1$
    list.add(o1); //$NON-NLS-1$
    list.add(o2); //$NON-NLS-1$
    list.sort();
    assertSame(o1, list.get(0));
    assertSame(o2, list.get(1));
    assertSame(o3, list.get(2));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#iterator()}.
   */
  public final void testIterator() {
    Object o = "o1"; //$NON-NLS-1$
    list.add(o);
    Object o2 = "o2"; //$NON-NLS-1$
    list.add(o2);
    Object o3 = "o3"; //$NON-NLS-1$
    list.add(o3);
    Iterator it = list.iterator();
    if (it.hasNext()) {
      assertEquals(o, it.next());
    } else {
      fail();
    }
    if (it.hasNext()) {
      assertEquals(o2, it.next());
    } else {
      fail();
    }
    if (it.hasNext()) {
      assertEquals(o3, it.next());
    } else {
      fail();
    }
    assertFalse(it.hasNext());
    try {
      assertEquals(null, it.next());
    } catch (Throwable t) { // NOPMD
      return;
    }
    fail();
  }

  /**
   * Test method for {@link net.wrka.util.SimpleArrayList#enumeration()}.
   */
  public final void testEnumeration() {
    Object o = "o1"; //$NON-NLS-1$
    list.add(o);
    Object o2 = "o2"; //$NON-NLS-1$
    list.add(o2);
    Object o3 = "o3"; //$NON-NLS-1$
    list.add(o3);
    Enumeration e = list.enumeration();
    if (e.hasMoreElements()) {
      assertEquals(o, e.nextElement());
    } else {
      fail();
    }
    if (e.hasMoreElements()) {
      assertEquals(o2, e.nextElement());
    } else {
      fail();
    }
    if (e.hasMoreElements()) {
      assertEquals(o3, e.nextElement());
    } else {
      fail();
    }
    assertFalse(e.hasMoreElements());
    try {
      assertEquals(null, e.nextElement());
    } catch (Throwable t) { // NOPMD
      return;
    }
    fail();
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    list = new SimpleArrayList(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    list = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 18:00:55 Slobodan
   Initial version

*****************************************************************************^*/
