/*
 * @(#)SimpleHashMapTest.java  1.0.0 / 2008-05-12
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util;

import java.util.Iterator;

import net.wrka.junit.ExtendedTestCase;

/**
 * SimpleHashMapTest Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SimpleHashMapTest extends ExtendedTestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private SimpleHashMap map = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SimpleHashMapTest object.
   *
   * @param name the test name
   */
  public SimpleHashMapTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#size()}.
   */
  public final void testSize() {
    assertEquals(0, map.size());
    map.put(new Object(), new Object());
    assertEquals(1, map.size());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#capacity()}.
   */
  public final void testCapacity() {
    assertEquals(7, map.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#empty()}.
   */
  public final void testEmpty() {
    assertTrue(map.empty());
    map.put(new Object(), new Object());
    assertFalse(map.empty());
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleHashMap#containsKey(java.lang.Object)}.
   */
  public final void testContainsKey() {
    Object key = new Object();
    Object value = new Object();
    map.put(key, value);
    assertTrue(map.containsKey(key));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleHashMap#containsValue(java.lang.Object)}.
   */
  public final void testContainsValue() {
    Object key = new Object();
    Object value = new Object();
    map.put(key, value);
    assertTrue(map.containsValue(value));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#get(java.lang.Object)}.
   */
  public final void testGet() {
    Object key = new Object();
    Object value = new Object();
    map.put(key, value);
    assertSame(value, map.get(key));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#
   *                                   put(java.lang.Object, java.lang.Object)}.
   */
  public final void testPut() {
    Object key = new Object();
    Object value = new Object();
    map.put(key, value);
    assertSame(value, map.put(key, value));
  }

  /**
   * Test method for
   * {@link net.wrka.util.SimpleHashMap#remove(java.lang.Object)}.
   */
  public final void testRemoveObject() {
    Object key = new Object();
    Object value = new Object();
    map.put(key, value);
    assertSame(value, map.get(key));
    map.remove(key);
    assertNull(map.get(key));
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#clear()}.
   */
  public final void testClear() {
    map.put(new Object(), new Object());
    map.put(new Object(), new Object());
    assertFalse(map.empty());
    map.clear();
    assertTrue(map.empty());
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#keys()}.
   */
  public final void testKeys() {
    for (int i = 0; i < 100; i++) {
      String s = String.valueOf(i);
      map.put(s, s);
    }
    Iterator it = map.keys();
    int count = 0;
    while (it.hasNext()) {
      String s = (String)it.next();
      int i = Integer.parseInt(s);
      if ((i < 0) || (i >= 100)) {
        fail("Invalid key"); //$NON-NLS-1$
      }
      count++;
    }
    assertSame(count, 100);
  }

  /**
   * Test method for {@link net.wrka.util.SimpleHashMap#values()}.
   */
  public final void testValues() {
    for (int i = 0; i < 100; i++) {
      String s = String.valueOf(i);
      map.put(s, s);
    }
    Iterator it = map.values();
    int count = 0;
    while (it.hasNext()) {
      String s = (String)it.next();
      int i = Integer.parseInt(s);
      if ((i < 0) || (i >= 100)) {
        fail("Invalid value"); //$NON-NLS-1$
      }
      count++;
    }
    assertSame(count, 100);
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    map = new SimpleHashMap(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    map = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-12 21:57:27 Slobodan
   Initial version

*****************************************************************************^*/
