/*
 * @(#)IntQueueTest.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

import junit.framework.TestCase;

/**
 * IntQueue Class Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class IntQueueTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private IntQueue queue = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new IntQueueTest object.
   *
   * @param name the test name
   */
  public IntQueueTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(queue.isEmpty());
    queue.insert(0);
    assertFalse(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#size()}.
   */
  public final void testSize() {
    assertEquals(0, queue.size());
    queue.insert(0);
    assertEquals(1, queue.size());
    queue.insert(1);
    assertEquals(2, queue.size());
    queue.clear();
    assertEquals(0, queue.size());
  }

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#capacity()}.
   */
  public final void testCapacity() {
    assertEquals(7, queue.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#clear()}.
   */
  public final void testClear() {
    queue.insert(1);
    assertFalse(queue.isEmpty());
    queue.insert(1);
    queue.clear();
    assertTrue(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#remove()}.
   */
  public final void testRemove() {
    queue.insert(0);
    queue.insert(1);
    assertEquals(0, queue.remove());
    assertEquals(1, queue.remove());
  }

  /**
   * Test method for {@link net.wrka.util.queue.IntQueue#insert(int)}.
   */
  public final void testInsert() {
    queue.insert(0);
    assertEquals(0, queue.remove());
    queue.insert(1);
    assertEquals(1, queue.remove());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    queue = new IntQueue(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    queue = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 19:36:29 Slobodan
   Initial version

*****************************************************************************^*/
