/*
 * @(#)ObjectQueueTest.java  1.0.0 / 2008-05-07
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

import junit.framework.TestCase;

/**
 * ObjectQueue Class Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ObjectQueueTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private ObjectQueue queue = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ObjectQueueTest object.
   *
   * @param name the test name
   */
  public ObjectQueueTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.queue.ObjectQueue#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(queue.isEmpty());
    queue.insert(new Object());
    assertFalse(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.ObjectQueue#size()}.
   */
  public final void testSize() {
    assertEquals(0, queue.size());
    queue.insert(new Object());
    assertEquals(1, queue.size());
    queue.insert(new Object());
    assertEquals(2, queue.size());
    queue.clear();
    assertEquals(0, queue.size());
  }

  /**
   * Test method for {@link net.wrka.util.queue.ObjectQueue#capacity()}.
   */
  public final void testCapacity() {
    assertEquals(7, queue.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.queue.ObjectQueue#clear()}.
   */
  public final void testClear() {
    queue.insert(new Object());
    assertFalse(queue.isEmpty());
    queue.insert(new Object());
    queue.clear();
    assertTrue(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.ObjectQueue#remove()}.
   */
  public final void testRemove() {
    Object first = new Object();
    Object second = new Object();
    queue.insert(first);
    queue.insert(second);
    assertSame(first, queue.remove());
    assertSame(second, queue.remove());
  }

  /**
   * Test method for
   * {@link net.wrka.util.queue.ObjectQueue#insert(java.lang.Object)}.
   */
  public final void testInsert() {
    Object first = new Object();
    Object second = new Object();
    queue.insert(first);
    assertSame(first, queue.remove());
    queue.insert(second);
    assertSame(second, queue.remove());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    queue = new ObjectQueue(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    queue = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-07 20:14:36 Slobodan
   Initial version

*****************************************************************************^*/
