/*
 * @(#)StringQueueTest.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

import junit.framework.TestCase;

/**
 * StringQueue Class Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StringQueueTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private StringQueue queue = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StringQueueTest object.
   *
   * @param name the test name
   */
  public StringQueueTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.queue.StringQueue#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(queue.isEmpty());
    queue.insert("s1"); //$NON-NLS-1$
    assertFalse(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.StringQueue#size()}.
   */
  public final void testSize() {
    assertEquals(0, queue.size());
    queue.insert("s1"); //$NON-NLS-1$
    assertEquals(1, queue.size());
    queue.insert("s2"); //$NON-NLS-1$
    assertEquals(2, queue.size());
    queue.clear();
    assertEquals(0, queue.size());
  }

  /**
   * Test method for {@link net.wrka.util.queue.StringQueue#capacity()}.
   */
  public final void testCapacity() {
    assertEquals(7, queue.capacity());
  }

  /**
   * Test method for {@link net.wrka.util.queue.StringQueue#clear()}.
   */
  public final void testClear() {
    queue.insert("s1"); //$NON-NLS-1$
    assertFalse(queue.isEmpty());
    queue.insert("s2"); //$NON-NLS-1$
    queue.clear();
    assertTrue(queue.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.queue.StringQueue#remove()}.
   */
  public final void testRemove() {
    String first = "s1"; //$NON-NLS-1$
    String second = "s1"; //$NON-NLS-1$
    queue.insert(first);
    queue.insert(second);
    assertSame(first, queue.remove());
    assertSame(second, queue.remove());
  }

  /**
   * Test method for
   * {@link net.wrka.util.queue.StringQueue#insert(java.lang.String)}.
   */
  public final void testInsert() {
    String first = "s1"; //$NON-NLS-1$
    String second = "s1"; //$NON-NLS-1$
    queue.insert(first);
    assertSame(first, queue.remove());
    queue.insert(second);
    assertSame(second, queue.remove());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    queue = new StringQueue(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    queue = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:09:43 Slobodan
   Initial version

*****************************************************************************^*/
