/*
 * @(#)SyncBlockingQueueTest.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.queue;

import junit.framework.TestCase;

/**
 * SyncBlockingQueue Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class SyncBlockingQueueTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private SyncBlockingQueue queue = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new SyncBlockingQueueTest object.
   *
   * @param name the test name
   */
  public SyncBlockingQueueTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.queue.SyncBlockingQueue#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(queue.isEmpty());
    queue.put(new Object());
    assertFalse(queue.isEmpty());
  }

  /**
   * Test method for
   * {@link net.wrka.util.queue.SyncBlockingQueue#put(java.lang.Object)}.
   */
  public final void testPut() {
    Object first = new Object();
    Object second = new Object();
    queue.put(first);
    assertSame(first, queue.get());
    queue.put(second);
    assertSame(second, queue.get());
  }

  /**
   * Test method for {@link net.wrka.util.queue.SyncBlockingQueue#get()}.
   */
  public final void testGet() {
    Object first = new Object();
    Object second = new Object();
    queue.put(first);
    queue.put(second);
    assertSame(first, queue.get());
    assertSame(second, queue.get());
  }

  /**
   * Test method for {@link net.wrka.util.queue.SyncBlockingQueue#front()}.
   */
  public final void testFront() {
    Object first = new Object();
    Object second = new Object();
    queue.put(first);
    queue.put(second);
    assertSame(first, queue.front());
  }

  /**
   * Test method for {@link net.wrka.util.queue.SyncBlockingQueue#rear()}.
   */
  public final void testRear() {
    Object first = new Object();
    Object second = new Object();
    queue.put(first);
    queue.put(second);
    assertSame(second, queue.rear());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    queue = new SyncBlockingQueue();
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    queue = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:18:00 Slobodan
   Initial version

*****************************************************************************^*/
