/*
 * @(#)IntStackTest.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

import junit.framework.TestCase;

/**
 * IntStack Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class IntStackTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private IntStack stack = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new IntStackTest object.
   *
   * @param name the test name
   */
  public IntStackTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(stack.isEmpty());
    stack.push(0);
    assertFalse(stack.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#size()}.
   */
  public final void testSize() {
    assertEquals(0, stack.size());
    stack.push(0);
    assertEquals(1, stack.size());
    stack.push(1);
    assertEquals(2, stack.size());
    stack.clear();
    assertEquals(0, stack.size());
  }

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#push(int)}.
   */
  public final void testPush() {
    stack.push(0);
    assertEquals(0, stack.pop());
    stack.push(1);
    assertEquals(1, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#pop()}.
   */
  public final void testPop() {
    stack.push(0);
    stack.push(1);
    assertEquals(1, stack.pop());
    assertEquals(0, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#top()}.
   */
  public final void testTop() {
    stack.push(0);
    stack.push(1);
    assertEquals(1, stack.top());
    assertEquals(1, stack.top()); // duplicate OK.
  }

  /**
   * Test method for {@link net.wrka.util.stack.IntStack#clear()}.
   */
  public final void testClear() {
    stack.push(1);
    assertFalse(stack.isEmpty());
    stack.push(1);
    stack.clear();
    assertTrue(stack.isEmpty());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    stack = new IntStack(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    stack = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:47:25 Slobodan
   Initial version

*****************************************************************************^*/
