/*
 * @(#)ObjectStackTest.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

import junit.framework.TestCase;

/**
 * ObjectStack Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class ObjectStackTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private ObjectStack stack = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new ObjectStackTest object.
   *
   * @param name the test name
   */
  public ObjectStackTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.stack.ObjectStack#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(stack.isEmpty());
    stack.push(new Object());
    assertFalse(stack.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.stack.ObjectStack#size()}.
   */
  public final void testSize() {
    assertEquals(0, stack.size());
    stack.push(new Object());
    assertEquals(1, stack.size());
    stack.push(new Object());
    assertEquals(2, stack.size());
    stack.clear();
    assertEquals(0, stack.size());
  }

  /**
   * Test method for
   * {@link net.wrka.util.stack.ObjectStack#push(java.lang.Object)}.
   */
  public final void testPush() {
    Object first = new Object();
    Object second = new Object();
    stack.push(first);
    assertEquals(first, stack.pop());
    stack.push(second);
    assertEquals(second, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.ObjectStack#pop()}.
   */
  public final void testPop() {
    Object first = new Object();
    Object second = new Object();
    stack.push(first);
    stack.push(second);
    assertEquals(second, stack.pop());
    assertEquals(first, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.ObjectStack#top()}.
   */
  public final void testTop() {
    Object first = new Object();
    Object second = new Object();
    stack.push(first);
    stack.push(second);
    assertEquals(second, stack.top());
    assertEquals(second, stack.top()); // duplicate OK.
  }

  /**
   * Test method for {@link net.wrka.util.stack.ObjectStack#clear()}.
   */
  public final void testClear() {
    stack.push(new Object());
    assertFalse(stack.isEmpty());
    stack.push(new Object());
    stack.clear();
    assertTrue(stack.isEmpty());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    stack = new ObjectStack(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    stack = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:55:50 Slobodan
   Initial version

*****************************************************************************^*/
