/*
 * @(#)StringStackTest.java  1.0.0 / 2008-05-08
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.util.stack;

import junit.framework.TestCase;

/**
 * StringStackTest Class.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class StringStackTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private StringStack stack = null;

/*^**************************************************************************^*/
/*- Public Constructor.                                                      -*/
/*^**************************************************************************^*/

  /**
   * Constructs new StringStackTest object.
   *
   * @param name the test name
   */
  public StringStackTest(String name) {
    super(name);
  }

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Test method for {@link net.wrka.util.stack.StringStack#isEmpty()}.
   */
  public final void testIsEmpty() {
    assertTrue(stack.isEmpty());
    stack.push("s1"); //$NON-NLS-1$
    assertFalse(stack.isEmpty());
  }

  /**
   * Test method for {@link net.wrka.util.stack.StringStack#size()}.
   */
  public final void testSize() {
    assertEquals(0, stack.size());
    stack.push("s1"); //$NON-NLS-1$
    assertEquals(1, stack.size());
    stack.push("s2"); //$NON-NLS-1$
    assertEquals(2, stack.size());
    stack.clear();
    assertEquals(0, stack.size());
  }

  /**
   * Test method for
   * {@link net.wrka.util.stack.StringStack#push(java.lang.String)}.
   */
  public final void testPush() {
    String first = "s1"; //$NON-NLS-1$
    String second = "s2"; //$NON-NLS-1$
    stack.push(first);
    assertEquals(first, stack.pop());
    stack.push(second);
    assertEquals(second, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.StringStack#pop()}.
   */
  public final void testPop() {
    String first = "s1"; //$NON-NLS-1$
    String second = "s2"; //$NON-NLS-1$
    stack.push(first);
    stack.push(second);
    assertEquals(second, stack.pop());
    assertEquals(first, stack.pop());
  }

  /**
   * Test method for {@link net.wrka.util.stack.StringStack#top()}.
   */
  public final void testTop() {
    String first = "s1"; //$NON-NLS-1$
    String second = "s2"; //$NON-NLS-1$
    stack.push(first);
    stack.push(second);
    assertEquals(second, stack.top());
    assertEquals(second, stack.top()); // duplicate OK.
  }

  /**
   * Test method for {@link net.wrka.util.stack.StringStack#clear()}.
   */
  public final void testClear() {
    stack.push("s1"); //$NON-NLS-1$
    assertFalse(stack.isEmpty());
    stack.push("s2"); //$NON-NLS-1$
    stack.clear();
    assertTrue(stack.isEmpty());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    stack = new StringStack(7);
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    stack = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-05-08 20:56:12 Slobodan
   Initial version

*****************************************************************************^*/
