/*
 * @(#)XMLElementStackTest.java  1.0.0 / 2008-04-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import junit.framework.TestCase;

/**
 * XMLElementStack Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLElementStackTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private XMLElementStack stack = null;

/*^**************************************************************************^*/
/*- Test Methods.                                                            -*/
/*^**************************************************************************^*/

  /**
   * Tests <tt>isEmpty</tt>, <tt>putElement</tt> and <tt>clear</tt> methods.
   */
  public void testPutClearEmpty() {
    assertTrue(stack.isEmpty());
    stack.putElement("Test", XMLElement.EMPTY_ELEMENT); //$NON-NLS-1$
    assertFalse(stack.isEmpty());
    stack.clear();
    assertTrue(stack.isEmpty());
  }

  /**
   * Tests <tt>putElement</tt>, <tt>firstElementName</tt> and
   * <tt>firstElementKind</tt> method.
   */
  public void testFirstElement() {
    String firstElementName = "Test"; //$NON-NLS-1$
    int firstElementKind = XMLElement.EMPTY_ELEMENT;
    stack.putElement(firstElementName, firstElementKind);
    assertEquals(firstElementName, stack.firstElementName());
    assertEquals(firstElementKind, stack.firstElementKind());
  }

  /**
   * Tests <tt>secondElementName</tt> and <tt>secondElementKind</tt> method.
   */
  public void testSecondElement() {
    String secondElementName = "Second"; //$NON-NLS-1$
    int secondElementKind = XMLElement.EMPTY_ELEMENT;
    stack.putElement(secondElementName, secondElementKind);
    stack.putElement("First", XMLElement.INLINE_ELEMENT); //$NON-NLS-1$
    assertEquals(secondElementName, stack.secondElementName());
    assertEquals(secondElementKind, stack.secondElementKind());
  }

  /**
   * Tests <tt>putElement</tt> and <tt>removeElement</tt> method.
   */
  public void testPutRemove() {
    stack.putElement("Empty", XMLElement.EMPTY_ELEMENT); //$NON-NLS-1$
    stack.putElement("Inline", XMLElement.INLINE_ELEMENT); //$NON-NLS-1$
    stack.putElement("Multiline", XMLElement.MULTILINE_ELEMENT); //$NON-NLS-1$
    assertEquals(XMLElement.MULTILINE_ELEMENT, stack.firstElementKind());
    stack.removeElement();
    assertEquals(XMLElement.INLINE_ELEMENT, stack.firstElementKind());
    stack.removeElement();
    assertEquals(XMLElement.EMPTY_ELEMENT, stack.firstElementKind());
    stack.removeElement();
    assertTrue(stack.isEmpty());
  }

  /**
   * Tests <tt>putElement</tt> and <tt>getElementCount</tt> method.
   */
  public void testGetElementCount() {
    for (int i = 0; i < XMLElementStack.DEFAULT_CAPACITY; i++) {
      stack.putEmptyElement(String.valueOf(i));
    }
    assertEquals(XMLElementStack.DEFAULT_CAPACITY, stack.getElementCount());
    assertEquals(String.valueOf(XMLElementStack.DEFAULT_CAPACITY - 1),
                 stack.firstElementName());
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    stack = new XMLElementStack();
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    stack = null;
    super.tearDown();
  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-23 20:19:20 Slobodan
   Initial version

*****************************************************************************^*/
