/*
 * @(#)XMLPrintServerTest.java  1.0.0 / 2008-04-23
 *
 * Copyright (c) 2008, Slobodan Vrkacevic. All rights reserved.
 *
 * This software is licensed under the LGPL, version 2.1, license.
 * A copy of the license is included in the file LICENSE-LGPL.txt.
 */
package net.wrka.xml.simple;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import junit.framework.TestCase;
import net.wrka.xml.simple.sax.DocumentHandler;
import net.wrka.xml.simple.sax.Parser;
import net.wrka.xml.simple.sax.ParsingException;

/**
 * XMLPrintServer Test.
 *
 * @author Slobodan Vrkacevic (slobodan.vrkacevic@gmail.com)
 * @version 1.0.0
 */
public class XMLPrintServerTest extends TestCase {

/*^**************************************************************************^*/
/*- Private Field.                                                           -*/
/*^**************************************************************************^*/

  private File testFile = null;

/*^**************************************************************************^*/
/*- Test Method.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Tests <tt>isEmpty</tt>, <tt>putElement</tt> and <tt>clear</tt> methods.
   */
  public void testXMLPrintServer() {
    try {
      FileWriter fw = new FileWriter(testFile);
      XMLPrintServer ps = new XMLPrintServer(fw);
      ps.beginDocument("comment", "<!-- Warning -->");
      // CS:OFF
        ps.beginElement("Root");
          ps.beginInlineElement("Inline");
          ps.setAttribute("a1", true);
          ps.appendContent(true);
          ps.appendContentSpace();
          ps.appendContent(true);
          ps.endElement();
          ps.beginElement("Multiline");
          ps.setAttribute("a2", 2);
          ps.appendContent("Content");
          ps.endElement();
        ps.endElement();
      // CS:ON
      ps.endDocument();
      fw.close();
      FileReader fr = new FileReader(testFile);
      TestHandler th = new TestHandler();
      Parser.parse(fr, th);
      fr.close();
      assertTrue(th.passed());
    } catch (IOException ioe) {
      fail(ioe.getMessage());
    } catch (ParsingException pe) {
      fail(pe.getMessage());
    }
  }

/*^**************************************************************************^*/
/*- Overridden Methods.                                                      -*/
/*^**************************************************************************^*/

  /**
   * {@inheritDoc}
   */
  protected void setUp() throws Exception {
    super.setUp();
    testFile = File.createTempFile("XMLPrintServerTest", ".xml");
  }

  /**
   * {@inheritDoc}
   */
  protected void tearDown() throws Exception {
    if (testFile != null) {
      testFile.delete();
      testFile = null;
    }
    super.tearDown();
  }

/*^**************************************************************************^*/
/*- Inner Class.                                                             -*/
/*^**************************************************************************^*/

  /**
   * Test Handler.
   */
  static final class TestHandler implements DocumentHandler {

    private int state;

    /**
     * Returns <tt>true</tt> if test has passed.
     *
     * @return <tt>true</tt> if test has passed
     */
    public boolean passed() {
      return (state == 6);
    }

    /**
     * {@inheritDoc}
     */
    public void startDocument() throws ParsingException {
      if (state != 0) {
        throw new ParsingException("Invalid parser state"); //$NON-NLS-1$ NOPMD
      }
      state = 1;
    }

    /**
     * {@inheritDoc}
     */
    public void endDocument() throws ParsingException {
      if (state != 5) {
        throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
      }
      state = 6;
    }

    /**
     * {@inheritDoc}
     */
    public void startElement(String tag, XMLAttributes attrs) throws
                                                             ParsingException {
      switch (state) {
        case 1:
          if (!"Root".equals(tag) || !attrs.isEmpty()) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          state = 2;
          break;
        case 2:
          if (!"Inline".equals(tag) || attrs.isEmpty() || //$NON-NLS-1$
              !attrs.getBoolean("a1", false)) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          state = 3;
          break;
        case 3:
          if (!"Multiline".equals(tag) || attrs.isEmpty() || //$NON-NLS-1$
              (attrs.getInteger("a2", 0) != 2)) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          state = 4;
          break;
        default:
          throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
      }
    }

    /**
     * {@inheritDoc}
     */
    public void endElement(String tag) throws ParsingException {
      switch (state) {
        case 2:
          if (!"Root".equals(tag)) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          break;
        case 3:
          if (!"Inline".equals(tag)) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          break;
        case 4:
          if (!"Multiline".equals(tag)) { //$NON-NLS-1$
            throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
          state = 5;
          break;
        default:
          break;
      }
    }

    /**
     * {@inheritDoc}
     */
    public void characters(String str) throws ParsingException {
      if (str != null) {
        str = str.trim();
        if  (str.length() > 0) {
          switch (state) {
            case 3:
              if (!"true true".equals(str)) { //$NON-NLS-1$
                throw new ParsingException(
                                         "Invalid parser state"); //$NON-NLS-1$
              }
              break;
            case 4:
              if (!"Content".equals(str)) { //$NON-NLS-1$
                throw new ParsingException(
                                         "Invalid parser state"); //$NON-NLS-1$
              }
              break;
            default:
              throw new ParsingException("Invalid parser state"); //$NON-NLS-1$
          }
        }
      }
    }

  }

}

/*^*****************************************************************************
  File History:

 - 2008-04-23 14:27:24 Slobodan
   Initial version

*****************************************************************************^*/
